/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 203                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 23:10:45 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file bs_bot.c
 *   @brief This is the Module file for BotServ BOT Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the BotServ BOT command.
 *
 *   The BotServ BOT Command allows the user to manage the<br/>
 *   Networkbots - (adds/deletes/changes) a NetworkBot.
 *
 *   Syntax is: /msg BotServ BOT { ADD | CHANGE | DEL }
 *
 *<table border="0">
 *    <tr><td> ADD		</td><td> Syntax is: /msg BotServ BOT ADD		</td><td> nick user host realname					</td></tr>
 *    <tr><td> DEL		</td><td> Syntax is: /msg BotServ BOT DEL		</td><td> nick										</td></tr>
 *    <tr><td> CHANGE	</td><td> Syntax is: /msg BotServ BOT CHANGE	</td><td> oldnick newnick [user [host [realname]]]	</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "bs"
#define MODUL "bot"
#define MODUL_FILE "bs_bot"
#define MODUL_VERSION "1.0"

MOD_BOOL bs_bot_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_bot_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_bot_change(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL bs_bot_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL bs_bot_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"bot option[1]","bot");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"bot",is_services_oper,INFINIT); /* just to add a displaylevel for /msg bs help
															  this should always be the lowest used Module level! */
	
	addCommand(thread,&Module,"bot add",bs_bot_add,is_services_oper,INFINIT,8,0);
	addCommand(thread,&Module,"bot del",bs_bot_del,is_services_oper,INFINIT,5,5);
	addCommand(thread,&Module,"bot change",bs_bot_change,is_services_oper,INFINIT,6,0);
	
	addCommand(thread,&Module,"help bot",bs_bot_help,is_services_oper,INFINIT,4,4);
	addCommand(thread,&Module,"help",bs_bot_help_listing,is_services_oper,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief BS BOT Add function for the BotServ BOT Command.
 * 
 * The BotServ BOT Add Command allows an admin to add a Bot to the Networks BotList
 *  
 * Syntax is: /msg BotServ BOT ADD nick user host realname
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_bot_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	
	/* Initialise Variables to (NULL) */

	/*
	 * Bot Add Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"bot add")){
		if(CheckOption(thread,snick,nick,READONLY)){
			if(!stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_bs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[5])))){
				if(!is_valid_chars(param[7],"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.-") && !is_valid_chars(param[6],"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_") && !is_valid_chars(param[5],"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_")){
					domysql(thread,"INSERT INTO `orion_bs_core` (`botnick`,`botident`,`bothost`,`botrealname`,`private`,`time`) VALUES ('%s','%s','%s','%s','0','%d')",mysql_quote(thread,param[5]),mysql_quote(thread,param[6]),mysql_quote(thread,param[7]),mysql_quote(thread,_dstrjoin(' ',param,8)),time(NULL));
					do_sqline(thread,m->servername,param[5],0,time(NULL),"nick used by orion");
					createuser(thread,1,param[5],param[6],param[7],IRC.BSBotsMode,_dstrjoin(' ',param,8));
					send_lang(thread,language,howtosend,snick,nick,"BS_BOT_BOT_X_ADDED",param[5]);
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION","add");
			}else send_lang(thread,language,howtosend,snick,nick,"BS_BOT_BOT_X_ALREADY_KNOWN",param[5]);
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS BOT Del function for the BotServ BOT Command.
 * 
 * The BotServ BOT Del Command allows an admin to delete a Bot from the Networks BotList
 *  
 * Syntax is: /msg BotServ BOT DEL nick
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_bot_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	
	/* Initialise Variables to (NULL) */

	/*
	 * Bot Del Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"bot del")){
		if(CheckOption(thread,snick,nick,READONLY)){
			if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_bs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[5])))){
				if(!is_valid_chars(param[5],"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_")){
					domysql(thread,"DELETE FROM `orion_bs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[5]));
					/* 
					InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
					domysql(thread,"DELETE FROM `botassign` WHERE `bot`='%s'",mysql_quote(thread,param[5]));
					domysql(thread,"DELETE FROM `orion_bs_badwords` WHERE `botnick`='%s'",mysql_quote(thread,param[5]));
					*/
					do_unsqline(thread,bs->nick,m->servername,param[5]);
					do_quit(thread,param[5],"bot deleted");
					send_lang(thread,language,howtosend,snick,nick,"BS_BOT_BOT_X_DELETED",param[5]);
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION","del");
			}else send_lang(thread,language,howtosend,snick,nick,"BS_BOT_BOT_X_NOT_KNOWN",param[5]);
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief BS BOT Change function for the BotServ BOT Command.
 * 
 * The BotServ BOT Change Command allows an admin to change a Bot from the Networks BotList
 *  
 * Syntax is: /msg BotServ BOT CHANGE ldnick newnick [user [host [realname]]]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_bot_change(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL bot;
	deepSQL channels;
	int i=0;
	
	/* Initialise Variables to (NULL) */
	bzero(&bot,sizeof(bot));
	bzero(&channels,sizeof(channels));

	/*
	 * Bot Change Command
	 */
 	//:shock PRIVMSG BotServ :bot change NICK IDENT HOST REALNAME
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"bot change")){
		if(CheckOption(thread,snick,nick,READONLY)){
		//:shock PRIVMSG BotServ :bot change oldnick NICK IDENT
			if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_bs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[5])))){
				if(!is_valid_chars(param[6],"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_")){
					if(param[9]){
						domysql(thread,"UPDATE `orion_bs_core` SET `botnick`='%s',`botident`='%s',`bothost`='%s',`botrealname`='%s' WHERE `botnick`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[7]),mysql_quote(thread,param[8]),mysql_quote(thread,_dstrjoin(' ',param,9)),mysql_quote(thread,param[5]));
						/* 
						InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
						domysql(thread,"UPDATE `botassign` SET `bot`='%s' WHERE `bot`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));
						domysql(thread,"UPDATE `orion_bs_badwords` SET `botnick`='%s' WHERE `botnick`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));
						*/
						delStayNick(thread,param[5]);
						do_unsqline(thread,bs->nick,m->servername,param[5]);
						do_quit(thread,param[5],"bot changed, it would be back in a second");
						
						do_sqline(thread,m->servername,param[6],0,time(NULL),"nick used by orion");
						createuser(thread,1,param[6],param[7],param[8],IRC.BSBotsMode,_dstrjoin(' ',param,9));
						
						channels.res    = doSelectMySQL(thread,"SELECT `channel`,`datecreate` FROM `orion_cs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[6]));
						channels.fields = mysql_num_fields(channels.res);
						
						if(mysql_num_rows(channels.res)>0){
							for(i=0;i<channels.fields;i++){
								channels.row = mysql_fetch_row(channels.res);
								
								do_join(thread,1,param[6],channels.row[0],(time_t)stol(channels.row[1]));
								do_mode(thread,param[6],channels.row[0],(time_t)stol(channels.row[1]),"+oaq %s %s %s",param[6],param[6],param[6]);
							}
						}
						mysql_free_result_myr(channels.res);
						
						send_lang(thread,language,howtosend,snick,nick,"BS_BOT_BOT_X_CHANGED",param[5]);
					}else if(param[8] && !param[9]){
						domysql(thread,"UPDATE `orion_bs_core` SET `botnick`='%s',`botident`='%s',`bothost`='%s' WHERE `botnick`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[7]),mysql_quote(thread,param[8]),mysql_quote(thread,param[5]));
						/* 
						InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
						domysql(thread,"UPDATE `botassign` SET `bot`='%s' WHERE `bot`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));
						domysql(thread,"UPDATE `orion_bs_badwords` SET `botnick`='%s' WHERE `botnick`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));
						*/
						delStayNick(thread,param[5]);
						do_unsqline(thread,bs->nick,m->servername,param[5]);
						do_quit(thread,param[5],"bot changed, it would be back in a second");
						
						do_sqline(thread,m->servername,param[6],0,time(NULL),"nick used by orion");
						createuser(thread,1,param[6],param[7],param[8],IRC.BSBotsMode,mysql_sel_var(thread,"SELECT `botrealname` FROM `orion_bs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[6])));
						
						channels.res    = doSelectMySQL(thread,"SELECT `channel`,`datecreate` FROM `orion_cs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[6]));
						channels.fields = mysql_num_fields(channels.res);
						
						if(mysql_num_rows(channels.res)>0){
							for(i=0;i<channels.fields;i++){
								channels.row = mysql_fetch_row(channels.res);
								
								do_join(thread,1,param[6],channels.row[0],(time_t)stol(channels.row[1]));
								do_mode(thread,param[6],channels.row[0],(time_t)stol(channels.row[1]),"+oaq %s %s %s",param[6],param[6],param[6]);
							}
						}
						mysql_free_result_myr(channels.res);
						
						send_lang(thread,language,howtosend,snick,nick,"BS_BOT_BOT_X_CHANGED",param[5]);
					}else if(param[7] && !param[8]){
						domysql(thread,"UPDATE `orion_bs_core` SET `botnick`='%s',`botident`='%s',`bothost`='%s' WHERE `botnick`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[7]),mysql_quote(thread,param[8]),mysql_quote(thread,param[5]));
						/* 
						InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore							
						domysql(thread,"UPDATE `botassign` SET `bot`='%s' WHERE `bot`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));
						domysql(thread,"UPDATE `orion_bs_badwords` SET `botnick`='%s' WHERE `botnick`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));
						*/
						delStayNick(thread,param[5]);
						do_unsqline(thread,bs->nick,m->servername,param[5]);
						do_quit(thread,param[5],"bot changed, it would be back in a second");
						
						do_sqline(thread,m->servername,param[6],0,time(NULL),"nick used by orion");
						
						bot.res = doSelectMySQL(thread,"SELECT `bothost`,`botrealname` FROM `orion_bs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[6]));
						if(mysql_num_rows(bot.res)>0){
							bot.row = mysql_fetch_row(bot.res);
							createuser(thread,1,param[6],param[7],bot.row[0],IRC.BSBotsMode,bot.row[1]);
						}
						mysql_free_result_myr(bot.res);
						
						channels.res    = doSelectMySQL(thread,"SELECT `channel`,`datecreate` FROM `orion_cs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[6]));
						channels.fields = mysql_num_fields(channels.res);
						
						if(mysql_num_rows(channels.res)>0){
							for(i=0;i<channels.fields;i++){
								channels.row = mysql_fetch_row(channels.res);
								
								do_join(thread,1,param[6],channels.row[0],(time_t)stol(channels.row[1]));
								do_mode(thread,param[6],channels.row[0],(time_t)stol(channels.row[1]),"+oaq %s %s %s",param[6],param[6],param[6]);
							}
						}
						mysql_free_result_myr(channels.res);
						
						send_lang(thread,language,howtosend,snick,nick,"BS_BOT_BOT_X_CHANGED",param[5]);
					}else if(param[6] && !param[7]){
						domysql(thread,"UPDATE `orion_bs_core` SET `botnick`='%s',`botident`='%s',`bothost`='%s' WHERE `botnick`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[7]),mysql_quote(thread,param[8]),mysql_quote(thread,param[5]));
						/* 
						InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
						domysql(thread,"UPDATE `botassign` SET `bot`='%s' WHERE `bot`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));
						domysql(thread,"UPDATE `orion_bs_badwords` SET `botnick`='%s' WHERE `botnick`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[5]));
						*/
						delStayNick(thread,param[5]);
						do_unsqline(thread,bs->nick,m->servername,param[5]);
						do_quit(thread,param[5],"bot changed, it would be back in a second");
						
						do_sqline(thread,m->servername,param[6],0,time(NULL),"nick used by orion");
						
						bot.res = doSelectMySQL(thread,"SELECT `bothost`,`botrealname`,`botident` FROM `orion_bs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[6]));
						if(mysql_num_rows(bot.res)>0){
							bot.row = mysql_fetch_row(bot.res);
							createuser(thread,1,param[6],bot.row[2],bot.row[0],IRC.BSBotsMode,bot.row[1]);
						}
						mysql_free_result_myr(bot.res);
						
						channels.res    = doSelectMySQL(thread,"SELECT `channel`,`datecreate` FROM `orion_cs_core` WHERE `botnick`='%s'",mysql_quote(thread,param[6]));
						channels.fields = mysql_num_fields(channels.res);
						
						if(mysql_num_rows(channels.res)>0){
							for(i=0;i<channels.fields;i++){
								channels.row = mysql_fetch_row(channels.res);
								
								do_join(thread,1,param[6],channels.row[0],(time_t)stol(channels.row[1]));
								do_mode(thread,param[6],channels.row[0],(time_t)stol(channels.row[1]),"+oaq %s %s %s",param[6],param[6],param[6]);
							}
						}
						mysql_free_result_myr(channels.res);
						
						send_lang(thread,language,howtosend,snick,nick,"BS_BOT_BOT_X_CHANGED",param[5]);
					}
				}else send_lang(thread,language,howtosend,snick,nick,"INVALID_OPTION","change");
			}else send_lang(thread,language,howtosend,snick,nick,"BS_BOT_BOT_X_NOT_KNOWN",param[5]);
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the BotServ BOT Command (/msg BotServ HELP BOT)
 * 
 * When the user tries to get help for this command, using /msg BotServ HELP BOT,<br/>
 * this function is executed.
 * 
 * Syntax: /msg BotServ HELP BOT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_bot_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Bot Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help bot"))
		send_lang(thread,language,howtosend,snick,nick,"BS_SERVADMIN_HELP_BOT",snick);
	else
		send_lang(thread,language,howtosend,snick,nick,"BS_NO_HELP_AVAIL",snick);
		
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief BotServ Help List Command - BOT part (/msg BotServ help)
 * 
 * When the user tries to get general BotServ help, using /msg BotServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the BOT command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL bs_bot_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_BS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Bot (Help Listing)
	 */
	if(Users_BS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"BS_HELP_CMD_BOT");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
