/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_akick.c
 *   @brief This is the Module file for ChanServ AKICK Command.
 *
 *   It contains all functions which are neccessary to initialize and use <br/>
 *   the ChanServ AKICK command.
 *
 *   The ChanServ AKICK Command allows the user to control the channels<br/>
 *   akicklist. (Controls autokicks, depending on hosts - <br/>
 *                useful to prevent unregistered users from joining the channel)
 *
 *   Syntax is: /msg Chanserv AKICK #channel { ADD | DEL | STICK | UNSTICK | VIEW | LIST | ENFORCE | CLEAR } ...
 *
 *<table border="0">
 *    <tr><td> ADD		</td><td> Syntax is: /msg ChanServ AKICK #channel ADD		</td><td> < mask > [ reason ]	</td></tr>
 *    <tr><td> STICK	</td><td> Syntax is: /msg ChanServ AKICK #channel STICK		</td><td> < mask >				</td></tr>
 *    <tr><td> UNSTICK	</td><td> Syntax is: /msg ChanServ AKICK #channel UNSTICK	</td><td> < mask >				</td></tr>
 *    <tr><td> DEL		</td><td> Syntax is: /msg ChanServ AKICK #channel DEL		</td><td> < mask >				</td></tr>
 *    <tr><td> LIST		</td><td> Syntax is: /msg ChanServ AKICK #channel LIST		</td><td> [ mask ]				</td></tr>
 *    <tr><td> VIEW		</td><td> Syntax is: /msg ChanServ AKICK #channel VIEW		</td><td> [ mask ]				</td></tr>
 *    <tr><td> CLEAR	</td><td> Syntax is: /msg ChanServ AKICK #channel CLEAR		</td><td>						</td></tr>
 *    <tr><td> ENFORCE	</td><td> Syntax is: /msg ChanServ AKICK #channel ENFORCE	</td><td>						</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "cs"
#define MODUL "akick"
#define MODUL_FILE "cs_akick"
#define MODUL_VERSION "1.0"

MOD_BOOL cs_akick_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_akick_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_akick_stick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_akick_unstick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_akick_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_akick_view(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_akick_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_akick_enforce(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL cs_akick_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_akick_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"akick * option[1]","akick");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);
	
	addLeastModuleLevels(thread,&Module,"akick",is_services_admin,is_services_user); /* just to add a displaylevel for /msg cs help
															  this should always be the lowest used Module level! */
	if(m->CSAutokickForceReason)											  
		addCommand(thread,&Module,"akick add",cs_akick_add,is_services_admin,is_services_user,7,0);
	else											  
		addCommand(thread,&Module,"akick add",cs_akick_add,is_services_admin,is_services_user,6,0);
		
	addCommand(thread,&Module,"akick del",cs_akick_del,is_services_admin,is_services_user,6,6);
	//addCommand(thread,&Module,"akick stick",cs_akick_stick,is_services_admin,is_services_user,6,6);
	//addCommand(thread,&Module,"akick unstick",cs_akick_unstick,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"akick list",cs_akick_list,is_services_admin,is_services_user,5,6);
	addCommand(thread,&Module,"akick view",cs_akick_view,is_services_admin,is_services_user,5,6);
	addCommand(thread,&Module,"akick clear",cs_akick_clear,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"akick enforce",cs_akick_enforce,is_services_admin,is_services_user,5,5);

	addCommand(thread,&Module,"help akick",cs_akick_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_akick_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief CS AKICK Add function for the ChanServ AKICK Command.
 * 
 * The ChanServ AKICK Add Command allowes a user to add a specific Mask to the Akick-List<br/>
 *  Users on the Autokick-List won't be able to join!
 *  
 * Syntax is: /msg ChanServ AKICK #channel ADD < mask > [ reason ]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_akick_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL akick;
	char CBot[MAX_NICK_LEN];
	int i   = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&akick,sizeof(akick));
	bzero(&CBot,sizeof(CBot));
	
	/*
	 * Akick Add Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"akick add")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			int sender_level=stoi(thread->c.chan.access.ownlevel);
				
			if(sender_level>=thread->c.chan.levels.akick || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"akick add") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick)){
                if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_cs_akick` WHERE `channel`='%s' AND `mask`='%s'",mysql_quote(thread,param[4]),mysql_quote(thread,param[6])))==0){
					
					if(m->CSAutokickForceReason)
										domysql(thread,"INSERT INTO `orion_cs_akick` (`channel`,`mask`,`flags`,`reason`,`addtime`) VALUES ('%s','%s','0','%s','%ld')",mysql_quote(thread,param[4]),mysql_quote(thread,param[6]),mysql_quote(thread,_dstrjoin(' ',param,7)),time(NULL));
					else{
						if(param[7])	domysql(thread,"INSERT INTO `orion_cs_akick` (`channel`,`mask`,`flags`,`reason`,`addtime`) VALUES ('%s','%s','0','%s','%ld')",mysql_quote(thread,param[4]),mysql_quote(thread,param[6]),mysql_quote(thread,_dstrjoin(' ',param,7)),time(NULL));
						else 			domysql(thread,"INSERT INTO `orion_cs_akick` (`channel`,`mask`,`flags`,`reason`,`addtime`) VALUES ('%s','%s','0','%s','%ld')",mysql_quote(thread,param[4]),mysql_quote(thread,param[6]),mysql_quote(thread,m->CSAutokickReason),time(NULL));
					}
					
					akick.res 		= AkickEnforce(thread, param[4]); //mysql.c
	        		akick.fields	= mysql_num_rows(akick.res);
	        		i = 1;
	        		    
	        		if(akick.fields){
	            		while((akick.row = mysql_fetch_row(akick.res))){
							do_mode(thread,CBot,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",akick.row[2]);
							do_kick(thread,CBot,thread->c.chan.name,akick.row[0],"Akicked: %s",akick.row[3]);
	            			i++;
	            		}
	            		send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_ENFORCED",param[4],i-1);
					}
						
					mysql_free_result_myr(akick.res); //free MySQL result
						
                    send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_MASK_ADDED_TO_AKICK",param[6],param[4]);
				}else send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_MASK_ALREADY_IN_AKICK",param[6],param[4]);
			}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS AKICK Stick function for the ChanServ AKICK Command.
 * 
 * The ChanServ AKICK Stick Command allowes a user to add a Mask permanently to the<br/>
 *  channels Banlist, if someone removes it - ChanServ will set it again
 *  
 * Syntax is: /msg ChanServ AKICK #channel STICK < mask >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_akick_stick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	char CBot[MAX_NICK_LEN];
	
	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));
	
	/*
	 * Akick Stick Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"akick stick")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			int sender_level=stoi(thread->c.chan.access.ownlevel);
				
			if(sender_level>=thread->c.chan.levels.akick || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"akick stick") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick)){
                if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_cs_akick` WHERE `channel`='%s' AND `mask`='%s'",mysql_quote(thread,param[4]),mysql_quote(thread,param[6])))==1){
					domysql(thread,"UPDATE `orion_cs_akick` LEFT OUTER JOIN `orion_cs_core` ON `orion_cs_core`.`channel`=`orion_cs_akick`.`channel` SET `orion_cs_akick`.``flags`='1', `orion_cs_core`.`last_used`='%ld' WHERE `orion_cs_akick`.`channel`='%s' AND `mask`='%s'",time(NULL),mysql_quote(thread,param[4]),mysql_quote(thread,param[6]));
                    send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_STICK_MASK",param[6],param[4]);
/* STICK: Set Channel Ban */
                    do_mode(thread,CBot,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",param[6]);

				}else send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_MASK_NOT_IN_AKICK",param[6],param[4]);
			}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS AKICK UnStick function for the ChanServ AKICK Command.
 * 
 * The ChanServ AKICK UnStick Command allowes a user to unforce an AkickMask<br/>
 *  If the a user matching that banmask joins, it will be set again and the user will be kicked.
 *  
 * Syntax is: /msg ChanServ AKICK #channel UNSTICK < mask >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_akick_unstick(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;
	int sender_level = 0;
	
	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));
	
	/*
	 * Akick UnStick Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"akick unstick")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			sender_level = stoi(thread->c.chan.access.ownlevel);
				
			if(sender_level>=thread->c.chan.levels.akick || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"akick unstick") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick)){
                if(stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_cs_akick` WHERE `channel`='%s' AND `mask`='%s'",mysql_quote(thread,param[4]),mysql_quote(thread,param[6])))==1){
					domysql(thread,"UPDATE `orion_cs_akick` LEFT OUTER JOIN `orion_cs_core` ON `orion_cs_core`.`channel`=`orion_cs_akick`.`channel` SET `orion_cs_akick`.``flags`='0', `orion_cs_core`.`last_used`='%ld' WHERE `orion_cs_akick`.`channel`='%s' AND `mask`='%s'",time(NULL),mysql_quote(thread,param[4]),mysql_quote(thread,param[6]));
                    send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_UNSTICK_MASK",param[6],param[4]);
/* UNSTICK: Did nothing on unstick */
				}else send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_MASK_NOT_IN_AKICK",param[6],param[4]);
			}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS AKICK Del function for the ChanServ AKICK Command.
 * 
 * The ChanServ AKICK Del Command allowes a user to delete a specific Mask from the Akick-List
 *  
 * Syntax is: /msg ChanServ AKICK #channel DEL < mask >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_akick_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	char CBot[MAX_NICK_LEN];
	
	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));
	
	/*
	 * Akick Del Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"akick del")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			int sender_level=stoi(thread->c.chan.access.ownlevel);
				
			if(sender_level>=thread->c.chan.levels.akick || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"akick del") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick)){
				if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_cs_akick` WHERE `mask`='%s' AND `channel`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[4])))==1){
					domysql(thread,"DELETE FROM `orion_cs_akick` WHERE `mask`='%s' AND `channel`='%s'",mysql_quote(thread,param[6]),mysql_quote(thread,param[4]));
	/* DEL: Clear that Channel Ban */
	                do_mode(thread,CBot,thread->c.chan.name,thread->c.chan.datecreate,"-%c %s",IRC.BanMode,param[6]);
	                send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_DELETED_MASK",param[6],param[4]);
				}else send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_MASK_NOT_IN_AKICK",param[6],param[4]);
			}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS AKICK List function for the ChanServ AKICK Command.
 * 
 * The ChanServ AKICK List Command allowes a user to dump the Akick-List<br/>
 *  for target channel.
 *  
 * Syntax is: /msg ChanServ AKICK #channel LIST [ mask ]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_akick_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL akick;
	//ChannelDB c;
	int i   = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&akick,sizeof(akick));
	//bzero(&c,sizeof(c));
	
	/*
	 * Akick List Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"akick list")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
    		if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.akick || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"akick list") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick)){
        		if(!param[6])
					akick.res		= doSelectMySQL(thread,"SELECT `mask`,`reason` FROM `orion_cs_akick` WHERE `channel`='%s'",mysql_quote(thread,param[4]));
        		if(param[6])
        			akick.res		= doSelectMySQL(thread,"SELECT `mask`,`reason` FROM `orion_cs_akick` WHERE `channel`='%s' AND `mask` LIKE '%c%s%c'",mysql_quote(thread,param[4]),'%',mysql_quote(thread,param[6]),'%');
        			
				akick.fields	= mysql_num_rows(akick.res);
        		i = 1;
        		    
        		if(akick.fields){
					send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_LIST_VIEW_HEADER",param[4]);
            		while((akick.row = mysql_fetch_row(akick.res))){
						send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_LIST",i,akick.row[0],akick.row[1]);
            			i++;
					}
            		send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_LIST_VIEW_FOOTER",i-1);
				}
				else send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_NO_AKICKS",param[4]);
					
        		mysql_free_result_myr(akick.res); //free MySQL result
        		    
        	}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS AKICK View function for the ChanServ AKICK Command.
 * 
 * The ChanServ AKICK View Command allowes a user to dump the Akick-List<br/>
 *  in more detail as in list, for target channel.
 *  
 * Syntax is: /msg ChanServ AKICK #channel VIEW [ mask ]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_akick_view(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL akick;
	//ChannelDB c;
	int i   = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&akick,sizeof(akick));
	//bzero(&c,sizeof(c));
	
	/*
	 * Akick View Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"akick view")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){

    		if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.akick || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"akick view") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick)){
        		if(!param[6])
					akick.res		= doSelectMySQL(thread,"SELECT `mask`,`creator`,`addtime`,`reason` FROM `orion_cs_akick` WHERE `channel`='%s'",mysql_quote(thread,param[4]));
				else if(param[6])
					akick.res		= doSelectMySQL(thread,"SELECT `mask`,`creator`,`addtime`,`reason` FROM `orion_cs_akick` WHERE `channel`='%s' AND `mask` LIKE '%c%s%c'",mysql_quote(thread,param[4]),'%',mysql_quote(thread,param[6]),'%');

        		akick.fields	= mysql_num_rows(akick.res);
				i = 1;
        		    
        		if(akick.fields){
					send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_LIST_VIEW_HEADER",param[4]);
            		while((akick.row = mysql_fetch_row(akick.res))){
						send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_VIEW",i,akick.row[0],akick.row[1],get_localtime((time_t)stol(akick.row[2])),akick.row[3]);
            			i++;
            		}
            		send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_LIST_VIEW_FOOTER",i-1);
				}
				else send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_NO_AKICKS",param[4]);
					
				mysql_free_result_myr(akick.res); //free MySQL result
        		    
        	}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS AKICK Clear function for the ChanServ AKICK Command.
 * 
 * The ChanServ AKICK Clear Command allowes a user to clear the Akick-List<br/>
 *  for target Channel
 *  
 * Syntax is: /msg ChanServ AKICK #channel CLEAR
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_akick_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL akick;
	char CBot[MAX_NICK_LEN];
	char nicks[BUFFER];
	char mode[4];
	int i   = 0;
	int j   = 0;
	int pos = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&akick,sizeof(akick));
	bzero(&CBot,sizeof(CBot));
	bzero(&mode,sizeof(mode));
	bzero(&nicks,sizeof(nicks));
	
	/*
	 * Akick Clear Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"akick clear")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){

    		if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"akick clear") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick) || streqnc(thread->c.chan.successor,thread->sql.aliases.authnick)){
        		akick.res		= doSelectMySQL(thread,"SELECT `mask`,`flags` FROM `orion_cs_akick` WHERE `channel`='%s'",mysql_quote(thread,param[4]));
        		akick.fields	= mysql_num_rows(akick.res);
        		i = 0, j = 0, pos = 0;
        		    
        		if(akick.fields){
					for(j=0;j<akick.fields;j++){
						akick.row = mysql_fetch_row(akick.res);
							
						if(i==0){
							bzero(&mode,sizeof(mode));
							bzero(&nicks,sizeof(nicks));
						}
						if(i<3){
							straddnc(mode,IRC.BanMode,sizeof(mode));
							straddns(nicks,akick.row[0],sizeof(nicks));
							straddnc(nicks,' ',sizeof(nicks));
							i++;
						}
						if(i==3 || pos >= (akick.fields-1)){
							do_mode(thread,CBot,param[4],thread->c.chan.datecreate,_dynadd("-%s %s",mode,nicks));
							i=0;
						}
						pos++;
					}
				}
	        	mysql_free_result_myr(akick.res); //free MySQL result
	        		    
	            domysql(thread,"DELETE FROM `orion_cs_akick` WHERE `channel`='%s'",mysql_quote(thread,param[4]));
	            send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_CLEARED",thread->c.chan.name);
        	}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS AKICK Enforce function for the ChanServ AKICK Command.
 * 
 * The ChanServ AKICK Enforce Command allowes a user to Check all users in the channel<br/>
 *  on any matching Akick-List-Entry, if an entry is found the user is kickbanned!
 *  
 * Syntax is: /msg ChanServ AKICK #channel ENFORCE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_akick_enforce(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL akick;
	char CBot[MAX_NICK_LEN];
	int i   = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&akick,sizeof(akick));
	bzero(&CBot,sizeof(CBot));
	//bzero(&c,sizeof(c));
	
	/*
	 * Akick Enforce Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"akick enforce")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
    		if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.akick || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"akick enforce") || streqnc(thread->c.chan.founder,thread->sql.aliases.authnick)){
				
				akick.res 		= AkickEnforce(thread, param[4]); //mysql.c
        		akick.fields	= mysql_num_rows(akick.res);
        		i = 1;
        		    
        		if(akick.fields){
					while((akick.row = mysql_fetch_row(akick.res))){
						do_mode(thread,CBot,thread->c.chan.name,thread->c.chan.datecreate,"+b %s",akick.row[2]);
						do_kick(thread,CBot,thread->c.chan.name,akick.row[0],"Akicked: %s",akick.row[3]);
            			i++;
            		}
            		send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_ENFORCED",thread->c.chan.name,i);
				}
				else send_lang(thread,language,howtosend,snick,nick,"CS_AKICK_NO_AKICKS",param[4]);
					
        		mysql_free_result_myr(akick.res); //free MySQL result
        		    
        	}else send_lang(thread,language,howtosend,snick,nick,"ACCESS_IN_CHANNEL_LOW",param[4]);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ AKICK Command (/msg ChanServ HELP AKICK)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP AKICK,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP AKICK
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_akick_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Akick Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help akick"))
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_AKICK",snick,snick);

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - AKICK part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the AKICK command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_akick_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Akick (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_AKICK");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
