/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_ban.c
 *   @brief This is the Module file for ChanServ BAN Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ BAN command.
 *
 *   The ChanServ BAN Command allows the user to Ban a user on target #channel
 *
 *   Syntax is: /msg ChanServ BAN #channel [nick] [reason]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "cs"
#define MODUL "ban"
#define MODUL_FILE "cs_ban"
#define MODUL_VERSION "1.0"

MOD_BOOL cs_ban(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_ban_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_ban_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by <br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"ban","ban");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"ban",cs_ban,is_services_admin,is_services_user,3,0);
	addCommand(thread,&Module,"help ban",cs_ban_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_ban_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ BAN Command.
 * 
 * The ChanServ BAN Command allows the user to Ban a user on target #channel
 *  
 * Syntax is: /msg ChanServ BAN #channel [nick] [reason]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_ban(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	deepSQL channels;
	char CBot[MAX_NICK_LEN];
	
	/* Initialise Variables to (NULL) */
	bzero(&channels,sizeof(channels));
	bzero(&CBot,sizeof(CBot));
	
	/*
	 * Ban Command
	 */
 	//:shock PRIVMSG Chanserv :ban [channel] [NICK] [reason]
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"ban")){
		if(param[5])
			/* Updates the Threads Channel Management */
			UpdateChannelInformation(thread,param[4],param[5]);
		else
			/* Updates the Threads Channel Management */
			UpdateChannelInformation(thread,param[4],NULL);

		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
			if(!param[5] && CheckOption(thread,snick,nick,READONLY) && (thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"ban") || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.banme)){

				msend(thread,howtosend,snick,nick,"Banning host %s on %s.",BuildBanMask(thread,&thread->sql.Cdata,CSAnalyseBantype(thread->c.chan.settings)),param[4]);
				do_mode(thread,CBot,param[4],thread->c.chan.datecreate,"+b %s",BuildBanMask(thread,&thread->sql.Cdata,CSAnalyseBantype(thread->c.chan.settings)));
				do_kick(thread,CBot,param[4],nick,"banned - self request");
				
				send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
			}else if(param[5] && CheckOption(thread,snick,nick,READONLY) && (thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"ban") || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.ban)){
				//aliases5 = sql_aliases(thread,param[5]);
				
				if(thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"ban") || (~thread->c.chan.settings&CS_PEACE_BIT || (thread->c.chan.settings&CS_PEACE_BIT && stoi(thread->c.chan.access.ownlevel)>stoi(thread->c.chan.access.targetlevel)))){
					
					if(thread->c.Cdata.nick[0]!='\0'){ //User online
						msend(thread,howtosend,snick,nick,"Banning host %s on %s.",BuildBanMask(thread,&thread->c.Cdata,CSAnalyseBantype(thread->c.chan.settings)),param[4]);
						do_mode(thread,CBot,param[4],thread->c.chan.datecreate,"+b %s",BuildBanMask(thread,&thread->c.Cdata,CSAnalyseBantype(thread->c.chan.settings)));
					
						if(param[6])
							do_kick(thread,CBot,param[4],param[5],_dstrjoin(' ',param,6));
						else
							do_kick(thread,CBot,param[4],param[5],"banned - without a certain reason");
							
						send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
					}else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_USER",param[5]);
				}else send_lang(thread,language,howtosend,snick,nick,"CS_PERMISSION_DENIED");
			}else if(!param[4]){
				channels.res    = doSelectMySQL(thread,
					"SELECT `orion_cs_access`.`channel`,`orion_cs_core`.`botnick`,`orion_cs_core`.`bs_set` "
					"FROM `orion_cs_access` "
					"LEFT JOIN `orion_cs_core` ON `orion_cs_access`.`channel` = `orion_cs_core`.`channel` "
					"LEFT JOIN `orion_chan_users` ON `orion_cs_access`.`channel` = `orion_chan_users`.`channel` "
					"WHERE `orion_cs_access`.`serv` = 'ChanServ' "
					"AND `orion_cs_access`.`authnick` = '%s' "
					"AND `orion_chan_users`.`nick` = '%s' "
					"AND `orion_cs_core`.`cs_banme` IS NOT NULL "
					"AND `orion_cs_access`.`level` >= `orion_cs_core`.`cs_banme`",
					mysql_quote(thread,thread->sql.aliases.authnick),
					mysql_quote(thread,nick)
				);
				channels.fields = mysql_num_rows(channels.res);
				while((channels.row = mysql_fetch_row(channels.res))){
					 do_mode(thread,CSCheckBotInPlace(channels.row[1],stol(channels.row[2])),channels.row[0],thread->c.chan.datecreate,"+b %s",BuildBanMask(thread,&thread->sql.Cdata,CSAnalyseBantype(thread->c.chan.settings)));
					 do_kick(thread,CSCheckBotInPlace(channels.row[1],stol(channels.row[2])),channels.row[0],nick,"banned - without a certain reason");
				}
				send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
				
				mysql_free_result_myr(channels.res); //free MySQL result
			}else send_lang(thread,language,howtosend,snick,nick,"CS_BAN_CHECK_SYNTAX",snick);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ BAN Command (/msg ChanServ HELP BAN)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP BAN,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP BAN
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_ban_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Ban Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help ban"))
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_BAN");

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - BAN part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the BAN command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_ban_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Ban (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_BAN");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
