/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_info.c
 *   @brief This is the Module file for ChanServ INFO Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ INFO command.
 *
 *   The ChanServ INFO Command allows the user to dump<br/>
 *   some Information about the Channel.
 *
 *   Syntax is: /msg ChanServ INFO #channel
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "cs"
#define MODUL "info"
#define MODUL_FILE "cs_info"
#define MODUL_FILE_CS_GETKEY "cs_getkey" //used to decide if we provide the channel key in info or not
#define MODUL_VERSION "1.0"

MOD_BOOL cs_info(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_info_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_info_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"info","info");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"info",cs_info,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help info",cs_info_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_info_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ INFO Command.
 * 
 * The ChanServ INFO Command allows the user to dump<br/>
 *   some Information about the Channel.
 *
 * Syntax is: /msg ChanServ INFO #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_info(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Info Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"info")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){
	
			/*
			(04:28) -ChanServ- Information for channel #ircops:
			(04:28) -ChanServ- Founder: NetSpLit
			(04:28) -ChanServ- Description: IRCops-Chan
			(04:28) -ChanServ- Registered: Jan 28 10:35:50 2003 CET
			(04:28) -ChanServ- Last used: Dec 31 03:51:29 2006 CET
			(04:28) -ChanServ- For more verbose information, type /msg ChanServ INFO #ircops ALL.*/
	
	    	send_lang(thread,language,howtosend,snick,nick,"CS_INFO_CHANNELINFO",param[4]);
	    	send_lang(thread,language,howtosend,snick,nick,"CS_INFO_FOUNDER",thread->c.chan.founder);
	    	if(thread->c.chan.successor[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"CS_INFO_SUCCESSOR",thread->c.chan.successor);
	    	send_lang(thread,language,howtosend,snick,nick,"CS_INFO_DESCRIPTION",thread->c.chan.description);
	    	send_lang(thread,language,howtosend,snick,nick,"CS_INFO_CREATED",get_localtime(thread->c.chan.regdate));
	    	send_lang(thread,language,howtosend,snick,nick,"CS_INFO_LASTUSED",get_localtime(thread->c.chan.last_used));
	    	if(thread->c.chan.url[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"CS_INFO_URL",thread->c.chan.url);
	    	if(thread->c.chan.email[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"CS_INFO_EMAIL",thread->c.chan.email);
	    	
			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.info || stoi(thread->c.chan.access.ownlevel)>=ACCESS_FOUNDER || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"info") || streqnc(thread->sql.aliases.authnick,thread->c.chan.founder)){
	    		int bantype=0;
			/*
			(04:28) -ChanServ- Information for channel #ircops:
			(04:28) -ChanServ- Founder: NetSpLit
			(04:28) -ChanServ- Description: IRCops-Chan
			(04:28) -ChanServ- Registered: Jan 28 10:35:50 2003 CET
			(04:28) -ChanServ- Last used: Dec 31 04:27:19 2006 CET
			(04:28) -ChanServ- Last topic: :: IRCops HQ :: Frohe Weihnachten euch Pappnasen :) - bitte um Q tests unter (./server -m dogabi2006.de:5880 d33pd3vhub) (account shock, pw: 34154424) fehler bitte an Manuel-Moser@aon.at - (Deepservices am ftp @cliebig dir:deepservices - fehlt nur ein make! -> testserver) - ne, geht nicht...
			(04:28) -ChanServ- Topic set by: cLiEbiG
			(04:28) -ChanServ- Ban type: 2
			(04:28) -ChanServ- Options: Topic Retention, Peace, Secure Founder
			(04:28) -ChanServ- Mode lock: +r
			(04:28) -ChanServ- This channel will not expire.*/
    			if(thread->c.chan.last_topic[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"CS_INFO_LASTTOPIC",thread->c.chan.last_topic);
    			if(thread->c.chan.welcome[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"CS_INFO_ENTRYMSG",thread->c.chan.welcome);
    			if(thread->c.chan.last_topic_setter[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"CS_INFO_LASTTOPICSETTER",thread->c.chan.last_topic_setter);
    			if(thread->c.chan.settings&CS_BANTYPE_1_BIT && thread->c.chan.settings&CS_BANTYPE_2_BIT) bantype=3;
   				else if(~thread->c.chan.settings&CS_BANTYPE_1_BIT && thread->c.chan.settings&CS_BANTYPE_2_BIT) bantype=2;
    			else if(thread->c.chan.settings&CS_BANTYPE_1_BIT && ~thread->c.chan.settings&CS_BANTYPE_2_BIT) bantype=1;
    			else if(~thread->c.chan.settings&CS_BANTYPE_1_BIT && ~thread->c.chan.settings&CS_BANTYPE_2_BIT) bantype=0;
    			send_lang(thread,language,howtosend,snick,nick,"CS_INFO_BANTYPE",bantype);

    			char options[300];
    			bzero(&options,sizeof(options));
    			if(thread->c.chan.settings&CS_XOP_BIT) straddns(options,"XOP, ",sizeof(options));
    			if(thread->c.chan.settings&CS_TOPICLOCK_BIT) straddns(options,"Topic Lock, ",sizeof(options));
    			if(thread->c.chan.settings&CS_KEEPTOPIC_BIT) straddns(options,"Topic Retention, ",sizeof(options));
    			if(thread->c.chan.settings&CS_SECUREFOUNDER_BIT) straddns(options,"Secure Founder, ",sizeof(options));
    			if(thread->c.chan.settings&CS_SECUREOPS_BIT) straddns(options,"Secure Ops, ",sizeof(options));
    			if(thread->c.chan.settings&CS_SECURE_BIT) straddns(options,"Secure, ",sizeof(options));
    			if(thread->c.chan.settings&CS_RESTRICTED_BIT) straddns(options,"Restricted, ",sizeof(options));
    			if(thread->c.chan.settings&CS_PRIVATE_BIT) straddns(options,"Private, ",sizeof(options));
    			if(thread->c.chan.settings&CS_PEACE_BIT) straddns(options,"Peace, ",sizeof(options));
    			if(thread->c.chan.settings&CS_OPNOTICE_BIT) straddns(options,"Opnotice, ",sizeof(options));
    			
    			if(thread->c.chan.settings&CS_SIGNKICK_1_BIT && thread->c.chan.settings&CS_SIGNKICK_2_BIT) straddns(options,"Signed kicks \"Level\", ",sizeof(options));
   				else if(thread->c.chan.settings&CS_SIGNKICK_1_BIT && !thread->c.chan.settings&CS_SIGNKICK_2_BIT) straddns(options,"Signed kicks, ",sizeof(options));
   				options[strlen(options)-2]='\0';
    			if(thread->c.chan.settings!=0) send_lang(thread,language,howtosend,snick,nick,"CS_INFO_OPTIONS",options);
    			bzero(&options,sizeof(options));
    			
    			if(thread->c.chan.mlock_positive[0]!='\0'){
					straddnc(options,'+',sizeof(options));
					straddns(options,thread->c.chan.mlock_positive,sizeof(options));
				}
                if(thread->c.chan.mlock_negative[0]!='\0'){
					straddnc(options,'-',sizeof(options));
					straddns(options,thread->c.chan.mlock_negative,sizeof(options));
				}
                if(options[0]!='\0') send_lang(thread,language,howtosend,snick,nick,"CS_INFO_MODELOCK",options);
                
                if(thread->c.chan.mode[0]!='\0')
                	send_lang(thread,language,howtosend,snick,nick,"CS_INFO_MODES",thread->c.chan.mode);
                
                if(thread->c.chan.key[0]!='\0' && (stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.getkey || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE_CS_GETKEY,"getkey")))
                	send_lang(thread,language,howtosend,snick,nick,"CS_INFO_KEY",thread->c.chan.key);
                	
    			if(thread->c.chan.expire) send_lang(thread,language,howtosend,snick,nick,"CS_INFO_EXPIRE",get_localtime ((thread->c.chan.last_used+expire(m->CSExpire))));
    			else send_lang(thread,language,howtosend,snick,nick,"CS_INFO_NOEXPIRE");
            }
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ INFO Command (/msg ChanServ HELP INFO)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP INFO,<br/> 
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP INFO
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_info_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Info Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help info"))
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_INFO");

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - INFO part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the INFO command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_info_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Info (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_INFO");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
