/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_levels.c
 *   @brief This is the Module file for ChanServ LEVELS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ LEVELS command.
 *
 *   The ChanServ LEVELS Command allows the user to fine control the meaning of<br/>
 *   the numeric access levels used for channels.<br/>
 *   With this command, you can define the access level required for most<br/>
 *   of ChanServ's functions.<br/>
 *     (The SET FOUNDER and SET PASSWORD commands,<br/>
 *      as well as this command, are always restricted to the channel founder.)
 *
 *   Syntax is: /msg ChanServ LEVELS #channel { SET | DISABLE | LIST | RESET } ...
 *
 *<table border="0">
 *    <tr><td> SET		</td><td> Syntax is: /msg ChanServ LEVELS #channel SET		</td><td> ChannelLevel < Level(figure) >	</td></tr>
 *    <tr><td> DISABLE	</td><td> Syntax is: /msg ChanServ LEVELS #channel DISABLE	</td><td> ChannelLevel						</td></tr>
 *    <tr><td> LIST		</td><td> Syntax is: /msg ChanServ LEVELS #channel LIST		</td><td>									</td></tr>
 *    <tr><td> RESET	</td><td> Syntax is: /msg ChanServ LEVELS #channel RESET	</td><td>									</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "cs"
#define MODUL "levels"
#define MODUL_FILE "cs_levels"
#define MODUL_VERSION "1.0"

MOD_BOOL cs_levels_set(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_levels_disable(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_levels_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_levels_reset(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL cs_levels_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_levels_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"levels * option[1]","levels");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"levels",is_services_admin,is_services_user); /* just to add a displaylevel for /msg cs help
															  this should always be the lowest used Module level! */
															  
	addCommand(thread,&Module,"levels set",cs_levels_set,is_services_admin,is_services_user,7,7);
	addCommand(thread,&Module,"levels disable",cs_levels_disable,is_services_admin,is_services_user,6,6);
	addCommand(thread,&Module,"levels list",cs_levels_list,is_services_admin,is_services_user,5,5);
	addCommand(thread,&Module,"levels reset",cs_levels_reset,is_services_admin,is_services_user,5,5);
	
	addCommand(thread,&Module,"help levels",cs_levels_help,is_services_user,is_services_user,4,5);
	addCommand(thread,&Module,"help",cs_levels_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief CS LEVELS Set function for the ChanServ LEVELS Command.
 * 
 * The ChanServ LEVELS Set Command allows the user to Set various Channellevels<br/>
 *    to levels of his choice.
 *  
 * Syntax is: /msg ChanServ LEVELS #channel SET ChannelLevel <level(figure)>
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_levels_set(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Levels Set Command
	 */
 	// :shock PRIVMSG chanserv :levels #channel set type level
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"levels set")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(CheckOption(thread,snick,nick,READONLY)){
				if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set){
					if(param[7] && stoi(param[7])>=ACCESS_UNKNOWN_END){
						if(streqnc(param[6],"autodeop")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.autodeop = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_autodeop`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"autohalfop")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.autohalfop = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_autohalfop`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"autoop")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.autoop = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_autoop`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"autoprotect")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.autoprotect = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_autoprotect`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"autovoice")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.autovoice = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_autovoice`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"nojoin")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.nojoin = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_nojoin`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"signkick")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.signkick = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_signkick`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"acc-list")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.acclist = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_acclist`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"acc-change")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.accchange = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_accchange`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"akick")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.akick = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_akick`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"set")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.set = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_set`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"ban")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.ban = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_ban`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"banme")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.banme = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_banme`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"clear")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.clear = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_cclear`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"getkey")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.getkey = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_getkey`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"halfop")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.halfop = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_halfop`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"halfopme")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.halfopme = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_halfopme`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"info")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.info = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_info`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"kick")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.kick = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_kick`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"kickme")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.kickme = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_kickme`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"invite")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.invite = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_invite`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"opdeop")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.opdeop = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_opdeop`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"opdeopme")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.opdeopme = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_opdeopme`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"protect")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.protect = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_protect`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"protectme")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.protectme = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_protectme`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"topic")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.topic = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_topic`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"unban")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.unban = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_unban`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"voice")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.voice = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_voice`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"voiceme")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.voiceme = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_voiceme`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"memo")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.memo = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_memo`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"assign")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.assign = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_botassign`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"badwords")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.badwords = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_badwords`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"fantasia")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.fantasia = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_fantasia`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"greet")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.greet = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_greet`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"nokick")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.nokick = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_nokick`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}else if(streqnc(param[6],"say")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.say = stoi(param[7]);
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_say`='%d' WHERE `channel`='%s'",time(NULL),stoi(param[7]),mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_SET_TO_X",_uc(param[6]),thread->c.chan.name,stoi(param[7]));
						}
						else send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_NO_SUCH_LEVEL",param[6]);
					}else send_lang(thread,language,howtosend,snick,nick,"MISSING_PARAMETER_OR_NONE_DIGIT");
				}else send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
			}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS LEVELS Disable function for the ChanServ LEVELS Command.
 * 
 * The ChanServ LEVELS Disable Command allows the user to Disable<br/>
 *    various Channellevels
 *  
 * Syntax is: /msg ChanServ LEVELS #channel DISABLE ChannelLevel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_levels_disable(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Levels Disable Command
	 */
 	// :shock PRIVMSG chanserv :levels #channel set type level
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"levels disable")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(CheckOption(thread,snick,nick,READONLY)){
				if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set){
					if(!param[7]){
						if(streqnc(param[6],"autodeop")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.autodeop = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_autodeop`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"autohalfop")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.autohalfop = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_autohalfop`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"autoop")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.autoop = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_autoop`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"autoprotect")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.autoprotect = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_autoprotect`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"autovoice")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.autovoice = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_autovoice`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"nojoin")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.nojoin = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_nojoin`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"signkick")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.signkick = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_signkick`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"acc-list")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.acclist = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_acclist`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"acc-change")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.accchange = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_accchange`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"akick")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.akick = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_akick`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"set")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.set = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_set`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"ban")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.ban = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_ban`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"banme")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.banme = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_banme`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"clear")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.clear = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_cclear`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"getkey")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.getkey = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_getkey`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"halfop")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.halfop = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_halfop`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"halfopme")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.halfopme = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_halfopme`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"info")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.info = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_info`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"kick")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.kick = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_kick`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"kickme")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.kickme = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_kickme`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"invite")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.invite = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_invite`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"opdeop")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.opdeop = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_opdeop`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"opdeopme")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.opdeopme = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_opdeopme`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"protect")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.protect = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_protect`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"protectme")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.protectme = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_protectme`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"topic")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.topic = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_topic`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"unban")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.unban = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_unban`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"voice")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.voice = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_voice`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"voiceme")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.voiceme = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_voiceme`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"memo")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.memo = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_memo`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"botassign")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.assign = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_botassign`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"badwords")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.badwords = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_badwords`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"fantasia")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.fantasia = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_fantasia`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"greet")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.greet = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_greet`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"nokick")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.nokick = ACCESS_INVALID;
#endif

							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_nokick`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else if(streqnc(param[6],"say")){
							/* Update Memory, for caching */
#ifndef USE_THREADS
							thread->c.chan.levels.say = ACCESS_INVALID;
#endif
							
							/* Update SQL as Live Update */
							domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`cs_say`='%d' WHERE `channel`='%s'",time(NULL),ACCESS_INVALID,mysql_quote(thread,param[4]));
							send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_LEVEL_X_ON_CHANNEL_X_DISABLED",_uc(param[6]),thread->c.chan.name);
						}else send_lang(thread,language,howtosend,snick,nick,"CS_LEVELS_NO_SUCH_LEVEL",param[6]);
					}else send_lang(thread,language,howtosend,snick,nick,"TOO_MANY_ARGUMENTS");
				}else send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
			}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS LEVELS List function for the ChanServ LEVELS Command.
 * 
 * The ChanServ LEVELS List Command allows the user to List the Channellevels
 *  
 * Syntax is: /msg ChanServ LEVELS #channel LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_levels_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Levels List Command
	 */
 	// :shock PRIVMSG chanserv :levels #channel set type level
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"levels list")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set){
				if(!param[7]){
			
					if(thread->c.chan.levels.autodeop!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"AUTODEOP %d",thread->c.chan.levels.autodeop);
					else msend(thread,howtosend,snick,nick,"AUTODEOP (disabled)");
			
					if(thread->c.chan.levels.autohalfop!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"AUTOHALFOP %d",thread->c.chan.levels.autohalfop);
					else msend(thread,howtosend,snick,nick,"AUTOHALFOP (disabled)");
			
					if(thread->c.chan.levels.autoop!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"AUTOOP %d",thread->c.chan.levels.autoop);
					else msend(thread,howtosend,snick,nick,"AUTOHALFOP (disabled)");
			
					if(thread->c.chan.levels.autoprotect!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"AUTOPROTECT %d",thread->c.chan.levels.autoprotect);
					else msend(thread,howtosend,snick,nick,"AUTOPROTECT (disabled)");
			
					if(thread->c.chan.levels.autovoice!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"AUTOVOICE %d",thread->c.chan.levels.autovoice);
					else msend(thread,howtosend,snick,nick,"AUTOVOICE (disabled)");
						
					if(thread->c.chan.levels.nojoin!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"NOJOIN %d",thread->c.chan.levels.nojoin);
					else msend(thread,howtosend,snick,nick,"NOJOIN (disabled)");
						
					if(thread->c.chan.levels.signkick!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"SIGNKICK %d",thread->c.chan.levels.signkick);
					else msend(thread,howtosend,snick,nick,"SIGNKICK (disabled)");
						
					if(thread->c.chan.levels.acclist!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"ACC-LIST %d",thread->c.chan.levels.acclist);
					else msend(thread,howtosend,snick,nick,"ACC-LIST (disabled)");
						
					if(thread->c.chan.levels.accchange!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"ACC-CHANGE %d",thread->c.chan.levels.accchange);
					else msend(thread,howtosend,snick,nick,"ACC-CHANGE (disabled)");
			
					if(thread->c.chan.levels.akick!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"AKICK %d",thread->c.chan.levels.akick);
					else msend(thread,howtosend,snick,nick,"AKICK (disabled)");
		
					if(thread->c.chan.levels.set!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"SET %d",thread->c.chan.levels.set);
					else msend(thread,howtosend,snick,nick,"SET (disabled)");
		
					if(thread->c.chan.levels.ban!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"BAN %d",thread->c.chan.levels.ban);
					else msend(thread,howtosend,snick,nick,"BAN (disabled)");
			
					if(thread->c.chan.levels.banme!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"BANME %d",thread->c.chan.levels.banme);
					else msend(thread,howtosend,snick,nick,"BANME (disabled)");
						
					if(thread->c.chan.levels.clear!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"CLEAR %d",thread->c.chan.levels.clear);
					else msend(thread,howtosend,snick,nick,"CLEAR (disabled)");
						
					if(thread->c.chan.levels.getkey!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"GETKEY %d",thread->c.chan.levels.getkey);
					else msend(thread,howtosend,snick,nick,"GETKEY (disabled)");
						
					if(thread->c.chan.levels.halfop!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"HALFOP %d",thread->c.chan.levels.halfop);
					else msend(thread,howtosend,snick,nick,"HALFOP (disabled)");
			
					if(thread->c.chan.levels.halfopme!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"HALFOPME %d",thread->c.chan.levels.halfopme);
					else msend(thread,howtosend,snick,nick,"HALFOPME (disabled)");
			
					if(thread->c.chan.levels.info!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"INFO %d",thread->c.chan.levels.info);
					else msend(thread,howtosend,snick,nick,"INFO (disabled)");
						
					if(thread->c.chan.levels.kick!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"KICK %d",thread->c.chan.levels.kick);
					else msend(thread,howtosend,snick,nick,"KICK (disabled)");
						
					if(thread->c.chan.levels.kickme!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"KICKME %d",thread->c.chan.levels.kickme);
					else msend(thread,howtosend,snick,nick,"KICKME (disabled)");
			
					if(thread->c.chan.levels.invite!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"INVITE %d",thread->c.chan.levels.invite);
					else msend(thread,howtosend,snick,nick,"INVITE (disabled)");
			
					if(thread->c.chan.levels.opdeop!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"OPDEOP %d",thread->c.chan.levels.opdeop);
					else msend(thread,howtosend,snick,nick,"OPDEOP (disabled)");
			
					if(thread->c.chan.levels.opdeopme!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"OPDEOPME %d",thread->c.chan.levels.opdeopme);
					else msend(thread,howtosend,snick,nick,"OPDEOPME (disabled)");
			
					if(thread->c.chan.levels.protect!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"PROTECT %d",thread->c.chan.levels.protect);
					else msend(thread,howtosend,snick,nick,"PROTECT (disabled)");
			
					if(thread->c.chan.levels.protectme!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"PROTECTME %d",thread->c.chan.levels.protectme);
					else msend(thread,howtosend,snick,nick,"PROTECTME (disabled)");
			
					if(thread->c.chan.levels.topic!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"TOPIC %d",thread->c.chan.levels.topic);
					else msend(thread,howtosend,snick,nick,"TOPIC (disabled)");
			
					if(thread->c.chan.levels.unban!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"UNBAN %d",thread->c.chan.levels.unban);
					else msend(thread,howtosend,snick,nick,"UNBAN (disabled)");
			
					if(thread->c.chan.levels.voice!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"VOICE %d",thread->c.chan.levels.voice);
					else msend(thread,howtosend,snick,nick,"VOICE (disabled)");
			
					if(thread->c.chan.levels.voiceme!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"VOICEME %d",thread->c.chan.levels.voiceme);
					else msend(thread,howtosend,snick,nick,"VOICEME (disabled)");
			
					if(thread->c.chan.levels.memo!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"MEMO %d",thread->c.chan.levels.memo);
					else msend(thread,howtosend,snick,nick,"MEMO (disabled)");
			
					if(thread->c.chan.levels.assign!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"ASSIGN %d",thread->c.chan.levels.assign);
					else msend(thread,howtosend,snick,nick,"ASSIGN (disabled)");
			
					if(thread->c.chan.levels.badwords!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"BADWORDS %d",thread->c.chan.levels.badwords);
					else msend(thread,howtosend,snick,nick,"BADWORDS (disabled)");
			
					if(thread->c.chan.levels.fantasia!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"FANTASIA %d",thread->c.chan.levels.fantasia);
					else msend(thread,howtosend,snick,nick,"FANTASIA (disabled)");
			
					if(thread->c.chan.levels.greet!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"GREET %d",thread->c.chan.levels.greet);
					else msend(thread,howtosend,snick,nick,"GREET (disabled)");
			
					if(thread->c.chan.levels.nokick!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"NOKICK %d",thread->c.chan.levels.nokick);
					else msend(thread,howtosend,snick,nick,"NOKICK (disabled)");
			
					if(thread->c.chan.levels.say!=ACCESS_INVALID) msend(thread,howtosend,snick,nick,"SAY %d",thread->c.chan.levels.say);
					else msend(thread,howtosend,snick,nick,"SAY (disabled)");
				}else send_lang(thread,language,howtosend,snick,nick,"TOO_MANY_ARGUMENTS");
			}else send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief CS LEVELS Reset function for the ChanServ LEVELS Command.
 * 
 * The ChanServ LEVELS Reset Command allows the user to Reset the Channellevels<br/>
 *    to the default ones.
 *  
 * Syntax is: /msg ChanServ LEVELS #channel RESET
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_levels_reset(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	//ChannelDB c;

	/* Initialise Variables to (NULL) */
	//bzero(&c,sizeof(c));

	/*
	 * Levels Reset Command
	 */
 	// :shock PRIVMSG chanserv :levels #channel set type level
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"levels reset")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(CheckOption(thread,snick,nick,READONLY)){
				if(stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.set){
					if(!param[6]){
						/* Update Memory, for caching */
						thread->c.chan.levels.set 			= CS_SET;
						thread->c.chan.levels.autodeop 		= CS_AUTODEOP;
						thread->c.chan.levels.autohalfop 	= CS_AUTOHALFOP;
						thread->c.chan.levels.autoop 		= CS_AUTOOP;
						thread->c.chan.levels.autoprotect 	= CS_AUTOPROTECT;
						thread->c.chan.levels.autovoice 	= CS_AUTOVOICE;
						thread->c.chan.levels.nojoin 		= CS_NOJOIN;
						thread->c.chan.levels.signkick 		= CS_SIGNKICK;
						thread->c.chan.levels.acclist 		= CS_ACCESS_LIST;
                        thread->c.chan.levels.accchange 	= CS_ACCESS_CHANGE;
                        thread->c.chan.levels.akick 		= CS_AKICK;
                        thread->c.chan.levels.ban 			= CS_BAN;
                        thread->c.chan.levels.banme 		= CS_BANME;
                        thread->c.chan.levels.clear 		= CS_CLEAR;
                        thread->c.chan.levels.getkey 		= CS_GETKEY;
                        thread->c.chan.levels.halfop 		= CS_HALFOP;
                        thread->c.chan.levels.halfopme 		= CS_HALFOPME;
                        thread->c.chan.levels.info 			= CS_INFO;
                        thread->c.chan.levels.kick 			= CS_KICK;
                        thread->c.chan.levels.kickme 		= CS_KICKME;
                        thread->c.chan.levels.invite 		= CS_INVITE;
                        thread->c.chan.levels.opdeop 		= CS_OPDEOP;
                        thread->c.chan.levels.opdeopme 		= CS_OPDEOPME;
                        thread->c.chan.levels.protect 		= CS_PROTECT;
                        thread->c.chan.levels.protectme 	= CS_PROTECTME;
                        thread->c.chan.levels.topic 		= CS_TOPIC;
                        thread->c.chan.levels.unban 		= CS_UNBAN;
                        thread->c.chan.levels.voice 		= CS_VOICE;
                        thread->c.chan.levels.voiceme 		= CS_VOICEME;
                        thread->c.chan.levels.memo 			= CS_MEMO;
                        thread->c.chan.levels.assign 		= CS_ASSIGN;
                        thread->c.chan.levels.badwords 		= CS_BADWORDS;
                        thread->c.chan.levels.fantasia 		= CS_FANTASIA;
                        thread->c.chan.levels.greet 		= CS_GREET;
                        thread->c.chan.levels.nokick 		= CS_NOKICK;
                        thread->c.chan.levels.say 			= CS_SAY;

						/* Update SQL as Live Update */
		    			domysql(thread,
							"UPDATE `orion_cs_core` SET "

								"`last_used`='%ld',"
								"`cs_set`='%d',`cs_autodeop`='%d',`cs_autohalfop`='%d',`cs_autoop`='%d',`cs_autoprotect`='%d',`cs_autovoice`='%d',"
								"`cs_nojoin`='%d',`cs_signkick`='%d',`cs_acclist`='%d',`cs_accchange`='%d',`cs_akick`='%d',`cs_ban`='%d',`cs_banme`='%d',"
								"`cs_cclear`='%d',`cs_getkey`='%d',`cs_halfop`='%d',`cs_halfopme`='%d',`cs_info`='%d',`cs_kick`='%d',`cs_kickme`='%d',`cs_invite`='%d',"
								"`cs_opdeop`='%d',`cs_opdeopme`='%d',`cs_protect`='%d',`cs_protectme `='%d',`cs_topic`='%d',`cs_unban`='%d',`cs_voice`='%d',"
								"`cs_voiceme`='%d',`cs_memo`='%d',`cs_botassign`='%d',`cs_badwords`='%d',`cs_fantasia`='%d',`cs_greet`='%d',`cs_nokick`='%d',"
								"`cs_say`='%d'"			

							" WHERE `channel`='%s' AND `serv`='ChanServ'",
							
							time(NULL),
							CS_SET,CS_AUTODEOP,CS_AUTOHALFOP,CS_AUTOOP,CS_AUTOPROTECT,CS_AUTOVOICE,
							CS_NOJOIN,CS_SIGNKICK,CS_ACCESS_LIST,CS_ACCESS_CHANGE,CS_AKICK,CS_BAN,CS_BANME,
							CS_CLEAR,CS_GETKEY,CS_HALFOP,CS_HALFOPME,CS_INFO,CS_KICK,CS_KICKME,CS_INVITE,
							CS_OPDEOP,CS_OPDEOPME,CS_PROTECT,CS_PROTECTME,CS_TOPIC,CS_UNBAN,CS_VOICE,
							CS_VOICEME,CS_MEMO,CS_ASSIGN,CS_BADWORDS,CS_FANTASIA,CS_GREET,CS_NOKICK,
							CS_SAY,
							mysql_quote(thread,thread->c.chan.name)
						);

					}else send_lang(thread,language,howtosend,snick,nick,"TOO_MANY_ARGUMENTS");
				}else send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
			}else send_lang(thread,language,howtosend,snick,nick,"READONLY_MODE");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ LEVELS Command (/msg ChanServ HELP LEVELS)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP LEVELS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP LEVELS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_levels_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Levels Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help levels")){
		if(param[5]==NULL)
			send_lang(thread,language,howtosend,snick,nick,"CS_HELP_LEVELS",snick);
		else if(streqnc(param[5],"desc"))
			send_lang(thread,language,howtosend,snick,nick,"CS_HELP_LEVELS_DESC");
		else
			send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	}
	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - LEVELS part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the LEVELS command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_levels_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Levels (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_LEVELS");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
