/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_suspend.c
 *   @brief This is the Module file for ChanServ SUSPEND Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ SUSPEND command.
 *
 *   The ChanServ SUSPEND Command allows an admin to disallow anyone from<br/>
 *   registering or using the given #channel.
 *
 *   May be cancelled by using the UNSUSPEND command to preserve all<br/>
 *   previous channel data/settings.<br/>
 *   (Reason may be required on certain networks.)<br/>
 *     ChanServ suspends a channel and kick bans all users (admins only).
 *
 *   Syntax is: /msg ChanServ SUSPEND #channel
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "cs"
#define MODUL "suspend"
#define MODUL_FILE "cs_suspend"
#define MODUL_VERSION "1.0"

MOD_BOOL cs_suspend(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_suspend_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_suspend_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"suspend","suspend");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"suspend",cs_suspend,is_services_admin,INFINIT,5,0);
	addCommand(thread,&Module,"help suspend",cs_suspend_help,is_services_admin,INFINIT,4,4);
	addCommand(thread,&Module,"help",cs_suspend_help_listing,is_services_admin,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ SUSPEND Command.
 * 
 * The ChanServ SUSPEND Command allows an admin to disallow anyone from<br/>
 *   registering or using the given #channel.
 *
 *   May be cancelled by using the UNSUSPEND command to preserve all<br/>
 *   previous channel data/settings.<br/>
 *   (Reason may be required on certain networks.)<br/>
 *     ChanServ suspends a channel and kick bans all users (admins only).
 *
 * Syntax is: /msg ChanServ SUSPEND #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_suspend(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	char CBot[MAX_NICK_LEN];
	deepSQL nicks;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));
	bzero(&nicks,sizeof(nicks));

	/*
	 * Suspend Command
	 */
	
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"suspend")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);

		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));

		if(thread->c.chan.fields && (thread->c.chan.settings & CS_FORBID_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(thread->sql.aliases.fields && streqnc(thread->c.chan.serv,"chanserv")){

			thread->c.chan.settings |= CS_SUSPEND_BIT;
			thread->c.chan.expire = 0;
			dynncpy(thread->c.chan.reason,_dstrjoin(' ',param,5),sizeof(thread->c.chan.reason));
			
			domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`expire`='0',`settings`='%ld',`reason`='%s' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),thread->c.chan.settings,mysql_quote(thread,thread->c.chan.reason),mysql_quote(thread,param[4]));
			
			nicks.res    = doSelectMySQL(thread,"SELECT `orion_core_condata`.`nick`,`orion_core_condata`.`ident`,`orion_core_condata`.`vhost` FROM `orion_core_condata` LEFT JOIN `orion_chan_users` ON (`orion_core_condata`.`nick` = `orion_chan_users`.`nick`) WHERE `orion_chan_users`.`channel`='%s'",mysql_quote(thread,param[4]));
			nicks.fields = mysql_num_rows(nicks.res);
			
			for(i=0;i<nicks.fields;i++){
				nicks.row = mysql_fetch_row(nicks.res);
				Condata Cdata;
				
				bzero(&Cdata,sizeof(Cdata));
				dynncpy(Cdata.nick,nicks.row[0],sizeof(Cdata.nick));
				dynncpy(Cdata.ident,nicks.row[1],sizeof(Cdata.ident));
				dynncpy(Cdata.vhost,nicks.row[2],sizeof(Cdata.vhost));
				
				do_mode(thread,CBot,param[4],thread->c.chan.datecreate,"+b %s",BuildBanMask(thread,&Cdata,CSAnalyseBantype(thread->c.chan.settings)));
				do_kick(thread,CBot,param[4],Cdata.nick,"channel is suspended reason: %s",thread->c.chan.reason);
			}
			mysql_free_result_myr(nicks.res);
			do_mode(thread,CBot,param[4],thread->c.chan.datecreate,IRC.OnSuspend);
			
			if(IRC.ChanRegMode)
				do_mode(thread,CBot,param[4],thread->c.chan.datecreate,IRC.ModeOnChannelDrop);
			send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ SUSPEND Command (/msg ChanServ HELP SUSPEND)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP SUSPEND,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP SUSPEND
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_suspend_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Suspend Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help suspend"))
		send_lang(thread,language,howtosend,snick,nick,"CS_SERVADMIN_HELP_SUSPEND",snick);
	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - SUSPEND part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SUSPEND command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_suspend_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Suspend (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_SUSPEND");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
