/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_topic.c
 *   @brief This is the Module file for ChanServ TOPIC Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ TOPIC command.
 *
 *   The ChanServ TOPIC Command allows the user to set the channel<br/>
 *   topic to the one specified. If "topic" is not given<br/>
 *   the current topic will be returned.
 *
 *   This command is most useful in conjunction<br/>
 *   with SET TOPICLOCK. See /msg ChanServ HELP SET TOPICLOCK<br/>
 *   for more information.
 *
 *   Syntax is: /msg ChanServ TOPIC #channel < Topic >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "cs"
#define MODUL "topic"
#define MODUL_FILE "cs_topic"
#define MODUL_VERSION "1.0"

MOD_BOOL cs_topic(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_topic_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_topic_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"topic","topic");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"topic",cs_topic,is_services_admin,is_services_user,4,0);
	addCommand(thread,&Module,"help topic",cs_topic_help,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help",cs_topic_help_listing,is_services_admin,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ TOPIC Command.
 * 
 * The ChanServ TOPIC Command allows the user to set the channel<br/>
 *   topic to the one specified. If "topic" is not given<br/>
 *   the current topic will be returned.
 *
 *   This command is most useful in conjunction<br/>
 *   with SET TOPICLOCK. See /msg ChanServ HELP SET TOPICLOCK<br/>
 *   for more information.
 *
 * Syntax is: /msg ChanServ TOPIC #channel < Topic >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_topic(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	char CBot[MAX_NICK_LEN];

	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));

	/*
	 * Topic Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"topic")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
		else if(param[5] && streqnc(thread->c.chan.serv,"chanserv")){
			
			if((thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"topic") || stoi(thread->c.chan.access.ownlevel)>=thread->c.chan.levels.topic)){
				send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
				do_topic(thread,CBot,CBot,thread->c.chan.name,time(NULL),_dstrjoin(' ',param,5));
			}else send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
		}else{
			if(thread->c.chan.fields && streqnc(thread->c.chan.name,param[4]) && streqnc(thread->c.chan.serv,"chanserv"))
				send_lang(thread,language,howtosend,snick,nick,"CS_TOPIC_SET_IS",thread->c.chan.last_topic);
			else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ TOPIC Command (/msg ChanServ HELP TOPIC)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP TOPIC,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP TOPIC
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_topic_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Topic Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help topic"))
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_TOPIC",snick,snick);

	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - TOPIC part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the TOPIC command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_topic_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Topic (Showcommands)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_TOPIC",command_level);

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
