/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file cs_unsuspend.c
 *   @brief This is the Module file for ChanServ UNSUSPEND Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the ChanServ UNSUSPEND command.
 *
 *   The ChanServ UNSUSPEND Command allows an admin to release a suspended channel.<br/>
 *   All data and settings are preserved from before the suspension. (admins only).
 *
 *   Syntax is: /msg ChanServ UNSUSPEND #channel
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "cs"
#define MODUL "unsuspend"
#define MODUL_FILE "cs_unsuspend"
#define MODUL_VERSION "1.0"

MOD_BOOL cs_unsuspend(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_unsuspend_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL cs_unsuspend_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"unsuspend","unsuspend");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"unsuspend",cs_unsuspend,is_services_admin,INFINIT,4,5);
	addCommand(thread,&Module,"help unsuspend",cs_unsuspend_help,is_services_admin,INFINIT,4,4);
	addCommand(thread,&Module,"help",cs_unsuspend_help_listing,is_services_admin,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the ChanServ UNSUSPEND Command.
 * 
 * The ChanServ UNSUSPEND Command allows an admin to release a suspended channel.<br/>
 *   All data and settings are preserved from before the suspension. (admins only).
 *
 * Syntax is: /msg ChanServ UNSUSPEND #channel
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_unsuspend(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Deklarate Variables */
	char CBot[MAX_NICK_LEN];
	char mlock[40];

	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));
	bzero(&mlock,sizeof(mlock));
	
	/*
	 * Unsuspend Command
	 */
	
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"unsuspend")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],NULL);
	
		/* Set ChanServ to our Bot, if BotServ is not loaded, or no Bot is assigned, or if FANTASY is OFF */
		CSCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));
		
		if(thread->c.chan.fields && (~thread->c.chan.settings & CS_SUSPEND_BIT)) send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_SUSPENDED");
		else if(streqnc(thread->c.chan.serv,"chanserv")){
			if(thread->c.chan.settings & CS_SUSPEND_BIT){
				thread->c.chan.settings &= ~CS_SUSPEND_BIT;
				thread->c.chan.expire = 1;
				bzero(&thread->c.chan.reason,sizeof(thread->c.chan.reason));

				domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`expire`='1',`reason`='',`settings`='%d' WHERE `channel`='%s' AND `serv`='ChanServ'",time(NULL),thread->c.chan.settings,mysql_quote(thread,param[4]));
			
				unbanall(thread,CBot,param[4]);
				do_mode(thread,CBot,param[4],thread->c.chan.datecreate,IRC.OnUnSuspend);
				do_mode(thread,CBot,param[4],thread->c.chan.datecreate,IRC.DefaultMLOCK);
			
				bzero(&mlock,sizeof(mlock));
				if(thread->c.chan.mlock_positive[0]!='\0'){
					straddnc(mlock,'+',sizeof(mlock));
					straddns(mlock,thread->c.chan.mlock_positive,sizeof(mlock));
				}
				if(thread->c.chan.mlock_negative[0]!='\0'){
					straddnc(mlock,'-',sizeof(mlock));
					straddns(mlock,thread->c.chan.mlock_negative,sizeof(mlock));
				}
				//perform Mlock
				if(mlock[0]!='\0'){
					char mlock_params[100];
					bzero(&mlock_params,sizeof(mlock_params));
					size_t i=0;
					bool type=0;
					for(i=0;i<dynlen(mlock);i++){
						char current=mlock[i];
						if(current=='+') type=1;
						else if(current=='-') type=0;
						else{
							if(type==1){
								if(strcin(current,IRC.MLockAbleModes)){
									if(current=='f'){
										straddns(mlock_params,thread->c.chan.mlock_flood,sizeof(mlock_params));
										straddnc(mlock_params,' ',sizeof(mlock_params));
									}else if(current=='k'){
										straddns(mlock_params,thread->c.chan.mlock_key,sizeof(mlock_params));
										straddnc(mlock_params,' ',sizeof(mlock_params));
									}else if(current=='l'){
										straddns(mlock_params,((char *)(long)thread->c.chan.mlock_limit),sizeof(mlock_params));
										straddnc(mlock_params,' ',sizeof(mlock_params));
									}
								}
							}
						}
					}
					mlock_params[dynlen(mlock_params)-1]='\0';
					do_mode(thread,CBot,param[4],thread->c.chan.datecreate,_dynadd("%s %s",mlock,mlock_params));
				}
				send_lang(thread,language,howtosend,snick,nick,"CS_COMMAND_DONE");
			}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_SUSPENDED");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_REGISTERED_FOR_CHANSERV");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the ChanServ UNSUSPEND Command (/msg ChanServ HELP UNSUSPEND)
 * 
 * When the user tries to get help for this command, using /msg ChanServ HELP UNSUSPEND,<br/>
 * this function is executed.
 * 
 * Syntax: /msg ChanServ HELP UNSUSPEND
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_unsuspend_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Unsuspend Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help unsuspend"))
		send_lang(thread,language,howtosend,snick,nick,"CS_SERVADMIN_HELP_UNSUSPEND",snick,snick);
	else send_lang(thread,language,howtosend,snick,nick,"CS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief ChanServ Help List Command - UNSUSPEND part (/msg ChanServ help)
 * 
 * When the user tries to get general ChanServ help, using /msg ChanServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the UNSUSPEND command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL cs_unsuspend_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_CS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Unsuspend (Help listing)
	 */
	if(Users_CS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"CS_HELP_CMD_UNSUSPEND");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
