/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ms_send.c
 *   @brief This is the Module file for MemoServ SEND Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the MemoServ SEND command.
 *
 *   The MemoServ SEND Command allows the user to send a memo to a<br/>
 *   channel / user.<br/>
 *   There are a 3 priority options:<br/>
 *   0 - Target user will get a notice that he/she has a new memo<br/>
 *       (all users on channel, who are allowed to read memos will get a notice/privmsg<br/>
 *       if the target is a channel. - notice or privmsg is dependent from /msg NickServ SET MSG
 *   
 *   1 - Target user will get a PRIVMSG that he/she has a new memo<br/>
 *       (all users on channel, who are allowed to read memos will get a PRIVMSG<br/>
 *       if the target is a channel. - the PRIVMSG mode is independent from /msg NickServ SET MSG
 *   
 *   2 - Target user will get a PRIVMSG that he/she has a new memo<br/>
 *       (all users on channel, who are allowed to read memos will get a PRIVMSG<br/>
 *       if the target is a channel. - the PRIVMSG mode is independent from /msg NickServ SET MSG<br/>
 *       AND an email will be set to the channels/users email address (within the sended memo)
 *
 *   Syntax is: /msg MemoServ SEND < nick / #channel > < priority[0,1,2] (may be disabled) > < memo >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define LOW 0
#define NORMAL 1
#define HIGH 2

#define SERVICE "ms"
#define MODUL "send"
#define MODUL_FILE "ms_send"
#define MODUL_VERSION "1.0"

MOD_BOOL ms_send(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_send_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_send_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"send","send");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"send",ms_send,is_services_oper,is_services_user,5,0);
	addCommand(thread,&Module,"help send",ms_send_help,is_services_oper,is_services_user,4,4);
	addCommand(thread,&Module,"help",ms_send_help_listing,is_services_oper,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the MemoServ SEND Command.
 * 
 * The MemoServ SEND Command allows the user to send a memo to a<br/>
 *   channel / user. 
 *
 * Syntax is: /msg MemoServ SEND < nick / #channel > < priority[0,1,2] (may be disabled) > < memo >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_send(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL chanusers;
	deepSQL memoSQL;
	ChannelDB c;
	Aliases aliases5;
	Access access5;
	
	MailInfo *mail = NULL;
	char From[80];
	char Subject[80];
	int priority 	= 0;
	int i        	= 0;
	size_t memojoin = 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&chanusers,sizeof(chanusers));
	bzero(&memoSQL,sizeof(memoSQL));
	bzero(&c,sizeof(c));
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
	bzero(&From,sizeof(From));
	bzero(&Subject,sizeof(Subject));

	/*
	 * Send Command
	 */
 	// :shock PRIVMSG memoserv :send authnick priority memo
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"send")){
		if(!digitcol(param[5]) || !m->MSUsePriorities)
			priority = LOW;
		else
			priority = stoi(param[5]);
			
		if(param[4][0]=='#'){
    		c = sql_chan_db(thread,param[4]);
    		if(c.serv[0]!='\0'){ 
				if(priority>=0 && priority<=2){
					if(!digitcol(param[5])){
						memojoin = 5;
						if(m->MSUsePriorities) send_lang(thread,language,howtosend,snick,nick,"MS_REMEMBER_PRIORITIES",snick);
					}else
						memojoin = 6;

					chanusers.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_chan_users` WHERE `channel`='%s'",mysql_quote(thread,c.name));
					chanusers.fields = mysql_num_rows(chanusers.res);
					
                    memoSQL.res        = doSelectMySQL(thread,"SELECT `memo` FROM `orion_ms_core` WHERE `receiver`='%s' ORDER BY `time` DESC",mysql_quote(thread,c.name));
					memoSQL.fields     = mysql_num_rows(memoSQL.res);
					
					if((memoSQL.fields+1<c.maxmemos || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"send"))){
    					domysql(thread,"INSERT INTO `orion_ms_core` (`sender`,`receiver`,`priority`,`rsend`,`mode`,`time`,`memo`) VALUES ('%s','%s','%d','0','0','%ld','%s')",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,c.name),priority,time(NULL),mysql_quote(thread,_dstrjoin(' ',param,memojoin)));
    					
                        for(i=0;i<chanusers.fields;i++){
                            chanusers.row = mysql_fetch_row(chanusers.res);
                            
                        	bzero(&aliases5,sizeof(aliases5));
                        	bzero(&access5,sizeof(access5));
                        	
    						aliases5 = sql_aliases(thread,chanusers.row[0]);
    						access5  = sql_access(thread,aliases5.authnick);
    						
    						if(aliases5.active && aliases5.authnick && (stoi(acclevel(thread,"ChanServ",c.name,aliases5.authnick))>=c.cs_memo || strcor(acclevel(thread,"Q",c.name,aliases5.authnick),"o") || strcor(acclevel(thread,"L",c.name,aliases5.authnick),"o"))){
    							if(priority==LOW) 			send_lang(thread,languages[access5.language],access5.settings&NS_SET_MSG ? 1 : 0,snick,chanusers.row[0],"MS_SEND_NEW_CHANNEL_MEMO",c.name,snick,c.name,memoSQL.fields+1,priority,snick);
    							else if(priority==NORMAL) 	send_lang(thread,languages[access5.language],1,snick,chanusers.row[0],"MS_SEND_NEW_CHANNEL_MEMO",c.name,snick,c.name,memoSQL.fields+1,priority,snick);
    							else if(priority==HIGH) 	send_lang(thread,languages[access5.language],1,snick,chanusers.row[0],"MS_SEND_NEW_CHANNEL_MEMO",c.name,snick,c.name,memoSQL.fields+1,priority,snick);
    						}
    					}
                        if(priority==HIGH && c.email[0]!='\0' && m->UseMail){
    						dynncpy(From,_dynadd("%s Network (%s) <%s>",m->network,ms->nick,m->SendFrom),sizeof(From));
    						dynncpy(Subject,_dynadd("New Memo for channel %s",c.name),sizeof(Subject));
    					
    						mail = o_email(thread,snick,From,c.email,Subject);
    						//Email					
    						fprintf(mail->pipe,"There is a new memo for channel: %s\n"
    						"Sent by MainNick / Nick: %s / %s\n"
    						"Memo Nr.: %d\n"
    						"Issue: %s\n",
    						c.name,thread->sql.aliases.authnick,nick,memoSQL.fields,_dstrjoin(' ',param,memojoin));
    						c_email(thread,mail);
    					}
    					send_lang(thread,language,howtosend,snick,nick,"MS_SEND_SENT_TO_CHANNEL",c.name);
					}else send_lang(thread,language,howtosend,snick,nick,"MS_SEND_FAILED_OVERRIDING_LIMIT",c.maxmemos);
					
            		/* mysql_free_result_myr(); - memos,  chanusers*/
            		mysql_free_result_myr(memoSQL.res);
            		mysql_free_result_myr(chanusers.res);
            		
                }else send_lang(thread,language,howtosend,snick,thread->sql.aliases.authnick,"MS_SEND_INVALID_PRIORITY_LEVEL",priority,snick);
            }else send_lang(thread,language,howtosend,snick,thread->sql.aliases.authnick,"CHANNEL_NOT_REGISTERED");
        }else{
    		aliases5 = sql_aliases(thread,param[4]);
    		access5  = sql_access(thread,aliases5.authnick);
    		if(aliases5.authnick[0]!='\0'){ 
				if(priority>=0 && priority<=2){ 
					if(!digitcol(param[5])){
						memojoin = 5;
						if(m->MSUsePriorities) send_lang(thread,language,howtosend,snick,nick,"MS_REMEMBER_PRIORITIES",snick);
					}else
						memojoin = 6;
					
                    memoSQL.res    = doSelectMySQL(thread,"SELECT `memo` FROM `orion_ms_core` WHERE `receiver`='%s' ORDER BY `time` DESC",mysql_quote(thread,aliases5.authnick));
					memoSQL.fields = mysql_num_rows(memoSQL.res);
                    
                    if(memoSQL.fields+1<access5.maxmemos || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"send")){
                        domysql(thread,"INSERT INTO `orion_ms_core` (`sender`,`receiver`,`priority`,`rsend`,`mode`,`time`,`memo`) VALUES ('%s','%s','%d','0','0','%ld','%s')",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,aliases5.authnick),priority,time(NULL),mysql_quote(thread,_dstrjoin(' ',param,memojoin)));
    					if(aliases5.active){
    						if(priority==LOW) 			send_lang(thread,languages[access5.language],access5.settings&NS_SET_MSG ? 1 : 0,snick,aliases5.nick,"MS_SEND_RECEIVED_NEW_MEMO_FROM_X",thread->sql.aliases.authnick,snick,memoSQL.fields+1,priority,snick);
    						else if(priority==NORMAL) 	send_lang(thread,languages[access5.language],1,snick,aliases5.nick,"MS_SEND_RECEIVED_NEW_MEMO_FROM_X",thread->sql.aliases.authnick,snick,memoSQL.fields+1,priority,snick);
    						else if(priority==HIGH) 	send_lang(thread,languages[access5.language],1,snick,aliases5.nick,"MS_SEND_RECEIVED_NEW_MEMO_FROM_X",thread->sql.aliases.authnick,snick,memoSQL.fields+1,priority,snick);
    					}
						if(priority==HIGH && access5.email[0]!='\0' && m->UseMail){
							dynncpy(From,_dynadd("%s Network (%s) <%s>",m->network,ms->nick,m->SendFrom),sizeof(From));
							dynncpy(Subject,_dynadd("New Memo for Group %s",aliases5.authnick),sizeof(Subject));
					
							mail = o_email(thread,From,access5.authnick,access5.email,Subject);
							//Email					
							fprintf(mail->pipe,"There is a new memo for Group: %s\n"
							"Sent by MainNick / Nick: %s / %s\n"
							"Memo Nr.: %d\n"
							"Issue: %s\n",
							aliases5.authnick,thread->sql.aliases.authnick,nick,memoSQL.fields,_dstrjoin(' ',param,memojoin));
							c_email(thread,mail);
						}
						send_lang(thread,language,howtosend,snick,nick,"MS_SEND_SENT_TO_NICK",aliases5.nick);
					}else send_lang(thread,language,howtosend,snick,nick,"MS_SEND_FAILED_OVERRIDING_LIMIT",access5.maxmemos);
					
            		/* mysql_free_result_myr(); - memos */
            		mysql_free_result_myr(memoSQL.res);
                }else send_lang(thread,language,howtosend,snick,nick,"MS_SEND_INVALID_PRIORITY_LEVEL",priority,snick);
            }else send_lang(thread,language,howtosend,snick,nick,"NO_AUTHNICK");
		}
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the MemoServ SEND Command (/msg MemoServ HELP SEND)
 * 
 * When the user tries to get help for this command, using /msg MemoServ HELP SEND,<br/>
 * this function is executed.
 * 
 * Syntax: /msg MemoServ HELP SEND
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_send_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Send Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help send"))
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_SEND",ns->nick,ns->nick,ns->nick);

	else send_lang(thread,language,howtosend,snick,nick,"MS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief MemoServ Help List Command - SEND part (/msg MemoServ help)
 * 
 * When the user tries to get general MemoServ help, using /msg MemoServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SEND command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_send_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_MS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Send (Help listing)
	 */
	if(Users_MS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_CMD_SEND");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
