/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ms_staff.c
 *   @brief This is the Module file for MemoServ STAFF Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the MemoServ STAFF command.
 *
 *   The MemoServ STAFF Command allows the user to send the services staff<br/>
 *   a memo. There are a 3 priority options:<br/>
 *   0 - Target user will get a notice that he/she has a new memo<br/>
 *       (all users on channel, who are allowed to read memos will get a notice/privmsg<br/>
 *       if the target is a channel. - notice or privmsg is dependent from /msg NickServ SET MSG
 *   
 *   1 - Target user will get a PRIVMSG that he/she has a new memo<br/>
 *       (all users on channel, who are allowed to read memos will get a PRIVMSG<br/>
 *       if the target is a channel. - the PRIVMSG mode is independent from /msg NickServ SET MSG
 *   
 *   2 - Target user will get a PRIVMSG that he/she has a new memo<br/>
 *       (all users on channel, who are allowed to read memos will get a PRIVMSG<br/>
 *       if the target is a channel. - the PRIVMSG mode is independent from /msg NickServ SET MSG<br/>
 *       AND an email will be set to the channels/users email address (within the sended memo)
 *
 *   Note: If you have opers on both the oper list and the<br/>
 *   admin list they will receive the memo twice.  The same<br/>
 *   applies for oper's on the Root list as well as other<br/>
 *   lists.
 *
 *   Syntax is: /msg MemoServ STAFF < priority[0,1,2] (may be disabled) > < memo >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define LOW 0
#define NORMAL 1
#define HIGH 2

#define SERVICE "ms"
#define MODUL "staff"
#define MODUL_FILE "ms_staff"
#define MODUL_VERSION "1.0"

MOD_BOOL ms_staff(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_staff_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ms_staff_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"staff","staff");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"staff",ms_staff,is_services_oper,is_services_user,4,0);
	addCommand(thread,&Module,"help staff",ms_staff_help,is_services_oper,is_services_user,4,4);
	addCommand(thread,&Module,"help",ms_staff_help_listing,is_services_oper,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the MemoServ STAFF Command.
 * 
 * The MemoServ STAFF Command allows the user to send the services staff<br/>
 *   a memo.
 *
 *   Note: If you have opers on both the oper list and the<br/>
 *   admin list they will receive the memo twice.  The same<br/>
 *   applies for oper's on the Root list as well as other<br/>
 *   lists.
 *
 * Syntax is: /msg MemoServ STAFF < priority[0,1,2] (may be disabled) > < memo >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_staff(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL staff;
	MailInfo *mail = NULL;
    char From[80];
    char Subject[80];
	char *unick   		= NULL;
	int memos    		= 0;
	int priority 		= 0;
	int i        		= 0;
	size_t memojoin 	= 0;
	
	/* Initialise Variables to (NULL) */
	bzero(&staff,sizeof(staff));
	bzero(&From,sizeof(From));
	bzero(&Subject,sizeof(Subject));
	
	/*
	 * Staff Command
	 */
 	// :shock PRIVMSG memoserv :staff priority memo
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"staff")){
		if(!digitcol(param[4]) || !m->MSUsePriorities)
			priority = LOW;
		else
			priority = stoi(param[4]);
		
		if(priority>=0 && priority<=2){
			if(!digitcol(param[4])){
				memojoin=4;
				if(m->MSUsePriorities) send_lang(thread,language,howtosend,snick,nick,"MS_REMEMBER_PRIORITIES",snick);
			}else
				memojoin=5;
				
			staff.res    = doSelectMySQL(thread,"SELECT `authnick`,`maxmemos`,`language`,`settings`,`email` FROM `orion_ns_core` WHERE `rights`>='%d'",is_services_oper);
            staff.fields = mysql_num_rows(staff.res);
            
            for(i=0;i<staff.fields;i++){
				staff.row = mysql_fetch_row(staff.res);
				
				memos = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_ms_core` WHERE `receiver`='%s'",mysql_quote(thread,staff.row[0])));
				
                if(memos+1<stoi(staff.row[1]) || thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"staff")){
					domysql(thread,"INSERT INTO `orion_ms_core` (`sender`,`receiver`,`priority`,`rsend`,`mode`,`time`,`memo`) VALUES ('%s','%s','%d','0','0','%ld','%s')",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,staff.row[0]),priority,time(NULL),mysql_quote(thread,_dstrjoin(' ',param,memojoin)));
                    unick = mysql_sel_var(thread,"SELECT `nick` FROM `orion_ns_aliases` WHERE `display`='%s' AND `active`='1'",mysql_quote(thread,staff.row[0]));
                    
                    if(unick != NULL){
    					if(priority==LOW)         	send_lang(thread,languages[stoi(staff.row[2])],stoi(staff.row[3])&NS_SET_MSG ? 1 : 0,snick,unick,"MS_SEND_RECEIVED_NEW_MEMO_FROM_X",thread->sql.aliases.authnick,snick,memos+1,priority,snick);
    					else if(priority==NORMAL)	send_lang(thread,languages[stoi(staff.row[2])],1,snick,unick,"MS_SEND_RECEIVED_NEW_MEMO_FROM_X",thread->sql.aliases.authnick,snick,memos+1,priority,snick);
    					else if(priority==HIGH){
							send_lang(thread,languages[stoi(staff.row[2])],1,snick,unick,"MS_SEND_RECEIVED_NEW_MEMO_FROM_X",thread->sql.aliases.authnick,snick,memos+1,priority,snick);
    						if(staff.row[4][0]!='\0' && m->UseMail){

    							dynncpy(From,_dynadd("%s Network (%s) <%s>",m->network,ms->nick,m->SendFrom),sizeof(From));
    							dynncpy(Subject,"New Memo To Staff Team",sizeof(Subject));
    					
    							mail = o_email(thread,From,"",staff.row[4],Subject);
    							//Email					
    							fprintf(mail->pipe,"There is a new memo for the Staff Team\n"
    							"Sent by MainNick / Nick: %s / %s\n"
    							"Memo Nr.: %d\n"
    							"Issue: %s\n",
    							thread->sql.aliases.authnick,nick,memos+1,_dstrjoin(' ',param,memojoin));
    							c_email(thread,mail);
    						}
                        }
					}
                }
			}
			
    		/* mysql_free_result_myr(); - staff */
    		mysql_free_result_myr(staff.res);
		}else send_lang(thread,language,howtosend,snick,thread->sql.aliases.authnick,"MS_SEND_INVALID_PRIORITY_LEVEL",priority,snick);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the MemoServ STAFF Command (/msg MemoServ HELP STAFF)
 * 
 * When the user tries to get help for this command, using /msg MemoServ HELP STAFF,<br/>
 * this function is executed.
 * 
 * Syntax: /msg MemoServ HELP STAFF
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_staff_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Staff Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help staff"))
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_STAFF");

	else send_lang(thread,language,howtosend,snick,nick,"MS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief MemoServ Help List Command - STAFF part (/msg MemoServ help)
 * 
 * When the user tries to get general MemoServ help, using /msg MemoServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the STAFF command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ms_staff_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_MS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Staff (Help listing)
	 */
	if(Users_MS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"MS_HELP_CMD_STAFF");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
