/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_confirm.c
 *   @brief This is the Module file for NickServ REGISTER Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ CONFIRM command.
 *
 *   The NickServ CONFIRM Command allows the user to verify that his Email was correct.<br/>
 *   (EAS - Email Activation System)
 *
 *   Syntax is: /msg NickServ CONFIRM < passcode (from the Email) >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "confirm"
#define MODUL_FILE "ns_confirm"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_confirm(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_confirm_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_confirm_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;

	if(!m->NSEmailReg){
		Module.ModErr = MOD_ERR_CANCELLED;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"confirm","confirm");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,USER);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"confirm",ns_confirm,is_user,is_user,3,5);
	addCommand(thread,&Module,"help confirm",ns_confirm_help,is_user,is_user,4,4);
	addCommand(thread,&Module,"help",ns_confirm_help_listing,is_user,is_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ CONFIRM Command.
 * 
 * The NickServ CONFIRM Command allows the user to verify that his Email was correct.<br/>
 *   (EAS - Email Activation System)
 *
 * Syntax is: /msg NickServ CONFIRM < passcode (from the Email) >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_confirm(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
    /* Global Variables */
    deepSQL request;
	
	/* Initialise Variables to (NULL) */
	bzero(&request,sizeof(request));

	/*
	 * Confirm Command
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"confirm")){ //cmd confirm
		request.res    = doSelectMySQL(thread,"SELECT `display`,`passcode`,`password`,`email`,`reqtime`,`reqsent`,`emailchange` FROM `orion_ns_request` WHERE `display`='%s'",mysql_quote(thread,nick));
		request.fields = mysql_num_rows(request.res);
		
		if(request.fields){
			request.row = mysql_fetch_row(request.res);
			if(streq(param[4],request.row[1]) && stoi(request.row[6])==0){
				
				domysql(thread,"INSERT INTO `orion_ns_core` (`authnick`,`password`,`qlevel`,"
												  "`olevel`,`slevel`,`llevel`,`email`,"
												  "`realname`,`language`,`rights`,"
												  "`maxmemos`,`settings`)"
												"VALUES ('%s','%s','1','0','0','1','%s','%s','%d','%d','%d','%d')",
							mysql_quote(thread,request.row[0]),mysql_quote(thread,request.row[2]),mysql_quote(thread,request.row[3]),mysql_quote(thread,thread->sql.Cdata.realname),m->language,is_services_user,m->maxmemos,NSCheckSettings());
				
				domysql(thread,"INSERT INTO `orion_ns_aliases` (`nick`,`display`,`last_usermask`,`last_realname`,"
										"`time_linked`,`last_login`,`last_used`,"
										"`expire`,`linked`,`active`,`forbid`)"
										" VALUES ('%s','%s','%s','%s','%ld','%ld','%ld',1,'0','1','0')",
							mysql_quote(thread,request.row[0]),mysql_quote(thread,request.row[0]),mysql_quote(thread,thread->sql.Cdata.vhost),mysql_quote(thread,thread->sql.Cdata.realname),time(NULL),time(NULL),time(NULL));
				
				domysql(thread,"DELETE FROM `orion_ns_request` WHERE `display`='%s'",mysql_quote(thread,request.row[0]));
				
				
				send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_NICK_REGISTERED",nick,thread->sql.Cdata.vhost);
				if(streqnc(orion_check_encryption(),NO_ENCRYPT))
					send_lang(thread,language,howtosend,snick,nick,"NS_REGISTER_NICK_PASSWORD_IS",request.row[2]);
				do_svs2mode(thread,ns->nick,nick,"+r");
				
				do_logchan(thread,snick,NULL,NULL,NULL,"group %s has been created",nick);
				do_logchan(thread,snick,NULL,NULL,NULL,"\'%s\' registered by %s@%s (email: %s)",nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,request.row[3]);
			}
			else if(streq(param[4],request.row[1]) && stoi(request.row[6])==1){
				
				domysql(thread,"UPDATE `orion_ns_core` SET `email`='%s' WHERE `authnick`='%s'",
							mysql_quote(thread,request.row[3]),mysql_quote(thread,request.row[0]));
				
				domysql(thread,"DELETE FROM `orion_ns_request` WHERE `display`='%s'",mysql_quote(thread,request.row[0]));
				
				send_lang(thread,language,howtosend,snick,nick,"NS_SET_CHANGED_EMAIL",request.row[3]);
				
				do_logchan(thread,snick,NULL,NULL,NULL,"\'%s\' validated new email: %s",request.row[0],request.row[3]);
			}
			else send_lang(thread,language,howtosend,snick,nick,"NS_CONFIRM_WRONG_PASSCODE");
		}else send_lang(thread,language,howtosend,snick,nick,"NICK_WAS_NOT_REQUESTED");

		/* MySQL Free Result */
		mysql_free_result_myr(request.res);
	}else{
	/*
	 *  Error Messages
	 */
	 	send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ CONFIRM Command (/msg NickServ HELP CONFIRM)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP CONFIRM,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP CONFIRM
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_confirm_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Confirm Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help confirm"))
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CONFIRM",snick,snick);

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - CONFIRM part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the CONFIRM command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_confirm_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Confirm (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_CONFIRM");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
