/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_drop.c
 *   @brief This is the Module file for NickServ DROP Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ DROP command.
 *
 *   The NickServ DROP Command allows the user to drop his nickname.<br/>
 *   (NOTE: If it's the last nick in your Group all settings<br/>
 *    and all channel access will be deleted)<br/>
 *   ServicesAdmins are able to give < nick > as parameter (to drop another users nick!)
 *
 *   Syntax is: /msg NickServ DROP [nick]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "drop"
#define MODUL_FILE "ns_drop"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_drop(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_drop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_drop_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"drop","drop");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"drop",ns_drop,is_services_admin,is_services_user,4,5);
	addCommand(thread,&Module,"help drop",ns_drop_help,is_services_admin,is_services_user,4,4);
	addCommand(thread,&Module,"help",ns_drop_help_listing,is_services_admin,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ DROP Command.
 * 
 * The NickServ DROP Command allows the user to drop his nickname.<br/>
 *   (NOTE: If it's the last nick in your Group all settings<br/>
 *    and all channel access will be deleted)<br/>
 *   ServicesAdmins are able to give < nick > as parameter (to drop another users nick!)
 *
 * Syntax is: /msg NickServ DROP [GROUP] [nick]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_drop(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
    Aliases aliases5;
    Access   access5;

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
	
	/*
	 * Drop Command
	 */
	 //:shock PRIVMSG NickServ :drop "GROUP" [shock]
	if(param[5] && streqnc(param[4],"group")){
		if(thread->sql.access.rights<ModuleFile_RootCommandLevel(thread,MODUL_FILE,"drop") && thread->sql.access.rights>=ModuleFile_UserCommandLevel(thread,MODUL_FILE,"drop")){
			aliases5 = sql_aliases(thread,param[5]);
			access5  = sql_access(thread,aliases5.authnick);
			/* User Drop - only in own group */
			if(streqnc(aliases5.authnick,thread->sql.aliases.authnick)){
				if(IRC.NickRegMode){
					do_svs2mode(thread,ns->nick,param[5],IRC.ModeOnNickDrop);
				}
						
				if(IRC.p10protocol){
					do_del_account(thread,GetUser(param[5])->uid);
				}
				
				DropAuthnick(thread, aliases5.authnick);
				
				send_lang(thread,language,howtosend,snick,nick,"NS_DROP_DROPPED_X",aliases5.authnick);
				do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"dropped nickname %s (group %s) (e-mail: %s)",aliases5.authnick,aliases5.authnick,access5.email);
				do_logchan(thread,snick,NULL,NULL,NULL,"%s deleted group %s (per command) (e-mail: %s)",nick,aliases5.authnick,access5.email);
			}else send_lang(thread,language,howtosend,snick,nick,"NICK_NOT_IN_YOUR_GROUP");
		}else if(param[4] && thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"drop")){
			aliases5 = sql_aliases(thread,param[5]);
			access5  = sql_access(thread,aliases5.authnick);
			
			if(m->WallDrop)
				do_global(thread,snick,"%s used drop for group %s",nick,aliases5.authnick);
				
			if(!m->NSSecureAdmins || (m->NSSecureAdmins && access5.rights<=is_services_oper) || streqnc(access5.authnick,thread->sql.access.authnick)){
				
				if(IRC.NickRegMode){
					do_svs2mode(thread,ns->nick,param[5],IRC.ModeOnNickDrop);
				}
						
				if(IRC.p10protocol){
					do_del_account(thread,GetUser(param[5])->uid);
				}
				
				DropAuthnick(thread, aliases5.authnick);
				
				send_lang(thread,language,howtosend,snick,nick,"NS_DROP_DROPPED_X",aliases5.authnick);
				do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"dropped nickname %s (group %s) (e-mail: %s)",aliases5.authnick,aliases5.authnick,access5.email);
				do_logchan(thread,snick,NULL,NULL,NULL,"%s deleted group %s (there aren't group members anymore) (e-mail: %s)",nick,aliases5.authnick,access5.email);
			}else send_lang(thread,language,howtosend,snick,nick,"NS_DROP_CANT_DROP_SA_SRA_X",aliases5.authnick);
		}
	}
	 //:shock PRIVMSG NickServ :drop [shock]
	else if(param[4] && thread->sql.access.rights<ModuleFile_RootCommandLevel(thread,MODUL_FILE,"drop") && thread->sql.access.rights>=ModuleFile_UserCommandLevel(thread,MODUL_FILE,"drop")){
		aliases5 = sql_aliases(thread,param[4]);
		access5  = sql_access(thread,aliases5.authnick);
		
		if(streqnc(aliases5.authnick,thread->sql.aliases.authnick)){
			if(streqnc(param[4],aliases5.authnick)){
				char newdisplay[MAX_NICK_LEN*2];
				bzero(&newdisplay,sizeof(newdisplay));
				dynncpy(newdisplay,mysql_sel_var(thread,"SELECT `nick` FROM `orion_ns_aliases` WHERE `display`='%s' AND `nick`!='%s' LIMIT 0,1",mysql_quote(thread,aliases5.authnick),mysql_quote(thread,aliases5.authnick)),sizeof(newdisplay));
				
				if(newdisplay[0]=='\0'){
					
					if(IRC.NickRegMode){
						do_svs2mode(thread,ns->nick,param[4],IRC.ModeOnNickDrop);
					}
						
					if(IRC.p10protocol){
						do_del_account(thread,GetUser(param[4])->uid);
					}
					
					DropAuthnick(thread, aliases5.authnick);
						
					send_lang(thread,language,howtosend,snick,nick,"NS_DROP_DROPPED_X",aliases5.authnick);
					do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"dropped nickname %s (group %s) (e-mail: %s)",nick,param[4],access5.email);
					do_logchan(thread,snick,NULL,NULL,NULL,"%s deleted group %s (there aren't group members anymore) (e-mail: %s)",nick,param[4],access5.email);
				}else{
					domysql(thread,"DELETE FROM `orion_ns_aliases` WHERE `nick`='%s'",mysql_quote(thread,param[4]));
					
					/* Update NickServ,MemoServ Information */
					domysql(thread,"UPDATE `orion_ns_core` LEFT OUTER JOIN `orion_ms_core` ON (`orion_ms_core`.`receiver`=`orion_ns_core`.`authnick`) SET `orion_ns_core`.`authnick`='%s',`orion_ms_core`.`receiver`='%s' WHERE `authnick`='%s'",mysql_quote(thread,newdisplay),mysql_quote(thread,newdisplay),mysql_quote(thread,param[4]));
					
					do_logchan(thread,snick,NULL,NULL,NULL,"%s changed %s group (display) to %s (newdisplay) (e-mail: %s)",nick,param[4],newdisplay,access5.email);
				}
				/* 
				InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
				domysql(thread,"DELETE FROM `orion_ns_aliases` WHERE `display`='%s'",mysql_quote(thread,aliases5.authnick));
				*/
				//send_lang(thread,language,howtosend,snick,nick,"NS_DROP_DROPPED_X",param[4]);
				//do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"dropped nickname %s (group %s) (e-mail: %s)",aliases5.authnick,aliases5.authnick,access5.email);

			}else if(streqnc(nick,aliases5.nick)){
				domysql(thread,"DELETE FROM `orion_ns_aliases` WHERE `nick`='%s'",mysql_quote(thread,param[4]));
				send_lang(thread,language,howtosend,snick,nick,"NS_DROP_DROPPED_X",nick);
				do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"dropped nickname %s (group %s) (e-mail: %s)",nick,aliases5.authnick,access5.email);
				
				if(IRC.NickRegMode){
					do_svs2mode(thread,ns->nick,param[4],IRC.ModeOnNickDrop);
				}
						
				if(IRC.p10protocol){
					do_del_account(thread,GetUser(param[4])->uid);
				}
				
			}else send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		}else send_lang(thread,language,howtosend,snick,nick,"NICK_NOT_IN_YOUR_GROUP");
	}
	else if(param[4] && thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"drop")){
		aliases5 = sql_aliases(thread,param[4]);
		access5  = sql_access(thread,aliases5.authnick);

		if(!m->NSSecureAdmins || (m->NSSecureAdmins && access5.rights<=is_services_oper) || streqnc(access5.authnick,thread->sql.access.authnick)){
			if(streqnc(param[4],aliases5.authnick)){
				if(m->WallDrop)
					do_global(thread,snick,"%s used drop for group %s",nick,aliases5.authnick);
				
				char newdisplay[MAX_NICK_LEN*2];
				bzero(&newdisplay,sizeof(newdisplay));
				dynncpy(newdisplay,mysql_sel_var(thread,"SELECT `nick` FROM `orion_ns_aliases` WHERE `display`='%s' AND `nick`!='%s' LIMIT 0,1",mysql_quote(thread,aliases5.authnick),mysql_quote(thread,aliases5.authnick)),sizeof(newdisplay));
				
				if(newdisplay[0]=='\0'){
					
					if(IRC.NickRegMode){
						do_svs2mode(thread,ns->nick,param[4],IRC.ModeOnNickDrop);
					}
				
					if(IRC.p10protocol){
						do_del_account(thread,GetUser(param[4])->uid);
					}
					
					DropAuthnick(thread, aliases5.authnick);
					
					send_lang(thread,language,howtosend,snick,nick,"NS_DROP_DROPPED_X",aliases5.authnick);
					do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"dropped nickname %s (group %s) (e-mail: %s)",nick,param[4],access5.email);
					do_logchan(thread,snick,NULL,NULL,NULL,"%s deleted group %s (there aren't group members anymore) (e-mail: %s)",nick,param[4],access5.email);
				}else{
					domysql(thread,"DELETE FROM `orion_ns_aliases` WHERE `nick`='%s'",mysql_quote(thread,param[4]));
					
					/* Update NickServ,MemoServ Information */
					domysql(thread,"UPDATE `orion_ns_core` LEFT OUTER JOIN `orion_ms_core` ON (`orion_ms_core`.`receiver`=`orion_ns_core`.`authnick`) SET `orion_ns_core`.`authnick`='%s',`orion_ms_core`.`receiver`='%s' WHERE `authnick`='%s'",mysql_quote(thread,newdisplay),mysql_quote(thread,newdisplay),mysql_quote(thread,param[4]));
					
					do_logchan(thread,snick,NULL,NULL,NULL,"%s changed %s group (display) to %s (newdisplay) (e-mail: %s)",nick,param[4],newdisplay,access5.email);
				}
				/* 
				InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
				domysql(thread,"DELETE FROM `orion_ns_aliases` WHERE `display`='%s'",mysql_quote(thread,aliases5.authnick));
				*/
				//send_lang(thread,language,howtosend,snick,nick,"NS_DROP_DROPPED_X",param[4]);
				//do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"dropped nickname %s (group %s) (e-mail: %s)",aliases5.authnick,aliases5.authnick,access5.email);
				
			}else if(streqnc(param[4],aliases5.nick)){
				if(m->WallDrop)
					do_global(thread,snick,"%s used drop for nick %s",nick,aliases5.nick);
				
				domysql(thread,"DELETE FROM `orion_ns_aliases` WHERE `nick`='%s'",mysql_quote(thread,param[4]));
				send_lang(thread,language,howtosend,snick,nick,"NS_DROP_DROPPED_X",param[4]);
				do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"dropped nickname %s (group %s) (e-mail: %s)",param[4],aliases5.authnick,access5.email);
				
				if(IRC.NickRegMode){
					do_svs2mode(thread,ns->nick,param[4],IRC.ModeOnNickDrop);
				}
				
				if(IRC.p10protocol){
					do_del_account(thread,GetUser(param[4])->uid);
				}
				
			}else send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
		}else send_lang(thread,language,howtosend,snick,nick,"NS_DROP_CANT_DROP_SA_SRA_X",aliases5.authnick);
	}
	else if(!param[4] && thread->sql.access.rights>=ModuleFile_UserCommandLevel(thread,MODUL_FILE,"drop")){
		if(streqnc(nick,thread->sql.aliases.authnick)){
			char newdisplay[MAX_NICK_LEN*2];
			bzero(&newdisplay,sizeof(newdisplay));
			dynncpy(newdisplay,mysql_sel_var(thread,"SELECT `nick` FROM `orion_ns_aliases` WHERE `display`='%s' AND `nick`!='%s' LIMIT 0,1",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,thread->sql.aliases.authnick)),sizeof(newdisplay));
			
			if(newdisplay[0]=='\0'){
				
				if(IRC.NickRegMode){
					do_svs2mode(thread,ns->nick,nick,IRC.ModeOnNickDrop);
				}
				
				if(IRC.p10protocol){
					do_del_account(thread,GetUser(nick)->uid);
				}
				
				DropAuthnick(thread, thread->sql.aliases.authnick);
				
				send_lang(thread,language,howtosend,snick,nick,"NS_DROP_DROPPED_X",thread->sql.aliases.authnick);
				do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"dropped nickname %s (group %s) (e-mail: %s)",nick,thread->sql.aliases.authnick,thread->sql.access.email);
				do_logchan(thread,snick,NULL,NULL,NULL,"%s deleted group %s (there aren't group members anymore) (e-mail: %s)",nick,thread->sql.aliases.authnick,thread->sql.access.email);
			}else{
				domysql(thread,"DELETE FROM `orion_ns_aliases` WHERE `nick`='%s'",mysql_quote(thread,nick));
				
				/* Update NickServ,MemoServ Information */
				domysql(thread,"UPDATE `orion_ns_core` LEFT OUTER JOIN `orion_ms_core` ON (`orion_ms_core`.`receiver`=`orion_ns_core`.`authnick`) SET `orion_ns_core`.`authnick`='%s',`orion_ms_core`.`receiver`='%s' WHERE `authnick`='%s'",mysql_quote(thread,newdisplay),mysql_quote(thread,newdisplay),mysql_quote(thread,nick));
					
				do_logchan(thread,snick,NULL,NULL,NULL,"%s changed %s group (display) to %s (newdisplay) (e-mail: %s)",nick,thread->sql.aliases.authnick,newdisplay,thread->sql.access.email);
			}
			/* 
			InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
			domysql(thread,"DELETE FROM `orion_ns_aliases` WHERE `display`='%s'",mysql_quote(thread,thread->sql.aliases.authnick));
			*/
			//send_lang(thread,language,howtosend,snick,nick,"NS_DROP_DROPPED_X",nick);
			//do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"dropped nickname %s (group %s) (e-mail: %s)",thread->sql.aliases.authnick,thread->sql.aliases.authnick,thread->sql.access.email);
			
		}else if(streqnc(nick,thread->sql.aliases.nick)){
			domysql(thread,"DELETE FROM `orion_ns_aliases` WHERE `nick`='%s'",mysql_quote(thread,nick));
			send_lang(thread,language,howtosend,snick,nick,"NS_DROP_DROPPED_X",nick);
			do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"dropped nickname %s (group %s) (e-mail: %s)",nick,thread->sql.aliases.authnick,thread->sql.access.email);
			
			if(IRC.NickRegMode){
				do_svs2mode(thread,ns->nick,nick,IRC.ModeOnNickDrop);
			}
				
			if(IRC.p10protocol){
				do_del_account(thread,GetUser(nick)->uid);
			}
			
		}else send_lang(thread,language,howtosend,snick,nick,"UNKNOWN_USER");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NOT_LOGGED_IN_OR_NOT_ENOUGH_RIGHTS");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ DROP Command (/msg NickServ HELP DROP)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP DROP,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP DROP
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_drop_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Drop Help
	 */
	if(strcor(thread->sql.Cdata.modes,"o") && thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"help drop"))
		send_lang(thread,language,howtosend,snick,nick,"NS_SERVADMIN_HELP_DROP",snick);
	else if(thread->sql.access.rights>=ModuleFile_UserCommandLevel(thread,MODUL_FILE,"help drop"))
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_DROP",snick,snick);

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - DROP part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the DROP command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_drop_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_UserCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Drop (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_DROP");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
