/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 203                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 23:10:45 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_identify.c
 *   @brief This is the Module file for NickServ IDENTIFY Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ IDENTIFY command.
 *
 *   The NickServ IDENTIFY Command allows the user to identify himself<br/>
 *   with NickServ, to ensure that the used nick is his!
 *
 *   Syntax is: /msg NickServ IDENTIFY < password >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "identify"
#define MODUL_FILE "ns_identify"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_identify(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_identify_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_identify_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"identify","identify");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,1,OPER_IF_DEFCON);
	addAccess (thread,&Module,USER);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"identify",ns_identify,is_user,is_user,4,4);
	addCommand(thread,&Module,"help identify",ns_identify_help,is_user,is_user,4,4);
	addCommand(thread,&Module,"help",ns_identify_help_listing,is_user,is_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ IDENTIFY Command.
 * 
 * The NickServ IDENTIFY Command allows the user to identify himself<br/>
 *   with NickServ, to ensure that the used nick is his!
 *
 * Syntax is: /msg NickServ IDENTIFY < password >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_identify(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL nickarray;
	deepSQL accesslist;
	deepSQL staff;
	MailInfo *mail 	= NULL;
	Users *user 	= NULL;

    char From[MAX_EMAIL_LEN];
    char Subject[MAX_DESCRIPTION_LEN];
	char strinstr[300];
	char regdate[200];
	char enc_password[ENC_PASSMAX];

	char *checker[10];
	char *unick = NULL;
	char *cmd 	= NULL;
	
	int memos 	= 0;
	int i    	= 0;
	time_t now 	= 0;

	/* Initialise Variables to (NULL) */
	bzero(&staff,sizeof(staff));
	bzero(&From,sizeof(From));
	bzero(&Subject,sizeof(Subject));
	
	bzero(&nickarray,sizeof(nickarray));
	bzero(&accesslist,sizeof(accesslist));
	bzero(&checker,sizeof(checker));
    bzero(&strinstr,sizeof(strinstr));
	bzero(&regdate,sizeof(regdate));
	bzero(&enc_password,sizeof(enc_password));

	/*
	 * Identify Command
	 */
	cmd=&param[3][1];
	//:shock PRIVMSG NickServ :identify PW
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"identify")){
		if(thread->sql.access.settings&NS_SUSPENDED){
			send_lang(thread,language,howtosend,snick,nick,"LOGIN_SUSPENDED");
			send_lang(thread,language,howtosend,snick,nick,"LOGIN_SUSPEND_REASON",thread->sql.access.reason);
			return MOD_CONT;
		}
        else if(thread->sql.aliases.forbid){
			send_lang(thread,language,howtosend,snick,nick,"NS_GROUP_NICK_FORBIDDEN_WITH_REASON",thread->sql.aliases.last_quit);
			return MOD_CONT;
		}
		
        else if(thread->sql.aliases.active){
			send_lang(thread,language,howtosend,snick,nick,"LOGGED_IN_ALREADY");
			return MOD_CONT;
		}

		if(orion_check_password(param[4],thread->sql.access.password)>0){		
			now = time(NULL);
			
			//if(thread->sql.aliases.expire!=0) domysql(thread,"UPDATE `orion_ns_aliases` SET `expire`='%d',`active`='1',`last_login`='%d',`last_used`='%d' WHERE `nick`='%s'",expire(m->nickexpiretime),time(NULL),time(NULL),mysql_quote(thread,thread->sql.aliases.nick));
			/*else */domysql(thread,"UPDATE `orion_ns_aliases` SET `active`='1',`last_login`='%ld',`last_used`='%ld',`last_usermask`='%s' WHERE `nick`='%s'",now,now,mysql_quote(thread,thread->sql.Cdata.vhost),mysql_quote(thread,thread->sql.aliases.nick));
			domysql(thread,"UPDATE `orion_core_condata` SET `ns_warntime`='0' WHERE `nick`='%s'",mysql_quote(thread,thread->sql.aliases.nick));
			/*  SELECT all nicks with authnick in '%s - param[4]' that authed earlier
			 *  to send them a notice that another user authed with the same authnick and password.
			 */
			 if(m->SecureIdentify){
				nickarray.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_ns_aliases` WHERE `display`='%s' AND `last_login`<'%ld' AND `active`='1'",mysql_quote(thread,thread->sql.aliases.authnick),now);
				nickarray.fields = mysql_num_rows(nickarray.res);
	            
	            if(nickarray.fields != 0 && thread->sql.Cdata.fields != 0){
							/////??????
					for(i=0;i<nickarray.fields;i++){
						nickarray.row = mysql_fetch_row(nickarray.res);
						
						send_lang(thread,language,howtosend,ns->nick,nickarray.row[0],"Q_AUTHED_WITH_YOUR_PASSWORD",nick,thread->sql.Cdata.ident,"@",thread->sql.Cdata.rhost,i,ns->nick,nick); 
							/////?????
					}
				}

				/* MySQL Free Result */
				mysql_free_result_myr(nickarray.res);
			}

			/* NickServ auth notice */
			send_lang(thread,language,howtosend,snick,nick,"LOGIN_SUCCESSFUL",m->network,snick,"@",ns->host);
			if(Q->loaded)
				send_lang(thread,language,howtosend,Q->nick,nick,"AUTHED_SUCCESSFUL");
			do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"identified for nick %s",nick);
			
			user 		 = GetUser(nick);
			
			if(IRC.NickRegMode){
				//printf("IRC.ModeOnNickReg == %s\n",IRC.ModeOnNickReg);
				do_svs2mode(thread,ns->nick,nick,IRC.ModeOnNickReg);
			}

			if(IRC.p10protocol){
				do_add_account(thread,user->uid,thread->sql.access.authnick);
			}
			
			user->rights = thread->sql.access.rights;
			/* Update our struct for Ignoring IRCOps > ServicesOperator */
			
			if(thread->sql.access.failed_logins && m->SecureIdentify){	//failed logins listing
				bzero(&strinstr,sizeof(strinstr));
				strmatch(thread->sql.access.password,thread->sql.access.last_failed_login_password,strinstr,sizeof(strinstr));
			
				send_lang(thread,language,howtosend,snick,nick,"NS_IDENTIFY_FAILED_LOGINS_LIST",get_localtime(thread->sql.access.last_failed_login_time),thread->sql.access.failed_logins);
				if(strinstr[0]!='\0' && streqnc(m->encrypt,"none"))
					send_lang(thread,language,howtosend,snick,nick,"NS_IDENTIFY_FAILED_LOGINS_ADDITIONAL_LIST",(100*dynlen(strinstr)/dynlen(thread->sql.access.password)),thread->sql.access.password,thread->sql.access.last_failed_login_password,strinstr);
				domysql(thread,"UPDATE `orion_ns_core` SET `failed_logins`='' WHERE `authnick`='%s'",thread->sql.access.authnick);
			}
			
			/* SELECT Q-chans */
			bzero(&accesslist,sizeof(accesslist));
			accesslist.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`channel`,`orion_cs_access`.`level`,`orion_cs_core`.`settings`,`orion_cs_core`.`datecreate` FROM `orion_cs_access` LEFT JOIN `orion_cs_core` ON `orion_cs_access`.`channel`=`orion_cs_core`.`channel` LEFT OUTER JOIN `orion_chan_users` ON `orion_cs_access`.`channel`=`orion_chan_users`.`channel` WHERE `orion_cs_access`.`authnick`='%s' AND `orion_cs_core`.`serv`='ChanServ' AND `orion_chan_users`.`nick`='%s'",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,nick));
			accesslist.fields = mysql_num_rows(accesslist.res);
            
            if(accesslist.fields){
				for(i=0;i<accesslist.fields;i++){
					accesslist.row = mysql_fetch_row(accesslist.res);
					
					alog(thread,"     Q - Auth: [channel] = %s\n",accesslist.row[0]);
					alog(thread,"     Q - Auth: acclevel for User(%s) = %s\n",thread->sql.access.authnick,accesslist.row[1]);
					alog(thread,"     Q - Auth: if User(%s) is on channel %s\n",nick,accesslist.row[0]);
					
					if(strcor(accesslist.row[1],"b")){
						do_mode(thread,Q->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+b %s",BuildBanMask(thread,&thread->sql.Cdata,CSAnalyseBantype(stoi(accesslist.row[2]))));
						do_kick(thread,Q->nick,accesslist.row[0],nick,"%s-banned",Q->nick);
                    }
					else if(strcor(accesslist.row[1],"a")){
						if(strcor(accesslist.row[1],"n")){
							if(strcor(accesslist.row[1],"p"))
								do_mode(thread,Q->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+qoa %s %s %s",nick,nick,nick);
							else
								do_mode(thread,Q->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+qo %s %s",nick,nick);
						}else if(strcor(accesslist.row[1],"om")){
							if(strcor(accesslist.row[1],"p"))
								do_mode(thread,Q->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+oa %s %s",nick,nick);
							else
								do_mode(thread,Q->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+o %s",nick);
						}else if(strcor(accesslist.row[1],"h")){
							if(strcor(accesslist.row[1],"p"))
								do_mode(thread,Q->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+ha %s %s",nick,nick);
							else
								do_mode(thread,Q->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+h %s",nick);
						}else if(strcor(accesslist.row[1],"v"))
							do_mode(thread,Q->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+v %s",nick);
					}
				}
			}
			
    		/* mysql_free_result_myr(); - accesslist */
    		mysql_free_result_myr(accesslist.res);
			
			/* SELECT L-chans */
			/*
			bzero(&accesslist,sizeof(accesslist));
			accesslist.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`channel`,`orion_cs_access`.`level`,`orion_cs_core`.`settings`,`orion_cs_core`.`datecreate` FROM `orion_cs_access` LEFT JOIN `orion_cs_core` ON `orion_cs_access`.`channel`=`orion_cs_core`.`channel` LEFT OUTER JOIN `orion_chan_users` ON `orion_cs_access`.`channel`=`orion_chan_users`.`channel` WHERE `orion_cs_access`.`authnick`='%s' AND `orion_cs_core`.`serv`='L' AND `orion_chan_users`.`nick`='%s'",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,nick));
			accesslist.fields = mysql_num_rows(accesslist.res);
            
            if(accesslist.fields){
				for(i=0;i<accesslist.fields;i++){
					accesslist.row = mysql_fetch_row(accesslist.res);
					
					alog(thread,"     L - Auth: [channel] = %s\n",accesslist.row[0]);
					alog(thread,"     L - Auth: acclevel for User(%s) = %s\n",param[4],accesslist.row[1]);
					alog(thread,"     L - Auth: if User(%s) is on channel %s\n",nick,accesslist.row[i]);

					if(strcor(accesslist.row[1],"a")){
						if(strcor(accesslist.row[1],"n")){
							if(strcor(accesslist.row[1],"p"))
								do_mode(thread,L->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+qoa %s %s %s",nick,nick,nick);
							else
								do_mode(thread,L->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+qo %s %s",nick,nick);
						}else if(strcor(accesslist.row[1],"om")){
							if(strcor(accesslist.row[1],"p"))
								do_mode(thread,L->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+oa %s %s",nick,nick);
							else
								do_mode(thread,L->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+o %s",nick);
						}else if(strcor(accesslist.row[1],"h")){
							if(strcor(accesslist.row[1],"p"))
								do_mode(thread,L->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+ha %s %s",nick,nick);
							else
								do_mode(thread,L->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+h %s",nick);
						}else if(strcor(accesslist.row[1],"v"))
							do_mode(thread,L->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+v %s",nick);
					}
				}
			}*/
			
    		/* mysql_free_result_myr(); - accesslist */
    		//mysql_free_result_myr(accesslist.res);
			
			/* SELECT ChanServ-chans */
			bzero(&accesslist,sizeof(accesslist));
            accesslist.res    = doSelectMySQL(thread,"SELECT `orion_cs_core`.`channel`,`orion_cs_access`.`level`,`orion_cs_core`.`settings`,`orion_cs_core`.`datecreate`,`orion_cs_core`.`cs_autovoice`,`orion_cs_core`.`cs_autoop`,`orion_cs_core`.`cs_autohalfop`,`orion_cs_core`.`cs_autoprotect` FROM `orion_cs_access` LEFT JOIN `orion_cs_core` ON `orion_cs_access`.`channel`=`orion_cs_core`.`channel` LEFT OUTER JOIN `orion_chan_users` ON `orion_cs_access`.`channel`=`orion_chan_users`.`channel` WHERE `orion_cs_access`.`authnick`='%s' AND `orion_cs_core`.`serv`='ChanServ' AND `orion_chan_users`.`nick`='%s'",mysql_quote(thread,thread->sql.aliases.authnick),mysql_quote(thread,nick));
			accesslist.fields = mysql_num_rows(accesslist.res);
            
            if(accesslist.fields){
				for(i=0;i<accesslist.fields;i++){
					accesslist.row = mysql_fetch_row(accesslist.res);
					
					alog(thread,"     ChanServ - Auth: [channel] = %s\n",accesslist.row[0]);
					alog(thread,"     ChanServ - Auth: acclevel for User(%s) = %s\n",param[4],accesslist.row[1]);
					alog(thread,"     ChanServ - Auth: if User(%s) is on channel %s\n",nick,accesslist.row[0]);
					
					if(stoi(accesslist.row[1])>=ACCESS_FOUNDER){
						if(stoi(accesslist.row[1])>=ACCESS_FOUNDER || (stoi(accesslist.row[1])>=stoi(accesslist.row[7])))
							do_mode(thread,cs->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+qoa %s %s %s",nick,nick,nick);
						else
							do_mode(thread,cs->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+qo %s %s",nick,nick);
					}
					
					else if(stoi(accesslist.row[1])>=stoi(accesslist.row[5])){
						if(stoi(accesslist.row[1])>=stoi(accesslist.row[7]))
							do_mode(thread,cs->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+oa %s %s",nick,nick);
						else
							do_mode(thread,cs->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+o %s",nick);
					}
					
					else if(stoi(accesslist.row[1])>=stoi(accesslist.row[6]))
						do_mode(thread,cs->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+h %s",nick);
					
					else if(stoi(accesslist.row[1])>=stoi(accesslist.row[4]))
						do_mode(thread,cs->nick,accesslist.row[0],(time_t)stol(accesslist.row[3]),"+v %s",nick);
				}
			}
			
    		/* mysql_free_result_myr(); - accesslist */
    		mysql_free_result_myr(accesslist.res);
			/************************ SEND identify to the eventhandler ****************************/
 			_ExecuteEvent(nick,param,"on_identify");
 			/***************************************************************************************/
		}else if(thread->sql.aliases.authnick[0]!='\0'){
			
			thread->sql.access.failed_logins++;
			send_lang(thread,language,howtosend,snick,nick,"WRONG_PASSWORD");
			domysql(thread,"UPDATE `orion_ns_core` SET `failed_logins`='%d',`last_failed_login`='%ld',`last_login_password`='%s' WHERE `authnick`='%s'",thread->sql.access.failed_logins,time(NULL),mysql_quote(thread,param[4]),mysql_quote(thread,thread->sql.access.authnick));
			do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"Failed IDENTIFY for %s",nick);
			
			if(m->BadPassLimit>0 && (thread->sql.access.failed_logins%m->BadPassLimit==0))
				do_kill(thread,m->servername,nick,"too many invalid passwords");
			
			if(m->SecureIdentify>0 && thread->sql.access.failed_logins==m->SecureIdentify){
/* Send a Memo to the Staff-Team */
				staff.res    = doSelectMySQL(thread,"SELECT `authnick`,`maxmemos`,`language`,`settings`,`email` FROM `orion_ns_core` WHERE `rights`>='%d'",is_services_oper);
	            staff.fields = mysql_num_rows(staff.res);
	            
	            for(i=0;i<staff.fields;i++){
					staff.row = mysql_fetch_row(staff.res);
					
					if(ms->loaded){
						memos = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_ms_core` WHERE `receiver`='%s'",mysql_quote(thread,staff.row[0])));
						
						domysql(thread,"INSERT INTO `orion_ms_core` (`sender`,`receiver`,`priority`,`rsend`,`mode`,`time`,`memo`) VALUES ('%s','%s','%d','0','0','%ld','%s')",mysql_quote(thread,ns->nick),mysql_quote(thread,staff.row[0]),2,time(NULL),
		    					mysql_quote(thread,
									_dynadd(
										"Fraud for Group / Nick: %s / %s\n"
				    					"Memo Nr.: %d\n"
				    					"Last known IP from fraudster: %s (check that with \"/whowas\" too)\n"
				    					"Issue: Possible fraud from target IP, trying to use target nick\n"
										"IRCOperators may check the IP for a dynamic/static account\n"
										"and inform the owner of the nick about a possible fraud.\n"
										"\n"
										"It is also recommanded to look through the matching percentage\n"
										"If it's neccesary an IRCOp should change the users password immediately, without\n"
										"informing the real owner.\n"
										"\n"
										"DO NOT DELETE THIS MEMO - take it for further details - to hand out\n"
										"the real nicks owner if possible!!"
			    					,thread->sql.aliases.authnick,nick,memos+1,thread->sql.Cdata.rhost)
								)
						);
					}
					
					unick = mysql_sel_var(thread,"SELECT `nick` FROM `orion_ns_aliases` WHERE `display`='%s' AND `active`='1'",mysql_quote(thread,staff.row[0]));  
					if(unick != NULL)
						send_lang(thread,languages[stoi(staff.row[2])],1,ms->nick,nick,"NS_IDENTIFY_POSSIBLE_NICK_ABUSE_FROM_X",thread->sql.aliases.authnick,ms->nick,memos+1,2,ms->nick);

    				if(staff.row[4][0]!='\0' && m->UseMail){
	
    					dynncpy(From,_dynadd("%s Network (%s) <%s>",m->network,ns->nick,m->SendFrom),sizeof(From));
    					dynncpy(Subject,"NickServ Abuse (possible fraud)",sizeof(Subject));
	    					
    					mail = o_email(thread,From,"",staff.row[4],Subject);
    					//Email					
    					fprintf(mail->pipe,
							"There is a new AbuseNotice for the Staff Team\n"
							"\n"
	    					"Fraud for Group / Nick: %s / %s\n"
	    					"Memo Nr.: %d\n"
	    					"Last known IP from fraudster: %s (check that with \"/whowas\" too)\n"
	    					"\n"
	    					"Issue: Possible fraud from target IP, trying to use target nick\n"
							"IRCOperators may check the IP for a dynamic/static account\n"
							"and inform the owner of the nick about a possible fraud.\n"
							"\n"
							"It is also recommanded to look through the matching percentage\n"
							"If it's neccesary an IRCOp should change the users password immediately, without\n"
							"informing the real owner.\n"
							"\n"
							"DO NOT DELETE THIS EMAIL - take it for further details - to hand out\n"
							"the real nicks owner if possible!!"
    					,thread->sql.aliases.authnick,nick,memos+1,thread->sql.Cdata.rhost);
    					c_email(thread,mail);
	                }
				}

				/* MySQL Free Result */
				mysql_free_result_myr(staff.res);
			}
		}else send_lang(thread,language,howtosend,snick,nick,"NO_LOGIN_OR_WRONG_PASSWORD");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ IDENTIFY Command (/msg NickServ HELP IDENTIFY)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP IDENTIFY,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP IDENTIFY
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_identify_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Identify Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help identify"))
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_IDENTIFY",snick);

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - IDENTIFY part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the IDENTIFY command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_identify_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Identify (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_IDENTIFY");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
