/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 203                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 23:10:45 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file ns_sendpass.c
 *   @brief This is the Module file for NickServ SENDPASS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the NickServ SENDPASS command.
 *
 *   The NickServ SENDPASS Command allows the user to request his<br/>
 *   password via email (it will be sent to the Email he registered with).<br/>
 *   This may be limited to Services Admins.<br/>
 *   (This feature is only trustable with EAS - Email Activation System)
 *
 *   Syntax is: /msg NickServ SENDPASS < nick >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "services.h"
#include "global.h"

#define SERVICE "ns"
#define MODUL "sendpass"
#define MODUL_FILE "ns_sendpass"
#define MODUL_VERSION "1.0"

MOD_BOOL ns_sendpass(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_sendpass_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL ns_sendpass_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;
	
	if(!m->UseMail){
		Module.ModErr = MOD_ERR_CANCELLED;
		Module.module = NULL;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"sendpass","sendpass");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"sendpass",ns_sendpass,is_services_admin,is_services_user,3,4);
	addCommand(thread,&Module,"help sendpass",ns_sendpass_help,is_services_user,is_services_user,4,4);
	addCommand(thread,&Module,"help",ns_sendpass_help_listing,is_services_user,is_services_user,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the NickServ SENDPASS Command.
 * 
 * The NickServ SENDPASS Command allows the user to request his<br/>
 *   password via email (it will be sent to the Email he registered with).
 *
 * Syntax is: /msg NickServ SENDPASS < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_sendpass(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	Aliases aliases5;
	Access access5;
	MailInfo *mail = NULL;
	char From[80];
	char Subject[80];
	char tmppasswd[MAX_PASSWORD_LEN];

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
	bzero(&From,sizeof(From));
	bzero(&Subject,sizeof(Subject));
	
	/*
	 * Sendpass Command
	 */
	if(param[4] && thread->sql.access.rights>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"sendpass") &&  m->UseMail){
        	aliases5 = sql_aliases(thread,param[4]);
        	access5  = sql_access(thread,aliases5.authnick);

			bzero(&tmppasswd,sizeof(tmppasswd));

			randomstring(tmppasswd,PASSWORD_LEN);
			dlog(thread,4,"generated pw = \"%s\"\n",tmppasswd);
			
			char enc_password[ENC_PASSMAX+1];
			bzero(&enc_password,sizeof(enc_password));
			
			if (orion_encrypt(tmppasswd, strlen(tmppasswd), enc_password, ENC_PASSMAX) < 0) {
				alog(thread,"%s: Failed to encrypt password for %s (register)",
					snick, nick);
				send_lang(thread,language,howtosend,snick,nick,"REGISTRATION_FAILED");
				return MOD_CONT;
			}

			domysql(thread,"UPDATE `orion_ns_core` SET `password`='%s' WHERE `authnick`='%s'",mysql_quote(thread,enc_password),mysql_quote(thread,aliases5.authnick));

			dynncpy(From,_dynadd("%s Network (%s) <%s>",m->network,ns->nick,m->SendFrom),sizeof(From));
			dynncpy(Subject,"Sendpass Request",sizeof(Subject));
			
			
			mail = o_email(thread,From,access5.authnick,access5.email,Subject);
			s_email(thread,language,mail->pipe,"NS_SENDPASSWORD_EMAIL",param[4],tmppasswd,m->network);
			c_email(thread,mail);
			
			do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"used SENDPASS on %s",param[4]);
			send_lang(thread,language,howtosend,snick,nick,"NS_SENDPASS_PASSWORD_SENDED_FOR_X_TO",param[4],access5.email);
	}else if(!param[4] && thread->sql.access.rights>=ModuleFile_UserCommandLevel(thread,MODUL_FILE,"sendpass") && m->UseMail && !m->RestrictMail){

			bzero(&tmppasswd,sizeof(tmppasswd));
			randomstring(tmppasswd,PASSWORD_LEN);
			
			char enc_password[ENC_PASSMAX+1];
			bzero(&enc_password,sizeof(enc_password));
			
			if (orion_encrypt(tmppasswd, strlen(tmppasswd), enc_password, ENC_PASSMAX) < 0) {
				alog(thread,"%s: Failed to encrypt password for %s (register)",
					snick, nick);
				send_lang(thread,language,howtosend,snick,nick,"REGISTRATION_FAILED");
				return MOD_CONT;
			}
			
			domysql(thread,"UPDATE `orion_ns_core` SET `password`='%s' WHERE `authnick`='%s'",mysql_quote(thread,enc_password),mysql_quote(thread,thread->sql.access.authnick));

			dynncpy(From,_dynadd("%s Network (%s) <%s>",m->network,ns->nick,m->SendFrom),sizeof(From));
			dynncpy(Subject,"Sendpass Request",sizeof(Subject));
			
			mail = o_email(thread,From,thread->sql.access.authnick,thread->sql.access.email,Subject);
			s_email(thread,language,mail->pipe,"NS_SENDPASSWORD_EMAIL",nick,tmppasswd,m->network);
			c_email(thread,mail);
			
			do_logchan(thread,snick,nick,thread->sql.Cdata.ident,thread->sql.Cdata.rhost,"used SENDPASS on %s",nick);
			send_lang(thread,language,howtosend,snick,nick,"NS_SENDPASS_PASSWORD_SENDED_FOR_X_TO",nick,thread->sql.access.email);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the NickServ SENDPASS Command (/msg NickServ HELP SENDPASS)
 * 
 * When the user tries to get help for this command, using /msg NickServ HELP SENDPASS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg NickServ HELP SENDPASS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_sendpass_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Sendpass Help
	 */
	if(thread->sql.access.rights>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help sendpass"))
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_SENDPASS",snick,snick,snick);

	else send_lang(thread,language,howtosend,snick,nick,"NS_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief NickServ Help List Command - SENDPASS part (/msg NickServ help)
 * 
 * When the user tries to get general NickServ help, using /msg NickServ help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SENDPASS command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL ns_sendpass_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_NS_Level=(thread->sql.aliases.active>0 ? thread->sql.access.rights : is_user);
	/*
	 * Sendpass (Help listing)
	 */
	if(Users_NS_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"NS_HELP_CMD_SENDPASS");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
