/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_broadcast.c
 *   @brief This is the Module file for O BROADCAST Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O BROADCAST command.
 *
 *   The O BROADCAST Command allows an admin to send GlobalMessages in various<br/>
 *   ways. (only local to target server / all nicks matching a mask / identified users only / etc.)
 *
 *   Syntax is: /msg O BROADCAST < OPTION > < Message >
 *
 *<table border="0">
 *    <tr><td> ALL			</td><td> Syntax is: /msg O BROADCAST ALL			</td><td> < message >					</td><td> (Sends a message to all users)						</td></tr>
 *    <tr><td> IDENTIFIED	</td><td> Syntax is: /msg O BROADCAST IDENTIFIED	</td><td> < message >					</td><td> (Sends a message to all identified users)				</td></tr>
 *    <tr><td> OPER			</td><td> Syntax is: /msg O BROADCAST OPER			</td><td> < message >					</td><td> (Sends a message to all IRCOps)						</td></tr>
 *    <tr><td> SERVER		</td><td> Syntax is: /msg O BROADCAST SERVER		</td><td> < server > < message >		</td><td> (Sends a message to all users on target < server >)	</td></tr>
 *    <tr><td> VHOST		</td><td> Syntax is: /msg O BROADCAST VHOST			</td><td> *vhostpattern* < message >	</td><td> (Sends the message to all users matching the pattern)	</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "broadcast"
#define MODUL_FILE "o_broadcast"
#define MODUL_VERSION "1.0"

MOD_BOOL O_broadcast_all(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_broadcast_identified(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_broadcast_oper(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_broadcast_server(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_broadcast_vhost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
//MOD_BOOL O_broadcast_rhost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_broadcast_all_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_broadcast_identified_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_broadcast_oper_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_broadcast_server_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_broadcast_vhost_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_broadcast_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_broadcast_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"broadcast option[1]","broadcast helpoption[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"broadcast",100,INFINIT); /* just to add a displaylevel for /msg o help
															  this should always be the lowest used Module level! */
	
	addCommand(thread,&Module,"broadcast all",O_broadcast_all,100,INFINIT,5,0);
	addCommand(thread,&Module,"broadcast identified",O_broadcast_identified,100,INFINIT,5,0);
	addCommand(thread,&Module,"broadcast oper",O_broadcast_oper,100,INFINIT,5,0);
	addCommand(thread,&Module,"broadcast server",O_broadcast_server,100,INFINIT,5,0);
	addCommand(thread,&Module,"broadcast vhost",O_broadcast_vhost,100,INFINIT,5,0);
	//addCommand(thread,&Module,"broadcast rhost",O_broadcast_rhost,100,INFINIT,5,0);
	
	addCommand(thread,&Module,"help broadcast",O_broadcast_help,100,INFINIT,4,4);
	addCommand(thread,&Module,"help broadcast all",O_broadcast_all_help,100,INFINIT,5,5);
	addCommand(thread,&Module,"help broadcast identified",O_broadcast_identified_help,100,INFINIT,5,5);
	addCommand(thread,&Module,"help broadcast oper",O_broadcast_oper_help,100,INFINIT,5,5);
	addCommand(thread,&Module,"help broadcast server",O_broadcast_server_help,100,INFINIT,5,5);
	addCommand(thread,&Module,"help broadcast vhost",O_broadcast_vhost_help,100,INFINIT,5,5);
	
	addCommand(thread,&Module,"help",O_broadcast_help_listing,100,INFINIT,3,3); //100 - each Admin is able to see it.
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O BROADCAST All function for the O BROADCAST Command.
 * 
 * The O BROADCAST All Command allows an admin to send out a Global to all users
 *  
 * Syntax is: /msg O BROADCAST ALL message
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_broadcast_all(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * BroadCast All Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"broadcast all")){
		
		if(m->AnonymousGlobal)
			msend(thread,0,snick,_dynadd("$*%s",m->host),"(Broadcast Message) %s",_dstrjoin(' ',param,5));
		else
			msend(thread,0,snick,_dynadd("$*%s",m->host),"(Broadcast Message - %s) %s",nick,_dstrjoin(' ',param,5));
			
		if(m->WallOSGlobal)
			do_global(thread,O->nick,"Global sent by %s",thread->sql.aliases.nick);
			
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O BROADCAST Identified function for the O BROADCAST Command.
 * 
 * The O BROADCAST Identified Command allows an admin to send out a Global<br/>
 *    to all identified users on the Network
 *  
 * Syntax is: /msg O BROADCAST IDENTIFIED message
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_broadcast_identified(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL users;
	char *message = NULL;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&users,sizeof(users));

	/*
	 * BroadCast Identified Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"broadcast identified")){
		message = _dstrjoin(' ',param,5);
		
		users.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_ns_aliases` WHERE `active`=1");
		users.fields = mysql_num_rows(users.res);
		
		for(i=0;i<users.fields;i++){
			users.row = mysql_fetch_row(users.res);
			
			msend(thread,howtosend,snick,users.row[0],message);
		}
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");

		/* mysql_free_result_myr(); - users */
		mysql_free_result_myr(users.res);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}
 
/*  ************************************************************************************************  */

/**
 * @brief O BROADCAST Oper function for the O BROADCAST Command.
 * 
 * The O BROADCAST Oper Command allows an admin to send out a Global<br/>
 *    to all IRCOps on the Network
 *  
 * Syntax is: /msg O BROADCAST OPER message
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_broadcast_oper(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL users;
	char *message = NULL;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&users,sizeof(users));

	/*
	 * BroadCast Oper Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"broadcast oper")){
		message = _dstrjoin(' ',param,5);
		
		users.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_core_condata` WHERE `modes` LIKE '%co%c'",'%','%');
		users.fields = mysql_num_rows(users.res);
		
		for(i=0;i<users.fields;i++){
			users.row = mysql_fetch_row(users.res);
			
			msend(thread,howtosend,snick,users.row[0],message);
		}
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");

		/* mysql_free_result_myr(); - users */
		mysql_free_result_myr(users.res);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O BROADCAST Server function for the O BROADCAST Command.
 * 
 * The O BROADCAST Server Command allows an admin to send out a Global<br/>
 *    to all users on the given Server 
 *  
 * Syntax is: /msg O BROADCAST SERVER message
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_broadcast_server(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	char noticeserv[150];
	int users = 0;

	/* Initialise Variables to (NULL) */
	bzero(&noticeserv,sizeof(noticeserv));

	/*
	 * BroadCast Server Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"broadcast server")){
		dynncpy(noticeserv,param[5],sizeof(noticeserv));

		strrepc(noticeserv,'*','%');

		users = stoi(mysql_sel_var(thread,"SELECT count(*) FROM `orion_core_condata` WHERE `srv` LIKE '%s'",mysql_quote(thread,noticeserv)));
		strrepc(noticeserv,'%','*');
		
		if(m->AnonymousGlobal)
			msend(thread,0,snick,_dynadd("$%s",noticeserv),"(Server Broadcast) %s",_dstrjoin(' ',param,6));
		else
			msend(thread,0,snick,_dynadd("$%s",noticeserv),"(Server Broadcast - %s) %s",nick,_dstrjoin(' ',param,6));
			
		if(m->WallOSGlobal)
			do_global(thread,O->nick,"Global sent by %s",thread->sql.aliases.nick);
			
		if(users != 1) send_lang(thread,language,howtosend,snick,nick,"O_BROADCAST_SERVER_SENT_MSG_TO_X_USERS",users);
		else send_lang(thread,language,howtosend,snick,nick,"O_BROADCAST_SERVER_NO_USERS_FOUND_NO_MESSAGES_SENT");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O BROADCAST Vhost function for the O BROADCAST Command.
 * 
 * The O BROADCAST Vhost Command allows an admin to send out a Global<br/>
 *    to all users matching the given Vhostpattern. (Message is sent over the Network)
 *  
 * Syntax is: /msg O BROADCAST VHOST *vhostpattern* message
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_broadcast_vhost(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL vhostlist;
	char pattern[BUFSIZE];
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&vhostlist,sizeof(vhostlist));
	bzero(&pattern,sizeof(pattern));

	/*
	 * BroadCast Vhost Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"broadcast vhost") && strcor(param[5],"*")){
		dyncpy (pattern,param[5]);
		strrepc(pattern,'*','%');

		vhostlist.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_core_condata` WHERE `vhost` LIKE '%s' LIMIT 100",mysql_quote(thread,pattern));
		vhostlist.fields = mysql_num_rows(vhostlist.res);
		
		for(i=0;i<vhostlist.fields;i++){
			vhostlist.row = mysql_fetch_row(vhostlist.res);
			
			if(m->AnonymousGlobal)
				msend(thread,howtosend,snick,vhostlist.row[0],"(Vhost Broadcast) %s",_dstrjoin(' ',param,6));
			else
				msend(thread,howtosend,snick,vhostlist.row[0],"(Vhost Broadcast - %s) %s",nick,_dstrjoin(' ',param,6));
		}
		if(m->WallOSGlobal)
			do_global(thread,O->nick,"Global sent by %s",thread->sql.aliases.nick);
		
		if(i != 0)
			send_lang(thread,language,howtosend,snick,nick,"O_BROADCAST_VHOST_SENT_MSG_TO_X_MATCHING_USERS",i);
		else
			send_lang(thread,language,howtosend,snick,nick,"O_BROADCAST_VHOST_NO_USERS_FOUND_NO_MESSAGES_SENT");

		mysql_free_result_myr(vhostlist.res);
	}else{
	/*
	 *  Error Messages
	 */
		if(!strcor(param[5],"*"))	send_lang(thread,language,howtosend,snick,nick,"WRONG_INPUT_PATTERN_INPUT",param[5]);
		else 						send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O BROADCAST Command (/msg O HELP BROADCAST)
 * 
 * When the user tries to get help for this command, using /msg O HELP BROADCAST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP BROADCAST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_broadcast_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Broadcast Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help broadcast")) {
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_BROADCAST");

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O BROADCAST Help ALL function for the O BROADCAST Command (/msg O HELP BROADCAST ALL)
 * 
 * When the user tries to get help for this command, using /msg O HELP BROADCAST ALL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP BROADCAST ALL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_broadcast_all_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Broadcast All Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help broadcast all")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"broadcast all"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_BROADCAST_ALL");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O BROADCAST Help IDENTIFIED function for the O BROADCAST Command (/msg O HELP BROADCAST IDENTIFIED)
 * 
 * When the user tries to get help for this command, using /msg O HELP BROADCAST IDENTIFIED,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP BROADCAST IDENTIFIED
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_broadcast_identified_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Broadcast Identified Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help broadcast identified")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"broadcast identified"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_BROADCAST_IDENTIFIED");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O BROADCAST Help OPER function for the O BROADCAST Command (/msg O HELP BROADCAST OPER)
 * 
 * When the user tries to get help for this command, using /msg O HELP BROADCAST OPER,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP BROADCAST OPER
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_broadcast_oper_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Broadcast Oper Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help broadcast oper")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"broadcast oper"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_BROADCAST_OPER");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O BROADCAST Help SERVER function for the O BROADCAST Command (/msg O HELP BROADCAST SERVER)
 * 
 * When the user tries to get help for this command, using /msg O HELP BROADCAST SERVER,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP BROADCAST SERVER
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_broadcast_server_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Broadcast Server Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help broadcast server")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"broadcast server"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_BROADCAST_SERVER");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O BROADCAST Help VHOST function for the O BROADCAST Command (/msg O HELP BROADCAST VHOST)
 * 
 * When the user tries to get help for this command, using /msg O HELP BROADCAST VHOST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP BROADCAST VHOST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_broadcast_vhost_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Broadcast Vhost Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help broadcast vhost")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"broadcast vhost"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_BROADCAST_VHOST");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - BROADCAST part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the BROADCAST command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_broadcast_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Broadcast (Overview Help)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_BROADCAST",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"broadcast"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
