/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_oper.c
 *   @brief This is the Module file for O OPER Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O OPER command.
 *
 *   The O OPER Command allows an admin to manage the OperList.<br/>
 *   (add, delete, view ServicesOps - PASS (with showing passwords) is limited to ServicesRoots (lvl: 1000))
 *
 *   Syntax is: /msg O OPER { ADD | DEL | LIST | PASS | MSG } [< nick >]
 *
 *<table border="0">
 *    <tr><td> ADD	</td><td> Syntax is: /msg O OPER ADD	</td><td> < nick > 		</td><td> (Allows ServicesAdmins to add a ServicesOps)							</td></tr>
 *    <tr><td> DEL	</td><td> Syntax is: /msg O OPER DEL	</td><td> < nick >		</td><td> (Allows ServicesAdmins to delete a ServicesOps)						</td></tr>
 *    <tr><td> LIST	</td><td> Syntax is: /msg O OPER LIST	</td><td>				</td><td> (Allows ServicesOps or higher to view all ServicesOps)				</td></tr>
 *    <tr><td> PASS	</td><td> Syntax is: /msg O OPER PASS	</td><td> [*nick*]		</td><td> (Allows ServicesRoots to view ServicesOps/Admins and their passwords)	</td></tr>
 *    <tr><td> MSG	</td><td> Syntax is :/msg O OPER MSG	</td><td> < message >	</td><td> (Sends a Message to all ServicesOps/ServicesAdmins/ServicesRoots)		</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "oper"
#define MODUL_FILE "o_oper"
#define MODUL_VERSION "1.0"

MOD_BOOL O_oper_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_oper_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_oper_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_oper_msg(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_oper_pass(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_oper_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_oper_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_oper_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_oper_msg_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_oper_pass_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_oper_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_oper_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"oper option[1]","oper helpoption[1]");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addLeastModuleLevels(thread,&Module,"oper",O_SA_LEVEL,INFINIT); /* just to add a displaylevel for /msg o help
															  this should always be the lowest used Module level! */
	
	addCommand(thread,&Module,"oper add",O_oper_add,O_SA_LEVEL,INFINIT,5,5);
	addCommand(thread,&Module,"oper del",O_oper_del,O_SA_LEVEL,INFINIT,5,5);
	addCommand(thread,&Module,"oper list",O_oper_list,O_SA_LEVEL,INFINIT,4,4);
	addCommand(thread,&Module,"oper msg",O_oper_msg,850,INFINIT,5,0);
	addCommand(thread,&Module,"oper pass",O_oper_pass,1000,INFINIT,4,5);

	addCommand(thread,&Module,"help oper",O_oper_help,850,INFINIT,4,4);
	addCommand(thread,&Module,"help oper add",O_oper_add_help,O_SA_LEVEL,INFINIT,5,5);
	addCommand(thread,&Module,"help oper del",O_oper_del_help,O_SA_LEVEL,INFINIT,5,5);
	addCommand(thread,&Module,"help oper list",O_oper_list_help,O_SA_LEVEL,INFINIT,5,5);
	addCommand(thread,&Module,"help oper msg",O_oper_msg_help,850,INFINIT,5,5);
	addCommand(thread,&Module,"help oper pass",O_oper_pass_help,1000,INFINIT,5,5);
	
	addCommand(thread,&Module,"help",O_oper_help_listing,O_SA_LEVEL,INFINIT,3,3); //100 - each Admin is able to see it.
	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O OPER Add function for the O OPER Command.
 * 
 * The O OPER Add Command allows a ServicesAdmin/ServicesRoot to add ServicesOps
 *  
 * Syntax is: /msg O OPER ADD < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_oper_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	Aliases aliases5;
	Access  access5;

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));

	/*
	 * Oper Add Command
	 */
	 //:shock PRIVMSG O :oper add shock
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"oper add")){
		aliases5 = sql_aliases(thread,param[5]);
		access5  = sql_access(thread,aliases5.authnick);
			
		if(aliases5.authnick[0]!='\0'){
			if(access5.fields && thread->sql.access.olevel>=O_SA_LEVEL && thread->sql.access.qlevel>=Q_SA_LEVEL){
				if(access5.rights<is_services_admin && access5.olevel<O_SA_LEVEL && access5.qlevel<Q_SA_LEVEL){
					domysql(thread,"UPDATE `orion_ns_core` SET `qlevel`='%d',`olevel`='%d',`llevel`='100',`slevel`='%d',`rights`='%d' WHERE `authnick`='%s'",Q_SO_LEVEL,O_SO_LEVEL,S_SO_LEVEL,is_services_oper,mysql_quote(thread,aliases5.authnick));
					send_lang(thread,language,howtosend,snick,nick,"O_OPER_ADD_ADDED_X_TO_SO",aliases5.authnick,Q_SO_LEVEL,O_SO_LEVEL,S_SO_LEVEL,nick);
				}else send_lang(thread,language,howtosend,snick,nick,"USERS_LEVEL_IS_HIGHER_THAN_SA");
			}else send_lang(thread,language,howtosend,snick,nick,"NOT_A_SERVICES_ADMIN");
		}else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_USER",aliases5.nick);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O OPER Del function for the O OPER Command.
 * 
 * The O OPER Del Command allows a ServicesAdmin/ServicesRoot to delete ServicesOps
 *  
 * Syntax is: /msg O OPER DEL < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_oper_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	Aliases aliases5;
	Access  access5;

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));

	/*
	 * Oper Del Command
	 */
	 //:shock PRIVMSG O :oper del shock
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"oper del")){
		aliases5 = sql_aliases(thread,param[5]);
		access5  = sql_access(thread,aliases5.authnick);
			
		if(aliases5.authnick[0]!='\0'){
			if(thread->sql.access.olevel>=access5.olevel && thread->sql.access.qlevel>=access5.qlevel && thread->sql.access.rights>=access5.rights){
				domysql(thread,"UPDATE `orion_ns_core` SET `qlevel`='1',`olevel`='0',`llevel`='1',`slevel`='1',`rights`='1' WHERE `authnick`='%s'",mysql_quote(thread,aliases5.authnick));
				send_lang(thread,language,howtosend,snick,nick,"O_OPER_DEL_DELETED_X_FROM_OPERS",aliases5.authnick,nick);
			}else send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
		}else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_USER",aliases5.authnick);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O OPER List function for the O OPER Command.
 * 
 * The O OPER List Command allows a ServicesAdmin/ServicesRoot to dump the List of ServicesOps
 *  
 * Syntax is: /msg O OPER LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_oper_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL adminlist;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&adminlist,sizeof(adminlist));

	/*
	 * Oper List Command
	 */
	 //:shock PRIVMSG O :oper list
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"oper list")){
		adminlist.res    = doSelectMySQL(thread,"SELECT `authnick`,`rights`,`qlevel`,`olevel`,`slevel` FROM `orion_ns_core` ORDER BY `rights` ASC");
		adminlist.fields = mysql_num_rows(adminlist.res);
		
		if(adminlist.fields){
			for(i=0;i<adminlist.fields;i++){
				adminlist.row = mysql_fetch_row(adminlist.res);
				
				if(adminlist.row[0][0]!='\0'){
					if(stoi(adminlist.row[1])==is_services_oper){
						msend(thread,howtosend,snick,nick,"SO: %s Q:%d O:%d S:%d",adminlist.row[0],stoi(adminlist.row[2]),stoi(adminlist.row[3]),stoi(adminlist.row[4]));
					}
				}
			}
			send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		}else send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");

		/* MySQL Free Result */
		mysql_free_result_myr(adminlist.res);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O OPER Msg function for the O OPER Command.
 * 
 * The O OPER Msg Command allows a ServicesAdmin/ServicesRoot to<br/>
 *    message all ServicesOps/ServicesAdmins/ServicesRoots
 *  
 * Syntax is: /msg O OPER MSG < message >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_oper_msg(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL opers;
	char *opermsg = NULL;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&opers,sizeof(opers));

	/*
	 * Oper Msg Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"oper msg")){

		opermsg      = _dstrjoin(' ',param,5);
		opers.res    = doSelectMySQL(thread,"SELECT `nick` FROM `orion_ns_aliases` LEFT JOIN `orion_ns_core` ON `orion_ns_aliases`.`display`=`orion_ns_core`.`authnick` WHERE `orion_ns_aliases`.`active`=1 AND `orion_ns_core`.`rights`>=%d",is_services_oper);
		opers.fields = mysql_num_rows(opers.res);
		
		for(i=0;i<opers.fields;i++){
			opers.row = mysql_fetch_row(opers.res);
			
			do_privmsg(thread,snick,opers.row[0],opermsg);
		}

		do_global(thread,snick,opermsg);
		send_lang(thread,language,howtosend,snick,nick,"O_OPER_MSG_MESSAGED_X_SERVICESOPS_OR_HIGHER",opers.fields);

		/* mysql_free_result_myr(); - opers */
		mysql_free_result_myr(opers.res);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O OPER Pass function for the O OPER Command.
 * 
 * The O OPER Pass Command allows a ServicesRoot to<br/>
 *    dump a List of ServicesOps/ServicesAdmins/ServicesRoots with their Passwords<br/>
 *    (or without Passwords by executing it as a ServicesOp)
 *  
 * Syntax is: /msg O OPER PASS [*nick*]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_oper_pass(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL opers;
	char pattern[BUFSIZE];
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&opers,sizeof(opers));
	bzero(&pattern,sizeof(pattern));

	/*
	 * Oper Pass Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"oper pass")){
		send_lang(thread,language,howtosend,snick,nick,"O_OPER_PASS_LIST_HEADER");

		if(!param[5])
			opers.res = doSelectMySQL(thread,"SELECT `authnick`,`password`,`qlevel`,`olevel`,`llevel`,`orion_ns_aliases`.`time_linked` FROM `orion_ns_core` LEFT JOIN `orion_ns_aliases` ON (`orion_ns_aliases`.`display`=`orion_ns_aliases`.`nick`=`orion_ns_core`.`authnick`) GROUP BY `orion_ns_core`.`authnick`");
		else{
			dyncpy (pattern,param[5]);
			strrepc(pattern,'*','%');
	 		opers.res = doSelectMySQL(thread,"SELECT `authnick`,`password`,`qlevel`,`olevel`,`llevel`,`orion_ns_aliases`.`time_linked` FROM `orion_ns_core` LEFT OUTER JOIN `orion_ns_aliases` ON (`orion_ns_aliases`.`display`=`orion_ns_aliases`.`nick`=`orion_ns_core`.`authnick`) WHERE `orion_ns_core`.`authnick` LIKE '%s' GROUP BY `orion_ns_core`.`authnick` LIMIT 100",mysql_quote(thread,pattern));
		}

		opers.fields = mysql_num_rows(opers.res);
		
		for(i=0;i<opers.fields;i++){
			opers.row = mysql_fetch_row(opers.res);
			
			if(stoi(opers.row[3])>=100){ // <- olevel
				if(streqnc(orion_check_encryption(),NO_ENCRYPT)){
					if(thread->sql.access.olevel>=1000)
						send_lang(thread,language,howtosend,snick,nick,"O_OPER_PASS_LIST_OPERS_AND_PASSWORDS",opers.row[0],opers.row[1],stoi(opers.row[2]),stoi(opers.row[3]),stoi(opers.row[4]),get_localtime((time_t)stol(opers.row[5])));
					else
						send_lang(thread,language,howtosend,snick,nick,"O_OPER_PASS_LIST_OPERS_WITHOUT_PASSWORDS",opers.row[0],stoi(opers.row[2]),stoi(opers.row[3]),stoi(opers.row[4]),get_localtime((time_t)stol(opers.row[5])));
				}
				else
					send_lang(thread,language,howtosend,snick,nick,"O_OPER_PASS_LIST_OPERS_WITHOUT_PASSWORDS",opers.row[0],stoi(opers.row[2]),stoi(opers.row[3]),stoi(opers.row[4]),get_localtime((time_t)stol(opers.row[5])));
			}
		}

		mysql_free_result_myr(opers.res);
		send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O OPER Command (/msg O HELP OPER)
 * 
 * When the user tries to get help for this command, using /msg O HELP OPER,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP OPER
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_oper_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Oper Help
	 */  
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help oper")) {
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_OPER",snick);
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O OPER Add Help function for the O OPER Command (/msg O HELP OPER ADD)
 * 
 * When the user tries to get help for this command, using /msg O HELP OPER ADD,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP OPER ADD
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_oper_add_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Oper Add Help
	 */  
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help oper add")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"oper add"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_OPER_ADD");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O OPER Del Help function for the O OPER Command (/msg O HELP OPER DEL)
 * 
 * When the user tries to get help for this command, using /msg O HELP OPER DEL,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP OPER DEL
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_oper_del_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Oper Del Help
	 */  
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help oper del")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"oper del"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_OPER_DEL");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O OPER List Help function for the O OPER Command (/msg O HELP OPER LIST)
 * 
 * When the user tries to get help for this command, using /msg O HELP OPER LIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP OPER LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_oper_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Oper List Help
	 */  
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help oper list")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"oper list"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_OPER_LIST");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O OPER Msg Help function for the O OPER Command (/msg O HELP OPER MSG)
 * 
 * When the user tries to get help for this command, using /msg O HELP OPER MSG,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP OPER MSG
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_oper_msg_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Oper Msg Help
	 */  
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help oper msg")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"oper msg"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_OPER_MSG");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O OPER Pass Help function for the O OPER Command (/msg O HELP OPER PASS)
 * 
 * When the user tries to get help for this command, using /msg O HELP OPER PASS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP OPER PASS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_oper_pass_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Oper Pass Help
	 */  
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help oper pass")) {
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"oper pass"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_OPER_PASS");
	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - OPER part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the OPER command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_oper_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Oper (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_OPER",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"oper"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
