/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_set.c
 *   @brief This is the Module file for O SET Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O SET command.
 *
 *   The O SET Functions allows an admin to change a few<br/>
 *   options to control the Network even better.
 *
 *   Syntax is: /msg O SET < OPTION > { ON | OFF }
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "set"
#define MODUL_FILE "o_set"
#define MODUL_VERSION "1.0"

MOD_BOOL O_set(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_set_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"set","set");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"set",O_set,900,INFINIT,4,5);
	addCommand(thread,&Module,"help set",O_set_help,900,INFINIT,4,5);
	addCommand(thread,&Module,"help",O_set_help_listing,900,INFINIT,3,3); //100 - each Admin is able to see it.

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the O SET Command.
 * 
 * The O SET Command allows an admin to set various SET-Options for controlling<br/>
 *   the services.
 *  
 * Syntax is: /msg O SET < OPTION > ...
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_set(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	char name[100];
	char date[50];
	/*
	 * Set Command
	 */
	bzero(&name,sizeof(name));
	bzero(&date,sizeof(date));
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set")){
		if(streqnc(param[4],"debug")){
			if(digitcol(param[5])){
				if(stoi(param[5])<=5 && stoi(param[5])>0 && m->debuglog[0]!='\0'){
					m->debug = stoi(param[5]);
					m->debuglogging = 1;
					send_lang(thread,language,howtosend,snick,nick,"O_SET_DEBUG_ON_MODE_X",m->debug);
				}else if(stoi(param[5])==0){
					m->debug = 0;
					m->debuglogging = 0;
					send_lang(thread,language,howtosend,snick,nick,"O_SET_DEBUG_OFF");
				}
			}
			else if(param[5] == NULL)
				send_lang(thread,language,howtosend,snick,nick,"MISSING_PARAMETER");
			else if(param[5][0]!='\0')
				send_lang(thread,language,howtosend,snick,nick,"O_SET_UNKNOWN_OPTION",param[5]);
		}else if(streqnc(param[4],"logchan")){
			if(streqnc(param[5],"on") && m->logchan[0]!='\0'){
				logchan = 1;
				send_lang(thread,language,howtosend,snick,nick,"O_SET_LOGCHAN_ON_CHANNEL_X",m->logchan);
			}
			else if(streqnc(param[5],"off")){
				logchan = 0;
				send_lang(thread,language,howtosend,snick,nick,"O_SET_LOGCHAN_OFF_CHANNEL_X",m->logchan);
			}
			else if(param[5] == NULL)
				send_lang(thread,language,howtosend,snick,nick,"MISSING_PARAMETER");
			else if(param[5][0]!='\0')
				send_lang(thread,language,howtosend,snick,nick,"O_SET_UNKNOWN_OPTION",param[5]);
		}else if(streqnc(param[4],"readonly")){
			if(streqnc(param[5],"on")){
				readonly = 1;
				send_lang(thread,language,howtosend,snick,nick,"O_SET_READONLY_ON");
			}
			else if(streqnc(param[5],"off")){
				readonly = 0;
				send_lang(thread,language,howtosend,snick,nick,"O_SET_READONLY_OFF");
			}
			else if(param[5] == NULL)
				send_lang(thread,language,howtosend,snick,nick,"MISSING_PARAMETER");
			else if(param[5][0]!='\0')
				send_lang(thread,language,howtosend,snick,nick,"O_SET_UNKNOWN_OPTION",param[5]);
		}else if(streqnc(param[4],"sqllogging") && m->sqllogfile[0]!='\0'){
			if(streqnc(param[5],"on")){
				sqllogging = 1;
				if (get_logdate(date, sizeof(date), NULL)){
					snprintf(name,sizeof(name),"logs/%s.",m->sqllogfile);
					straddns(name,date,sizeof(name));
					send_lang(thread,language,howtosend,snick,nick,"O_SET_SQLLOGGING_ON_FILE_X",name);
				}else
					send_lang(thread,language,howtosend,snick,nick,"O_SET_SQLLOGGING_ON_FILE_X",m->sqllogfile);
			}
			else if(streqnc(param[5],"off")){
				sqllogging = 0;
				if (get_logdate(date, sizeof(date), NULL)){
					snprintf(name,sizeof(name),"logs/%s.",m->sqllogfile);
					straddns(name,date,sizeof(name));
					send_lang(thread,language,howtosend,snick,nick,"O_SET_SQLLOGGING_OFF_FILE_X",name);
				}else
					send_lang(thread,language,howtosend,snick,nick,"O_SET_SQLLOGGING_OFF_FILE_X",m->sqllogfile);
			}
			else if(param[5] == NULL)
				send_lang(thread,language,howtosend,snick,nick,"MISSING_PARAMETER");
			else if(param[5][0]!='\0')
				send_lang(thread,language,howtosend,snick,nick,"O_SET_UNKNOWN_OPTION",param[5]);
		}else if(streqnc(param[4],"cmdlogging") && m->logfile[0]!='\0'){
			if(streqnc(param[5],"on")){
				logging = 1;
				if (get_logdate(date, sizeof(date), NULL)){
					snprintf(name,sizeof(name),"logs/%s.",m->logfile);
					straddns(name,date,sizeof(name));
					send_lang(thread,language,howtosend,snick,nick,"O_SET_LOGGING_ON_FILE_X",name);
				}else
					send_lang(thread,language,howtosend,snick,nick,"O_SET_LOGGING_ON_FILE_X",m->logfile);
			}
			else if(streqnc(param[5],"off")){
				logging = 0;
				if (get_logdate(date, sizeof(date), NULL)){
					snprintf(name,sizeof(name),"logs/%s.",m->logfile);
					straddns(name,date,sizeof(name));
					send_lang(thread,language,howtosend,snick,nick,"O_SET_LOGGING_OFF_FILE_X",name);
				}else
					send_lang(thread,language,howtosend,snick,nick,"O_SET_LOGGING_OFF_FILE_X",m->logfile);
			}
			else if(param[5] == NULL)
				send_lang(thread,language,howtosend,snick,nick,"MISSING_PARAMETER");
			else if(param[5][0]!='\0')
				send_lang(thread,language,howtosend,snick,nick,"O_SET_UNKNOWN_OPTION",param[5]);
		}else if(streqnc(param[4],"list") && param[5][0]=='\0'){

			if(m->debug) send_lang(thread,language,howtosend,snick,nick,"O_SET_LIST_DEBUG_ON_MODE_X",m->debug);
			else send_lang(thread,language,howtosend,snick,nick,"O_SET_LIST_DEBUG_OFF");
			
			if(logchan) send_lang(thread,language,howtosend,snick,nick,"O_SET_LIST_LOGCHAN_ON_CHANNEL_X",m->logchan);
			else send_lang(thread,language,howtosend,snick,nick,"O_SET_LIST_LOGCHAN_OFF");
			
			if(readonly) send_lang(thread,language,howtosend,snick,nick,"O_SET_LIST_READONLY_ON");
			else send_lang(thread,language,howtosend,snick,nick,"O_SET_LIST_READONLY_OFF");
			
			if(sqllogging) send_lang(thread,language,howtosend,snick,nick,"O_SET_LIST_SQLLOGGING_ON_FILE_X",m->sqllogfile);
			else send_lang(thread,language,howtosend,snick,nick,"O_SET_LIST_SQLLOGGING_OFF");
			
			if(logging) send_lang(thread,language,howtosend,snick,nick,"O_SET_LIST_LOGGING_ON_FILE_X",m->logfile);
			else send_lang(thread,language,howtosend,snick,nick,"O_SET_LIST_LOGGING_OFF");
			
		}else send_lang(thread,language,howtosend,snick,nick,"O_SET_UNKNOWN_OPTION",param[4]);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O SET Command (/msg O HELP SET)
 * 
 * When the user tries to get help for this command, using /msg O HELP SET,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP SET
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_set_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Set Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help set")){
		if(streqnc(param[5],"readonly")) send_lang(thread,language,howtosend,snick,nick,"O_HELP_SET_READONLY");
		else if(streqnc(param[5],"debug")) send_lang(thread,language,howtosend,snick,nick,"O_HELP_SET_DEBUG");
		else if(streqnc(param[5],"logchan")) send_lang(thread,language,howtosend,snick,nick,"O_HELP_SET_LOGCHAN");
		else if(streqnc(param[5],"sqllogging")) send_lang(thread,language,howtosend,snick,nick,"O_HELP_SET_SQLLOGGING");
		else if(streqnc(param[5],"cmdlogging")) send_lang(thread,language,howtosend,snick,nick,"O_HELP_SET_CMDLOGGING");
		else if(streqnc(param[5],"list")) send_lang(thread,language,howtosend,snick,nick,"O_HELP_SET_LIST");
		else{
			send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_RootCommandLevel(thread,MODUL_FILE,"set"));
			send_lang(thread,language,howtosend,snick,nick,"O_HELP_SET",snick);
		}
	}else send_lang(thread,language,howtosend,snick,nick,"O_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - SET part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SET command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_set_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Set (Overview Help)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_SET",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"set"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
