/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 203                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 23:10:45 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_szline.c
 *   @brief This is the Module file for O SZLINE Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O SZLINE command.
 *
 *   The O SZLINE Command allows an admin to prevent<br/>
 *   the user with target ip from connecting the Network
 *
 *   Syntax is: /msg O SZLINE { ADD | DEL | LIST | CLEAR } [ip [< duration (2d) > < reason >]]
 *
 *<table border="0">
 *    <tr><td> ADD		</td><td> Syntax is: /msg O SZLINE ADD		</td><td> < ip > < duration(i.e. 2d) > < reason >	</td></tr>
 *    <tr><td> DEL		</td><td> Syntax is: /msg O SZLINE DEL		</td><td> < ip >									</td></tr>
 *    <tr><td> LIST		</td><td> Syntax is: /msg O SZLINE LIST		</td><td>											</td></tr>
 *    <tr><td> CLEAR	</td><td> Syntax is: /msg O SZLINE CLEAR	</td><td>											</td></tr>
 *</table>
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "szline"
#define MODUL_FILE "o_szline"
#define MODUL_VERSION "1.0"

MOD_BOOL O_szline_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_szline_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_szline_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_szline_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);

MOD_BOOL O_szline(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_szline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_szline_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module;
	
	if(!IRC.SupportSZlines){
		Module.ModErr = MOD_ERR_CANCELLED;
		Module.module = NULL;
		return Module;
	}
	
	Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"szline option[1]","szline option[1]");
	
	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"szline add",O_szline_add,700,INFINIT,6,0);
	addCommand(thread,&Module,"szline del",O_szline_del,700,INFINIT,4,7);	
	addCommand(thread,&Module,"szline list",O_szline_list,700,INFINIT,4,4);
	addCommand(thread,&Module,"szline clear",O_szline_clear,700,INFINIT,4,4);

	addCommand(thread,&Module,"szline",NULL,700,INFINIT,4,0);
	addCommand(thread,&Module,"help szline",O_szline_help,700,INFINIT,4,5);
	addCommand(thread,&Module,"help",O_szline_help_listing,700,INFINIT,3,3); //100 - each Admin is able to see it.

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief O SZLINE Add function for the O SZLINE Command.
 * 
 * The O SZLINE Add Command allows an admin to<br/>
 *   add an Szline for target Ip
 *  
 * Syntax is: /msg O SZLINE ADD < ip > < duration(i.e. 2d) > < reason >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL O_szline_add(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
		//NULL
		
	/* Initialise Variables to (NULL) */
		//NULL
		
	/*
	 * Szline Add Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"szline add") && param[7]){
		if(check_expire(param[6])){
            if(is_valid_chars(param[5],"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_.*")){
                send_lang(thread,language,howtosend,snick,nick,"INVALID_PARAMETER",param[5]);
				return MOD_CONT;
			}
            
			do_szline(thread,m->servername,param[5],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),expire(param[6]) == 0 ? 0 : expire(param[6])+time(NULL),time(NULL),_dstrjoin(' ',param,7));
			send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
		}else
			send_lang(thread,language,howtosend,snick,nick,"INVALID_EXPIRESTRING",param[6]);
	}else{
	/*
	 *  Error Messages
	 */
		if(!param[7])	send_lang(thread,language,howtosend,snick,nick,"NO_REASON");
		else 			send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SZLINE Del function for the O SZLINE Command.
 * 
 * The O SZLINE Del Command allows an admin to<br/>
 *   delete an Szline for target ip
 *  
 * Syntax is: /msg O SZLINE DEL < ip >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL O_szline_del(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
		//NULL
		
	/* Initialise Variables to (NULL) */
		//NULL
		
	/*
	 * Szline Del Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"szline del")){
		do_unszline(thread,m->servername,param[5],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),time(NULL));
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SZLINE List function for the O SZLINE Command.
 * 
 * The O SZLINE List Command allows an admin to<br/>
 *   dump the Szline-List
 *  
 * Syntax is: /msg O SZLINE LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL O_szline_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	deepSQL szline;
	int i = 0;

	/* Initialise Variables to (NULL) */
	bzero(&szline,sizeof(szline));

	/*
	 * Szline List Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"szline list")){
		szline.res    = doSelectMySQL(thread,"SELECT `zline`,`duration`,`addedby`,`reason` FROM `orion_os_szlines`");
		szline.fields = mysql_num_rows(szline.res);
		
		for(i=0;i<szline.fields;i++){
			szline.row = mysql_fetch_row(szline.res);
			
			if(stol(szline.row[1])!=0)
				msend(thread,howtosend,snick,nick,"%d %s %s [%s]",i+1,szline.row[0],szline.row[3],get_localtime((time_t)stol(szline.row[1])));
			else
				msend(thread,howtosend,snick,nick,"%d %s %s",i+1,szline.row[0],szline.row[3]);
		}
		send_lang(thread,language,howtosend,snick,nick,"END_OF_STATS_REPORT");

		mysql_free_result_myr(szline.res); //free MySQL result
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief O SZLINE Clear function for the O SZLINE Command.
 * 
 * The O SZLINE Clear Command allows an admin to<br/>
 *   delete all Szlines.
 *  
 * Syntax is: /msg O SZLINE CLEAR
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 **/
MOD_BOOL O_szline_clear(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */

	/* Initialise Variables to (NULL) */

	/*
	 * Szline Clear Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"szline clear")){
		domysql(thread,"TRUNCATE `orion_os_szlines`");
		send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O SZLINE Command (/msg O HELP SZLINE)
 * 
 * When the user tries to get help for this command, using /msg O HELP SZLINE,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP SZLINE
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_szline_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Szline Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help szline")) {
		if(!param[5])
			send_lang(thread,language,howtosend,snick,nick,"O_HELP_SZLINE");
			
		else 
			send_lang(thread,language,howtosend,snick,nick,"O_NO_HELP_AVAIL");

	}else send_lang(thread,language,howtosend,snick,nick,"O_UNKNOWN_COMMAND",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - SZLINE part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the SZLINE command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_szline_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Szline (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_SZLINE",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"szline"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
