/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file o_whois.c
 *   @brief This is the Module file for O WHOIS Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the O WHOIS command.
 *
 *   The O WHOIS Command allows an admin to view<br/>
 *   some common Information about the given user. (Connecting from, realname, etc.)
 *
 *   Syntax is: /msg O WHOIS < nick >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "O"
#define MODUL "whois"
#define MODUL_FILE "o_whois"
#define MODUL_VERSION "1.1"

MOD_BOOL O_whois(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_whois_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL O_whois_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"whois","whois");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"whois",O_whois,700,INFINIT,4,4);
	addCommand(thread,&Module,"help whois",O_whois_help,700,INFINIT,4,4);
	addCommand(thread,&Module,"help",O_whois_help_listing,700,INFINIT,3,3); //100 - each Admin is able to see it.

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the O WHOIS Command.
 * 
 * The O WHOIS Command allows an admin to view<br/>
 *   some common Information about the given user. (Connecting from, realname, etc.)
 *
 * Syntax is: /msg O WHOIS < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_whois(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global Variables */
	Aliases aliases5;
	Access access5;
	Condata Cdata;
	char regtime[200], regtime2[200];
	char *regs[5];
	char *checker[ARRAY_SIZE];

	/* Initialise Variables to (NULL) */
	bzero(&aliases5,sizeof(aliases5));
	bzero(&access5,sizeof(access5));
	bzero(&Cdata,sizeof(Cdata));
	bzero(&regtime,sizeof(regtime));
	bzero(&regtime2,sizeof(regtime2));

	bzero(regs,sizeof(regs));
	bzero(checker,sizeof(checker));

	/*
	 * Whois Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"whois")){
		aliases5	= sql_aliases(thread,param[4]);
		access5		= sql_access(thread,aliases5.authnick);
		Cdata		= sql_condata(thread,param[4]);
		
		if(Cdata.fields && Cdata.time && Cdata.ident[0]!='\0' && Cdata.nick[0]!='\0'){
			if(access5.settings&NS_SUSPENDED){
				send_lang(thread,language,howtosend,snick,nick,"LOGIN_X_SUSPENDED",param[4]);
			}else{
				//bzero(&regtime,sizeof(regtime));
				bzero(regs,sizeof(regs));
				
				dynncpy(regtime,get_localtime(Cdata.time),sizeof(regtime));
				strclean(regtime);
				strsplit(regtime,regs,' ');
				snprintf(regtime2,sizeof(regtime2),"%s %s %s %s %s",regs[0],regs[2],regs[1],regs[4],regs[3]);

				send_lang(thread,language,howtosend,snick,nick,"O_WHOIS_WHOISHEADER",param[4]);

				if(aliases5.fields) send_lang(thread,language,howtosend,snick,nick,"O_WHOIS_IS_AUTHED_AS",param[4],aliases5.authnick);
				if(access5.fields) send_lang(thread,language,howtosend,snick,nick,"O_WHOIS_HAS_O_AUTHLEVEL",param[4],access5.olevel);
				if(access5.fields) send_lang(thread,language,howtosend,snick,nick,"O_WHOIS_REGISTERED_SINCE",get_localtime(aliases5.time_linked));

				send_lang(thread,language,howtosend,snick,nick,"O_WHOIS_RETURN_INFORMATION",regtime2,Cdata.realname,Cdata.rident,Cdata.ident,Cdata.ip,Cdata.rhost,Cdata.vhost,Cdata.subnet,Cdata.server);
				send_lang(thread,language,howtosend,snick,nick,"O_COMMAND_DONE");
			}
		}else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_USER",param[4]);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the O WHOIS Command (/msg O HELP WHOIS)
 * 
 * When the user tries to get help for this command, using /msg O HELP WHOIS,<br/>
 * this function is executed.
 * 
 * Syntax: /msg O HELP WHOIS
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_whois_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Whois Help
	 */ 
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help whois")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"whois"));
		send_lang(thread,language,howtosend,snick,nick,"O_HELP_WHOIS");
	}else send_lang(thread,language,howtosend,snick,nick,"O_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief O Help List Command - WHOIS part (/msg O help)
 * 
 * When the user tries to get general O help, using /msg O help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the WHOIS command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL O_whois_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_O_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Whois (Help listing)
	 */
	if(Users_O_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"O_SHOWCOMMANDS_WHOIS",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"whois"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
