/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 204                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-03 20:33:44 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file orion_rehash.c
 *   @brief This is the Module file for Orion REHASH Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Orion REHASH command.
 *
 *   The Orion REHASH Command allows an admin to rehash<br/>
 *   the services configuration file. (reloads the services.conf)
 *
 *   Syntax is: /msg Orion REHASH
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "orion"
#define MODUL "rehash"
#define MODUL_FILE "orion_rehash"
#define MODUL_VERSION "1.0"

MOD_BOOL orion_rehash(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_rehash_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_rehash_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"rehash","rehash");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,4,LOGINONLY,OPER_IF_DEFCON,OPERONLY,READONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"rehash",orion_rehash,1000,INFINIT,3,3);
	addCommand(thread,&Module,"help rehash",orion_rehash_help,1000,INFINIT,4,4);
	addCommand(thread,&Module,"help",orion_rehash_help_listing,1000,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}


/*  ************************************************************************************************  */

/**
 * @brief Main function for the Orion REHASH Command.
 * 
 * The Orion REHASH Command allows an admin to rehash<br/>
 *   the services configuration file. (reloads the services.conf)
 *
 * Syntax is: /msg Orion REHASH
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL orion_rehash(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int debug = 0;
	/*
	 * Rehash Command
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"rehash")){
		loadconf(thread,"services");
		
		debug = m->debug;
		m->debug=1;
		
		if(proof_config(thread)==0){
			dlog(thread,0,
				"You haven't defined a required variable in the config file, check all variables that are \"REQUIRED\", and restart\n"
				"You'll also find this text in the debuglog!\n"
				);
			do_privmsg(thread,m->nick,nick,
				"ERROR - You haven't defined a required variable in the config file, check all variables that are \"REQUIRED\", and rehash/restart\n"
				"You'll also find this text in the debuglog (look at the debuglog for more details)!\n"
			);
		}
		else{
			zero_config_values(thread);
			loadconf(thread,"services");
			rehashModuleConfigs(thread);
			do_privmsg(thread,m->nick,nick,"Done. - Config loaded without any Errors");
		}
			
		m->debug = debug;
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,1,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Orion REHASH Command (/msg Orion HELP REHASH)
 * 
 * When the user tries to get help for this command, using /msg Orion HELP REHASH,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Orion HELP REHASH
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL orion_rehash_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Rehash Help
	 */
	if(thread->sql.access.olevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help rehash")){
		send_lang(thread,language,1,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"rehash"));
		send_lang(thread,language,1,snick,nick,"ORION_HELP_REHASH",snick);
	}else send_lang(thread,language,1,snick,nick,"ORION_NO_HELP_AVAIL");
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**

 * @brief Orion Help List Command - REHASH part (/msg Orion help)
 * 
 * When the user tries to get general Orion help, using /msg Orion help, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to include<br/>
 * the information for the REHASH command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL orion_rehash_help_listing(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help");
	int Users_ORION_Level=(thread->sql.aliases.active>0 ? thread->sql.access.olevel : is_user);
	/*
	 * Rehash (Help listing)
	 */
	if(Users_ORION_Level>=command_level)
		send_lang(thread,language,1,snick,nick,"ORION_HELP_CMD_REHASH",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"rehash"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
