/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_autolimit.c
 *   @brief This is the Module file for Q AUTOLIMIT Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q AUTOLIMIT command.
 *
 *   The Q AUTOLIMIT Command allows the user to set a<br/>
 *   autolimit for his channel.<br/>
 *   (This limit is updated - per default - all 90 seconds)<br/>
 *   Can be set to avoid join floods.
 *
 *   Syntax is: /msg Q AUTOLIMIT #channel < #limit(# and a digit's) >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "Q"
#define MODUL "autolimit"
#define MODUL_FILE "q_autolimit"
#define MODUL_VERSION "1.0"

MOD_BOOL Q_autolimit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_autolimit_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_autolimit_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"autolimit","autolimit");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,LOGINONLY,OPER_IF_DEFCON);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"autolimit",Q_autolimit,300,1,4,5);
	addCommand(thread,&Module,"help autolimit",Q_autolimit_help,100,1,4,4);
	addCommand(thread,&Module,"showcommands",Q_autolimit_showcommands,100,1,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

//> :Aftershock PRIVMSG q :autolimit #psc #3
/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q AUTOLIMIT Command.
 * 
 * The Q AUTOLIMIT Command allows the user to set a<br/>
 *   autolimit for his channel.<br/>
 *   (This limit is updated - per default - all 90 seconds)<br/>
 *   Can be set to avoid join floods.
 *
 * Syntax is: /msg Q AUTOLIMIT #channel < #limit(# and a digit's) >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_autolimit(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */
	char CBot[MAX_NICK_LEN];
	long limit  = 0;
	long alimit = 0;

	/* Initialise Variables to (NULL) */
	bzero(&CBot,sizeof(CBot));
	
	/*
	 * Autolimit Command
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"autolimit")){
		/* Updates the Threads Channel-Management */
		UpdateChannelInformation(thread,param[4],NULL);
		
		/* Set Q to our Bot, if BotServ is not loaded, or no Bot is assigned, or if SYMBIOSIS is OFF */
		QCheckBot(CBot,thread->c.chan.bot,thread->c.chan.botserv.set,sizeof(CBot));

		if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"q")){
			if((thread->c.chan.settings & CS_FORBID_BIT))		send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
			else if((thread->c.chan.settings & CS_SUSPEND_BIT))	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
			else if(!param[5]){
				if(thread->c.chan.alimit != 0) send_lang(thread,language,howtosend,snick,nick,"Q_AUTOLIMIT_SET",thread->c.chan.alimit);
				else send_lang(thread,language,howtosend,snick,nick,"Q_AUTOLIMIT_NONE");
			}else if(dynlen(param[5])>1 && strcor(param[5],"#") && param[5][0]=='#' && CheckOption(thread,snick,nick,READONLY)){
				alimit = stol(&param[5][1]);
				
				if(thread->sql.access.qlevel>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"autolimit") || strcor(thread->c.chan.access.ownlevel,"nmo")){
					/* Update Memory, for caching */
#ifndef USE_THREADS
					thread->c.chan.alimit = alimit;
#endif
					
					/* Update SQL as Live Update */
					domysql(thread,"UPDATE `orion_cs_core` SET `last_used`='%ld',`alimit`='%ld' WHERE `channel`='%s' AND `serv`='Q'",time(NULL),alimit,mysql_quote(thread,param[4]));
					if(strcor(thread->c.chan.flags,"c") && !strcor(thread->c.chan.flags,"l")){
						limit	= stol(mysql_sel_var(thread,"SELECT count(*) FROM `orion_chan_users` WHERE `channel`='%s'",param[4]));
						limit	+= alimit;
						do_mode(thread,CBot,param[4],thread->c.chan.datecreate,"+l %d",limit);
					}
					send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
				}else send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
			}else send_lang(thread,language,howtosend,snick,nick,"Q_HELP_AUTOLIMIT",snick,snick);
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_EXISTING");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q AUTOLIMIT Command (/msg Q HELP AUTOLIMIT)
 * 
 * When the user tries to get help for this command, using /msg Q HELP AUTOLIMIT,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP AUTOLIMIT
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_autolimit_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Autolimit Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help autolimit"))
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_AUTOLIMIT",snick);

	else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - AUTOLIMIT part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the AUTOLIMIT command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_autolimit_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=0, Users_Q_Level=0;
	command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Autolimit (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_AUTOLIMIT",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"autolimit"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
