/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_list.c
 *   @brief This is the Module file for Q LIST Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q LIST command.
 *
 *   The Q LIST Command allows an admin to dump a list of registered Q channel<br/>
 *   which match the given < *channel-pattern* >.
 *
 *   Syntax is: /msg Q LIST [ < *channel-pattern* > ]
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "Q"
#define MODUL "list"
#define MODUL_FILE "q_list"
#define MODUL_VERSION "1.0"

MOD_BOOL Q_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_list_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"list","list");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,OPERONLY);
	addAccess (thread,&Module,ADMIN);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"list",Q_list,950,INFINIT,3,4);
	addCommand(thread,&Module,"help list",Q_list_help,950,INFINIT,4,4);
	addCommand(thread,&Module,"showcommands",Q_list_showcommands,950,INFINIT,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q LIST Command.
 * 
 * The Q LIST Command allows an admin to dump a list of registered Q channel<br/>
 *   which match the given < *channel-pattern* >.
 *
 * Syntax is: /msg Q LIST [ < *channel-pattern* > ]
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_list(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */
	deepSQL chans;
	char channel[MAX_CHANNELNAME_LEN];

	/* Initialise Variables to (NULL) */
	bzero(&chans,sizeof(chans));
	bzero(&channel,sizeof(channel));

	/*
	 * List Command
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list")){
		if(param[4]){
			dynncpy(channel,param[4],sizeof(channel));
			strdelc(channel,'*');
			chans.res = doSelectMySQL(thread,"SELECT `channel`,`settings`,`reason`,`description`,`founder` FROM `orion_cs_core` WHERE `serv`='Q' AND `channel` LIKE '%c%s%c'",'%',channel,'%');
		}else 
			chans.res = doSelectMySQL(thread,"SELECT `channel`,`settings`,`reason`,`description`,`founder` FROM `orion_cs_core` WHERE `serv`='Q'");
		chans.fields  = mysql_num_rows(chans.res);

		if(chans.fields) {
			send_lang(thread,language,howtosend,snick,nick,"Q_LIST_LIST_X_CHANNELS",chans.fields);

			while((chans.row = mysql_fetch_row(chans.res))){
				if(stoi(chans.row[1]) & CS_SUSPEND_BIT)
					send_lang(thread,language,howtosend,snick,nick,"Q_LIST_LIST_SUSPENDED",chans.row[0],chans.row[2]);
				else
					send_lang(thread,language,howtosend,snick,nick,"Q_LIST_LIST_UNSUSPENDED",chans.row[0],get_localtime((time_t)stol(chans.row[1])),chans.row[3],chans.row[4]);
			}
			send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
		}
		else send_lang(thread,language,howtosend,snick,nick,"Q_LIST_NO_ADDED_CHANS");

		/* mysql_free_result_myr(); - chans */
		mysql_free_result_myr(chans.res);
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_OPERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q LIST Command (/msg Q HELP LIST)
 * 
 * When the user tries to get help for this command, using /msg Q HELP LIST,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP LIST
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_list_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * List Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help list")){
		send_lang(thread,language,howtosend,snick,nick,"REQUIRED_LEVEL",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list"));
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_LIST",snick,snick);
	}else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - LIST part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the LIST command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_list_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	int Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * List (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_LIST",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"list"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
