/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_removeuser.c
 *   @brief This is the Module file for Q REMOVEUSER Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q REMOVEUSER command.
 *
 *   The Q REMOVEUSER Command allows the user to remove another<br/>
 *   user from the channel's access list.
 *
 *   Syntax is: /msg Q REMOVEUSER #channel < nick >
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "Q"
#define MODUL "removeuser"
#define MODUL_FILE "q_removeuser"
#define MODUL_VERSION "1.0"

MOD_BOOL Q_removeuser(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_removeuser_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_removeuser_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"removeuser","removeuser");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,3,LOGINONLY,OPER_IF_DEFCON,READONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"removeuser",Q_removeuser,550,1,5,5);
	addCommand(thread,&Module,"help removeuser",Q_removeuser_help,1,1,4,4);
	addCommand(thread,&Module,"showcommands",Q_removeuser_showcommands,1,1,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q REMOVEUSER Command.
 * 
 * The Q REMOVEUSER Command allows the user to remove another<br/>
 *   user from the channel's access list.
 *
 * Syntax is: /msg Q REMOVEUSER #channel < nick >
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_removeuser(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */

	/* Initialise Variables to (NULL) */

	/*
	 * Removeuser Command
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"removeuser")){
		/* Updates the Threads Channel Management */
		UpdateChannelInformation(thread,param[4],param[5]);
		
		// > :shock PRIVMSG Q :removeuser #lala shock
		if(thread->c.chan.fields && streqnc(thread->c.chan.serv,"q")){
			if((thread->c.chan.settings & CS_FORBID_BIT))		send_lang(thread,language,howtosend,snick,nick,"CHANNEL_FORBIDDEN",thread->c.chan.serv,thread->c.chan.reason);
			else if((thread->c.chan.settings & CS_SUSPEND_BIT))	send_lang(thread,language,howtosend,snick,nick,"CHANNEL_SUSPENDED",thread->c.chan.serv,thread->c.chan.reason);
			
			else if(thread->sql.access.qlevel>=ModuleFile_RootCommandLevel(thread,MODUL_FILE,"removeuser") || strcor(thread->c.chan.access.ownlevel,"n")){
				if(thread->c.chan.access.targetlevel[0]!='\0'){
					
					/* Update memory for caching */
#ifndef USE_THREADS
					bzero(&thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
					bzero(&thread->c.chan.access.targetauthnick,sizeof(thread->c.chan.access.targetauthnick));
#endif
					
					/* Update SQL as live update */
					domysql(thread,"DELETE FROM `orion_cs_access` WHERE `channel`='%s' AND `authnick`='%s' AND `serv`='Q'",mysql_quote(thread,param[4]),mysql_quote(thread,param[5]));
					send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
				}else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_USER",param[5]);
			}
			else if(strcor(thread->c.chan.access.ownlevel,"m")){
				if(strcor(thread->c.chan.access.targetlevel,"n")) 		send_lang(thread,language,howtosend,snick,nick,"Q_RIGHT_OWNER_TO_DELETE_FLAGS");
				else if(strcor(thread->c.chan.access.targetlevel,"m")) 	send_lang(thread,language,howtosend,snick,nick,"Q_RIGHT_OWNER_TO_DELETE_MASTERS_FLAGS");
				
				else if(thread->c.chan.access.targetlevel[0]!='\0'){
					
					/* Update memory for caching */
#ifndef USE_THREADS
					bzero(&thread->c.chan.access.targetlevel,sizeof(thread->c.chan.access.targetlevel));
					bzero(&thread->c.chan.access.targetauthnick,sizeof(thread->c.chan.access.targetauthnick));
#endif
					
					/* Update SQL as live update */
					domysql(thread,"DELETE FROM `orion_cs_access` WHERE `channel`='%s' AND `authnick`='%s' AND `serv`='Q'",mysql_quote(thread,param[4]),mysql_quote(thread,param[5]));
					send_lang(thread,language,howtosend,snick,nick,"Q_COMMAND_DONE");
				}else send_lang(thread,language,howtosend,snick,nick,"NOT_KNOWN_USER",param[5]);
			}
			else send_lang(thread,language,howtosend,snick,nick,"LEVEL_NOT_HIGH_ENOUGH");
		}else send_lang(thread,language,howtosend,snick,nick,"CHANNEL_NOT_EXISTING");
	}else{
	/*
	 *  Error Messages
	 */
		send_lang(thread,language,howtosend,snick,nick,"NO_ACCESS_TO_THIS_USERCOMMAND");
	}
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q REMOVEUSER Command (/msg Q HELP REMOVEUSER)
 * 
 * When the user tries to get help for this command, using /msg Q HELP REMOVEUSER,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP REMOVEUSER
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_removeuser_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Removeuser Help
	 */
	if(thread->sql.access.qlevel>=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"help removeuser"))
		send_lang(thread,language,howtosend,snick,nick,"Q_HELP_REMOVEUSER",snick,snick);

	else send_lang(thread,language,howtosend,snick,nick,"Q_NO_HELP_AVAIL",snick);
	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - REMOVEUSER part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the REMOVEUSER command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_removeuser_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	int Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Removeuser (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_REMOVEUSER",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"removeuser"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
