/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net,
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file q_whoami.c
 *   @brief This is the Module file for Q WHOAMI Command.
 *
 *   It contains all functions which are neccessary to initialize and use<br/>
 *   the Q WHOAMI command.
 *
 *   The Q WHOAMI allows the user to dump some<br/>
 *   information about himself. (last usermask, last seen, ... )<br/>
 *   It will also show up a list of channel's you have access to.
 *
 *   Syntax is: /msg Q WHOAMI
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "osocket.h"
#include "modules.h"
#include "global.h"
#include "services.h"

#define SERVICE "Q"
#define MODUL "whoami"
#define MODUL_FILE "q_whoami"
#define MODUL_VERSION "1.0"

MOD_BOOL Q_whoami(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_whoami_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL Q_whoami_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]);
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action);

/*  ************************************************************************************************  */

/**
 * @brief Initialize the module in the orion DB
 * 
 * This function loads the module with all its functions into memory, so that it can be used by<br/>
 * the services. At the same time, important options like access permissions, etc. are loaded.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param dl_handle The file handle for the .so file
 * @return ModInfo
 */
ModInfo orion_Init(Threads *thread, void *dl_handle){
	ModInfo Module = ModuleInit(thread,dl_handle,MOD_UNIQUE,SERVICE,MODUL_FILE,MODUL,"whoami","whoami");

	addAuthor (thread,&Module,"shock (C)%s %s",PROJECTNAME,YEAR);
	addVersion(thread,&Module,"%s version %s, ModulVersion: %s v%s",PROJECTNAME,VERSION_NUMBER,MODUL,MODUL_VERSION);
	addOptions(thread,&Module,2,OPER_IF_DEFCON,LOGINONLY);
	addAccess (thread,&Module,BOTH);
	addMODType(thread,&Module,CORE);

	addCommand(thread,&Module,"whoami",Q_whoami,0,0,3,3);
	addCommand(thread,&Module,"help whoami",Q_whoami_help,0,0,4,4);
	addCommand(thread,&Module,"showcommands",Q_whoami_showcommands,0,0,3,3);

	addFinish(thread,&Module,orion_Fini);
	VerifyLevels(thread,&Module);
	return Module;
}

/*  ************************************************************************************************  */

/**
 * @brief Main function for the Q WHOAMI Command.
 * 
 * The Q WHOAMI allows the user to dump some<br/>
 *   information about himself. (last usermask, last seen, ... )<br/>
 *   It will also show up a list of channel's you have access to.
 *
 * Syntax is: /msg Q WHOAMI
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_whoami(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/* Global variables */
	deepSQL Chanaccess;
	char* currentmodes;
	
	/* Initialise Variables to (NULL) */
	bzero(&Chanaccess,sizeof(Chanaccess));
	
	/*
	 * Whoami Command
	 */
	currentmodes = currmodes(thread,nick);
	
	send_lang(thread,language,howtosend,snick,nick,"Q_WHOAMI_GLOBAL_LEVEL",thread->sql.access.qlevel);
	send_lang(thread,language,howtosend,snick,nick,"Q_WHOAMI_CURRENT_MODES",currentmodes);
	send_lang(thread,language,howtosend,snick,nick,"Q_WHOAMI_EMAIL",thread->sql.access.email);
	send_lang(thread,language,howtosend,snick,nick,"Q_WHOAMI_AUTHED_AS",thread->sql.aliases.authnick);
	send_lang(thread,language,howtosend,snick,nick,"Q_WHOAMI_REGISTERED_SINCE",get_localtime(thread->sql.aliases.time_linked));
	send_lang(thread,language,howtosend,snick,nick,"Q_WHOAMI_LAST_AUTH",get_localtime(thread->sql.aliases.last_login));
	Chanaccess.res = doSelectMySQL(thread,"SELECT `authnick`,`channel`,`level` FROM `orion_cs_access` WHERE `authnick`='%s'  AND `serv`='Q'",mysql_quote(thread,thread->sql.aliases.authnick));
	while((Chanaccess.row = mysql_fetch_row(Chanaccess.res))){
		send_lang(thread,language,howtosend,snick,nick,"Q_WHOAMI_ACCESS_LISTINGS",Chanaccess.row[2],Chanaccess.row[1]);
	}
	send_lang(thread,language,howtosend,snick,nick,"END_OF_LIST");
	/* mysql_free_result_myr(); - Chanaccess */
	mysql_free_result_myr(Chanaccess.res);

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Main Help function for the Q WHOAMI Command (/msg Q HELP WHOAMI)
 * 
 * When the user tries to get help for this command, using /msg Q HELP WHOAMI,<br/>
 * this function is executed.
 * 
 * Syntax: /msg Q HELP WHOAMI
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If its 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_whoami_help(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	/*
	 * Whoami Help
	 */
	send_lang(thread,language,howtosend,snick,nick,"Q_HELP_WHOAMI");

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Q SHOWCOMMANDS Command - WHOAMI part (/msg Q showcommands)
 * 
 * When the user tries to get general Q help, using /msg Q showcommands, all loaded commands with<br/>
 * a short description are returned. This function is executed, when the help function tries to<br/>
 * include the information for the WHOAMI command.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language in which the information should be returned
 * @param howtosend If it is 0, the returned information on IRC is sent as PRIVMSG, otherwise as NOTICE
 * @param snick The ServicesClient nick
 * @param nick The Nickname of the user who executed the command
 * @param param The array which holds the parameters of the whole IRCd-message
 * @return MOD_BOOL
 */
MOD_BOOL Q_whoami_showcommands(Threads *thread, void *language, bool howtosend, char *snick, char *nick, char *param[]){
	int command_level=ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"showcommands");
	int Users_Q_Level=(thread->sql.aliases.active>0 ? thread->sql.access.qlevel : is_user);
	/*
	 * Whoami (Showcommands)
	 */
	if(Users_Q_Level>=command_level)
		send_lang(thread,language,howtosend,snick,nick,"Q_SHOWCOMMANDS_WHOAMI",ModuleFile_DynamicCommandLevel(thread,MODUL_FILE,"whoami"));

	return MOD_CONT;
}

/*  ************************************************************************************************  */

/**
 * @brief Function which is executing while unloading the Module
 * 
 * These statements are executed when someone is unloading this Module, e.g. while unloading parts<br/>
 * of the services or the whole services.
 * 
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int MOD_SUCCESS
 */
MOD_BOOL orion_Fini(Threads *thread, DO_ACTION Action){
	return MOD_CONT;
}

/*  ************************************************************************************************  */
