/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 194                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-16 18:33:19 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file modes.c
 *   @brief This is the Module file for managing channel-user modes.
 *
 *   It contains all functions which are neccessary to
 *   handle user's modes.
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

/**
 * @brief getModeNick:  Searches in a given Struct-Array for a given nick (used in SJOIN / MODE)
 *
 * @param ucmodes The Struct-Array where we search for our user
 * @param ucmodes_count The last Arrayelement, so we can't override the limit
 * @param nick The nick we want the modes for.
 * @return int32 Returns the array-number, where the user was found (or -1 for ERROR)
 **/
int getModeNick(UserChanModes *ucmodes, size_t ucmodes_count, char *nick){
	int i=0;
	
	if(nick == NULL || nick[0]=='\0')	return -1;
	if(ucmodes_count==0) 				return -1;
	
	for(i=0;i<ucmodes_count;i++){
		//printf("nick = %s\n",nick);
		//printf("ucmodes[i].nick = %s\n",ucmodes[i].nick);
		if(streqnc(ucmodes[i].nick,nick)) return i;
	}
	return -1;
}

/*  ************************************************************************************************  */

/**
 * @brief multiply_occurance:  proofes if a parameter occurs more than once
 *
 * @param nr Number of the potentially first occurance of our parameter
 * @param max The last Arrayelement, to avoid over-riding
 * @param param The parameter-Array in which we are searching
 * @return int Returns 1 if the Parameter occured at least twice (ERROR) or
 *              0 if it occured only once (SUCCESS) - as used in mode.c
 **/
bool multiply_occurance(int nr, unsigned int max, char *param[]){
	int i=0;
	int counter=0;
	for(i=nr;i<max;i++){
		if(streqnc(param[nr],param[i])) counter++;
	}
	if(counter>1) return 1;
	return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief do_rawmode:  Sends a mode string (without updating the Database)
 *                Used on SJOIN where we act before storing data
 *                i.e. DEOP (data stored to memory) and then INSERT-ED to DB without "+o"
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nickname who executes it (ServicesNick)
 * @param target The target (Channel) in which we want to set these modes
 * @param time_in The channels creation time (if unknown, give 0)
 * @param what The mode string (i.e "+ao shock shock")
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_rawmode(Threads *thread, char *nick, char *target, time_t time_in, char *what, ...){
	/* Deklarate Variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));

	/* Join reason string */
	va_start	(va,  what);
	vsnprintf	(tmp, sizeof(tmp)-1, what, va);
	va_end		(va);

	if(orion_WRCtranslate_valid_chan(thread, target) == false)
		return CMD_ERROR;
	
	return orion_WRCtranslate_cmd_rawmode(thread,nick,target,time_in,tmp);
}

/*  ************************************************************************************************  */

/**
 * @brief do_raw2mode:  Sends a mode string (without updating the Database)
 *                Used on SJOIN where we act before storing data
 *                i.e. DEOP (data stored to memory) and then INSERT-ED to DB without "+o"
 *                NOTE: ONLY use that for Channel-User Modes! 
 *                  (Defined in the protocol-file as: SupportedU2ChanModes)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param source The Sender (ServicesNick)
 * @param channel The Channel in which we want to set these modes (for target user)
 * @param modes The modes we want to give/take the user in target channel
 * @param targetnick The Nickname of target user (the modes will be set/unset for him)
 * @param time_in The channels creation time (if unknown, give 0)
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_raw2mode(Threads *thread, char *source, char *channel, char *modes, char *targetnick, time_t time_in){
	char newmode[MAX_MODES];
	char outbuf[512];
	bzero(&newmode,sizeof(newmode));
	bzero(&outbuf,sizeof(outbuf));
	
	size_t i=0, a=0, end = dynlen(modes);
	
	if(orion_WRCtranslate_valid_chan(thread, channel) == false)
		return CMD_ERROR;
	
	straddnc(newmode,modes[i],sizeof(newmode));
	
	for(i=1;i<end;i++){
		if(IRC.SupportHalfOp && modes[i]=='h')
			straddnc(newmode,(char)IRC.HOpMode,sizeof(newmode));
		else if(IRC.HasOwner && modes[i]=='q')
			straddnc(newmode,(char)IRC.OwnerMode,sizeof(newmode));
		else if(IRC.HasProtect && modes[i]=='a')
			straddnc(newmode,(char)IRC.ProtectMode,sizeof(newmode));
		else if(modes[i]=='o')
			straddnc(newmode,(char)IRC.OpMode,sizeof(newmode));
		else if(modes[i]=='v')
			straddnc(newmode,(char)IRC.VMode,sizeof(newmode));
	}
	
	a = dynlen(newmode)-1;
	straddns(outbuf,newmode,sizeof(outbuf));
	straddnc(outbuf,' ',sizeof(outbuf));
	
	for(i=0;i<a;i++){
		if(IRC.p10protocol || IRC.p10new)
			straddns(outbuf,GetUser(targetnick)->uid,sizeof(outbuf));
		else
			straddns(outbuf,targetnick,sizeof(outbuf));
		straddnc(outbuf,' ',sizeof(outbuf));
	}
	outbuf[dynlen(outbuf)-1] = '\0';
	
	return orion_WRCtranslate_cmd_rawmode(thread,source,channel,time_in,outbuf);
}


/*  ************************************************************************************************  */

/**
 * @brief do_modecompress:  Sends a compressed mode string (WITH updating the Database)
 *                NOTE: ONLY use that for Channel-User Modes! 
 *                  (Defined in the protocol-file as: SupportedU2ChanModes)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param source The Sender (ServicesNick)
 * @param channel The Channel in which we want to set these modes (for target user)
 * @param modes The modes we want to give/take the user in target channel
 * @param targetnick The Nickname of target user (the modes will be set/unset for him)
 * @param time_in The channels creation time (if unknown, give 0)
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_modecompress(Threads *thread, char *source, char *channel, char *modes, char *targetnick, time_t time_in){
	char newmode[MAX_MODES];
	char outbuf[512];
	bzero(&newmode,sizeof(newmode));
	bzero(&outbuf,sizeof(outbuf));
	
	size_t i=0, a=0, end = dynlen(modes);
	
	if(orion_WRCtranslate_valid_chan(thread, channel) == false)
		return CMD_ERROR;
	
	straddnc(newmode,modes[i],sizeof(newmode));
	
	for(i=1;i<end;i++){
		if(IRC.SupportHalfOp && modes[i]=='h')
			straddnc(newmode,(char)IRC.HOpMode,sizeof(newmode));
		else if(IRC.HasOwner && modes[i]=='q')
			straddnc(newmode,(char)IRC.OwnerMode,sizeof(newmode));
		else if(IRC.HasProtect && modes[i]=='a')
			straddnc(newmode,(char)IRC.ProtectMode,sizeof(newmode));
		else if(modes[i]=='o')
			straddnc(newmode,(char)IRC.OpMode,sizeof(newmode));
		else if(modes[i]=='v')
			straddnc(newmode,(char)IRC.VMode,sizeof(newmode));
	}
	
	a = dynlen(newmode)-1;
	straddns(outbuf,newmode,sizeof(outbuf));
	straddnc(outbuf,' ',sizeof(outbuf));
	
	for(i=0;i<a;i++){
		straddns(outbuf,targetnick,sizeof(outbuf));
		straddnc(outbuf,' ',sizeof(outbuf));
	}
	outbuf[dynlen(outbuf)-1] = '\0';
	
	return do_mode(thread,source,channel,time_in,outbuf);
}

/*  ************************************************************************************************  */

/**
 * @brief do_rawkill:  Kills a user without deleting him from the Database.
 *               (Used if the User was deleted from the Database before)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nickname which is executing the KILL (ServicesNick)
 * @param target The target-Nick we want to kill
 * @param message The Kill-Message
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_rawkill(Threads *thread, char *nick, char *target, const char *message, ...){
	/* Let'z check out inputs - are they valid? */	
	if(nick==NULL || target==NULL || message==NULL)
		return CMD_ERROR;
		
	if(orion_WRCtranslate_valid_nick(thread, target) == false)
		return CMD_ERROR;
	
	/* Deklarate Variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));

	/* Join reason string */
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, message, va);
	va_end		(va);
	
	return orion_WRCtranslate_cmd_kill(thread,nick,target,tmp);
}

/*  ************************************************************************************************  */

/**
 * @brief do_rawsvsmode:  Does a raw SVSMode (without updating the Database)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param source The Service where the mode comes from
 * @param nick The Nickname to receive the new modes
 * @param modes The new modes
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_rawsvsmode(Threads *thread, char *source, char *nick, char *modes){
	if(orion_WRCtranslate_valid_nick(thread, nick) == false)
		return CMD_ERROR;
	
	return orion_WRCtranslate_cmd_rawsvsmode(thread,source,nick,modes);
}

/*  ************************************************************************************************  */

/**
 * @brief do_rawtopic:  Does a raw Topic-change (without updating the Database)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param source The Service where the mode comes from
 * @param nick The Nickname to receive the new modes
 * @param modes The new modes
 * @return void
 **/
PROTOCOL_BOOL do_rawtopic(Threads *thread, char *whosets, char *chan, char *whosetit, char *topic, time_t when){
	if(orion_WRCtranslate_valid_chan(thread, chan) == false)
		return CMD_ERROR;
	
	return orion_WRCtranslate_cmd_topic(thread,whosets,chan,whosetit,topic,when);
}

/*  ************************************************************************************************  */
