/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 207                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 12:32:05 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file nick.c
 *   @brief This is the Module file for NICK (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the NICK command.
 *
 *   The NICK File is used to process NICKS sent from Servers
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Nick functions for the server - manages the DB Status (sockscontrol - in orion-xp).)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

/**
 * @brief do_nick:  Sends out a NICK-Change (depending on the Loaded Protocol) - using NICK
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param oldnick The Nick we like to change (only services nicks)
 * @param newnick The Nick we like to set to our Service
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_nick(Threads *thread, char *oldnick, char *newnick){
	PROTOCOL_BOOL analyzer;
	
	/* Let'z check out inputs - are they valid? */
	if(oldnick=='\0' || newnick=='\0')
		return CMD_ERROR;

	if(orion_WRCtranslate_valid_nick(thread, newnick) == false)
		return CMD_ERROR;
		
	if(GetUser(oldnick)==NULL)
		return CMD_ERROR;

	/* Analyze Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_chg_nick(thread,oldnick,newnick);
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;

	/* Command supported .. we go on running it */
	chgnick(thread,oldnick,newnick);
	
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief do_svsnick:  Sends out a NICK-Change (depending on the Loaded Protocol) - using SVSNICK
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param snick The Nick which sends out the NICK-Change (services-nick)
 * @param oldnick The Nick we like to change
 * @param newnick The Nick we like to set to target user
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_svsnick(Threads *thread, char *snick, char *oldnick, char *newnick){
	PROTOCOL_BOOL analyzer;
	
	/* Let'z check out inputs - are they valid? */
	if(snick=='\0' || oldnick=='\0' || newnick=='\0')
		return CMD_ERROR;

	if(orion_WRCtranslate_valid_nick(thread, newnick) == false)
		return CMD_ERROR;
		
	if(GetUser(oldnick)==NULL)
		return CMD_ERROR;

	/* Analyze Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_svsnick(thread, snick, oldnick, newnick, time(NULL));
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;

	/* Command supported .. we go on running it */
	if(!IRC.DoesGlobalChangeAfterSVS)
		chgnick(thread,oldnick,newnick);
		
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief nick:  Processes X-NICK Changes for updating the Database
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param oldnick The Nick which was changed
 * @param newnick The Nick to which the oldnick was changed
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL chgnick(Threads *thread, char *oldnick, char *newnick){
	PROTOCOL_BOOL analyzer;
	
	/* Let'z check out inputs - are they valid? */
	if(GetUser(oldnick)==NULL)
		return CMD_ERROR;
	
	/* Declare Variables */
	Condata Cdata;
	char authnick[MAX_NICK_LEN*2];
	time_t now = time(NULL);

	/* Initialise Variables to (NULL) */
	bzero(&Cdata,sizeof(Cdata));
	bzero(&authnick,sizeof(authnick));
	
	/* 
	InnoDB: using ON UPDATE/DELETE CASCADE -> Update/Delete not neccessary anymore
	NOTE: mode system changed, old tables not neccessary anylonger...
	*/
	
	Users *user = ChangeUser(oldnick,newnick);
	
	if(m->OSNickFloodProt){
		if(user->rights < is_services_oper && getUline(thread,user->server)!=NULL){
			if (now - user->LastNFloodCheck > m->OSNickFloodtime) {
				user->LastNFloodCheck 	= time(NULL);
				user->NickFloodLines 	= 0;
			}
				
			user->NickFloodLines++;
			if (user->NickFloodLines >= m->OSNickFloodlines) {
				printf("local DB killing = %s\n",oldnick);
				_kill(thread,oldnick);
				do_kill(thread,m->servername,user->nick,"killed in cause of a Nickflood");
				/* We did 
					'do_kill(thread,m->servername,user->nick,"killed in cause of a Nickflood");'
					and had the problem that db said after connect: Duplicate entry 'MaTRiX1' for key 1..
					*
					We had to change it to 
					_kill(thread,oldnick);
					'do_kill(thread,m->servername,user->nick,"killed in cause of a Nickflood");'
					Cauze we do Nickkilling before setting the Database to current status... so we have to kill the oldnick in
					the Database as well.
				*/
				return CMD_EXECUTED;
			}
		}
	}
	
	Cdata = sql_condata(thread,oldnick);
	
	/* 	Update our Cache	*/
	if(streqnc(thread->c.Cdata.nick,oldnick))
		bzero(&thread->c,sizeof(thread->c));
	else if(streqnc(thread->sql.Cdata.nick,oldnick))
		bzero(&thread->sql,sizeof(thread->sql));
	/**			***			**/
	
	if(Cdata.ns_warntime)
		domysql(thread,"UPDATE `orion_core_condata` SET `nick`='%s',`ns_warntime`='0' WHERE `nick`='%s'",mysql_quote(thread,newnick),mysql_quote(thread,oldnick));
	else
		domysql(thread,"UPDATE `orion_core_condata` SET `nick`='%s' WHERE `nick`='%s'",mysql_quote(thread,newnick),mysql_quote(thread,oldnick));
		
	domysql(thread,"UPDATE `orion_chan_users` SET `nick`='%s' WHERE `nick`='%s'",mysql_quote(thread,newnick),mysql_quote(thread,oldnick));
	
	if(m->UseWhowas){
		domysql(thread,"INSERT INTO `orion_core_condata_log` (`nick`,`time`,`localtime`,`ident`,`rhost`,`srv`,`vhost`,`realname`,`subnet`,`faketype`) VALUES "
		"('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)"
		,mysql_quote(thread,Cdata.nick),Cdata.time,time(NULL),mysql_quote(thread,Cdata.ident),mysql_quote(thread,Cdata.rhost),mysql_quote(thread,Cdata.server),mysql_quote(thread,Cdata.vhost),mysql_quote(thread,Cdata.realname),mysql_quote(thread,Cdata.subnet),Cdata.faketype);
	}
	
	//printf("m->NSUseTempLinks = %d\n",m->NSUseTempLinks);
	//printf("IRC.p10protocol = %d\n",IRC.p10protocol);
	
	if(m->NSUseTempLinks || IRC.p10protocol){
		
		if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_ns_aliases` WHERE `nick`='%s'",mysql_quote(thread,newnick)))==0){ /* THE NEW-NICK ISN'T REGGED YET */

			/* We proof if the oldnick was active (temp.linked or perm.linked) */
			if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_ns_aliases` WHERE `nick`='%s' AND `active`='1'",mysql_quote(thread,oldnick)))>0){

				/* If it was active we set a new temp.link for the newnick */
				domysql(thread,"INSERT INTO `orion_ns_aliases` (`nick`,`display`,`last_usermask`,`last_realname`,"
					"`time_linked`,`last_login`,`last_used`,"
					"`expire`,`linked`,`active`,`forbid`)"
					"VALUES ('%s','%s','%s','%s','%ld','%ld','%ld','1','1','1','0')",
					mysql_quote(thread,newnick),mysql_quote(thread,selectaliases(thread,"display",oldnick)),mysql_quote(thread,Cdata.vhost),mysql_quote(thread,Cdata.realname),time(NULL),time(NULL),time(NULL));

				/* We proof if the oldnick was temp.linked */
				if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_ns_aliases` WHERE `nick`='%s' AND `linked`='1'",mysql_quote(thread,oldnick)))>0)

					/* If it was temp.linked we delete the old temp.link now */
					domysql(thread,"DELETE FROM `orion_ns_aliases` WHERE `nick`='%s'",mysql_quote(thread,oldnick));
				else
				
					/* If the oldnick wasn't temp. linked it has to be perm. linked (in the same nickgroup) - so we don't delete it, we just set the active state OFF */
					domysql(thread,"UPDATE `orion_ns_aliases` SET `active`='0' WHERE `nick`='%s' AND `linked`='0'",mysql_quote(thread,oldnick));
				
				/* Now we mark the newnick as identified by adding it usermode +r */
				if(IRC.NickRegMode){
					do_svs2mode(thread,m->nick,newnick,IRC.ModeOnNickReg);
				}
			}
		}else{ /* THE NEW-NICK IS REGISTERED YET */
			/* Get Display of Newnick to "authnick" */
			dynncpy(authnick,mysql_sel_var(thread,"SELECT `display` FROM `orion_ns_aliases` WHERE `nick`='%s'"/* AND `active`='0'"*/,mysql_quote(thread,newnick)),sizeof(authnick));
	
			/* check if Authnick(from Newnick) is Authnick(from Oldnick) */
			if(streqnc(authnick,mysql_sel_var(thread,"SELECT `display` FROM `orion_ns_aliases` WHERE `nick`='%s' AND `active`='1'",mysql_quote(thread,oldnick)))){
				/* If authnicks are equal (nick is in the same group) */
				
				/* We set the newnick to active (ONLY if it's permanently linked - means really in the same group already) */
				domysql(thread,"UPDATE `orion_ns_aliases` SET `active`='1' WHERE `nick`='%s' AND `linked`='0'",mysql_quote(thread,newnick));

				/* We proof if the oldnick was temp. linked */
				if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_ns_aliases` WHERE `nick`='%s' AND `linked`='1'",mysql_quote(thread,oldnick)))>0)

					/* if it was we delete the temp. link (which was added before for our oldnick) */
					domysql(thread,"DELETE FROM `orion_ns_aliases` WHERE `nick`='%s' AND `linked`='1'",mysql_quote(thread,oldnick));
				else
					
					/* If the oldnick wasn't temp. linked it has to be perm. linked (in the same nickgroup) - so we don't delete it, we just set the active state OFF */
					domysql(thread,"UPDATE `orion_ns_aliases` SET `active`='0' WHERE `nick`='%s' AND `linked`='0'",mysql_quote(thread,oldnick));
				
				/* We set the nick the user changed to (newnick) +r (mark as identified) */
				if(IRC.NickRegMode && !strcin(IRC.RegMode,Cdata.modes)){
					do_svs2mode(thread,m->nick,newnick,IRC.ModeOnNickReg);
				}
			}
			
			/* If the two authnicks aren't equal (means that nick does NOT belong to the group of "newnick" */
			else{
				/* We set the newnick -r (not identifed - if wasn't not already set to) */
				if(IRC.NickRegMode && strcin(IRC.RegMode,Cdata.modes)){
					do_svs2mode(thread,m->nick,newnick,IRC.ModeOnNickDrop);
				}
				
				/* If we don't use +r/-r (i.e P10 protocol) we delete the account in the IRCd instead of using -r */
				if(IRC.p10protocol){
					analyzer = orion_WRCtranslate_cmd_del_account(thread,GetUser(newnick)->uid);
					if(analyzer >= CMD_NOT_SUPPORTED)
						return analyzer;
				}
				
				/* We proof if the oldnick was temp. linked */
				if(stoi(mysql_sel_var(thread,"SELECT count(1) FROM `orion_ns_aliases` WHERE `nick`='%s' AND `linked`='1'",mysql_quote(thread,oldnick)))>0)

					/* if it was we delete the temp. link (which was added before for our oldnick) */
					domysql(thread,"DELETE FROM `orion_ns_aliases` WHERE `nick`='%s' AND `linked`='1'",mysql_quote(thread,oldnick));
				else
					
					/* If the oldnick wasn't temp. linked it has to be perm. linked (in the same nickgroup) - so we don't delete it, we just set the active state OFF */
					domysql(thread,"UPDATE `orion_ns_aliases` SET `active`='0' WHERE `nick`='%s' AND `linked`='0'",mysql_quote(thread,oldnick));
				
				/* Now we provide the warning that the newnick is already registered (by another user) */
				send_lang(thread,languages[m->language],0,ns->nick,newnick,"NS_WARN_NICK_REGISTERED_BY_ANOTHER_USER");//Keine Rechte - Warnen mit NickServ dieser nick ist nicht in der selben gruppe - geh�t wem anderen (nswarntime setzen)
				domysql(thread,"UPDATE `orion_core_condata` SET `ns_warntime`='%ld' WHERE `nick`='%s'",time(NULL),mysql_quote(thread,newnick));
			}
		}
	}else{
		
		/* Here we aren't using P10 or TempLinks (we handle nicks normally) */
		
		/* Get the Newnicks authnick to "authnick" */
		dynncpy(authnick,mysql_sel_var(thread,"SELECT `display` FROM `orion_ns_aliases` WHERE `nick`='%s' AND `active`='0'",mysql_quote(thread,newnick)),sizeof(authnick));

		/* We set the oldnicks active state OFF */
		domysql(thread,"UPDATE `orion_ns_aliases` SET `active`='0' WHERE `nick`='%s' AND `linked`='0'",mysql_quote(thread,oldnick));
		
		/* If the Newnicks authnick isn't equal to the oldnicks authnick */
		if(!streqnc(authnick,mysql_sel_var(thread,"SELECT `display` FROM `orion_ns_aliases` WHERE `nick`='%s'",mysql_quote(thread,oldnick)))){

			/* We proof if autnick of the Newnick is existing anyways (if it exists the Newnick is already registered) */
			if(authnick[0]!='\0'){
				send_lang(thread,languages[m->language],0,ns->nick,newnick,"NS_WARN_NICK_REGISTERED_BY_ANOTHER_USER");//Keine Rechte - Warnen mit NickServ dieser nick ist nicht in der selben gruppe - geh�t wem anderen (nswarntime setzen)
				//printf("Ja warntime update\n");
				domysql(thread,"UPDATE `orion_core_condata` SET `ns_warntime`='%ld' WHERE `nick`='%s'",time(NULL),mysql_quote(thread,newnick));
			}
			
			/* Authnick of Newnick isn't existing - so the Newnick isn't regged by another user */
			/* We allow the nickchange without acting */
		}
		
		/* Now we set the newnick to INACTIVE state - user has to identify again */
		
		/* If we aren't on P10 protocol and use modes */
		if(IRC.NickRegMode && strcin(IRC.RegMode,Cdata.modes)){
			
			/* This is just to make our mode change visible to users on UnrealIRCd */
			if(streqnc(m->protocol,"UnrealIRCd"))
				send_cmd(thread,ns->nick, "SVSMODE %s %s", newnick, IRC.ModeOnNickReg);
			
			/* Here we set the newnick to -r (visible for users - cauze it's svs2mode) */
			do_svs2mode(thread,ns->nick,newnick,IRC.ModeOnNickDrop); //IRC.ModeOnNickDrop
		}
		
		/* If we use P10 protocol */
		if(IRC.p10protocol){
			
			/* we jsut delete the new users account */
			analyzer = orion_WRCtranslate_cmd_del_account(thread,GetUser(newnick)->uid);
			if(analyzer >= CMD_NOT_SUPPORTED)
				return analyzer;
		}
			
		user->rights = is_user;
	}
	
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief cnick:  Receives a Connecting nick - Received, so what are we doing now?
 *            YEAH - we process it, that's what we are doing here ^^ *-shock
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The Nick which connects
 * @param vident The VIdent of connecting user if it has a virtual ident on connect
 * @param ident The Ident of connecting user
 * @param modes The Modes of connecting user
 * @param time_in The Time when the user connected
 * @param rhost The realhost of connecting user
 * @param vhost The Vhost of connecting user
 * @param server The Server where the Nick connects
 * @param realname The Realname of connecting user
 * @param ip The IP of connecting user
 * @param fake The faketype (to tell the program it's a nick that was created by itself) usually >0
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL cnick(Threads *thread, char *nick, char *vident, char *ident, char *modes, time_t time_in, char *rhost, char *vhost, char *server, char *realname, char *ip, int fake){
	char *eventarray[9];
	char OScompletehost[MAX_NICK_LEN+1+MAX_IDENT_LEN+1+MAX_HOST_LEN+1];
	bool killed			= 0;
	bool sessioncheck	= 0;
	time_t Nick_time	= time(NULL);
	Users *user			= NULL;
	
	deepSQL gline;
	bzero(&gline,sizeof(gline));
	bzero(&OScompletehost,sizeof(OScompletehost));
	//NICK tunix25 3 1089515018 atte python.netsplit.de mafia.de.eu.deepirc.net 0 +x pA2A90BE0.dialin.deepirc.net :Your IRC network is currently listed at irc.netspl
	
	//if(GetUser(nick)!=NULL) /* If it leads to a desync enable it, inspircd1.2.x protcol will suffer from it */
	//	return;
	
	if(defcon==1 && !fake && !thread->Webform) do_kill(thread,m->nick,nick,m->DefConKillReason);
	else if(!fake && m->OSUseRNlineIDline && !thread->Webform){
		killed=false;
		
		/* Build our regexp banmask */
		char *regexp_check_structure = _dynadd("%s!%s@%s#%s=%s",nick,ident,rhost,ip == NULL ? rhost : ip,realname);
		
		if(dynlen(regexp_check_structure)<MAX_GLINE_REGEXP_LEN){
			gline.res    = doSelectMySQL(thread,"SELECT `rng`,`gline`,`duration`,`addedby`,`reason` FROM `orion_core_glines` WHERE `rng`!='0' AND '%s' REGEXP `gline`",mysql_quote(thread,regexp_check_structure));
			gline.fields = mysql_num_rows(gline.res);

			if(gline.fields>0){
				dynncpy(OScompletehost,_dynadd("%s!%s@%s",O->nick,O->ident,O->host),sizeof(OScompletehost));
				gline.row = mysql_fetch_row(gline.res);
				
				/* If connecting user should be glined */
				if(stoi(gline.row[0])==1 || stoi(gline.row[0])==3 || stoi(gline.row[0])==5){
					do_gline(thread,m->servername,_dynadd("*@%s",rhost),OScompletehost,stoi(gline.row[2]),time(NULL),gline.row[4]);
					killed=true;
				}
				
				/* If only kill */
				else{
					do_kill(thread,O->nick,nick,"%s",gline.row[4]);
					killed=true;
				}		
			}
			mysql_free_result_myr(gline.res);
		}else
			dlog(thread,1,"NICK %s With RegEXP Mask [%s] failed length check (increase your \"MAX_GLINE_REGEXP_LEN\" size in services.h and recompile!!!)\n",nick,regexp_check_structure);
	}
	if(killed==false){
//NICK tunix25 3 1089515018 atte python.netsplit.de mafia.de.eu.deepirc.net 0 +x pA2A90BE0.dialin.deepirc.net :Your IRC network is currently listed at irc.netspl
		long limit=getHostLimit(rhost);
		dlog(thread,1,"Limit for Host = %ld\n",limit);
		//printf("vor dem host system\n");
#ifdef USE_THREADS
		my_pthread_mutex_lock(&globvar_lock);
#endif

		if(DuringServerlink && !streqnc(linkserver,m->servername) && m->UseWhowas){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&globvar_lock);
#endif
			if(getUline(thread,server)!=NULL && !fake && !thread->Webform && stol(mysql_sel_var(thread,"SELECT count(1) FROM `orion_core_condata_tmp` LEFT JOIN `orion_core_condata` ON `orion_core_condata`.`rhost`=`orion_core_condata_tmp`.`rhost` WHERE `orion_core_condata_tmp`.`rhost`='%s'",mysql_quote(thread,rhost)))>=limit)
				sessioncheck=1;
		}
		else{
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&globvar_lock);
#endif
			if(getUline(thread,server)!=NULL && !fake && !thread->Webform && stol(mysql_sel_var(thread,"SELECT count(1) FROM `orion_core_condata` WHERE `rhost`='%s'",mysql_quote(thread,rhost)))>=limit)
				sessioncheck=1;
		}
		
		if(sessioncheck){
			if(m->OSGlineOnTrustViolation){
				if(m->SessionLimitExceeded[0]!='\0')
					do_gline(thread,m->servername,_dynadd("*@%s",rhost),_dynadd("%s!%s@%s",O->nick,O->ident,O->host),(m->SessionGlineExpiry+time(NULL)),time(NULL),_dynadd(m->SessionLimitExceeded,rhost));
				else
					do_gline(thread,m->servername,_dynadd("*@%s",rhost),_dynadd("%s!%s@%s",O->nick,O->ident,O->host),(m->SessionGlineExpiry+time(NULL)),time(NULL),"Session limit exceeded");
				
				return CMD_EXECUTED;
				
			}
			else{
				if(m->SessionLimitExceeded[0]!='\0')
					do_kill(thread,m->servername,nick,_dynadd(m->SessionLimitExceeded,rhost));//#letzten killen
				else
					do_kill(thread,m->servername,nick,"Session limit exceeded");//#letzten killen
					
				return CMD_EXECUTED;
			}
			
			
/*
	Set Gline here if trust limit is overridden to often in a short sequence
*/
	
	
		}else{
			//printf("in dem host system\n");
			
			//Add User to Global Flood System
			user = GetUser(nick);
			if(user == NULL)
				user = addUser(nick,server);

			dynncpy(user->host,rhost,sizeof(user->host));
			dynncpy(user->ip,ip,sizeof(user->ip));
        	dynncpy(user->ident,ident,sizeof(user->ident));
        	dynncpy(user->vident,vident,sizeof(user->vident));
        	dynncpy(user->vhost,vhost,sizeof(user->vhost));
            user->faketype = fake;
			user->time = time_in;
			
			char nowhost[250];
			dynncpy(nowhost,rhost,sizeof(nowhost));
			char *ip_tok[10];
			size_t index=strsplit(nowhost,ip_tok,'.');
			if(digitcol(ip_tok[0])){
				char *subnet;
				subnet=_dynadd("%s.%s.%s.*",ip_tok[0],ip_tok[1],ip_tok[2]);
				char newmode[50];dynncpy(newmode,modes,sizeof(newmode));
				substr(newmode,0);
				/* Connecting User */
					dlog(thread,4,"nick(); [IP] - nick = %s\n",nick);
					dlog(thread,4,"nick(); [IP] - time = %ld\n",time_in);
					dlog(thread,4,"nick(); [IP] - ident = %s\n",ident);
					dlog(thread,4,"nick(); [IP] - rhost = %s\n",rhost);
					dlog(thread,4,"nick(); [IP] - srv = %s\n",server);
					if(newmode[0]!='\0') dlog(thread,4,"nick(); [IP] - modes = %s\n",newmode);
					dlog(thread,4,"nick(); [IP] - vhost = %s\n",vhost);
					dlog(thread,4,"nick(); [IP] - realname = %s\n",realname);
					dlog(thread,4,"nick(); [IP] - subnet = %s\n",subnet);
				/*	
					nick = shock
					time = 1111666289
					ident = cion
					rhost = M895P027.adsl.highway.telekom.at
					srv = segfault.fl.us.deepirc.net
					modes = +owghaACxeWt
					vhost = techadmin.deepirc.net
					realname = nMe2.1final
					subnet = *.(null).(null)
				*/
				//printf("vor dem add der ip\n");
				//
				if(!DuringServerlink){
					if(newmode[0]=='\0'){
						if(m->UseWhowas)
							domysql(thread,"INSERT INTO `orion_core_condata_log` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
	
						domysql(thread,"INSERT INTO `orion_core_condata` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
					}else{
						if(noop && !fake && !thread->Webform){
							if(strcor("o",newmode)) do_kill(thread,m->nick,nick,"your olines are nooped."); //noop
							else{
								if(m->UseWhowas)
									domysql(thread,"INSERT INTO `orion_core_condata_log` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`modes`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,newmode),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
	
								domysql(thread,"INSERT INTO `orion_core_condata` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`modes`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,newmode),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
							
							}
						}
						else{
							if(m->UseWhowas)
								domysql(thread,"INSERT INTO `orion_core_condata_log` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`modes`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,newmode),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
	
							domysql(thread,"INSERT INTO `orion_core_condata` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`modes`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,newmode),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
						}
					}
				}else{
					//printf("TO MEMORY\n");
					if(newmode[0]=='\0'){
						doBufferedInsert(thread,1,SQL_BUFFER_CONDATA_NO_MODES,"INSERT INTO `orion_core_condata` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
					}
					else{
						if(noop && !fake && !thread->Webform){
							if(strcor("o",newmode)) do_kill(thread,m->nick,nick,"your olines are nooped."); //noop
							else{
								doBufferedInsert(thread,1,SQL_BUFFER_CONDATA,"INSERT INTO `orion_core_condata` (`nick`,`localtime`,`rident`,`time`,`ident`,`rhost`,`srv`,`modes`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,newmode),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
							}
						}else{
							doBufferedInsert(thread,1,SQL_BUFFER_CONDATA,"INSERT INTO `orion_core_condata` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`modes`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,newmode),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
						}
					}
				}
				/* Sockscontrol check */
				//dlog(thread,1,"1--fake: %d, sockemergency: %d, sockstartcheck: %d \n",fake,sockemergency,sockstartcheck);
				if(!fake && !thread->Webform && sockemergency==0 && sockstartcheck==0){
					sockstartcheck = Nick_time;
					printf("starting sockscontrol1...\n");
				}
			}else{
				//printf("habe eine host\n");
				//host = M895P027.adsl.highway.telekom.at
				
				//Add User to Global Flood System
				user = GetUser(nick);
				if(user == NULL)
					user = addUser(nick,server);
					
				dynncpy(user->host,rhost,sizeof(user->host));
				dynncpy(user->ip,ip,sizeof(user->ip));
        		dynncpy(user->ident,ident,sizeof(user->ident));
        		dynncpy(user->vident,vident,sizeof(user->vident));
        		dynncpy(user->vhost,vhost,sizeof(user->vhost));
            	user->faketype = fake;
				user->time = time_in;
				
				char *subnet;
				subnet=_dynadd("*.%s.%s",ip_tok[index-2],ip_tok[index-1]);
				char newmode[50];dynncpy(newmode,modes,sizeof(newmode));
				substr(newmode,0);
				/* Connecting User */
					dlog(thread,4,"nick(); [HOST] - nick = %s\n",nick);
					dlog(thread,4,"nick(); [HOST] - time = %ld\n",time_in);
					dlog(thread,4,"nick(); [HOST] - ident = %s\n",ident);
					dlog(thread,4,"nick(); [HOST] - rhost = %s\n",rhost);
					dlog(thread,4,"nick(); [HOST] - srv = %s\n",server);
					if(newmode[0]!='\0') dlog(thread,4,"nick(); [HOST] - modes = %s\n",newmode);
					dlog(thread,4,"nick(); [HOST] - vhost = %s\n",vhost);
					dlog(thread,4,"nick(); [HOST] - realname = %s\n",realname);
					dlog(thread,4,"nick(); [HOST] - subnet = %s\n",subnet);
				/*	
					nick = shock
					time = 1111666289
					ident = cion
					rhost = M895P027.adsl.highway.telekom.at
					srv = segfault.fl.us.deepirc.net
					modes = +owghaACxeWt
					vhost = techadmin.deepirc.net
					realname = nMe2.1final
					subnet = *.(null).(null)
				*/
				if(!DuringServerlink){
					if(newmode[0]=='\0'){
						if(m->UseWhowas)
							domysql(thread,"INSERT INTO `orion_core_condata_log` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
	
						domysql(thread,"INSERT INTO `orion_core_condata` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
					}else{
						if(noop && !fake && !thread->Webform){
							if(strcor("o",newmode)) do_kill(thread,m->nick,nick,"your olines are nooped."); //noop
							else{
								if(m->UseWhowas)
									domysql(thread,"INSERT INTO `orion_core_condata_log` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`modes`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,newmode),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
	
								domysql(thread,"INSERT INTO `orion_core_condata` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`modes`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,newmode),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
							}
						}else{
							if(m->UseWhowas)
								domysql(thread,"INSERT INTO `orion_core_condata_log` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`modes`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,newmode),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
							
							domysql(thread,"INSERT INTO `orion_core_condata` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`modes`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,newmode),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
						}
					}
				}
				else{
					//printf("TO MEMORY 2\n");
					if(newmode[0]=='\0'){
						doBufferedInsert(thread,1,SQL_BUFFER_CONDATA_NO_MODES,"INSERT INTO `orion_core_condata` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
					}else{
						if(noop && !fake && !thread->Webform){
							if(strcor("o",newmode)) do_kill(thread,m->nick,nick,"your olines are nooped."); //noop
							else{
								doBufferedInsert(thread,1,SQL_BUFFER_CONDATA,"INSERT INTO `orion_core_condata` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`modes`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%d','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,newmode),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
							}
						}else{
							doBufferedInsert(thread,1,SQL_BUFFER_CONDATA,"INSERT INTO `orion_core_condata` (`nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`modes`,`vhost`,`realname`,`subnet`,`ip`,`faketype`) VALUES ('%s','%ld','%ld','%s','%s','%s','%s','%s','%s','%s','%s','%s','%d') ON DUPLICATE KEY UPDATE `nick`=VALUES(`nick`)",mysql_quote(thread,nick),time_in,time(NULL),mysql_quote(thread,ident),vident!=NULL ? mysql_quote(thread,vident) : mysql_quote(thread,ident),mysql_quote(thread,rhost),mysql_quote(thread,server),mysql_quote(thread,newmode),mysql_quote(thread,vhost),mysql_quote(thread,realname),mysql_quote(thread,subnet),ip!=NULL ? mysql_quote(thread,ip) : "",fake);
						}
					}
				}
				/* Sockscontrol check */
				//dlog(thread,1,"2--fake: %d, sockemergency: %d, sockstartcheck: %d \n",fake,sockemergency,sockstartcheck);
				if(!fake && sockemergency==0 && sockstartcheck==0 && !thread->Webform){
					sockstartcheck = Nick_time;
					printf("starting sockscontrol2...\n");
				}else{
					//sockendtime = time(NULL);
				}
				/* ---- */
			}
			
			//NICK tunix25 3 1089515018 atte python.netsplit.de mafia.de.eu.deepirc.net 0 +x pA2A90BE0.dialin.deepirc.net :Your IRC network is currently listed at irc.netspl
			if(!fake && !thread->Webform && !strcin(IRC.RegMode,modes)){
				bzero(eventarray,sizeof(eventarray));
				
				eventarray[0] = nick;
				eventarray[1] = ident;
				eventarray[2] = vhost;
				eventarray[3] = rhost;
				eventarray[4] = server;
				eventarray[5] = modes;
				eventarray[6] = realname;
				eventarray[7] = _dynadd("%d",fake);
				eventarray[8] = '\0';
				/**************************** SEND NICK to the eventhandler ****************************/
				_ExecuteEvent(nick,eventarray,"on_nick");
				/***************************************************************************************/
			}
		}
	}
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */
