/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 203                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 23:10:45 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file send.c
 *   @brief This is the Module file for SEND (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the SEND command.
 *
 *   The SEND File is used to send NOTICES and PRIVMSGS
 *   to Servers. (Protocol-Process-File -> PPF)
 *   (Send is used to send out langauge depending messages and normal messages,
 *    by making a decision between privmsg/notice.)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

/**
 * @brief msend:  Sends out a NOTICE/PRIVMSG (depending on the Loaded Protocol and on howtosend) - using NOTICE/PRIVMSG
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param howtosend The Bool which is given to decide wheater we send a NOTICE (0) or a PRIVMSG (1)
 * @param nick The Nick (only services nicks) that sends out the NOTICE/PRIVMSG
 * @param target The target (NICK/CHANNEL) to receive that NOTICE/PRIVMSG
 * @param message The notice/privmsg message itself
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL msend(Threads *thread, bool howtosend, char *nick, char *target, const char *message, ...){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	
	/* Let'z check out inputs - are they valid? */
	if(nick==NULL || target==NULL || message==NULL)
		return CMD_ERROR;
		
	/* Deklarate Variables */
	va_list va;
	char tmp[BUFFER+1];
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));

	/* Join reason string */
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, message, va);
	va_end		(va);

	/* Analyze Command - is it supported from the IRCd !? */
	if(howtosend == true)
		analyzer = orion_WRCtranslate_cmd_privmsg(thread,nick,target,tmp);
	else if(howtosend == false)
		analyzer = orion_WRCtranslate_cmd_notice(thread,nick,target,tmp);
		
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief send_lang:  Sends out a NOTICE/PRIVMSG (depending on the Loaded Protocol and on howtosend) - using NOTICE/PRIVMSG
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param lang The Language in which we like to provide Information
 * @param howtosend The Bool which is given to decide wheater we send a NOTICE (0) or a PRIVMSG (1)
 * @param nick The Nick (only services nicks) that sends out the NOTICE/PRIVMSG
 * @param target The target (NICK/CHANNEL) to receive that NOTICE/PRIVMSG
 * @param message The notice/privmsg message itself
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL send_lang(Threads *thread, void *lang, bool howtosend, char *nick, char *target, const char *message, ...){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	
	/* Let'z check out inputs - are they valid? */	
	if(nick==NULL || target==NULL || message==NULL)
		return CMD_ERROR;
		
	/* Deklarate Variables */
	va_list va;
	char tmp[PUTSERVER_BUF_LEN+1];
	char strout[PUTSERVER_BUF_LEN+1];
	char *param[ARRAY_SIZE];
	void *define 	= NULL;
	size_t items	= 0;
	size_t i		= 0;
	
	/* Initialise Variables to (NULL) */	
	bzero(&tmp,sizeof(tmp));
	bzero(&strout,sizeof(strout));
	bzero(param,sizeof(param));

	/* Join reason string */
	define	=	dynsym(thread,lang,(char*)message);
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, (const char*)define, va);
	va_end		(va);

	items=strsplit(tmp,param,'\n');
	for(i=0;i<items;i++){

		bzero(&strout,sizeof(strout));
		if(param[i]!=NULL){
			straddns(strout,param[i],sizeof(strout));
			straddns(strout,"\r\n",sizeof(strout));
	
			if(howtosend == true && strout[0]!='\0')
				analyzer = orion_WRCtranslate_cmd_privmsg(thread,nick,target,strout);
			else if(howtosend == false && strout[0]!='\0')
				analyzer = orion_WRCtranslate_cmd_notice(thread,nick,target,strout);
				
			/* Analyze it - was it sent?, is that command supported !? */
			if(analyzer >= CMD_NOT_SUPPORTED)
				return analyzer;
		}
	}
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief moduleSendLang:  Sends out a NOTICE/PRIVMSG (depending on the set language) - using NOTICE/PRIVMSG
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The Language in which we like to provide Information
 * @param howtosend The Bool which is given to decide wheater we send a NOTICE (0) or a PRIVMSG (1)
 * @param nick The Nick (only services nicks) that sends out the NOTICE/PRIVMSG
 * @param target The target (NICK/CHANNEL) to receive that NOTICE/PRIVMSG
 * @param message The notice/privmsg message itself
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL moduleSendLang(Threads *thread, int language, bool howtosend, char *nick, char *target, char *message, ...){
	PROTOCOL_BOOL analyzer = CMD_ERROR;
	/* Let'z check out inputs - are they valid? */
	if(nick==NULL || target==NULL || message==NULL)
		return CMD_ERROR;

	/* Deklarate Variables */
	va_list va;
	char tmp[PUTSERVER_BUF_LEN+1];
	char strout[PUTSERVER_BUF_LEN+1];
	char *param[ARRAY_SIZE];
	char *mymsg 	= NULL;
	size_t items	= 0;
	size_t i		= 0;

	/* Initialise Variables to (NULL) */
	bzero(&tmp,sizeof(tmp));
	bzero(&strout,sizeof(strout));
	bzero(param,sizeof(param));

	/* Join reason string */
	mymsg	=	GetSharedLangstring(thread,language,message);
	va_start	(va,  message);
	vsnprintf	(tmp, sizeof(tmp)-1, (const char*)mymsg, va);
	va_end		(va);

	items=strsplit(tmp,param,'\n');
	for(i=0;i<items;i++){

		bzero(&strout,sizeof(strout));
		if(param[i]!=NULL){
			straddns(strout,param[i],sizeof(strout));
			straddns(strout,"\r\n",sizeof(strout));

			if(howtosend == true && strout[0]!='\0')
				analyzer = orion_WRCtranslate_cmd_privmsg(thread,nick,target,strout);
			else if(howtosend == false && strout[0]!='\0')
				analyzer = orion_WRCtranslate_cmd_notice(thread,nick,target,strout);

			/* Analyze it - was it sent?, is that command supported !? */
			if(analyzer >= CMD_NOT_SUPPORTED)
				return analyzer;
		}
	}
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief vsend_cmd: Sends a command to the server.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param source Orgin of the Message (some times NULL)
 * @param fmt Format of the Message
 * @param args List of the arguments
 * @return PROTOCOL_BOOL
 */
PROTOCOL_BOOL vsend_cmd(Threads *thread, const char *source, const char *fmt, va_list args)
{
    char buf[BUFSIZE];
    *buf = '\0';

    if (fmt) {
        vsnprintf(buf, sizeof(buf) - 1, fmt, args);

        if (*buf=='\0') {
            return CMD_ERROR;
        }

        if (source) {
			if(IRC.p10protocol)
            	main_raw(thread,"%s %s", source, buf);
            else
            	main_raw(thread,":%s %s", source, buf);

        } else {
            main_raw(thread,"%s", buf);
        }
        return CMD_EXECUTED;
    }
    return CMD_ERROR;
}

/*  ************************************************************************************************  */

/**
 * @brief send_cmd: Sends a command to the server.  The two forms here are like
 *              printf()/vprintf() and friends.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param source Orgin of the Message (some times NULL)
 * @param fmt Format of the Message
 * @param ... any number of parameters
 * @return PROTOCOL_BOOL
 */
PROTOCOL_BOOL send_cmd(Threads *thread, const char *source, const char *fmt, ...)
{
    va_list args;

    if (fmt) {
        va_start(args, fmt);
        vsend_cmd(thread, source, fmt, args);
        va_end(args);
        return CMD_EXECUTED;
    }
    return CMD_ERROR;
}

/*  ************************************************************************************************  */
