/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 275                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-02-05 12:47:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file sharedlanguages.c
 *   @brief This is the Module file for managing language-strings for Modules.
 *
 *   It contains all functions which are neccessary to
 *   add/delete shared language strings.
 */
/*  ************************************************************************************************  */

#include "sysconf.h"		//ifdef USE_THREADS
#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"

SharedLanguage *ModuleLang[NUM_LANGS];

/**
 * @brief InitializeSharedLanguages:  Used to initialize the usage of Shared language strings
 *
 * @param void
 * @return void
 **/
void InitializeSharedLanguages(void){
	int i = 0;
	for(i=0;i<LASTLANG;i++){
        ModuleLang[i] = NULL;
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief addModuleLanguage:  Adds a shared language string to the list
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param Module The module which holds information about the modulefile for which we should add the string
 * @param lang The language in which the string is given
 * @param item The identifier to search for afterwards
 * @param value The string in that specific language
 * @return MOD_ERROR
 **/
bool addModuleLanguage(Threads *thread, ModInfo *ModHandle, Languages lang, char *item, char *value){
	if(item == NULL || value == NULL || thread == NULL || ModHandle == NULL )
		return false;

	if(dynlen(item)>sizeof(ModuleLang[0]->item))
	    return false;

	SharedLanguage *sl = NULL;
	size_t block_len = dynlen(value);

#ifdef USE_THREADS
	my_pthread_mutex_lock(&languages_lock);
#endif
	sl = (SharedLanguage*)malloc(sizeof(SharedLanguage));
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&languages_lock);
#endif
    bzero(sl,sizeof(SharedLanguage));

    dynncpy(sl->item,item,sizeof(sl->item));
    sl->value = (char*)malloc(block_len + 1);
    bzero(sl->value, block_len + 1);
    
    dynncpy(sl->value,value,block_len);
    //dlog(thread,0,"SharedLanguage-Value ADDED from %s\n",value);
    //dlog(thread,0,"SharedLanguage-Value ADDED = %s\n",sl->value);
    
    dynncpy(sl->modulefile,ModHandle->module->filename,sizeof(sl->modulefile));

#ifdef USE_THREADS
	my_pthread_mutex_lock(&languages_lock);
#endif
    sl->next = ModuleLang[lang];
    sl->prev = NULL;
    if (ModuleLang[lang])
        ModuleLang[lang]->prev = sl;

    ModuleLang[lang] = sl;

#ifdef USE_THREADS
	my_pthread_mutex_unlock(&languages_lock);
#endif
	return true;
}

/*  ************************************************************************************************  */

/**
 * @brief GetSharedLangstring:  Returns a shared language string
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param language The language number of the language in which we want to reply
 * @param message The message string we are searching for
 * @return char* Langstring
 **/
char *GetSharedLangstring(Threads *thread, int language, char *message){
	SharedLanguage *sl = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&languages_lock);
#endif
	for(sl = ModuleLang[language]; sl; sl = sl->next){
		if(streqnc(sl->item,message)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&languages_lock);
#endif
			//printf("SharedLang-Value = %s\n",sl->value);
		    return sl->value;
		}
	}
	
	/* If we haven't found anything we return the english one */
	for(sl = ModuleLang[EN]; sl; sl = sl->next){
		if(streqnc(sl->item,message)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&languages_lock);
#endif
		    return sl->value;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&languages_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief delSharedLanguage:  Deletes all language strings for target modulefile
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param modulefile The modulefile for which the shared language string was added
 * @return void
 **/
void delSharedLanguage(Threads *thread, char *modulefile){
	SharedLanguage *sl = NULL;
	int i = 0;
	
	for(i = 0; i<LASTLANG; i++){
#ifdef USE_THREADS
		my_pthread_mutex_lock(&languages_lock);
#endif
		for (sl = ModuleLang[i]; sl; sl = sl->next) {

			if (!sl){
#ifdef USE_THREADS
				my_pthread_mutex_unlock(&languages_lock);
#endif
				break;
			}

			if(streqnc(sl->modulefile,modulefile)){
			
				if (sl->prev)
					sl->prev->next = sl->next;

				else
					ModuleLang[i] = sl->next;

				if (sl->next)
					sl->next->prev = sl->prev;

				free(sl->value);
				free(sl);
			}
		}
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&languages_lock);
#endif
	}
	return;
}

/*  ************************************************************************************************  */
