/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file sockutil.c
 *   @brief This is the Module file to manage Socket Inputs/Outputs.
 *
 *   It contains all functions which are neccessary to buffer
 *   socket-actions - this buffering is used cauze it increases
 *   execution speeds. (memory is faster then network receiving ;)
 *
 *   (Protocol-Process-File -> PPF)
 *   (Socket Functions - Taken from Anope Services - since they have coded it pretty nice!)
 */
/*  ************************************************************************************************  */
 
#include "datatypes.h"
#include "global.h"		// for m-> handles!
#include "services.h"
#include "osocket.h"

# undef FD_ZERO
# define FD_ZERO(p) memset((p), 0, sizeof(*(p)))

/*  ************************************************************************************************  */

/*static char read_netbuf[NET_BUFSIZE];
static char *read_curpos = read_netbuf; // Next byte to return
static char *read_bufend = read_netbuf; // Next position for data from socket
static char *const read_buftop = read_netbuf + NET_BUFSIZE;
int32 total_read = 0;
static char write_netbuf[NET_BUFSIZE];
static char *write_curpos = write_netbuf;       // Next byte to write to socket
static char *write_bufend = write_netbuf;       // Next position for data to socket
static char *const write_buftop = write_netbuf + NET_BUFSIZE;
static int write_fd = -1;
int32 total_written;
static int lastchar = EOF;*/

/*  ************************************************************************************************  */

/**
 * @brief read_buffer_len: Return amount of data in read buffer.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int32
 */
int32 read_buffer_len(Threads *thread)
{
    if (thread->read_bufend >= thread->read_curpos) {
        return thread->read_bufend - thread->read_curpos;
    } else {
        return (thread->read_bufend + NET_BUFSIZE) - thread->read_curpos;
    }
}

/*  ************************************************************************************************  */

/**
 * @brief buffered_read: Reads data buffered from Socket buffer.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param buf Buffer
 * @param int Length of buffer
 * @return int
 */
static int buffered_read(Threads *thread, char *buf, int len)
{
    int nread, left = len;
    fd_set fds;
    struct timeval tv = { 0, 0 };
    int errno_save = orion_sockgeterr();

    if (thread->Sock < 0) {
        orion_sockseterr(SOCKERR_EBADF);
        return -1;
    }
    while (left > 0) {
        struct timeval *tvptr = (thread->read_bufend == thread->read_curpos ? NULL : &tv);
        FD_ZERO(&fds);
        FD_SET(thread->Sock, &fds);
        while (thread->read_bufend != thread->read_curpos - 1
               && !(thread->read_curpos == thread->read_netbuf
                    && thread->read_bufend == thread->read_buftop - 1)
               && select(thread->Sock + 1, &fds, 0, 0, tvptr) == 1) {
            int maxread;
            tvptr = &tv;        /* don't wait next time */
            if (thread->read_bufend < thread->read_curpos)      /* wrapped around? */
                maxread = (thread->read_curpos - 1) - thread->read_bufend;
            else if (thread->read_curpos == thread->read_netbuf)
                maxread = thread->read_buftop - thread->read_bufend - 1;
            else
                maxread = thread->read_buftop - thread->read_bufend;
            nread = orion_sockread(thread->Sock, thread->read_bufend, maxread);
            errno_save = orion_sockgeterr();
            dlog(thread,6,"debug: buffered_read wanted %d, got %d\n", maxread,
                     nread);
            if (nread <= 0)
                break;
            thread->read_bufend += nread;
            if (thread->read_bufend == thread->read_buftop)
                thread->read_bufend = thread->read_netbuf;
        }
        if (thread->read_curpos == thread->read_bufend) /* No more data on socket */
            break;
        /* See if we can gobble up the rest of the buffer. */
        if (thread->read_curpos + left >= thread->read_buftop && thread->read_bufend < thread->read_curpos) {
            nread = thread->read_buftop - thread->read_curpos;
            memcpy(buf, thread->read_curpos, nread);
            buf += nread;
            left -= nread;
            thread->read_curpos = thread->read_netbuf;
        }
        /* Now everything we need is in a single chunk at read_curpos. */
        if (thread->read_bufend > thread->read_curpos && thread->read_bufend - thread->read_curpos < left)
            nread = thread->read_bufend - thread->read_curpos;
        else
            nread = left;
        if (nread) {
            memcpy(buf, thread->read_curpos, nread);
            buf += nread;
            left -= nread;
            thread->read_curpos += nread;
        }
    }
    thread->total_read += len - left;
    dlog(thread,6,"debug: buffered_read(%d,%p,%d) returning %d\n",
             thread->Sock, buf, len, len - left);
             
    orion_sockseterr(errno_save);
    return len - left;
}

/*  ************************************************************************************************  */

/**
 * @brief buffered_read_one: Optimized version of the above for reading a single character; returns
 *                      the character in an int or EOF, like fgetc().
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int
 */
static int buffered_read_one(Threads *thread)
{
    int nread;
    fd_set fds;
    struct timeval tv = { 0, 0 };
    char c;
    struct timeval *tvptr = (thread->read_bufend == thread->read_curpos ? NULL : &tv);
    int errno_save = orion_sockgeterr();

    if (thread->Sock < 0) {
        orion_sockseterr(SOCKERR_EBADF);
        return -1;
    }
    FD_ZERO(&fds);
    FD_SET(thread->Sock, &fds);
    while (thread->read_bufend != thread->read_curpos - 1
           && !(thread->read_curpos == thread->read_netbuf
                && thread->read_bufend == thread->read_buftop - 1)
           && select(thread->Sock + 1, &fds, 0, 0, tvptr) == 1) {
        int maxread;
        tvptr = &tv;            /* don't wait next time */
        if (thread->read_bufend < thread->read_curpos)  /* wrapped around? */
            maxread = (thread->read_curpos - 1) - thread->read_bufend;
        else if (thread->read_curpos == thread->read_netbuf)
            maxread = thread->read_buftop - thread->read_bufend - 1;
        else
            maxread = thread->read_buftop - thread->read_bufend;
        nread = orion_sockread(thread->Sock, thread->read_bufend, maxread);
        errno_save = orion_sockgeterr();
        dlog(thread,6,"debug: buffered_read_one wanted %d, got %d\n", maxread,
                 nread);
        if (nread <= 0)
            break;
        thread->read_bufend += nread;
        if (thread->read_bufend == thread->read_buftop)
            thread->read_bufend = thread->read_netbuf;
    }
    if (thread->read_curpos == thread->read_bufend) {   /* No more data on socket */
        dlog(thread,6,"debug: buffered_read_one(%d) returning %d\n", thread->Sock, EOF);
        orion_sockseterr(errno_save);
        return EOF;
    }
    c = *thread->read_curpos++;
    if (thread->read_curpos == thread->read_buftop)
        thread->read_curpos = thread->read_netbuf;
    thread->total_read++;
    dlog(thread,6,"debug: buffered_read_one(%d) returning %d\n", thread->Sock, c);
    return (int) c & 0xFF;
}

/*  ************************************************************************************************  */

/**
 * @brief write_buffer_len: Return amount of data in write buffer.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int32
 */
int32 write_buffer_len(Threads *thread)
{
    if (thread->write_bufend >= thread->write_curpos) {
        return thread->write_bufend - thread->write_curpos;
    } else {
        return (thread->write_bufend + NET_BUFSIZE) - thread->write_curpos;
    }
}

/*  ************************************************************************************************  */

/**
 * @brief flush_write_buffer: Helper routine to try and write up to one chunk of data from the buffer
 *                      to the socket.  Return how much was written.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param wait Wait
 * @return int
 */
static int flush_write_buffer(Threads *thread, bool wait)
{
    fd_set fds;
    struct timeval tv = { 0, 0 };
    int errno_save = orion_sockgeterr();

    if (thread->write_bufend == thread->write_curpos || thread->write_fd == -1)
        return 0;
    FD_ZERO(&fds);
    FD_SET(thread->write_fd, &fds);
    if (select(thread->write_fd + 1, 0, &fds, 0, wait ? NULL : &tv) == 1) {
        int maxwrite, nwritten;
        if (thread->write_curpos > thread->write_bufend)        /* wrapped around? */
            maxwrite = thread->write_buftop - thread->write_curpos;
        else if (thread->write_bufend == thread->write_netbuf)
            maxwrite = thread->write_buftop - thread->write_curpos - 1;
        else
            maxwrite = thread->write_bufend - thread->write_curpos;
        nwritten = orion_sockwrite(thread->write_fd, thread->write_curpos, maxwrite);
        errno_save = orion_sockgeterr();
        dlog(thread,6,"debug: flush_write_buffer wanted %d, got %d\n", maxwrite,
                 nwritten);
        if (nwritten > 0) {
            thread->write_curpos += nwritten;
            if (thread->write_curpos == thread->write_buftop)
                thread->write_curpos = thread->write_netbuf;
            thread->total_written += nwritten;
            return nwritten;
        }
    }
    orion_sockseterr(errno_save);
    return 0;
}

/*  ************************************************************************************************  */

/**
 * @brief buffered_write: Writes data to the socket buffer.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param buf Buffer to write
 * @param len Length to write
 * @return int
 */
static int buffered_write(Threads *thread, char *buf, int32 len)
{
    int nwritten, left = len;
    int errno_save = orion_sockgeterr();

    if (thread->Sock < 0) {
        errno = EBADF;
        return -1;
    }
    thread->write_fd = thread->Sock;

    while (left > 0) {

        /* Don't try putting anything in the buffer if it's full. */
        if (thread->write_curpos != thread->write_bufend + 1 &&
            (thread->write_curpos != thread->write_netbuf
             || thread->write_bufend != thread->write_buftop - 1)) {
            /* See if we need to write up to the end of the buffer. */
            if (thread->write_bufend + left >= thread->write_buftop
                && thread->write_curpos <= thread->write_bufend) {
                nwritten = thread->write_buftop - thread->write_bufend;
                memcpy(thread->write_bufend, buf, nwritten);
                buf += nwritten;
                left -= nwritten;
                thread->write_bufend = thread->write_netbuf;
            }
            /* Now we can copy a single chunk to write_bufend. */
            if (thread->write_curpos > thread->write_bufend
                && thread->write_curpos - thread->write_bufend - 1 < left)
                nwritten = thread->write_curpos - thread->write_bufend - 1;
            else
                nwritten = left;
            if (nwritten) {
                memcpy(thread->write_bufend, buf, nwritten);
                buf += nwritten;
                left -= nwritten;
                thread->write_bufend += nwritten;
            }
        }

        /* Now write to the socket as much as we can. */
        if (thread->write_curpos == thread->write_bufend + 1 ||
            (thread->write_curpos == thread->write_netbuf
             && thread->write_bufend == thread->write_buftop - 1))
            flush_write_buffer(thread,1);
        else
            flush_write_buffer(thread,0);
        errno_save = errno;
        if (thread->write_curpos == thread->write_bufend + 1 ||
            (thread->write_curpos == thread->write_netbuf
             && thread->write_bufend == thread->write_buftop - 1)) {
            /* Write failed on full buffer */
            break;
        }
    }

    dlog(thread,6,"debug: buffered_write(%d,%p,%d) returning %d\n",
             thread->Sock, buf, len, len - left);
             
    orion_sockseterr(errno_save);
    return len - left;
}


/*  ************************************************************************************************  */

/**
 * @brief buffered_write_one: Optimized version of the above for writing a single character; returns
 *                      the character in an int or EOF, like fputc().  Commented out because it
 *                      isn't currently used.
 *
 * @param int to write
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int
 */
#if 0
static int buffered_write_one(int c, Threads *thread)
{
    struct timeval tv = { 0, 0 };

    if (thread->Sock < 0) {
        orion_sockseterr(SOCKERR_EBADF);
        return -1;
    }
    thread->write_fd = thread->Sock;

    /* Try to flush the buffer if it's full. */
    if (thread->write_curpos == thread->write_bufend + 1 ||
        (thread->write_curpos == thread->write_netbuf
         && thread->write_bufend == thread->write_buftop - 1)) {
        flush_write_buffer(thread,1);
        if (thread->write_curpos == thread->write_bufend + 1 ||
            (thread->write_curpos == thread->write_netbuf
             && thread->write_bufend == thread->write_buftop - 1)) {
            /* Write failed */
            dlog(thread,6,"debug: buffered_write_one(%d) returning %d\n", thread->Sock, EOF);
            return EOF;
        }
    }

    /* Write the character. */
    *thread->write_bufend++ = c;
    if (thread->write_bufend == thread->write_buftop)
        thread->write_bufend = thread->write_netbuf;

    /* Move it to the socket if we can. */
    flush_write_buffer(thread,0);

    dlog(thread,6,"debug: buffered_write_one(%d) returning %d\n", thread->Sock, c);
    return (int) c & 0xFF;
}
#endif                          /* 0 */

/*  ************************************************************************************************  */

/**
 * @brief sgetc: Reads one charakter from the Socket-buffer
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int Returns the ASCII-Value of the received charakter
 */
int sgetc(Threads *thread)
{
    int c;

    if (thread->lastchar != EOF) {
        c = thread->lastchar;
        thread->lastchar = EOF;
        return c;
    }
    return buffered_read_one(thread);
}

/*  ************************************************************************************************  */

/**
 * @brief sungetc: Reads one charakter from the Socket-buffer (Sun System)
 *
 * @param c The charakter we save as lastchar.
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int Returns the ASCII-Value of the received charakter
 */
int sungetc(int c, Threads *thread)
{
    return thread->lastchar = c;
}

/*  ************************************************************************************************  */

/**
 * @brief sgets: If connection was broken, return NULL.  If the read timed out, return
 *                 (char *)-1.
 *
 * @param buf Buffer to get
 * @param len Length
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return char* buffer
 */
char *sgets(char *buf, int len, Threads *thread)
{
    int c = 0;
    struct timeval tv;
    fd_set fds;
    char *ptr = buf;

    flush_write_buffer(thread,0);

    if (len == 0)
        return NULL;
    FD_SET(thread->Sock, &fds);
    tv.tv_sec = m->ReadTimeout;
    tv.tv_usec = m->ReadTimeout_us;
    while (read_buffer_len(thread) == 0 &&
           (c = select(thread->Sock + 1, &fds, NULL, NULL, &tv)) < 0) {
        if (orion_sockgeterr() != EINTR)
            break;
        flush_write_buffer(thread,0);
    }
    if (read_buffer_len(thread) == 0 && c == 0)
        return (char *) -1;
    c = sgetc(thread);
    while (--len && (*ptr++ = c) != '\n' && (c = sgetc(thread)) >= 0);
    if (c < 0)
        return NULL;
    *ptr = 0;
    return buf;
}

/*  ************************************************************************************************  */

/**
 * @brief sgets2:  Read a line of text from a socket, and strip newline and
 *           carriage return characters from the end of the line.
 *
 * @param buf Buffer to get
 * @param len Length
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return char* buffer
 */
char *sgets2(char *buf, int len, Threads *thread)
{
	double strlength;
	
    dlog(thread,5,"sgets(); NOT done\n");
    char *str = sgets(buf, len, thread);
    dlog(thread,5,"sgets(); done\n");
    if (!str || str == (char *) -1)
        return str;
	
    str = buf + dynlen(buf) - 1;
    if (*str == '\n')
        *str-- = 0;
    if (*str == '\r')
        *str = 0;
        
	strlength = dynlen(buf);
        
/** Determines Traffic Input - Socket reading **/
	m->TrafficIn+=((strlength/1000)/1000);
	m->TrafficIn_bps+=strlength;
/***********************************************/
    return buf;
}

/*  ************************************************************************************************  */

/**
 * @brief sread: Read from a socket.  (Use this instead of read() because it has
 *            buffering.)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param buf Buffer to get
 * @param len Length
 * @return int
 */
int sread(Threads *thread, char *buf, int len)
{
    return buffered_read(thread, buf, len);
}

/*  ************************************************************************************************  */

/**
 * @brief sputs: write buffer
 *
 * @param str Buffer to write
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return int
 */
int sputs(char *str, Threads *thread)
{
/** Determines Traffic Output - Socket wrinting **/
	double strlength = dynlen(str);
	m->TrafficOut+=((strlength/1000)/1000);
	m->TrafficOut_bps+=strlength;
/*************************************************/
	if(m->debug>=1)
		dlog(thread,1,"< %s",str);
		
	else if(!m->debug && dofork) 
		alog(thread,"< %s", str);

    return buffered_write(thread, str, dynlen(str));
}

/*  ************************************************************************************************  */

/**
 * @brief main_raw: a socket writting method
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param fmt format of message
 * @param ... various args
 * @return void
 */
void main_raw(Threads *thread, const char *fmt, ...)
{
    va_list args;
    char buf[16384];            //Really huge, to try and avoid truncation
    int len;

    va_start(args, fmt);
    len = vsnprintf(buf, sizeof(buf)-1, fmt, args);
    va_end(args);
    
    buf[len]='\0';
    strclean(buf);
    straddns(buf,"\r\n",sizeof(buf));
    
    dlog(thread,4,"sputs-buf: %s",buf);
    sputs(buf, thread); 
    return;
}

/*  ************************************************************************************************  */

/**
 * @brief startlink: Starts to link the IRCd
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param server The server (host/ip) where we want to connect to
 * @param port The port on which we like to connect the IRCd
 * @param linkpass The link password for the ServicesServer
 * @param servername The Servername for the ServicesServer
 * @param serverlink The Hopcount we are using
 * @param description Description for the ServicesServer ( Config )
 * @return int Returns the Sockethandle.
 */
int startlink(Threads *thread,char *server,long port,char *linkpass,char *servername,int serverlink,char *description){
	//printf("server = %s\n",server);
	//printf("port = %d\n",port);
	
	struct sockaddr_in cli_name;
	thread->Sock  = socket(PF_INET, SOCK_STREAM, 0);
	
	if (thread->Sock < 0){
		perror ("Opening channel");
		dlog(thread,0,"Socket Closed, in case of socket();");
		close(thread->Sock);
		_orion_exit();
	}

	bzero(&cli_name, sizeof(cli_name));
	cli_name.sin_family = AF_INET;
	cli_name.sin_addr.s_addr = inet_addr(server);
	cli_name.sin_port = htons(port);
	
	if (connect(thread->Sock, (struct sockaddr *)&cli_name, sizeof(cli_name)) < 0){
		perror ("Establishing communications\n");
		dlog(thread,0,"Sockect Closed, in case of connect();");
		close(thread->Sock);
		_orion_exit();
	}
	
    /*main_raw("PASS %s",linkpass);
	main_raw("PROTOCTL NICKv2 SJOIN SJOIN2 SJ3 NOQUIT VHP"); //SJOIN SJOIN2 SJ3
	main_raw("SERVER %s %d :%s",servername,serverlink,description);*/
	//printf("orion BanMode = %d\n",IRC.HasOwner);
	orion_WRCtranslate_cmd_initialize(thread);
	return thread->Sock;
}

/*  ************************************************************************************************  */
/* Windows support functions */

#ifdef _WIN32
/* Microsoft makes things nice and fun for us! */
struct u_WSA_errors {
    int error_code;
    char *error_string;
};

/* Must be sorted ascending by error code */
struct u_WSA_errors WSAErrors[] = {
    {WSAEINTR, "Interrupted system call"},
    {WSAEBADF, "Bad file number"},
    {WSAEACCES, "Permission denied"},
    {WSAEFAULT, "Bad address"},
    {WSAEINVAL, "Invalid argument"},
    {WSAEMFILE, "Too many open sockets"},
    {WSAEWOULDBLOCK, "Operation would block"},
    {WSAEINPROGRESS, "Operation now in progress"},
    {WSAEALREADY, "Operation already in progress"},
    {WSAENOTSOCK, "Socket operation on non-socket"},
    {WSAEDESTADDRREQ, "Destination address required"},
    {WSAEMSGSIZE, "Message too long"},
    {WSAEPROTOTYPE, "Protocol wrong type for socket"},
    {WSAENOPROTOOPT, "Bad protocol option"},
    {WSAEPROTONOSUPPORT, "Protocol not supported"},
    {WSAESOCKTNOSUPPORT, "Socket type not supported"},
    {WSAEOPNOTSUPP, "Operation not supported on socket"},
    {WSAEPFNOSUPPORT, "Protocol family not supported"},
    {WSAEAFNOSUPPORT, "Address family not supported"},
    {WSAEADDRINUSE, "Address already in use"},
    {WSAEADDRNOTAVAIL, "Can't assign requested address"},
    {WSAENETDOWN, "Network is down"},
    {WSAENETUNREACH, "Network is unreachable"},
    {WSAENETRESET, "Net connection reset"},
    {WSAECONNABORTED, "Software caused connection abort"},
    {WSAECONNRESET, "Connection reset by peer"},
    {WSAENOBUFS, "No buffer space available"},
    {WSAEISCONN, "Socket is already connected"},
    {WSAENOTCONN, "Socket is not connected"},
    {WSAESHUTDOWN, "Can't send after socket shutdown"},
    {WSAETOOMANYREFS, "Too many references, can't splice"},
    {WSAETIMEDOUT, "Connection timed out"},
    {WSAECONNREFUSED, "Connection refused"},
    {WSAELOOP, "Too many levels of symbolic links"},
    {WSAENAMETOOLONG, "File name too long"},
    {WSAEHOSTDOWN, "Host is down"},
    {WSAEHOSTUNREACH, "No route to host"},
    {WSAENOTEMPTY, "Directory not empty"},
    {WSAEPROCLIM, "Too many processes"},
    {WSAEUSERS, "Too many users"},
    {WSAEDQUOT, "Disc quota exceeded"},
    {WSAESTALE, "Stale NFS file handle"},
    {WSAEREMOTE, "Too many levels of remote in path"},
    {WSASYSNOTREADY, "Network subsystem is unavailable"},
    {WSAVERNOTSUPPORTED, "Winsock version not supported"},
    {WSANOTINITIALISED, "Winsock not yet initialized"},
    {WSAHOST_NOT_FOUND, "Host not found"},
    {WSATRY_AGAIN, "Non-authoritative host not found"},
    {WSANO_RECOVERY, "Non-recoverable errors"},
    {WSANO_DATA, "Valid name, no data record of requested type"},
    {WSAEDISCON, "Graceful disconnect in progress"},
#ifdef WSASYSCALLFAILURE
    {WSASYSCALLFAILURE, "System call failure"},
#endif
    {0, NULL}
};

char *orion_sockstrerror(int error)
{
    static char unkerr[64];
    int start = 0;
    int stop = sizeof(WSAErrors) / sizeof(WSAErrors[0]) - 1;
    int mid;

    /* Microsoft decided not to use sequential numbers for the error codes,
     * so we can't just use the array index for the code. But, at least
     * use a binary search to make it as fast as possible. 
     */
    while (start <= stop) {
        mid = (start + stop) / 2;
        if (WSAErrors[mid].error_code > error)
            stop = mid - 1;

        else if (WSAErrors[mid].error_code < error)
            start = mid + 1;
        else
            return WSAErrors[mid].error_string;
    }
    sprintf(unkerr, "Unknown Error: %d", error);
    return unkerr;
}

int orion_socksetnonb(orion_socket_t fd)
{
    u_long i = 1;
    return (!ioctlsocket(fd, FIONBIO, &i) ? -1 : 1);
}
#endif
