/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file trusts.c
 *   @brief This is the Module file for managing Trusts/Trustgroups.
 *
 *   It contains all functions which are neccessary to
 *   control Trust dumping/storing. 
 *   (We request a copy of the DB-Data to manage them faster;
 *    this is necessary =/)
 *
 *	 Trusts structs and Trustmanagement
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"

static Trusts *trusts           = NULL;
static Trustgroups *trustgroups = NULL;

/**
 * @brief add_Trust:  Adds a Trust to the list
 *
 * @param host The host which we should trust
 * @param description The description for the trust
 * @param limit The limit for target host
 * @param art It is given to decide wheather its a trust or a trustdeny
 * @param fields Fields of the SQL-Select
 * @return void
 **/
void add_Trust(char *host, char* description, char* limit, Trust_art art, int fields)
{
#ifdef USE_THREADS
	my_pthread_mutex_lock(&trusts_lock);
#endif
	Trusts *trust = (Trusts*)malloc(sizeof(Trusts));
	bzero(trust,sizeof(Trusts));

	dynncpy(trust->host,host,sizeof(trust->host));
	dynncpy(trust->description,description,sizeof(trust->description));
	dynncpy(trust->limit,limit,sizeof(trust->limit));
	trust->art = art;
	trust->fields = fields;
	
	
	trust->next = trusts;
	trust->prev = NULL;
	if (trusts)
		trusts->prev = trust;
	trusts = trust;
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&trusts_lock);
#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief add_Trustgroup:  Adds a Trustgroup to the list
 *
 * @param group The group which we like to create
 * @param limit The limit for target group
 * @param fields Fields of the SQL-Select
 * @return void
 **/
void add_Trustgroup(char *group, long limit, int32 fields)
{
#ifdef USE_THREADS
	my_pthread_mutex_lock(&trustgroups_lock);
#endif
	Trustgroups *trustgroup = (Trustgroups*)malloc(sizeof(Trustgroups));
	bzero(trustgroup,sizeof(Trustgroups));

	dynncpy(trustgroup->group,group,sizeof(trustgroup->group));
	trustgroup->limit = limit;
	trustgroup->fields = fields;
	trustgroup->next = trustgroups;

	trustgroup->prev = NULL;
	if (trustgroups)
		trustgroups->prev = trustgroup;
	trustgroups = trustgroup;
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&trustgroups_lock);
#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief del_Trust:  Deletes a Trust from the list
 *
 * @param trust The pointer to the Trust we like to delete
 * @return bool Returns 1 if trust was deleted and 0 if there was an error in deleting
 **/
bool del_Trust(Trusts *trust)
{
	Trusts *ptr = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&trusts_lock);
#endif
	for (ptr = trusts; ptr; ptr = ptr->next) {
		if (ptr == trust)
			break;
	}
	if (!ptr){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&trusts_lock);
#endif
		return 0;
	}
	if (trust->prev)
		trust->prev->next = trust->next;
	else
		trusts = trust->next;
	if (trust->next)
		trust->next->prev = trust->prev;
	free(trust);
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&trusts_lock);
#endif
	return 1;
}

/*  ************************************************************************************************  */

/**
 * @brief del_Trust_by_Group:  Deletes all Trusts of one group (Trust or Trustdeny)
 *
 * @param group The group we like to delete
 * @param T_art It is given to decide wheather to delete the Trusts or the Trustdeny's
 * @return int Returns the number of delete Trusts / Trustdeny's
 **/
int del_Trust_by_Group(char *group, Trust_art T_art){
	Trusts *trust = NULL;
	int i = 0;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&trusts_lock);
#endif
	for (trust = trusts; trust; trust = trust->next) {
		if(streqnc(trust->limit,group) && trust->art==T_art){
			del_Trust(trust);
			i++;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&trusts_lock);
#endif
	return i;
}

/*  ************************************************************************************************  */

/**
 * @brief edit_Trust_by_Group:  Changes a group
 *
 * @param group The group we like to rename
 * @param newgroup The newname for the group
 * @param T_art It is given to decide wheather to rename the Trusts or the Trustdeny's
 * @return int Returns the number of matched Trusts / Trustdeny's
 **/
int edit_Trust_by_Group(char *group, char *newgroup, Trust_art T_art){
	Trusts *trust = NULL;
	int i=0;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&trusts_lock);
#endif
	for (trust = trusts; trust; trust = trust->next) {
		if(streqnc(trust->limit,group) && trust->art==T_art){
			dynncpy(trust->limit,newgroup,sizeof(trust->limit));
			i++;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&trusts_lock);
#endif
	return i;
}

/*  ************************************************************************************************  */

/**
 * @brief del_Trustgroup:  Deletes a Trustgroup from the list
 *
 * @param trustgroup The pointer to the Trustgroup we like to delete
 * @return bool Returns 1 if trustgroup was deleted and 0 if there was an error in deleting
 **/
bool del_Trustgroup(Trustgroups *trustgroup)
{
	Trustgroups *ptr = NULL;

#ifdef USE_THREADS
	my_pthread_mutex_lock(&trustgroups_lock);
#endif
	for (ptr = trustgroups; ptr; ptr = ptr->next) {
		if (ptr == trustgroup)
			break;
	}
	if (!ptr){
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&trustgroups_lock);
#endif
		return 0;
	}
	if (trustgroup->prev)
		trustgroup->prev->next = trustgroup->next;
	else
		trustgroups = trustgroup->next;
	if (trustgroup->next)
		trustgroup->next->prev = trustgroup->prev;
	free(trustgroup);
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&trustgroups_lock);
#endif
	return 1;
}

/*  ************************************************************************************************  */

/**
 * @brief DelTrustgroups:  Deletes all Trustgroups from the list
 *
 * @return void
 **/
void DelTrustgroups(void){
	Trustgroups *trustgroup = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&trustgroups_lock);
#endif
	for (trustgroup = trustgroups; trustgroup; trustgroup = trustgroup->next) {
		if(!del_Trustgroup(trustgroup)) break;
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&trustgroups_lock);
#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief DelTrusts:  Deletes all Trusts from the list
 *
 * @return void
 **/
void DelTrusts(void){
	Trusts *trust = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&trusts_lock);
#endif	
	for (trust = trusts; trust; trust = trust->next) {
		if(!del_Trust(trust)) break;
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&trusts_lock);
#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief GetTrustsSql:  Dumpes all Trusts from the DB to the list again
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void GetTrustsSql(Threads *thread){
	/* Global variables */
	deepSQL gettrusts;
	int fields = 0;

	/* Initialise Variables to (NULL) */
	bzero(&gettrusts,sizeof(gettrusts));
	
	gettrusts.res    = doSelectMySQL(thread,"SELECT `rhost`,`limit`,`description`,`art` FROM `orion_os_trusts`");
	gettrusts.fields = mysql_num_fields(gettrusts.res);

	if(gettrusts.fields){
		while((gettrusts.row = mysql_fetch_row(gettrusts.res))){
			add_Trust(gettrusts.row[0],gettrusts.row[2],gettrusts.row[1],(Trust_art)stoi(gettrusts.row[3]),fields);
		}
	}
	mysql_free_result_myr(gettrusts.res);
}

/*  ************************************************************************************************  */

/**
 * @brief GetTrustgroupsSql:  Dumpes all Trustgroups from the DB to the list again
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void GetTrustgroupsSql(Threads *thread){
	/* Global variables */
	deepSQL gettrusts;

	/* Initialise Variables to (NULL) */
	bzero(&gettrusts,sizeof(gettrusts));

	gettrusts.res    = doSelectMySQL(thread,"SELECT `group`,`limit` FROM `orion_os_trustgroups`");
	gettrusts.fields = mysql_num_fields(gettrusts.res);
	
	if(gettrusts.fields){
		while((gettrusts.row = mysql_fetch_row(gettrusts.res))){
			add_Trustgroup(gettrusts.row[0],stol(gettrusts.row[1]),gettrusts.fields);
		}
	}
	mysql_free_result_myr(gettrusts.res);
}

/*  ************************************************************************************************  */

/**
 * @brief getTrust:  Searches the list for a trusted host
 *
 * @param host The host which we are searching for
 * @param T_art It is given to decide wheather we are looking for a trust or a trustdeny
 * @return Trusts* Returns a pointer to the found Trust
 **/
Trusts *getTrust(char *host, Trust_art T_art){
	Trusts *trust   = NULL;
	char *temprhost = NULL;
	bool found=0;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&trusts_lock);
#endif
	for (trust = trusts; trust; trust = trust->next) {
		if(streqnc(trust->host,host) && trust->art==T_art){
			temprhost = host;
			found=1;
			break;
		}
		if(strins(trust->host,"*.")){
			char tmprepl[200];
			dynncpy(tmprepl,trust->host,sizeof(tmprepl));
			strdels(tmprepl,'n',"*.");
			if(strins(host,tmprepl)){
				temprhost=trust->host;
				found=1;
				break;
			}
		}
	}
	if(found && temprhost){
		if(streq(trust->host,temprhost) && trust->art==T_art){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&trusts_lock);
#endif
			return trust;
		}else{
			for (trust = trusts; trust; trust = trust->next) {
				if(streqnc(trust->host,temprhost) && trust->art==T_art){
#ifdef USE_THREADS
					my_pthread_mutex_unlock(&trusts_lock);
#endif
					return trust;
				}
			}
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&trusts_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief getTrustgroup:  Searches the list for a trusted host
 *
 * @param group The group which we are searching for
 * @return Trustgroups* Returns a pointer to the found Trustgroup
 **/
Trustgroups *getTrustgroup(char *group){
	Trustgroups *trustgroup = NULL;
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&trustgroups_lock);
#endif
	for (trustgroup = trustgroups; trustgroup; trustgroup = trustgroup->next) {
		if(streqnc(trustgroup->group,group)){
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&trustgroups_lock);
#endif
			return trustgroup;
		}
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&trustgroups_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief getHostLimit:  Get a limit for a trusted host from the list
 *
 * @param host The host which we are searching for
 * @return int32 Returns the limit for the given host
 **/
long getHostLimit(char *host){
	Trustgroups *trustgroup = NULL;
	Trusts *trust           = getTrust(host,TRUST);
	Trusts *trust2          = getTrust(host,DENY);
	
	if(trust!=NULL && trust->limit[0]!='\0' && trust2==NULL){
		if(strcor("#",trust->limit)){
			trustgroup=getTrustgroup(trust->limit);
			if(trustgroup != NULL && trustgroup->limit)
				return trustgroup->limit;
			else 
				return m->maxconnections;
		}
		else if(digitcol(trust->limit))
			return stol(trust->limit);
	}
	else
		return m->maxconnections;
		
	return m->maxconnections;
}

/*  ************************************************************************************************  */
