/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 194                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-16 18:33:19 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file zline.c
 *   @brief This is the Module file for ZLINE (Server Side) Command.
 *
 *   It contains all functions which are neccessary to process and use 
 *   the ZLINE command.
 *
 *   The ZLINE File is used to process ZLINES sent from Servers
 *   and to Servers. (Protocol-Process-File -> PPF)
 *   (Zline is used to handle Szlines - to set/remove SZLINES)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"

#include "services.h"
#include "modules.h"
#include "osocket.h"

/**
 * @brief do_szline:  Sends out a ZLINE (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servername The ServerName (services-server) which sends the gline information
 * @param ipmask The Ip which we like to put a gline on
 * @param nick The Nick which added the gline
 * @param in_expiretime The Time when the gline should expire
 * @param in_hubtime The Time when the gline was set
 * @param in_reason The Reason why we gline that Host / IP
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_szline(Threads *thread, char *servername, char *ipmask, char *nick, time_t in_expiretime, time_t in_hubtime, char *in_reason){
	PROTOCOL_BOOL analyzer;

	/* Declare variables */
	char addedby[MAX_NICK_LEN];
	char ident_at_host[MAX_IDENT_LEN+1+MAX_HOST_LEN];
	time_t expire = 0;
	
	/* Initialize variables to NULL */
	bzero(&addedby,sizeof(addedby));
	bzero(&ident_at_host,sizeof(ident_at_host));
	
	/* Clean the delay in our Duration */
							expire = in_expiretime-in_hubtime;
	if(in_expiretime > 0) 	expire+=time(NULL);
	else 					expire = 0;

	/*
	 * Build response.
	 */
	
	/* Analyze Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_szline(thread,servername,ipmask,nick,in_hubtime,in_expiretime,in_reason);
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;

	/* Command supported .. we go on running it */
	sscanf(nick,"%[^!]!%s",addedby,ident_at_host);
	//addedby = NICK ohne !ident@host.net ....
	
	domysql(thread,"INSERT INTO `orion_core_lines` (`type`,`mask`,`addedby`,`duration`,`reason`) VALUES ('Z','%s','%s','%ld','%s') ON DUPLICATE KEY UPDATE "
	"`addedby`=VALUES(`addedby`),"
	"`duration`=VALUES(`duration`),"
	"`reason`=VALUES(`reason`)"
	,mysql_quote(thread,ipmask),mysql_quote(thread,addedby),expire,mysql_quote(thread,in_reason));
	
	if(streqnc(addedby,O->nick) || streqnc(addedby,SC->nick) || streqnc(addedby,Q->nick))			
		domysql(thread,"INSERT INTO `orion_os_szlines` (`zline`,`duration`,`addedby`,`reason`) VALUES ('%s','%ld','%s','%s') ON DUPLICATE KEY UPDATE "
		"`duration`=VALUES(`duration`),"
		"`addedby`=VALUES(`addedby`),"
		"`reason`=VALUES(`reason`)"
		,mysql_quote(thread,ipmask),expire,mysql_quote(thread,addedby),mysql_quote(thread,in_reason));				
		
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief do_unszline:  Sends out an UNZLINE (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servername The ServerName (services-server) which sends the unzline information
 * @param ipmask The Ip which we like to unzline
 * @param nick The Nick which added the zline
 * @param in_hubtime The Time when the zline was set
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL do_unszline(Threads *thread, char *servername, char *ipmask, char *nick, time_t in_hubtime){
	PROTOCOL_BOOL analyzer;
	
	/*
	 * Build response.
	 */
	 
	/* Analyze Command - is it supported from the IRCd !? */
	analyzer = orion_WRCtranslate_cmd_unszline(thread,servername,ipmask,nick,in_hubtime);
	if(analyzer >= CMD_NOT_SUPPORTED)
		return analyzer;
	
	/* Command supported .. we go on running it */
	domysql(thread,
		"DELETE `orion_os_szlines`,`orion_core_lines` FROM `orion_os_szlines` "
		"LEFT OUTER JOIN `orion_core_lines` ON (`orion_core_lines`.`mask`=`orion_os_szlines`.`zline` AND `orion_core_lines`.`type`='Z') "
		"WHERE `orion_os_szlines`.`zline`='%s'"
		,mysql_quote(thread,ipmask));
	return analyzer;
}

/*  ************************************************************************************************  */

/**
 * @brief on_szline:  Receives a ZLINE (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servername The ServerName which sends us the zline information
 * @param ipmask The Ip which was zlined
 * @param nick The Nick which added the zline
 * @param in_expiretime The Time when the zline should expire
 * @param in_hubtime The Time when the zline was set
 * @param in_reason The Reason why we that Host / IP was zlined
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_szline(Threads *thread, char *servername, char *ipmask, char *nick, time_t in_expiretime, time_t in_hubtime, char *in_reason){
	/* Declare variables */
	char addedby[MAX_NICK_LEN];
	char ident_at_host[MAX_IDENT_LEN+1+MAX_HOST_LEN];
	time_t expire = 0;

	/* Initialize variables to NULL */
	bzero(&addedby,sizeof(addedby));
	bzero(&ident_at_host,sizeof(ident_at_host));

	/* Clean the delay in our Duration */
	expire = in_expiretime == 0 ? 0 : (time(NULL)+(in_expiretime-in_hubtime));
	
	/* Get the Zlines setter */
	sscanf(nick,"%[^!]!%s",addedby,ident_at_host);
	
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_SJOIN,SQL_BUFFER_SJOIN); /* Flush Inserts - Since on serverlink or not; the SJOIN
																Event comes before TKL! */

	domysql(thread,"INSERT INTO `orion_core_lines` (`type`,`mask`,`addedby`,`duration`,`reason`) VALUES ('Z','%s','%s','%ld','%s') ON DUPLICATE KEY UPDATE "
	"`addedby`=VALUES(`addedby`),"
	"`duration`=VALUES(`duration`),"
	"`reason`=VALUES(`reason`)"
	,mysql_quote(thread,ipmask),mysql_quote(thread,addedby),expire,mysql_quote(thread,in_reason));
	
	if(streqnc(addedby,O->nick) || streqnc(addedby,SC->nick) || streqnc(addedby,Q->nick))
		domysql(thread,"INSERT INTO `orion_os_szlines` (`zline`,`duration`,`addedby`,`reason`) VALUES ('%s','%ld','%s','%s') ON DUPLICATE KEY UPDATE "
		"`duration`=VALUES(`duration`),"
		"`addedby`=VALUES(`addedby`),"
		"`reason`=VALUES(`reason`)"
		,mysql_quote(thread,ipmask),expire,mysql_quote(thread,addedby),mysql_quote(thread,in_reason));

	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */

/**
 * @brief on_unszline:  Receives an UNZLINE (depending on the Loaded Protocol)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param servername The ServerName (services-server) which sends the ungline information
 * @param ipmask The Ip which we like unzline
 * @return PROTOCOL_BOOL
 **/
PROTOCOL_BOOL on_unszline(Threads *thread, char *servername, char *ipmask){
	/* Declare variables */
	deepSQL getSQL;
	
	/* Initialize variables to NULL */
	bzero(&getSQL,sizeof(getSQL));
	
	if(DuringServerlink)
		MySQL_FlushInserts(thread,SQL_BUFFER_SJOIN,SQL_BUFFER_SJOIN); /* Flush Inserts - Since on serverlink or not; the SJOIN
																Event comes before TKL! */
																
	domysql(thread,"DELETE FROM `orion_core_lines` WHERE `type`='Z' AND `mask`='%s'",mysql_quote(thread,ipmask));
	
	/* MySQL */
	getSQL.res    = doSelectMySQL(thread,"SELECT `zline`,`addedby`,`duration`,`reason` FROM `orion_os_szlines` WHERE `zline`='%s'",mysql_quote(thread,ipmask));
	getSQL.fields = mysql_num_rows(getSQL.res);
	getSQL.row 	  = mysql_fetch_row(getSQL.res);
	
	if(streqnc(getSQL.row[1],O->nick))
		do_szline(thread,m->servername,getSQL.row[0],_dynadd("%s!%s@%s",O->nick,O->ident,O->host),stol(getSQL.row[2]),time(NULL),getSQL.row[3]);
	
	else if(streqnc(getSQL.row[1],Q->nick))
		do_szline(thread,m->servername,getSQL.row[0],_dynadd("%s!%s@%s",Q->nick,Q->ident,Q->host),stol(getSQL.row[2]),time(NULL),getSQL.row[3]);
	
	else if(streqnc(getSQL.row[1],SC->nick))
		do_szline(thread,m->servername,getSQL.row[0],_dynadd("%s!%s@%s",SC->nick,SC->ident,SC->host),stol(getSQL.row[2]),time(NULL),getSQL.row[3]);

	mysql_free_result_myr(getSQL.res);
	return CMD_EXECUTED;
}

/*  ************************************************************************************************  */
