/*  ************************************************************************************************
 *
 *  Copyright (C) 2004-2009
 *      Manuel Moser (alias: shock), Email: shock@deepirc.net
 *      Christian Liebig (alias: cLiEbiG), Email: cLiEbiG@deepirc.net
 * 
 *  ************************************************************************************************
 *
 *  Please read COPYING and README for further details.
 *  Based on the original code of Services by Andy Church. 
 *
 *  This program is free software; you can redistribute it and/or modify it under the terms of 
 *  the GNU General Public License as published by the Free Software Foundation; either version 3 
 *  of the License, or (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See 
 *  the GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License along with this program; 
 *  if not, see <http://www.gnu.org/licenses/>. 
 *
 *  ************************************************************************************************
 * 
 *  SVN Status Information:
 *
 *      $Rev:: 202                   $:	Rivision of last commit
 *      $Author:: shockzZ            $:	Author of last commit
 *      $Date:: 2009-01-24 17:29:41 #$:	Date of last commit
 *
 *  ************************************************************************************************
 * 
 *  The following part is important for the Doxygen Documentation:
 */
 /** @file mysql.c
 *   @brief This is the Module file for communication with the MySQL-Database.
 *
 *   It contains all functions which are neccessary to
 *   control the MySQL-Database.
 *
 *	 Manages the Database. (Select/Insert/Update/Delete/Truncate/Reset/Flush)
 */
/*  ************************************************************************************************  */

#include "datatypes.h"
#include "global.h"
#include "services.h"
#include "modules.h"
#include "osocket.h"

MYSQL mysql;              /* MySQL Handler */
MYSQL_FIELD *fields;      /* MySQL Fields  */
MYSQL_RES *socks_res;     /* MySQL Result  */
MYSQL_ROW socks_row;      /* MySQL Row     */

MySQLBuf MySQLSave[MAX_BUFFER_TABLES];

/**
 * @brief mysql_connect:  Opens a connection to the MySQL-Server
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void mysql_connect(Threads *thread){
   	mysql_init(MYSQL_CONNECTION_HANDLE);
   	
#ifdef USE_THREADS
   	my_pthread_mutex_lock(&mysql_lock);
#endif
	if(mysql_real_connect(MYSQL_CONNECTION_HANDLE,m->mysqlsrv,m->mysqluser,m->mysqlpass,m->mysqlDB,m->mysqlport,m->mysqlsocket,0)==NULL){
		fprintf (stderr, "  * ERROR: Failed to connect through mysql_real_connect():"
			"%u (%s)\n",mysql_errno (MYSQL_CONNECTION_HANDLE), mysql_error (MYSQL_CONNECTION_HANDLE));
		exit(0);
	}else{
		printf(" [*] MySQL - Connected\n");
	}
	mysql_select_db(MYSQL_CONNECTION_HANDLE, m->mysqlDB);
	
#ifdef USE_THREADS
   	my_pthread_mutex_unlock(&mysql_lock);
#endif
	printf(" [*] Using DB: %s\n",m->mysqlDB);
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief mysql_disconnect:  Opens a connection to the MySQL-Server
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @return void
 **/
void mysql_disconnect(Threads *thread){
//#ifdef USE_THREADS
//	my_pthread_mutex_lock(&mysql_lock);
//#endif
	mysql_close(MYSQL_CONNECTION_HANDLE);
//#ifdef USE_THREADS
//	my_pthread_mutex_unlock(&mysql_lock);
//#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief mysql_free_result_myr:  Thread-Safe version of mysql_free_result (only Orion application)
 *
 * @param SQLres The MySQL Result pointer which we want to free.
 * @return void
 **/
void mysql_free_result_myr(SQLres *res){
	if(res==NULL)
	    return;

#ifdef USE_THREADS
	//my_pthread_mutex_lock(&mysql_lock);
	mysql_free_result(res);
	//my_pthread_mutex_unlock(&mysql_lock);
#else
	mysql_free_result(res);
#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief mysql_quote:  Quotes a string, so that it is understandable for MySQL
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param sql The string to be quoted, before sending it to the MySQL-Server
 * @return char* sqlQuote[QUOTES] - A Pointer to the last quoted element
 **/
char* mysql_quote(Threads *thread, char *sql){
	int i=0;
	size_t slen=0;

	thread->QUOTES++;
	if(thread->QUOTES>=SQL_MAX_ITEM_QUOTE){
		for(i=0;thread->QUOTES>=0 && i<=thread->QUOTES;i++){
			
#ifdef USE_THREADS
			my_pthread_mutex_lock(&globvar_lock);
#endif
			if(m->logquotes[0]!='\0')
				fprintf(quotelog,"Freeing Quote[%d] = \"%s\"\n",i,thread->sqlQuote[i]);
				
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&globvar_lock);
#endif
			sfree(thread->sqlQuote[i]);
		}
		thread->QUOTES=0;
	}
	if (!sql) {
		thread->QUOTES--;
		return '\0';
	}
	slen = strlen(sql);
	thread->sqlQuote[thread->QUOTES] = (char*)malloc((1 + (slen * 2)));
	mysql_escape_string(thread->sqlQuote[thread->QUOTES], sql, slen);
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&globvar_lock);
#endif
	if(m->logquotes[0]!='\0')
		fprintf(quotelog,"Added Quote[%d] = \"%s\"\n",thread->QUOTES,thread->sqlQuote[thread->QUOTES]);
		
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&globvar_lock);
#endif
	return thread->sqlQuote[thread->QUOTES];
}

/*  ************************************************************************************************  */

/**
 * @brief mysql_free_quotes:  Frees all quoted Strings
 *
 * @param thread The Thread for which we like to free all quotes
 * @return void
 **/
void mysql_free_quotes(Threads *thread){
	int i=0;
	
	if(thread->QUOTES>=0){
		for(i=0;thread->QUOTES>=0 && i<=thread->QUOTES;i++){
#ifdef USE_THREADS
			my_pthread_mutex_lock(&globvar_lock);
#endif
			if(m->logquotes[0]!='\0')
				fprintf(quotelog,"Freeing Quote[%d] = \"%s\"\n",i,thread->sqlQuote[i]);
				
#ifdef USE_THREADS
			my_pthread_mutex_unlock(&globvar_lock);
#endif
			sfree(thread->sqlQuote[i]);
		}
		thread->QUOTES=-1;
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief ExecuteQuery:  Executes a query
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param query The query that you want to execute (never use this function your own)
 *                 other functions attempt to use it for executing the built query
 * @param len The length of the query that you want to execute
 * @return void
 **/
void ExecuteQuery(Threads *thread, char *query, size_t len){
	sqllog(thread,"qry: %s\n",query);
	if(m->debug>=4)
		printf("qry: %s\n",query);
		
#ifdef USE_THREADS	
	if(mysql_query(MYSQL_CONNECTION_HANDLE, query)){ //executing Query
#else
	if(mysql_real_query(MYSQL_CONNECTION_HANDLE, query, len)){ //executing Query
#endif
		dlog(thread,0,mysql_error(MYSQL_CONNECTION_HANDLE));
		do_backtrace(thread,0,10);
		
		_orion_exit();
	}
}

/*  ************************************************************************************************  */

/**
 * @brief ExecuteQuery_r:  Executes a query (Thread Safe)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param query The query that you want to execute (never use this function your own)
 *                 other functions attempt to use it for executing the built query
 * @param len The length of the query that you want to execute
 * @return void
 **/
void ExecuteQuery_r(Threads *thread, char *query, size_t len){
	sqllog(thread,"qry: %s\n",query);
	if(m->debug>=4)
		printf("qry: %s\n",query);
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&mysql_lock);
#endif
#ifdef USE_THREADS
	if(mysql_query(MYSQL_CONNECTION_HANDLE, query)){ //executing Query
#else
	if(mysql_real_query(MYSQL_CONNECTION_HANDLE, query, len)){ //executing Query
#endif
		dlog(thread,0,mysql_error(MYSQL_CONNECTION_HANDLE));
		do_backtrace(thread,0,10);
		
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&mysql_lock);
#endif
		_orion_exit();
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&mysql_lock);
#endif
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief InlineExecuteQuery_r:  Executes a query (Thread Safe)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param query The query that you want to execute (never use this function your own)
 *                 other functions attempt to use it for executing the built query
 * @param len The length of the query that you want to execute
 * @return char* mysql_error
 **/
char *InlineExecuteQuery_r(Threads *thread, char *query, size_t len){
	sqllog(thread,"qry: %s\n",query);
	if(m->debug>=4)
		printf("qry: %s\n",query);
	
#ifdef USE_THREADS
	my_pthread_mutex_lock(&mysql_lock);
#endif
#ifdef USE_THREADS
	if(mysql_query(MYSQL_CONNECTION_HANDLE, query)){ //executing Query
#else
	if(mysql_real_query(MYSQL_CONNECTION_HANDLE, query, len)){ //executing Query
#endif
#ifdef USE_THREADS
		my_pthread_mutex_unlock(&mysql_lock);
#endif
		return (char*)mysql_error(MYSQL_CONNECTION_HANDLE);
	}
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&mysql_lock);
#endif
	return NULL;
}

/*  ************************************************************************************************  */

/**
 * @brief sql_mainAAC:  Selects all common information about
 *                  a single user (aliases, access, condata)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information an the SQL Struct
 * @param nick The nick (normal user nick) for the internal SELECT query
 * @return void
 **/
void sql_mainAAC(Threads *thread, char* nick){ //main Aliases Access Condata
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/

	char nickdata[MAX_NICK_LEN*2], query[SQL_BUF];
                                                
	thread->sql.access.fields=0;
	thread->sql.access.authnick[0]='\0';
	thread->sql.access.password[0]='\0';
	thread->sql.access.reason[0]='\0';
	thread->sql.access.qlevel=0;
	thread->sql.access.olevel=0;
	thread->sql.access.slevel=0;
	thread->sql.access.llevel=0;
	thread->sql.access.email[0]='\0';
	thread->sql.access.greet[0]='\0';
	thread->sql.access.url[0]='\0';
	thread->sql.access.icq=0;
	thread->sql.access.yahoo[0]='\0';
	thread->sql.access.location[0]='\0';
	thread->sql.access.birthday[0]='\0';
	thread->sql.access.realname[0]='\0';
	thread->sql.access.gender=0;
	thread->sql.access.interests[0]='\0';
	thread->sql.access.profession[0]='\0';
	thread->sql.access.slogan[0]='\0';
	thread->sql.access.language=m->language;
	thread->sql.access.rights=is_user;
	thread->sql.access.settings=0;
	//thread->sql.access.hide=0;
	//thread->sql.access.secure=0;
	//thread->sql.access.privatenick=0;
	//thread->sql.access.settings&NS_SET_MSG ? 1 : 0=0;
	thread->sql.access.maxmemos=0;
	thread->sql.access.failed_logins=0;
	thread->sql.access.last_failed_login_time=(time_t)0;
	thread->sql.access.last_failed_login_password[0]='\0';
	
	thread->sql.aliases.fields=0;
	thread->sql.aliases.nick[0]='\0';
	thread->sql.aliases.authnick[0]='\0';
	thread->sql.aliases.last_vhost[0]='\0';
	thread->sql.aliases.last_realname[0]='\0';
	thread->sql.aliases.last_quit[0]='\0';
	thread->sql.aliases.time_linked=(time_t)0;
	thread->sql.aliases.last_login=(time_t)0;
	thread->sql.aliases.last_used=(time_t)0;
	thread->sql.aliases.active=0;
	thread->sql.aliases.expire=0;
	thread->sql.aliases.forbid=0;
	thread->sql.aliases.linked=1;
	
	thread->sql.Cdata.fields=0;
	thread->sql.Cdata.nick[0]='\0';
	thread->sql.Cdata.time=(time_t)0;
	thread->sql.Cdata.localtime=(time_t)0;
	thread->sql.Cdata.rident[0]='\0';
	thread->sql.Cdata.ident[0]='\0';
	thread->sql.Cdata.rhost[0]='\0';
	thread->sql.Cdata.server[0]='\0';
	thread->sql.Cdata.modes[0]='\0';
	thread->sql.Cdata.vhost[0]='\0';
	thread->sql.Cdata.realname[0]='\0';
	thread->sql.Cdata.swhois[0]='\0';
	thread->sql.Cdata.subnet[0]='\0';
	thread->sql.Cdata.ip[0]='\0';
	thread->sql.Cdata.ns_warntime=(time_t)0;
                                                
	if(nick==NULL) return;
	
	bzero(nickdata,sizeof(nickdata));
	bzero(query,sizeof(query));
	
	/*
	 Aliases x
	 Access y
	 Condata z
	*/
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, nickdata, nick, strlen(nick)); //escape authnick -> nickdata
//	sprintf(query, "SELECT x.nick, `authnick` , `last_usermask` , `last_realname` , `last_quit` , `time_linked` , `last_login` , `failed_logins` , `last_failed_login` , `last_login_password` , x.expire, `linked` , `active` , x.forbid , x.suspend , `password` , `time_registered` , `qlevel` , `olevel` , `slevel` , `llevel` , `email` , `greet` , `url` , `icq` , `yahoo` , `location` , `birthday`, y.realname , `gender` , `interests` ,`profession` , `slogan` , `language` , `rights` , `kill` , `set_kill` , `hide` , `secure` , `private` , `msg` , y.suspend , `maxmemos` , `time` , `ident` , `rhost` , `srv` , `modes` , `vhost` , z.realname, `swhois` , `subnet` , `ns_warntime` FROM orion_ns_aliases x, orion_ns_core y, orion_core_condata z WHERE (x.nick='%s' OR x.nick='NULL') AND (y.authnick=x.display OR y.authnick=NULL) AND (z.nick='%s' OR z.nick='NULL')",nickdata,nickdata); //prepare query
	snprintf(query,sizeof(query), "SELECT x.nick, y.nick, `authnick` , `last_usermask` , `last_realname` , `last_quit` , `time_linked` , `last_login` , `last_used` , `failed_logins` , `last_failed_login` , `last_login_password` , y.expire, `linked` , `active` , y.forbid , `password` , `reason` , `qlevel` , `olevel` , `slevel` , `llevel` , `email` , `greet` , `url` , `icq` , `yahoo` , `location` , `birthday`, z.realname , `gender` , `interests` ,`profession` , `slogan` , `language` , `rights` , `settings` , `maxmemos` , `time`, `localtime` , `rident` , `ident` , `rhost` , `srv` , `modes` , `vhost` , x.realname, `swhois` , `subnet` , `ip` , `ns_warntime` FROM orion_core_condata x LEFT OUTER JOIN orion_ns_aliases y LEFT OUTER JOIN orion_ns_core z ON (z.`authnick`=y.`display`) ON (y.`nick`=x.`nick`) WHERE x.nick='%s'",nickdata); //prepare query
	
	dlog(thread,4,"SQL: %s",query);
	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to MYSQL result pointer
	
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res); //fetch result to the MYSQL **row array
		thread->sql.access.fields=mysql_num_fields(res); //get number of Array entries
		thread->sql.aliases.fields=thread->sql.access.fields; //get number of Array entries
		thread->sql.Cdata.fields=thread->sql.access.fields; //get number of Array entries
		
		if(row[0]!=NULL) dynncpy(thread->sql.Cdata.nick,row[0],sizeof(thread->sql.Cdata.nick));
		if(row[1]!=NULL) dynncpy(thread->sql.aliases.nick,row[1],sizeof(thread->sql.aliases.nick));
		
		if(row[2]!=NULL) dynncpy(thread->sql.access.authnick,row[2],sizeof(thread->sql.access.authnick));
		if(row[2]!=NULL) dynncpy(thread->sql.aliases.authnick,thread->sql.access.authnick,sizeof(thread->sql.aliases.authnick));
		
		if(row[3]!=NULL) dynncpy(thread->sql.aliases.last_vhost,row[3],sizeof(thread->sql.aliases.last_vhost));
		if(row[4]!=NULL) dynncpy(thread->sql.aliases.last_realname,row[4],sizeof(thread->sql.aliases.last_realname));
		if(row[5]!=NULL) dynncpy(thread->sql.aliases.last_quit,row[5],sizeof(thread->sql.aliases.last_quit));
		if(row[6]!=NULL) thread->sql.aliases.time_linked=(time_t)stol(row[6]);
		if(row[7]!=NULL) thread->sql.aliases.last_login=(time_t)stol(row[7]);
		if(row[8]!=NULL) thread->sql.aliases.last_used=(time_t)stol(row[8]);
		/* Moved failed logins to access! */
		if(row[9]!=NULL) thread->sql.access.failed_logins=stoi(row[9]);
		if(row[10]!=NULL) thread->sql.access.last_failed_login_time=(time_t)stol(row[10]);
		if(row[11]!=NULL) dynncpy(thread->sql.access.last_failed_login_password,row[11],sizeof(thread->sql.access.last_failed_login_password));
		/**********************************/
		if(row[12]!=NULL) thread->sql.aliases.expire=stoi(row[12]);
		if(row[13]!=NULL) thread->sql.aliases.linked=stoi(row[13]); //0-permanent, 1-temporarily
		if(row[14]!=NULL) thread->sql.aliases.active=stoi(row[14]);
		if(row[15]!=NULL) thread->sql.aliases.forbid=stoi(row[15]);
		
		if(row[16]!=NULL) dynncpy(thread->sql.access.password,row[16],sizeof(thread->sql.access.password));
		if(row[17]!=NULL) dynncpy(thread->sql.access.reason,row[17],sizeof(thread->sql.access.reason));
		if(row[18]!=NULL) thread->sql.access.qlevel=stoi(row[18]);
		if(row[19]!=NULL) thread->sql.access.olevel=stoi(row[19]);
		if(row[20]!=NULL) thread->sql.access.slevel=stoi(row[20]);
		if(row[21]!=NULL) thread->sql.access.llevel=stoi(row[21]);
		if(row[22]!=NULL) dynncpy(thread->sql.access.email,row[22],sizeof(thread->sql.access.email));
		if(row[23]!=NULL) dynncpy(thread->sql.access.greet,row[23],sizeof(thread->sql.access.greet));
		if(row[24]!=NULL) dynncpy(thread->sql.access.url,row[24],sizeof(thread->sql.access.url));
		if(row[25]!=NULL) thread->sql.access.icq=stol(row[25]);
		if(row[26]!=NULL) dynncpy(thread->sql.access.yahoo,row[26],sizeof(thread->sql.access.yahoo));
		if(row[27]!=NULL) dynncpy(thread->sql.access.location,row[27],sizeof(thread->sql.access.location));
		if(row[28]!=NULL) dynncpy(thread->sql.access.birthday,row[28],sizeof(thread->sql.access.birthday));
		if(row[29]!=NULL) dynncpy(thread->sql.access.realname,row[29],sizeof(thread->sql.access.realname));
		if(row[30]!=NULL) thread->sql.access.gender=stoi(row[30]);
		if(row[31]!=NULL) dynncpy(thread->sql.access.interests,row[31],sizeof(thread->sql.access.interests));
		if(row[32]!=NULL) dynncpy(thread->sql.access.profession,row[32],sizeof(thread->sql.access.profession));
		if(row[33]!=NULL) dynncpy(thread->sql.access.slogan,row[33],sizeof(thread->sql.access.slogan));
		if(row[34]!=NULL) thread->sql.access.language=stoi(row[34]);
		if(row[35]!=NULL) thread->sql.access.rights=stoi(row[35]);
		if(row[36]!=NULL) thread->sql.access.settings=stoi(row[36]);
		if(row[37]!=NULL) thread->sql.access.maxmemos=stoi(row[37]);
		
		if(row[38]!=NULL) thread->sql.Cdata.time=(time_t)stol(row[38]);
		if(row[39]!=NULL) thread->sql.Cdata.localtime=(time_t)stol(row[39]);
		if(row[40]!=NULL) dynncpy(thread->sql.Cdata.rident,row[40],sizeof(thread->sql.Cdata.rident));
		if(row[41]!=NULL) dynncpy(thread->sql.Cdata.ident,row[41],sizeof(thread->sql.Cdata.ident));
		if(row[42]!=NULL) dynncpy(thread->sql.Cdata.rhost,row[42],sizeof(thread->sql.Cdata.rhost));
		if(row[43]!=NULL) dynncpy(thread->sql.Cdata.server,row[43],sizeof(thread->sql.Cdata.server));
		if(row[44]!=NULL) dynncpy(thread->sql.Cdata.modes,row[44],sizeof(thread->sql.Cdata.modes));
		if(row[45]!=NULL) dynncpy(thread->sql.Cdata.vhost,row[45],sizeof(thread->sql.Cdata.vhost));
		if(row[46]!=NULL) dynncpy(thread->sql.Cdata.realname,row[46],sizeof(thread->sql.Cdata.realname));
		if(row[47]!=NULL) dynncpy(thread->sql.Cdata.swhois,row[47],sizeof(thread->sql.Cdata.swhois));
		if(row[48]!=NULL) dynncpy(thread->sql.Cdata.subnet,row[48],sizeof(thread->sql.Cdata.subnet));
		if(row[49]!=NULL) dynncpy(thread->sql.Cdata.ip,row[49],sizeof(thread->sql.Cdata.ip));
		
		if(row[50]!=NULL) thread->sql.Cdata.ns_warntime=(time_t)stol(row[50]);
	}
	mysql_free_result_myr(res); //free result
	return; //return access struct
}

/*  ************************************************************************************************  */

/**
 * @brief sql_chan_db:  Selects all information for
 *                  a channel (levels, channel, bs-settings)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The channel for which we like the information
 * @return ChannelDB The Struct wirh channeldata from the Channel-Database
 **/
ChannelDB sql_chan_db(Threads *thread, char* channel){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
	char query[SQL_BUF], chandata[MAX_CHANNELNAME_LEN*2];
        
	/* STRUCT */ ChannelDB c; /* STRUCT */
	bzero(&c,sizeof(c));
	
	
	c.fields=0;
	c.name[0]='\0';
	c.bot[0]='\0';
	c.serv[0]='\0';
	c.founder[0]='\0';
	c.successor[0]='\0';
	c.datecreate=(time_t)0;
	c.regdate=(time_t)0;
	c.description[0]='\0';
	c.mode[0]='\0';
	c.mlock_positive[0]='\0';
	c.mlock_negative[0]='\0';
	c.mlock_limit=0;
	c.mlock_key[0]='\0';
	c.mlock_flood[0]='\0';
	c.mlock_redirect[0]='\0';
	c.link[0]='\0';
	c.limit=0;
	c.key[0]='\0';
	c.flood[0]='\0';
	c.last_topic[0]='\0';
	c.last_topic_setter[0]='\0';
	c.last_topic_time=(time_t)0;
	c.url[0]='\0';
	c.alimit=0;
	c.flags[0]='\0';
	c.welcome[0]='\0';
	c.last_used=(time_t)0;
	c.reason[0]='\0';
	c.settings=0;
	c.email[0]='\0';
	c.expire=1;
	c.maxmemos=0;

	c.cs_accchange=0;
	c.cs_acclist=0;
	c.cs_akick=0;
	c.cs_autodeop=0;
	c.cs_autohalfop=0;
	c.cs_autoop=0;
	c.cs_autoprotect=0;
	c.cs_autovoice=0;
	c.cs_badwords=0;
	c.cs_ban=0;
	c.cs_banme=0;
	c.cs_assign=0;
	c.cs_clear=0;
	c.cs_fantasia=0;
	c.cs_getkey=0;
	c.cs_greet=0;
	c.cs_halfop=0;
	c.cs_halfopme=0;
	c.cs_info=0;
	c.cs_invite=0;
	c.cs_kick=0;
	c.cs_kickme=0;
	c.cs_memo=0;
	c.cs_nojoin=0;
	c.cs_nokick=0;
	c.cs_opdeop=0;
	c.cs_opdeopme=0;
	c.cs_protect=0;
	c.cs_protectme=0;
	c.cs_say=0;
	c.cs_set=0;
	c.cs_signkick=0;
	c.cs_topic=0;
	c.cs_unban=0;
	c.cs_voice=0;
	c.cs_voiceme=0;
	
	c.bs_set=-1;
	c.bs_caps_ttb=-1;
	c.bs_caps_min=-1;
	c.bs_caps_per=-1;
	c.bs_flood_ttb=-1;
	c.bs_flood_ln=-1;
	c.bs_flood_secs=-1;
	c.bs_repeat_ttb=-1;
	c.bs_repeat=-1;
	c.bs_colors=-1;
	c.bs_underlines=-1;
	c.bs_reverses=-1;
	c.bs_badwords=-1;
	c.bs_bolds=-1;
	
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, chandata, channel, strlen(channel)); //escape channel -> chandata
	snprintf(query,sizeof(query), 

	"SELECT "
	
	"`channel`,"
	"`botnick`,"
	"`serv`,"
	"`founder`,"
	"`successor`,"
	"`email`,"
	"`settings`,"
	"`regdate`,"
	"`datecreate`,"
	"`description`,"
	"`mode`,"
	"`mlock_positive`,"
	"`mlock_negative`,"
	"`mlock_limit`,"
	"`mlock_key`,"
	"`mlock_flood`,"
	"`mlock_redirect`,"
	"`link`,"
	"`limit`,"
	"`key`,"
	"`flood`,"
	"`last_topic`,"
	"`last_topic_setter`,"
	"`last_topic_time`,"
	"`url`,"
	"`alimit`,"
	"`flags`,"
	"`welcome`,"
	"`last_used`,"
	"`expire`,"
	"`maxmemos`,"
	"`reason`,"
	
	"`cs_set`,"
	"`cs_autodeop`,"
	"`cs_autohalfop`,"
	"`cs_autoop`,"
	"`cs_autoprotect`,"
	"`cs_autovoice`,"
	"`cs_nojoin`,"
	"`cs_signkick`,"
	"`cs_acclist`,"
	"`cs_accchange`,"
	"`cs_akick`,"
	"`cs_ban`,"
	"`cs_banme`,"
	"`cs_cclear`,"
	"`cs_getkey`,"
	"`cs_halfop`,"
	"`cs_halfopme`,"
	"`cs_info`,"
	"`cs_kick`,"
	"`cs_kickme`,"
	"`cs_invite`,"
	"`cs_opdeop`,"
	"`cs_opdeopme`,"
	"`cs_protect`,"
	"`cs_protectme`,"
	"`cs_topic`,"
	"`cs_unban`,"
	"`cs_voice`,"
	"`cs_voiceme`,"
	"`cs_memo`,"
	"`cs_botassign`,"
	"`cs_badwords`,"
	"`cs_fantasia`,"
	"`cs_greet`,"
	"`cs_nokick`,"
	"`cs_say`,"
	
	"`bs_set`,"
	"`bs_caps_ttb`,"
	"`bs_caps_min`,"
	"`bs_caps_per`,"
	"`bs_flood_ttb`,"
	"`bs_flood_ln`,"
	"`bs_flood_secs`,"
	"`bs_repeat_ttb`,"
	"`bs_repeat`,"
	"`bs_colors`,"
	"`bs_underlines`,"
	"`bs_reverses`,"
	"`bs_badwords`,"
	"`bs_bolds`"

	"FROM `orion_cs_core` WHERE `channel`='%s' LIMIT 0,1"
	
	,chandata);

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to MYSQL result pointer
	c.fields=mysql_num_rows(res);

	//printf("fields = %d\n",c.fields);
	if(c.fields){
		row=mysql_fetch_row(res); //

		if(row[0]!=NULL) dynncpy(c.name,row[0],sizeof(c.name));
		if(row[1]!=NULL) dynncpy(c.bot,row[1],sizeof(c.bot));
		if(row[2]!=NULL) dynncpy(c.serv,row[2],sizeof(c.serv));
		if(row[3]!=NULL) dynncpy(c.founder,row[3],sizeof(c.founder));
		if(row[4]!=NULL) dynncpy(c.successor,row[4],sizeof(c.successor));
		if(row[5]!=NULL) dynncpy(c.email,row[5],sizeof(c.email));
		if(row[6]!=NULL) c.settings=stoi(row[6]);
		if(row[7]!=NULL) c.regdate=(time_t)stol(row[7]);
		if(row[8]!=NULL) c.datecreate=(time_t)stol(row[8]);
		if(row[9]!=NULL) dynncpy(c.description,row[9],sizeof(c.description));
		if(row[10]!=NULL) dynncpy(c.mode,row[10],sizeof(c.mode));
		if(row[11]!=NULL) dynncpy(c.mlock_positive,row[11],sizeof(c.mlock_positive));
		if(row[12]!=NULL) dynncpy(c.mlock_negative,row[12],sizeof(c.mlock_negative));
		if(row[13]!=NULL) c.mlock_limit=stol(row[13]);
		if(row[14]!=NULL) dynncpy(c.mlock_key,row[14],sizeof(c.mlock_key));
		if(row[15]!=NULL) dynncpy(c.mlock_flood,row[15],sizeof(c.mlock_flood));
		if(row[16]!=NULL) dynncpy(c.mlock_redirect,row[16],sizeof(c.mlock_redirect));
		
		if(row[17]!=NULL) dynncpy(c.link,row[17],sizeof(c.link));
		if(row[18]!=NULL) c.limit=stol(row[18]);
		if(row[19]!=NULL) dynncpy(c.key,row[19],sizeof(c.key));
		if(row[20]!=NULL) dynncpy(c.flood,row[20],sizeof(c.flood));
		if(row[21]!=NULL) dynncpy(c.last_topic,row[21],sizeof(c.last_topic));
		if(row[22]!=NULL) dynncpy(c.last_topic_setter,row[22],sizeof(c.last_topic_setter));
		if(row[23]!=NULL) c.last_topic_time=(time_t)stol(row[23]);
		if(row[24]!=NULL) dynncpy(c.url,row[24],sizeof(c.url));
		if(row[25]!=NULL) c.alimit=stol(row[25]);
		if(row[26]!=NULL) dynncpy(c.flags,row[26],sizeof(c.flags));
		if(row[27]!=NULL) dynncpy(c.welcome,row[27],sizeof(c.welcome));
		if(row[28]!=NULL) c.last_used=(time_t)stol(row[28]);
		if(row[29]!=NULL) c.expire=stoi(row[29]);
		if(row[30]!=NULL) c.maxmemos=stoi(row[30]);
		if(row[31]!=NULL) dynncpy(c.reason,row[31],sizeof(c.reason));
		
		if(row[32]!=NULL) c.cs_accchange=stoi(row[32]);
		if(row[33]!=NULL) c.cs_acclist=stoi(row[33]);
		if(row[34]!=NULL) c.cs_akick=stoi(row[34]);
		if(row[35]!=NULL) c.cs_autodeop=stoi(row[35]);
		if(row[36]!=NULL) c.cs_autohalfop=stoi(row[36]);
		if(row[37]!=NULL) c.cs_autoop=stoi(row[37]);
		if(row[38]!=NULL) c.cs_autoprotect=stoi(row[38]);
		if(row[39]!=NULL) c.cs_autovoice=stoi(row[39]);
		if(row[40]!=NULL) c.cs_badwords=stoi(row[40]);
		if(row[41]!=NULL) c.cs_ban=stoi(row[41]);
		if(row[42]!=NULL) c.cs_banme=stoi(row[42]);
		if(row[43]!=NULL) c.cs_assign=stoi(row[43]);
		if(row[44]!=NULL) c.cs_clear=stoi(row[44]);
		if(row[45]!=NULL) c.cs_fantasia=stoi(row[45]);
		if(row[46]!=NULL) c.cs_getkey=stoi(row[46]);
		if(row[47]!=NULL) c.cs_greet=stoi(row[47]);
		if(row[48]!=NULL) c.cs_halfop=stoi(row[48]);
		if(row[49]!=NULL) c.cs_halfopme=stoi(row[49]);
		if(row[50]!=NULL) c.cs_info=stoi(row[50]);
		if(row[51]!=NULL) c.cs_invite=stoi(row[51]);
		if(row[52]!=NULL) c.cs_kick=stoi(row[52]);
		if(row[53]!=NULL) c.cs_kickme=stoi(row[53]);
		if(row[54]!=NULL) c.cs_memo=stoi(row[54]);
		if(row[55]!=NULL) c.cs_nojoin=stoi(row[55]);
		if(row[56]!=NULL) c.cs_nokick=stoi(row[56]);
		if(row[57]!=NULL) c.cs_opdeop=stoi(row[57]);
		if(row[58]!=NULL) c.cs_opdeopme=stoi(row[58]);
		if(row[59]!=NULL) c.cs_protect=stoi(row[59]);
		if(row[60]!=NULL) c.cs_protectme=stoi(row[60]);
		if(row[61]!=NULL) c.cs_say=stoi(row[61]);
		if(row[62]!=NULL) c.cs_set=stoi(row[62]);
		if(row[63]!=NULL) c.cs_signkick=stoi(row[63]);
		if(row[64]!=NULL) c.cs_topic=stoi(row[64]);
		if(row[65]!=NULL) c.cs_unban=stoi(row[65]);
		if(row[66]!=NULL) c.cs_voice=stoi(row[66]);
		if(row[67]!=NULL) c.cs_voiceme=stoi(row[67]);
			
		if(row[68]!=NULL) c.bs_set=stoi(row[68]);
		if(row[69]!=NULL) c.bs_caps_ttb=stoi(row[69]);
		if(row[70]!=NULL) c.bs_caps_min=stoi(row[70]);
		if(row[71]!=NULL) c.bs_caps_per=stoi(row[71]);
		
		if(row[72]!=NULL) c.bs_flood_ttb=stoi(row[72]);
		if(row[73]!=NULL) c.bs_flood_ln=stoi(row[73]);
		if(row[74]!=NULL) c.bs_flood_secs=stoi(row[74]);
		
		if(row[75]!=NULL) c.bs_repeat_ttb=stoi(row[75]);
		if(row[76]!=NULL) c.bs_repeat=stoi(row[76]);
		if(row[77]!=NULL) c.bs_colors=stoi(row[77]);
		if(row[78]!=NULL) c.bs_underlines=stoi(row[78]);
		
		if(row[79]!=NULL) c.bs_reverses=stoi(row[79]);
		if(row[80]!=NULL) c.bs_badwords=stoi(row[80]);
		if(row[81]!=NULL) c.bs_bolds=stoi(row[81]);
	}
	mysql_free_result_myr(res);
	return c;
}

/*  ************************************************************************************************  */

/**
 * @brief sql_chan_data:  Selects information for a channel
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param starttype The parameter whith which we like to start
 * @param parameters The number of different information which should be returned
 * @param ... following types are allowed: CHANNEL,WITH_LEVELS,WITH_BOTSERV,COUNTUSERS,OWNACCLEVEL,TARGETACCLEVEL
 * @return DB3 The Struct wirh channeldata from the Channel-Database
 **/
DB3 sql_chan_data(Threads *thread, char* starttype, int parameters, ...){
	int i=0,j=0,adddata=0,fields=0,param_now=0,join_aliases=0;
	int start_from[MAX_CHANDATA_TYPES],needed_param[MAX_CHANDATA_TYPES],num_fields[MAX_CHANDATA_TYPES];
	char temp_fields[SQL_BUF],select_fields[SQL_BUF],select_from[SQL_BUF],select_join[SQL_BUF],select_where[SQL_BUF],nick[MAX_NICK_LEN],channel[MAX_CHANNELNAME_LEN];
	char* arg;
	deepSQL chandata;
	va_list options;
	/* STRUCT */ DB3 c; /* STRUCT */
	
	bzero(&select_fields,sizeof(select_fields));
	bzero(&select_from,sizeof(select_from));
	bzero(&select_join,sizeof(select_join));
	bzero(&select_where,sizeof(select_where));
	bzero(&nick,sizeof(nick));
	bzero(&channel,sizeof(channel));
	bzero(&c,sizeof(c));
	bzero(&start_from,sizeof(start_from));
	bzero(&needed_param,sizeof(needed_param));
	bzero(&num_fields,sizeof(num_fields));
	
	nick[0]='\0';
	channel[0]='\0';
	
	// reset the returning ChannelDB structure
	c.access.fields = 0;
	c.aliases.fields = 0;
	c.chan.fields = 0;
	c.Cdata.fields = 0;
	
	// build the SQL query...
	va_start(options, parameters);

	for(i=0;i<parameters;i++){
		adddata+=va_arg(options,int);
	}

	if(strcor(starttype,"#")){
		if(!(adddata&CHANNEL)) adddata+=CHANNEL;
		dynncpy(channel,starttype,sizeof(channel));
	}else{
		if(!(adddata&CONDATA)) adddata+=CONDATA;
		dynncpy(nick,starttype,sizeof(nick));
	}

	if(adddata&WITH_LEVELS){
		if(!(adddata&CHANNEL)) adddata+=CHANNEL;

		c.chan.levels.accchange=0;
		c.chan.levels.acclist=0;
		c.chan.levels.akick=0;
		c.chan.levels.autodeop=0;
		c.chan.levels.autohalfop=0;
		c.chan.levels.autoop=0;
		c.chan.levels.autoprotect=0;
		c.chan.levels.autovoice=0;
		c.chan.levels.badwords=0;
		c.chan.levels.ban=0;
		c.chan.levels.banme=0;
		c.chan.levels.assign=0;
		c.chan.levels.clear=0;
		c.chan.levels.fantasia=0;
		c.chan.levels.getkey=0;
		c.chan.levels.greet=0;
		c.chan.levels.halfop=0;
		c.chan.levels.halfopme=0;
		c.chan.levels.info=0;
		c.chan.levels.invite=0;
		c.chan.levels.kick=0;
		c.chan.levels.kickme=0;
		c.chan.levels.memo=0;
		c.chan.levels.nojoin=0;
		c.chan.levels.nokick=0;
		c.chan.levels.opdeop=0;
		c.chan.levels.opdeopme=0;
		c.chan.levels.protect=0;
		c.chan.levels.protectme=0;
		c.chan.levels.say=0;
		c.chan.levels.set=0;
		c.chan.levels.signkick=0;
		c.chan.levels.topic=0;
		c.chan.levels.unban=0;
		c.chan.levels.voice=0;
		c.chan.levels.voiceme=0;

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,
				"`orion_cs_core`.`cs_accchange`,`orion_cs_core`.`cs_acclist`,"
				"`orion_cs_core`.`cs_akick`,`orion_cs_core`.`cs_autodeop`,`orion_cs_core`.`cs_autohalfop`,`orion_cs_core`.`cs_autoop`,"
				"`orion_cs_core`.`cs_autoprotect`,`orion_cs_core`.`cs_autovoice`,`orion_cs_core`.`cs_badwords`,`orion_cs_core`.`cs_ban`,`orion_cs_core`.`cs_banme`,"
				"`orion_cs_core`.`cs_botassign`,`orion_cs_core`.`cs_cclear`,`orion_cs_core`.`cs_fantasia`,`orion_cs_core`.`cs_getkey`,`orion_cs_core`.`cs_greet`,`orion_cs_core`.`cs_halfop`,`orion_cs_core`.`cs_halfopme`,"
				"`orion_cs_core`.`cs_info`,`orion_cs_core`.`cs_invite`,`orion_cs_core`.`cs_kick`,`orion_cs_core`.`cs_kickme`,`orion_cs_core`.`cs_memo`,`orion_cs_core`.`cs_nojoin`,"
				"`orion_cs_core`.`cs_nokick`,`orion_cs_core`.`cs_opdeop`,`orion_cs_core`.`cs_opdeopme`,`orion_cs_core`.`cs_protect`,"
				"`orion_cs_core`.`cs_protectme`,`orion_cs_core`.`cs_say`,`orion_cs_core`.`cs_set`,`orion_cs_core`.`cs_signkick`,"
				"`orion_cs_core`.`cs_topic`,`orion_cs_core`.`cs_unban`,`orion_cs_core`.`cs_voice`,`orion_cs_core`.`cs_voiceme`",
		sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));

		//printf("CLevels fields = %d - 36\n",findchar(temp_fields,',')+1); // 36
		num_fields[WITH_LEVELS_ARR]=findchar(temp_fields,',')+1;
		
		start_from[WITH_LEVELS_ARR]=fields;
		fields+=num_fields[WITH_LEVELS_ARR];
	}
	if(adddata&WITH_BOTSERV){
		if(!(adddata&CHANNEL)) adddata+=CHANNEL;

		c.chan.botserv.set=-1;
		c.chan.botserv.caps_ttb=-1;
		c.chan.botserv.caps_min=-1;
		c.chan.botserv.caps_per=-1;
		c.chan.botserv.flood_ttb=-1;
		c.chan.botserv.flood_ln=-1;
		c.chan.botserv.flood_secs=-1;
		c.chan.botserv.repeat_ttb=-1;
		c.chan.botserv.repeat=-1;
		c.chan.botserv.colors=-1;
		c.chan.botserv.underlines=-1;
		c.chan.botserv.reverses=-1;
		c.chan.botserv.badwords=-1;
		c.chan.botserv.bolds=-1;

		//num_fields[WITH_BOTSERV_ARR]=14;
		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,
			"`orion_cs_core`.`bs_set`,`orion_cs_core`.`bs_caps_ttb`,`orion_cs_core`.`bs_caps_min`,`orion_cs_core`.`bs_caps_per`,`orion_cs_core`.`bs_flood_ttb`,`orion_cs_core`.`bs_flood_ln`,`orion_cs_core`.`bs_flood_secs`,`orion_cs_core`.`bs_repeat_ttb`,`orion_cs_core`.`bs_repeat`,`orion_cs_core`.`bs_colors`,"
			"`orion_cs_core`.`bs_underlines`,`orion_cs_core`.`bs_reverses`,`orion_cs_core`.`bs_badwords`,`orion_cs_core`.`bs_bolds`",
		sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("BotServ fields = %d - 14\n",findchar(temp_fields,',')+1); //14
		num_fields[WITH_BOTSERV_ARR]=findchar(temp_fields,',')+1;
		
		start_from[WITH_BOTSERV_ARR]=fields;
		fields+=num_fields[WITH_BOTSERV_ARR];
	}
	if(adddata&COUNTUSERS){
		if(!(adddata&CHANNEL)) adddata+=CHANNEL;

		c.chan.countusers = -1;
		c.chan.botinchan = 0;

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,"COUNT(`orion_chan_users`.`nick`) as `count_users`, SUM(IF(`orion_chan_users`.`nick`=`orion_cs_core`.`botnick`,1,0)) as `botinchan`",sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
				
		//printf("Countusers fields = %d - 2\n",findchar(temp_fields,',')+1-2); //2
		num_fields[COUNTUSERS_ARR]=findchar(temp_fields,',')+1-2;
		
		start_from[COUNTUSERS_ARR]=fields;
		fields+=num_fields[COUNTUSERS_ARR];
		needed_param[param_now++]=COUNTUSERS;
	}
	if(adddata&OWNACCLEVEL){
		if(!(adddata&CHANNEL)) adddata+=CHANNEL;

		c.chan.access.ownlevel[0]='\0';

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,"`own`.`level` as `ownacclevel`",sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("ownacclevel fields = %d - 1\n",findchar(temp_fields,',')+1); //1
		num_fields[OWNACCLEVEL_ARR]=findchar(temp_fields,',')+1;
		
		start_from[OWNACCLEVEL_ARR]=fields;
		fields+=num_fields[OWNACCLEVEL_ARR];
		needed_param[param_now++]=OWNACCLEVEL;
	}
	if(adddata&OWNACCLEVEL_FROM_NICK){
		c.chan.access.ownlevel[0]='\0';
		c.chan.access.ownauthnick[0]='\0';

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,"`own`.`authnick` as `authnick`,`own`.`level` as `ownacclevel`",sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("ownacclevel from nick fields = %d - 2\n",findchar(temp_fields,',')+1); //2
		num_fields[OWNACCLEVEL_FROM_NICK_ARR]=findchar(temp_fields,',')+1;
		
		start_from[OWNACCLEVEL_FROM_NICK_ARR]=fields;
		fields+=num_fields[OWNACCLEVEL_FROM_NICK_ARR];
		needed_param[param_now++]=OWNACCLEVEL_FROM_NICK;
	}
	if(adddata&TARGETACCLEVEL){
		c.chan.access.targetlevel[0]='\0';

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,"`target`.`level` as `targetlevel`",sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("Targetlevel fields = %d - 1\n",findchar(temp_fields,',')+1); //1
		num_fields[TARGETACCLEVEL_ARR]=findchar(temp_fields,',')+1;
		
		start_from[TARGETACCLEVEL_ARR]=fields;
		fields+=num_fields[TARGETACCLEVEL_ARR];
		needed_param[param_now++]=TARGETACCLEVEL;
	}
	if(adddata&ALIASES){
		c.aliases.nick[0]='\0';
		c.aliases.authnick[0]='\0';
		c.aliases.last_vhost[0]='\0';
		c.aliases.last_realname[0]='\0';
		c.aliases.last_quit[0]='\0';
		c.aliases.time_linked=(time_t)0;
		c.aliases.last_login=(time_t)0;
		c.aliases.last_used=(time_t)0;
		c.aliases.active=0;
		c.aliases.expire=0;
		c.aliases.forbid=0;
		c.aliases.linked=1;

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,"`orion_ns_aliases`.`nick`,`orion_ns_aliases`.`display`,`orion_ns_aliases`.`last_usermask`,`orion_ns_aliases`.`last_realname`,`orion_ns_aliases`.`last_quit`,`orion_ns_aliases`.`time_linked`,`orion_ns_aliases`.`last_login`,`orion_ns_aliases`.`last_used`,`orion_ns_aliases`.`expire`,`orion_ns_aliases`.`linked`,`orion_ns_aliases`.`active`,`orion_ns_aliases`.`forbid`",sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("Aliases fields = %d - 12\n",findchar(temp_fields,',')+1); //12
		num_fields[ALIASES_ARR]=findchar(temp_fields,',')+1;
		
		start_from[ALIASES_ARR]=fields;
		fields+=num_fields[ALIASES_ARR];
		needed_param[param_now++]=ALIASES;
	}
	if(adddata&ACCESS || adddata&ACCESS_FROM_NICK){
		int temp_arr = (adddata&ACCESS ? ACCESS_ARR : ACCESS_FROM_NICK_ARR);
		int temp = (adddata&ACCESS ? ACCESS : ACCESS_FROM_NICK);

		c.access.authnick[0]='\0';
		c.access.password[0]='\0';
		c.access.reason[0]='\0';
		c.access.qlevel=0;
		c.access.olevel=0;
		c.access.slevel=0;
		c.access.llevel=0;
		c.access.email[0]='\0';
		c.access.greet[0]='\0';
		c.access.url[0]='\0';
		c.access.icq=0;
		c.access.yahoo[0]='\0';
		c.access.location[0]='\0';
		c.access.birthday[0]='\0';
		c.access.realname[0]='\0';
		c.access.gender=0;
		c.access.interests[0]='\0';
		c.access.profession[0]='\0';
		c.access.slogan[0]='\0';
		c.access.language=m->language;
		c.access.rights=is_user;
		c.access.settings=0;
		c.access.maxmemos=0;
		c.access.failed_logins=0;
		c.access.last_failed_login_time=(time_t)0;
		c.access.last_failed_login_password[0]='\0';

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,"`orion_ns_core`.`authnick`,`orion_ns_core`.`password`,`orion_ns_core`.`reason`,`orion_ns_core`.`qlevel`,`orion_ns_core`.`olevel`,`orion_ns_core`.`slevel`,`orion_ns_core`.`llevel`,`orion_ns_core`.`email`,`orion_ns_core`.`greet`,`orion_ns_core`.`url`,`orion_ns_core`.`icq`,`orion_ns_core`.`yahoo`,`orion_ns_core`.`location`,`orion_ns_core`.`birthday`,`orion_ns_core`.`realname`,`orion_ns_core`.`gender`,`orion_ns_core`.`interests`,`orion_ns_core`.`profession`,`orion_ns_core`.`slogan`,`orion_ns_core`.`language`,`orion_ns_core`.`rights`,`orion_ns_core`.`settings`,`orion_ns_core`.`maxmemos`,`orion_ns_core`.`failed_logins`,`orion_ns_core`.`last_failed_login`,`orion_ns_core`.`last_login_password`",sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("Access fields = %d - 26\n",findchar(temp_fields,',')+1); //26
		num_fields[temp_arr]=findchar(temp_fields,',')+1;
		
		start_from[temp_arr]=fields;
		fields+=num_fields[temp_arr];
		needed_param[param_now++]=temp;
	}
	if(adddata&CHANNEL){
		c.chan.fields=0;
		c.chan.name[0]='\0';
		c.chan.bot[0]='\0';
		c.chan.serv[0]='\0';
		c.chan.founder[0]='\0';
		c.chan.successor[0]='\0';
		c.chan.datecreate=(time_t)0;
		c.chan.regdate=(time_t)0;
		c.chan.description[0]='\0';
		c.chan.mode[0]='\0';
		c.chan.mlock_positive[0]='\0';
		c.chan.mlock_negative[0]='\0';
		c.chan.mlock_limit=0;
		c.chan.mlock_key[0]='\0';
		c.chan.mlock_flood[0]='\0';
		c.chan.mlock_redirect[0]='\0';
		c.chan.link[0]='\0';
		c.chan.limit=0;
		c.chan.key[0]='\0';
		c.chan.flood[0]='\0';
		c.chan.last_topic[0]='\0';
		c.chan.last_topic_setter[0]='\0';
		c.chan.last_topic_time=(time_t)0;
		c.chan.url[0]='\0';
		c.chan.alimit=0;
		c.chan.flags[0]='\0';
		c.chan.welcome[0]='\0';
		c.chan.last_used=0;
		c.chan.settings=(time_t)0;
		c.chan.email[0]='\0';
		c.chan.expire=1;
		c.chan.maxmemos=0;
		c.chan.reason[0]='\0';
		c.chan.countusers=-1;

		c.chan.levels.fields=0;
		c.chan.access.fields=0;
		c.chan.botserv.fields=0;

		if(channel[0]!='\0'){
			straddns(select_from,"`orion_cs_core`",sizeof(select_from));
			snprintf(select_where,sizeof(select_where), "`orion_cs_core`.`channel` = '%s'",mysql_quote(thread,channel));
		}else
			needed_param[param_now++]=CHANNEL;

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,
			"`orion_cs_core`.`serv`,`orion_cs_core`.`botnick`,`orion_cs_core`.`channel`,`orion_cs_core`.`founder`,`orion_cs_core`.`successor`,`orion_cs_core`.`datecreate`,`orion_cs_core`.`regdate`,"
			"`orion_cs_core`.`description`,`orion_cs_core`.`mode`,`orion_cs_core`.`mlock_positive`,`orion_cs_core`.`mlock_negative`,"
			"`orion_cs_core`.`mlock_limit`,`orion_cs_core`.`mlock_key`,`orion_cs_core`.`mlock_flood`,`orion_cs_core`.`mlock_redirect`,`orion_cs_core`.`link`,"
			"`orion_cs_core`.`limit`,`orion_cs_core`.`key`,`orion_cs_core`.`flood`,`orion_cs_core`.`last_topic`,`orion_cs_core`.`last_topic_setter`,"
			"`orion_cs_core`.`last_topic_time`,`orion_cs_core`.`url`,`orion_cs_core`.`alimit`,`orion_cs_core`.`flags`,`orion_cs_core`.`welcome`,"
			"`orion_cs_core`.`last_used`,`orion_cs_core`.`settings`,"
			"`orion_cs_core`.`email`,`orion_cs_core`.`expire`,`orion_cs_core`.`maxmemos`,`orion_cs_core`.`reason`",
		sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("Channel fields = %d - 32\n",findchar(temp_fields,',')+1); //32
		num_fields[CHANNEL_ARR]=findchar(temp_fields,',')+1;
		
		start_from[CHANNEL_ARR]=fields;
		fields+=num_fields[CHANNEL_ARR];
	}
	if(adddata&CONDATA){
		c.Cdata.nick[0]='\0';
		c.Cdata.time=(time_t)0;
		c.Cdata.localtime=(time_t)0;
		c.Cdata.rident[0]='\0';
		c.Cdata.ident[0]='\0';
		c.Cdata.rhost[0]='\0';
		c.Cdata.server[0]='\0';
		c.Cdata.modes[0]='\0';
		c.Cdata.vhost[0]='\0';
		c.Cdata.realname[0]='\0';
		c.Cdata.swhois[0]='\0';
		c.Cdata.subnet[0]='\0';
		c.Cdata.ns_warntime=(time_t)0;
		c.Cdata.faketype=0;

		if(nick[0]!='\0'){
			straddns(select_from,"`orion_core_condata`",sizeof(select_from));
			snprintf(select_where,sizeof(select_where),"`orion_core_condata`.`nick` = '%s'",mysql_quote(thread,nick));
		}else
			needed_param[param_now++]=CONDATA;
			
		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,"`orion_core_condata`.`nick`,`orion_core_condata`.`time`,`orion_core_condata`.`localtime`,`orion_core_condata`.`rident`,`orion_core_condata`.`ident`,`orion_core_condata`.`rhost`,`orion_core_condata`.`srv`,`orion_core_condata`.`modes`,`orion_core_condata`.`vhost`,`orion_core_condata`.`realname`,`orion_core_condata`.`swhois`,`orion_core_condata`.`subnet`,`orion_core_condata`.`ip`,`orion_core_condata`.`ns_warntime`,`orion_core_condata`.`faketype`",sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("Condata fields = %d - 13\n",findchar(temp_fields,',')+1); //13
		num_fields[CONDATA_ARR]=findchar(temp_fields,',')+1;
		
		start_from[CONDATA_ARR]=fields;
		fields+=num_fields[CONDATA_ARR];
	}

	for(i=0;i<param_now;i++){
		//printf("PARAM: %d -- %d \n",i,needed_param[i]);
		switch(needed_param[i]){
			case CHANNEL:
				if(channel[0]=='\0') dynncpy(channel,arg = va_arg(options,char *),sizeof(channel));
				stradds(select_from,_dynadd(" LEFT OUTER JOIN `orion_cs_core` ON `orion_cs_core`.`channel` ='%s'",mysql_quote(thread,channel)));
				break;
			case CONDATA:
				if(nick[0]=='\0') dynncpy(nick,arg = va_arg(options,char *),sizeof(nick));
				stradds(select_from,_dynadd(" LEFT OUTER JOIN `orion_core_condata` ON `orion_core_condata`.`nick` ='%s'",mysql_quote(thread,nick)));
				break;
			case COUNTUSERS:
				stradds(select_join," LEFT OUTER JOIN `orion_chan_users` ON `orion_chan_users`.`channel` = `orion_cs_core`.`channel`");
				stradds(select_where," GROUP BY `orion_chan_users`.`channel`");
				break;
			case OWNACCLEVEL:
				arg = va_arg(options,char *);
				stradds(select_join,_dynadd(" LEFT OUTER JOIN `orion_cs_access` AS `own` ON ( `own`.`channel` = `orion_cs_core`.`channel` AND `own`.`authnick` = '%s')",mysql_quote(thread,arg)));
				break;
			case OWNACCLEVEL_FROM_NICK:
				if(nick[0]=='\0') dynncpy(nick,arg = va_arg(options,char *),sizeof(nick));
				join_aliases=1;
				stradds(select_join,_dynadd(" LEFT OUTER JOIN `orion_cs_access` AS `own` ON (`orion_ns_aliases`.`display`=`own`.`authnick` AND `own`.`channel`='%s')",mysql_quote(thread,channel)));
				break;
			case TARGETACCLEVEL:
				arg = va_arg(options,char *);
				stradds(select_join,_dynadd(" LEFT OUTER JOIN `orion_cs_access` AS `target` ON ( `target`.`channel` = `orion_cs_core`.`channel` AND `target`.`authnick` = '%s')",mysql_quote(thread,arg)));
				break;
			case ALIASES:
				if(nick[0]=='\0') dynncpy(nick,arg = va_arg(options,char *),sizeof(nick));
				join_aliases=1;
				break;
			case ACCESS:
				arg = va_arg(options,char *);
				stradds(select_join,_dynadd(" LEFT OUTER JOIN `orion_ns_core` ON `orion_ns_core`.`authnick`='%s'",mysql_quote(thread,arg)));
				break;
			case ACCESS_FROM_NICK:
				if(nick[0]=='\0') dynncpy(nick,arg = va_arg(options,char *),sizeof(nick));
				join_aliases=1;
				stradds(select_join," LEFT OUTER JOIN `orion_ns_core` ON `orion_ns_core`.`authnick`=`orion_ns_aliases`.`display`");
				break;
		}
	}

	va_end(options);

	if(join_aliases==1) stradds(select_from,_dynadd(" LEFT OUTER JOIN `orion_ns_aliases` ON (`orion_ns_aliases`.`nick` = '%s' AND `orion_ns_aliases`.`active`=1)",mysql_quote(thread,nick)));

	// get the requested chandata..
	//dlog(thread,4,"QRY: SELECT %s FROM %s%s WHERE %s LIMIT 0,1 \n",select_fields,select_from,select_join,select_where);
	chandata.res = doSelectMySQL(thread,"SELECT %s FROM %s%s WHERE %s LIMIT 0,1",select_fields,select_from,select_join,select_where);
	
	if(mysql_num_rows(chandata.res)>0){
		chandata.row = mysql_fetch_row(chandata.res);
		c.fields = mysql_num_fields(chandata.res);
		
		dlog(thread,4,"mysql_num_fields: %d \n",mysql_num_fields(chandata.res));
		
		for(i=0;i<MAX_CHANDATA_TYPES;i++){
			if(num_fields[i]==0) continue;
			j=start_from[i];
			switch((1<<i)){
				case CHANNEL:
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.serv,chandata.row[(j-1)],sizeof(c.chan.serv));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.bot,chandata.row[(j-1)],sizeof(c.chan.bot));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.name,chandata.row[(j-1)],sizeof(c.chan.name));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.founder,chandata.row[(j-1)],sizeof(c.chan.founder));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.successor,chandata.row[(j-1)],sizeof(c.chan.successor));
					if(chandata.row[j++]!=NULL) c.chan.datecreate=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.regdate=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.description,chandata.row[(j-1)],sizeof(c.chan.description));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.mode,chandata.row[(j-1)],sizeof(c.chan.mode));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.mlock_positive,chandata.row[(j-1)],sizeof(c.chan.mlock_positive));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.mlock_negative,chandata.row[(j-1)],sizeof(c.chan.mlock_negative));
					if(chandata.row[j++]!=NULL) c.chan.mlock_limit=stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.mlock_key,chandata.row[(j-1)],sizeof(c.chan.mlock_key));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.mlock_flood,chandata.row[(j-1)],sizeof(c.chan.mlock_flood));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.mlock_redirect,chandata.row[(j-1)],sizeof(c.chan.mlock_redirect));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.link,chandata.row[(j-1)],sizeof(c.chan.link));
					if(chandata.row[j++]!=NULL) c.chan.limit=stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.key,chandata.row[(j-1)],sizeof(c.chan.key));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.flood,chandata.row[(j-1)],sizeof(c.chan.flood));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.last_topic,chandata.row[(j-1)],sizeof(c.chan.last_topic));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.last_topic_setter,chandata.row[(j-1)],sizeof(c.chan.last_topic_setter));
					if(chandata.row[j++]!=NULL) c.chan.last_topic_time=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.url,chandata.row[(j-1)],sizeof(c.chan.url));
					if(chandata.row[j++]!=NULL) c.chan.alimit=stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.flags,chandata.row[(j-1)],sizeof(c.chan.flags));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.welcome,chandata.row[(j-1)],sizeof(c.chan.welcome));
					if(chandata.row[j++]!=NULL) c.chan.last_used=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.settings=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.email,chandata.row[(j-1)],sizeof(c.chan.email));
					if(chandata.row[j++]!=NULL) c.chan.expire=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.maxmemos=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.reason,chandata.row[(j-1)],sizeof(c.chan.reason));
					c.chan.fields=num_fields[i];
					break;
				case WITH_LEVELS:
					if(chandata.row[j++]!=NULL) c.chan.levels.accchange=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.acclist=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.akick=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.autodeop=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.autohalfop=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.autoop=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.autoprotect=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.autovoice=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.badwords=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.ban=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.banme=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.assign=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.clear=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.fantasia=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.getkey=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.greet=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.halfop=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.halfopme=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.info=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.invite=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.kick=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.kickme=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.memo=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.nojoin=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.nokick=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.opdeop=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.opdeopme=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.protect=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.protectme=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.say=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.set=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.signkick=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.topic=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.unban=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.voice=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.voiceme=stoi(chandata.row[(j-1)]);
					c.chan.levels.fields=num_fields[i];
					break;
				case WITH_BOTSERV:
					if(chandata.row[j++]!=NULL) c.chan.botserv.set=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.caps_ttb=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.caps_min=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.caps_per=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.flood_ttb=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.flood_ln=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.flood_secs=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.repeat_ttb=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.repeat=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.colors=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.underlines=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.reverses=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.badwords=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.bolds=stoi(chandata.row[(j-1)]);
					c.chan.botserv.fields=num_fields[i];
					break;
				case COUNTUSERS:
					if(chandata.row[j++]!=NULL) c.chan.countusers=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botinchan=stoi(chandata.row[(j-1)]);
					c.chan.fields=num_fields[i];
					break;
				case OWNACCLEVEL:
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.access.ownlevel,chandata.row[(j-1)],sizeof(c.chan.access.ownlevel));
					c.chan.access.fields=num_fields[i];
					break;
				case OWNACCLEVEL_FROM_NICK:
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.access.ownauthnick,chandata.row[(j-1)],sizeof(c.chan.access.ownauthnick));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.access.ownlevel,chandata.row[(j-1)],sizeof(c.chan.access.ownlevel));
					c.chan.access.fields=num_fields[i];
					//c.chan.access.fields++;
					break;
				case TARGETACCLEVEL:
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.access.targetlevel,chandata.row[(j-1)],sizeof(c.chan.access.targetlevel));
					c.chan.access.fields=num_fields[i];
					break;
				case CONDATA:
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.nick,chandata.row[(j-1)],sizeof(c.Cdata.nick));
					if(chandata.row[j++]!=NULL) c.Cdata.time=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.Cdata.localtime=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.rident,chandata.row[(j-1)],sizeof(c.Cdata.rident));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.ident,chandata.row[(j-1)],sizeof(c.Cdata.ident));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.rhost,chandata.row[(j-1)],sizeof(c.Cdata.rhost));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.server,chandata.row[(j-1)],sizeof(c.Cdata.server));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.modes,chandata.row[(j-1)],sizeof(c.Cdata.modes));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.vhost,chandata.row[(j-1)],sizeof(c.Cdata.vhost));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.realname,chandata.row[(j-1)],sizeof(c.Cdata.realname));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.swhois,chandata.row[(j-1)],sizeof(c.Cdata.swhois));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.subnet,chandata.row[(j-1)],sizeof(c.Cdata.subnet));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.ip,chandata.row[(j-1)],sizeof(c.Cdata.ip));
					if(chandata.row[j++]!=NULL) c.Cdata.ns_warntime=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.Cdata.faketype=stoi(chandata.row[(j-1)]);
					c.Cdata.fields=num_fields[i];
					break;
				case ALIASES:
					if(chandata.row[j++]!=NULL) dynncpy(c.aliases.nick,chandata.row[(j-1)],sizeof(c.aliases.nick));
					if(chandata.row[j++]!=NULL) dynncpy(c.aliases.authnick,chandata.row[(j-1)],sizeof(c.aliases.authnick));
					if(chandata.row[j++]!=NULL) dynncpy(c.aliases.last_vhost,chandata.row[(j-1)],sizeof(c.aliases.last_vhost));
					if(chandata.row[j++]!=NULL) dynncpy(c.aliases.last_realname,chandata.row[(j-1)],sizeof(c.aliases.last_realname));
					if(chandata.row[j++]!=NULL) dynncpy(c.aliases.last_quit,chandata.row[(j-1)],sizeof(c.aliases.last_quit));
					if(chandata.row[j++]!=NULL) c.aliases.time_linked=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.aliases.last_login=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.aliases.last_used=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.aliases.expire=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.aliases.linked=stoi(chandata.row[(j-1)]); //0-permanent, 1-temporarily
					if(chandata.row[j++]!=NULL) c.aliases.active=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.aliases.forbid=stoi(chandata.row[(j-1)]);
					c.aliases.fields=num_fields[i];
					break;
				case ACCESS:
				case ACCESS_FROM_NICK:
					if(chandata.row[j++]!=NULL) dynncpy(c.access.authnick,chandata.row[(j-1)],sizeof(c.access.authnick));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.password,chandata.row[(j-1)],sizeof(c.access.password));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.reason,chandata.row[(j-1)],sizeof(c.access.reason));
					if(chandata.row[j++]!=NULL) c.access.qlevel=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.olevel=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.slevel=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.llevel=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.access.email,chandata.row[(j-1)],sizeof(c.access.email));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.greet,chandata.row[(j-1)],sizeof(c.access.greet));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.url,chandata.row[(j-1)],sizeof(c.access.url));
					if(chandata.row[j++]!=NULL) c.access.icq=stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.access.yahoo,chandata.row[(j-1)],sizeof(c.access.yahoo));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.location,chandata.row[(j-1)],sizeof(c.access.location));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.birthday,chandata.row[(j-1)],sizeof(c.access.birthday));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.realname,chandata.row[(j-1)],sizeof(c.access.realname));
					if(chandata.row[j++]!=NULL) c.access.gender=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.access.interests,chandata.row[(j-1)],sizeof(c.access.interests));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.profession,chandata.row[(j-1)],sizeof(c.access.profession));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.slogan,chandata.row[(j-1)],sizeof(c.access.slogan));
					if(chandata.row[j++]!=NULL) c.access.language=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.rights=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.settings=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.maxmemos=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.failed_logins=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.last_failed_login_time=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.access.last_failed_login_password,chandata.row[(j-1)],sizeof(c.access.last_failed_login_password));
					c.access.fields=num_fields[i];
					break;
			}
		}
	}
	mysql_free_result_myr(chandata.res);
	return c;
}

/*  ************************************************************************************************  */

/**
 * @brief sql_chanmodes:  Selects all common information about
 *                  a channel (levels, channel info)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The channel for which we like the information
 * @param c The channelstruct to be filled
 * @param cLev The chanlevels struct to be filled with the channel levels
 * @return void
 **/
void sql_chanmodes(Threads *thread, char* channel, Channel *c, ChanLevels *cLev){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
        
	char query[SQL_BUF], chandata[MAX_CHANNELNAME_LEN*2];
	
	c->fields=0;
	c->serv[0]='\0';
	c->name[0]='\0';
	c->founder[0]='\0';
	c->successor[0]='\0';
	c->datecreate=(time_t)0;
	c->regdate=(time_t)0;
	c->description[0]='\0';
	c->mode[0]='\0';
	c->mlock_positive[0]='\0';
	c->mlock_negative[0]='\0';
	c->mlock_limit=0;
	c->mlock_key[0]='\0';
	c->mlock_flood[0]='\0';
	c->mlock_redirect[0]='\0';
	c->link[0]='\0';
	c->limit=0;
	c->key[0]='\0';
	c->flood[0]='\0';
	c->last_topic[0]='\0';
	c->last_topic_setter[0]='\0';
	c->last_topic_time=(time_t)0;
	c->url[0]='\0';
	c->alimit=0;
	c->flags[0]='\0';
	c->welcome[0]='\0';
	c->reason[0]='\0';
	c->last_used=(time_t)0;
	c->settings=0;
	c->email[0]='\0';
	c->expire=1;
	c->maxmemos=0;

	cLev->fields=0;
	cLev->accchange=0;
	cLev->acclist=0;
	cLev->akick=0;
	cLev->autodeop=0;
	cLev->autohalfop=0;
	cLev->autoop=0;
	cLev->autoprotect=0;
	cLev->autovoice=0;
	cLev->badwords=0;
	cLev->ban=0;
	cLev->banme=0;
	cLev->assign=0;
	cLev->clear=0;
	cLev->fantasia=0;
	cLev->getkey=0;
	cLev->greet=0;
	cLev->halfop=0;
	cLev->halfopme=0;
	cLev->info=0;
	cLev->invite=0;
	cLev->kick=0;
	cLev->kickme=0;
	cLev->memo=0;
	cLev->nojoin=0;
	cLev->nokick=0;
	cLev->opdeop=0;
	cLev->opdeopme=0;
	cLev->protect=0;
	cLev->protectme=0;
	cLev->say=0;
	cLev->set=0;
	cLev->signkick=0;
	cLev->topic=0;
	cLev->unban=0;
	cLev->voice=0;
	cLev->voiceme=0;
	
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, chandata, channel, strlen(channel)); //escape channel -> chandata
	snprintf(query, sizeof(query),

	"SELECT "
	
		"`serv`,`channel`,`founder`,`successor`,`datecreate`,"
		"`description`,`mode`,`mlock_positive`,`mlock_negative`,"
		"`mlock_limit`,`mlock_key`,`mlock_flood`,`link`,"
		"`limit`,`key`,`flood`,`last_topic`,`last_topic_setter`,"
		"`last_topic_time`,`url`,`alimit`,`flags`,`welcome`,"
		"`mlock_redirect`,`last_used`,`reason`,`settings`,"
		"`email`,`expire`,`maxmemos`,"
	
		"`cs_accchange`,`cs_acclist`,"
	    "`cs_akick`,`cs_autodeop`,`cs_autohalfop`,`cs_autoop`,"
	    "`cs_autoprotect`,`cs_autovoice`,`cs_badwords`,`cs_ban`,`cs_banme`,"
	    "`cs_botassign`,`cs_cclear`,`cs_fantasia`,`cs_getkey`,`cs_greet`,`cs_halfop`,`cs_halfopme`,"
	    "`cs_info`,`cs_invite`,`cs_kick`,`cs_kickme`,`cs_memo`,`cs_nojoin`,"
	    "`cs_nokick`,`cs_opdeop`,`cs_opdeopme`,`cs_protect`,"
	    "`cs_protectme`,`cs_say`,`cs_set`,`cs_signkick`,"
		"`cs_topic`,`cs_unban`,`cs_voice`,`cs_voiceme`,"
		
		"`regdate` "
	
	"FROM `orion_cs_core` WHERE `channel`='%s' LIMIT 0,1"

	,chandata);

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res    = mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to MYSQL result pointer

	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res); //
		cLev->fields = mysql_num_fields(res);
		c->fields    = cLev->fields;
		
		dynncpy(c->serv,row[0],sizeof(c->serv));
		dynncpy(c->name,row[1],sizeof(c->name));
		dynncpy(c->founder,row[2],sizeof(c->founder));
		dynncpy(c->successor,row[3],sizeof(c->successor));
		c->datecreate=(time_t)stol(row[4]);
		dynncpy(c->description,row[5],sizeof(c->description));
		dynncpy(c->mode,row[6],sizeof(c->mode));
		dynncpy(c->mlock_positive,row[7],sizeof(c->mlock_positive));
		dynncpy(c->mlock_negative,row[8],sizeof(c->mlock_negative));
		c->mlock_limit=stol(row[9]);
		dynncpy(c->mlock_key,row[10],sizeof(c->mlock_key));
		dynncpy(c->mlock_flood,row[11],sizeof(c->mlock_flood));
		dynncpy(c->link,row[12],sizeof(c->link));
		c->limit=stol(row[13]);
		dynncpy(c->key,row[14],sizeof(c->key));
		dynncpy(c->flood,row[15],sizeof(c->flood));
		dynncpy(c->last_topic,row[16],sizeof(c->last_topic));
		dynncpy(c->last_topic_setter,row[17],sizeof(c->last_topic_setter));
		c->last_topic_time=(time_t)stol(row[18]);
		dynncpy(c->url,row[19],sizeof(c->url));
		c->alimit=stol(row[20]);
		dynncpy(c->flags,row[21],sizeof(c->flags));
		dynncpy(c->welcome,row[22],sizeof(c->welcome));
		dynncpy(c->mlock_redirect,row[23],sizeof(c->mlock_redirect));
		c->last_used=(time_t)stol(row[24]);
		dynncpy(c->reason,row[25],sizeof(c->reason));
		c->settings=stoi(row[26]);
		dynncpy(c->email,row[27],sizeof(c->email));
		c->expire=stoi(row[28]);
		c->maxmemos=stoi(row[29]);
		
		cLev->accchange=stoi(row[30]);
		cLev->acclist=stoi(row[31]);
		cLev->akick=stoi(row[32]);
		cLev->autodeop=stoi(row[33]);
		cLev->autohalfop=stoi(row[34]);
		cLev->autoop=stoi(row[35]);
		cLev->autoprotect=stoi(row[36]);
		cLev->autovoice=stoi(row[37]);
		cLev->badwords=stoi(row[38]);
		cLev->ban=stoi(row[39]);
		cLev->banme=stoi(row[40]);
		cLev->assign=stoi(row[41]);
		cLev->clear=stoi(row[42]);
		cLev->fantasia=stoi(row[43]);
		cLev->getkey=stoi(row[44]);
		cLev->greet=stoi(row[45]);
		cLev->halfop=stoi(row[46]);
		cLev->halfopme=stoi(row[47]);
		cLev->info=stoi(row[48]);
		cLev->invite=stoi(row[49]);
		cLev->kick=stoi(row[50]);
		cLev->kickme=stoi(row[51]);
		cLev->memo=stoi(row[52]);
		cLev->nojoin=stoi(row[53]);
		cLev->nokick=stoi(row[54]);
		cLev->opdeop=stoi(row[55]);
		cLev->opdeopme=stoi(row[56]);
		cLev->protect=stoi(row[57]);
		cLev->protectme=stoi(row[58]);
		cLev->say=stoi(row[59]);
		cLev->set=stoi(row[60]);
		cLev->signkick=stoi(row[61]);
		cLev->topic=stoi(row[62]);
		cLev->unban=stoi(row[63]);
		cLev->voice=stoi(row[64]);
		cLev->voiceme=stoi(row[65]);
		
		c->regdate=(time_t)stol(row[66]);
	}
	mysql_free_result_myr(res);
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief sql_CA: 	Selects all information about a channel together with the channel access level
 * 		of a given user (and if needed, the channel access level of another (target) user)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The Channel for which the information are requested
 * @param authnick The Authnick of the user who is trying to do something in the channel
 * @param target The Authnick of the user who is the target of the action (optional)
 * @return ChanCA Returns a struct that includes the information about the channel and channel access levels
 **/
ChanCA sql_chandata(Threads *thread, char* channel,char* authnick,char* target){
	/* Initialize SQL */
	char query[SQL_BUF];
	deepSQL chanSQL;
	bzero(&query,sizeof(query));
	bzero(&chanSQL,sizeof(chanSQL));
	
	/* STRUCT */ ChanCA chandata; /* STRUCT */
	bzero(&chandata,sizeof(chandata));
	
	chandata.fields=0;
	chandata.serv[0]='\0';
	chandata.name[0]='\0';
	chandata.founder[0]='\0';
	chandata.successor[0]='\0';
	chandata.datecreate=(time_t)0;
	chandata.regdate=(time_t)0;
	chandata.description[0]='\0';
	chandata.mode[0]='\0';
	chandata.mlock_positive[0]='\0';
	chandata.mlock_negative[0]='\0';
	chandata.mlock_limit=0;
	chandata.mlock_key[0]='\0';
	chandata.mlock_flood[0]='\0';
	chandata.mlock_redirect[0]='\0';
	chandata.link[0]='\0';
	chandata.limit=0;
	chandata.key[0]='\0';
	chandata.flood[0]='\0';
	chandata.last_topic[0]='\0';
	chandata.last_topic_setter[0]='\0';
	chandata.last_topic_time=(time_t)0;
	chandata.url[0]='\0';
	chandata.alimit=0;
	chandata.flags[0]='\0';
	chandata.welcome[0]='\0';
	chandata.last_used=(time_t)0;
	chandata.settings=0;
	chandata.email[0]='\0';
	chandata.expire=1;
	chandata.maxmemos=0;
	chandata.reason[0]='\0';
	
	chandata.acclevel[0]='\0';
	chandata.targetlevel[0]='\0';

	if(channel==NULL) return chandata;
	if(authnick[0]!='\0'){
		if(target[0]!='\0'){
			snprintf(query, sizeof(query),"SELECT `orion_cs_core`.`serv`,`orion_cs_core`.`channel`,`orion_cs_core`.`founder`,`orion_cs_core`.`successor`,`orion_cs_core`.`datecreate`,"
				"`orion_cs_core`.`description`,`orion_cs_core`.`mode`,`orion_cs_core`.`mlock_positive`,`orion_cs_core`.`mlock_negative`,`orion_cs_core`.`mlock_limit`,"
				"`orion_cs_core`.`mlock_key`,`orion_cs_core`.`mlock_flood`,`orion_cs_core`.`link`,`orion_cs_core`.`limit`,`orion_cs_core`.`key`,`orion_cs_core`.`flood`,"
				"`orion_cs_core`.`last_topic`,`orion_cs_core`.`last_topic_setter`,`orion_cs_core`.`last_topic_time`,`orion_cs_core`.`url`,`orion_cs_core`.`alimit`,"
				"`orion_cs_core`.`flags`,`orion_cs_core`.`welcome`,`orion_cs_core`.`mlock_redirect`,`orion_cs_core`.`last_used`,`orion_cs_core`.`reason`,`orion_cs_core`.`settings`,"
				"`orion_cs_core`.`email`,`orion_cs_core`.`expire`,`orion_cs_core`.`maxmemos`,`orion_cs_core`.`regdate`, "
				"`ca1`.`level` as `levelinfo_auth`,`ca2`.`level` as `levelinfo_target` "
				"FROM `orion_cs_core` "
				"LEFT OUTER JOIN `orion_cs_access` AS `ca1` ON ( `ca1`.`channel` = '%1$s' AND `ca1`.`authnick` = '%2$s') "
				"LEFT OUTER JOIN `orion_cs_access` AS `ca2` ON ( `ca2`.`channel` = '%1$s' AND `ca2`.`authnick` = '%3$s') "
				"WHERE `orion_cs_core`.`channel`='%1$s' LIMIT 0,1",mysql_quote(thread,channel),mysql_quote(thread,authnick),mysql_quote(thread,target)
			);
		}else{
			snprintf(query, sizeof(query),"SELECT `orion_cs_core`.`serv`,`orion_cs_core`.`channel`,`orion_cs_core`.`founder`,`orion_cs_core`.`successor`,`orion_cs_core`.`datecreate`,"
				"`orion_cs_core`.`description`,`orion_cs_core`.`mode`,`orion_cs_core`.`mlock_positive`,`orion_cs_core`.`mlock_negative`,`orion_cs_core`.`mlock_limit`,"
				"`orion_cs_core`.`mlock_key`,`orion_cs_core`.`mlock_flood`,`orion_cs_core`.`link`,`orion_cs_core`.`limit`,`orion_cs_core`.`key`,`orion_cs_core`.`flood`,"
				"`orion_cs_core`.`last_topic`,`orion_cs_core`.`last_topic_setter`,`orion_cs_core`.`last_topic_time`,`orion_cs_core`.`url`,`orion_cs_core`.`alimit`,"
				"`orion_cs_core`.`flags`,`orion_cs_core`.`welcome`,`orion_cs_core`.`mlock_redirect`,`orion_cs_core`.`last_used`,`orion_cs_core`.`reason`,`orion_cs_core`.`settings`,"
				"`orion_cs_core`.`email`,`orion_cs_core`.`expire`,`orion_cs_core`.`maxmemos`,`orion_cs_core`.`regdate`, "
				"`ca1`.`level` as `levelinfo_auth` "
				"FROM `orion_cs_core` "
				"LEFT OUTER JOIN `orion_cs_access` AS `ca1` ON ( `ca1`.`channel` = '%1$s' AND `ca1`.`authnick` = '%2$s') "
				"WHERE `orion_cs_core`.`channel`='%1$s' LIMIT 0,1",mysql_quote(thread,channel),mysql_quote(thread,authnick)
			);
		}
	}else{
		snprintf(query, sizeof(query),"SELECT `orion_cs_core`.`serv`,`orion_cs_core`.`channel`,`orion_cs_core`.`founder`,`orion_cs_core`.`successor`,`orion_cs_core`.`datecreate`,"
			"`orion_cs_core`.`description`,`orion_cs_core`.`mode`,`orion_cs_core`.`mlock_positive`,`orion_cs_core`.`mlock_negative`,`orion_cs_core`.`mlock_limit`,"
			"`orion_cs_core`.`mlock_key`,`orion_cs_core`.`mlock_flood`,`orion_cs_core`.`link`,`orion_cs_core`.`limit`,`orion_cs_core`.`key`,`orion_cs_core`.`flood`,"
			"`orion_cs_core`.`last_topic`,`orion_cs_core`.`last_topic_setter`,`orion_cs_core`.`last_topic_time`,`orion_cs_core`.`url`,`orion_cs_core`.`alimit`,"
			"`orion_cs_core`.`flags`,`orion_cs_core`.`welcome`,`orion_cs_core`.`mlock_redirect`,`orion_cs_core`.`last_used`,`orion_cs_core`.`reason`,`orion_cs_core`.`settings`,"
			"`orion_cs_core`.`email`,`orion_cs_core`.`expire`,`orion_cs_core`.`maxmemos`,`orion_cs_core`.`regdate` "
			"FROM `orion_cs_core` "
			"WHERE `orion_cs_core`.`channel`='%s' LIMIT 0,1",mysql_quote(thread,channel)
		);
	}
	
	chanSQL.res = doSelectMySQL(thread,query);
	if(mysql_num_rows(chanSQL.res)>0){
		chanSQL.row=mysql_fetch_row(chanSQL.res);
		chandata.fields=mysql_num_fields(chanSQL.res);
		dynncpy(chandata.serv,chanSQL.row[0],sizeof(chandata.serv));
		dynncpy(chandata.name,chanSQL.row[1],sizeof(chandata.name));
		dynncpy(chandata.founder,chanSQL.row[2],sizeof(chandata.founder));
		dynncpy(chandata.successor,chanSQL.row[3],sizeof(chandata.successor));
		chandata.datecreate=(time_t)stol(chanSQL.row[4]);
		dynncpy(chandata.description,chanSQL.row[5],sizeof(chandata.description));
		dynncpy(chandata.mode,chanSQL.row[6],sizeof(chandata.mode));
		dynncpy(chandata.mlock_positive,chanSQL.row[7],sizeof(chandata.mlock_positive));
		dynncpy(chandata.mlock_negative,chanSQL.row[7],sizeof(chandata.mlock_negative));
		chandata.mlock_limit=stol(chanSQL.row[9]);
		dynncpy(chandata.mlock_key,chanSQL.row[10],sizeof(chandata.mlock_key));
		dynncpy(chandata.mlock_flood,chanSQL.row[11],sizeof(chandata.mlock_flood));
		dynncpy(chandata.link,chanSQL.row[12],sizeof(chandata.link));
		chandata.limit=stol(chanSQL.row[13]);
		dynncpy(chandata.key,chanSQL.row[14],sizeof(chandata.key));
		dynncpy(chandata.flood,chanSQL.row[15],sizeof(chandata.flood));
		dynncpy(chandata.last_topic,chanSQL.row[16],sizeof(chandata.last_topic));
		dynncpy(chandata.last_topic_setter,chanSQL.row[17],sizeof(chandata.last_topic_setter));
		chandata.last_topic_time=(time_t)stol(chanSQL.row[18]);
		dynncpy(chandata.url,chanSQL.row[19],sizeof(chandata.url));
		chandata.alimit=stol(chanSQL.row[20]);
		dynncpy(chandata.flags,chanSQL.row[21],sizeof(chandata.flags));
		dynncpy(chandata.welcome,chanSQL.row[22],sizeof(chandata.welcome));
		dynncpy(chandata.mlock_redirect,chanSQL.row[23],sizeof(chandata.mlock_redirect));
		chandata.last_used=(time_t)stol(chanSQL.row[24]);
		dynncpy(chandata.reason,chanSQL.row[25],sizeof(chandata.reason));
		chandata.settings=stoi(chanSQL.row[26]);
		dynncpy(chandata.email,chanSQL.row[27],sizeof(chandata.email));
		chandata.expire=stoi(chanSQL.row[28]);
		chandata.maxmemos=stoi(chanSQL.row[29]);
		chandata.regdate=(time_t)stol(chanSQL.row[30]);

		if(authnick[0]!='\0')	dynncpy(chandata.acclevel,chanSQL.row[31],sizeof(chandata.acclevel));
		if(target[0]!='\0')	dynncpy(chandata.targetlevel,chanSQL.row[32],sizeof(chandata.targetlevel));
	}
	mysql_free_result_myr(chanSQL.res);
	return chandata;
}

/*  ************************************************************************************************  */

/**
 * @brief sql_access:  Selects all information about
 *                  that authnick from table `orion_ns_core`
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param authnick The authnick (display) of target user (connected, or not)
 * @return Access Returns a struct that includes the information from `orion_ns_core` for a single user
 **/
Access sql_access(Threads *thread, char* authnick){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
       
	char nickdata[MAX_NICK_LEN*2], query[SQL_BUF];
	/* STRUCT */ Access access; /* STRUCT */
                                                
	access.fields=0;
	access.authnick[0]='\0';
	access.password[0]='\0';
	access.reason[0]='\0';
	access.qlevel=0;
	access.olevel=0;
	access.slevel=0;
	access.llevel=0;
	access.email[0]='\0';
	access.greet[0]='\0';
	access.url[0]='\0';
	access.icq=0;
	access.yahoo[0]='\0';
	access.location[0]='\0';
	access.birthday[0]='\0';
	access.realname[0]='\0';
	access.gender=0;
	access.interests[0]='\0';
	access.profession[0]='\0';
	access.slogan[0]='\0';
	access.language=m->language;
	access.rights=is_user;
	access.settings=0;
	access.maxmemos=0;
	access.failed_logins=0;
	access.last_failed_login_time=(time_t)0;
	access.last_failed_login_password[0]='\0';
                                                
	if(authnick==NULL) return access;
	
	/* Access */
	bzero(nickdata,sizeof(nickdata));
	bzero(query,sizeof(query));
	
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, nickdata, authnick, strlen(authnick)); //escape authnick -> nickdata
	snprintf(query, sizeof(query),"SELECT `authnick`,`password`,`reason`,`qlevel`,`olevel`,`slevel`,`llevel`,`email`,`greet`,`url`,`icq`,`yahoo`,`location`,`birthday`,`realname`,`gender`,`interests`,`profession`,`slogan`,`language`,`rights`,`settings`,`maxmemos`,`failed_logins`,`last_failed_login`,`last_login_password` FROM `orion_ns_core` WHERE `authnick`='%s'",nickdata); //prepare query

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to MYSQL result pointer
	
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res); //fetch result to the MYSQL **row array
		access.fields=mysql_num_fields(res); //get number of Array entries
		dynncpy(access.authnick,row[0],sizeof(access.authnick));
		dynncpy(access.password,row[1],sizeof(access.password));
		dynncpy(access.reason,row[2],sizeof(access.reason));
		access.qlevel=stoi(row[3]);
		access.olevel=stoi(row[4]);
		access.slevel=stoi(row[5]);
		access.llevel=stoi(row[6]);
		dynncpy(access.email,row[7],sizeof(access.email));
		dynncpy(access.greet,row[8],sizeof(access.greet));
		dynncpy(access.url,row[9],sizeof(access.url));
		access.icq=stol(row[10]);
		dynncpy(access.yahoo,row[11],sizeof(access.yahoo));
		dynncpy(access.location,row[12],sizeof(access.location));
		dynncpy(access.birthday,row[13],sizeof(access.birthday));
		dynncpy(access.realname,row[14],sizeof(access.realname));
		access.gender=stoi(row[15]);
		dynncpy(access.interests,row[16],sizeof(access.interests));
		dynncpy(access.profession,row[17],sizeof(access.profession));
		dynncpy(access.slogan,row[18],sizeof(access.slogan));
		access.language=stoi(row[19]);
		access.rights=stoi(row[20]);
		access.settings=stoi(row[21]);
		access.maxmemos=stoi(row[22]);
		
		access.failed_logins=stoi(row[23]);
		access.last_failed_login_time=(time_t)stol(row[24]);
		dynncpy(access.last_failed_login_password,row[25],sizeof(access.last_failed_login_password));
	}
	mysql_free_result_myr(res); //free result
	return access; //return access struct
}

/*  ************************************************************************************************  */

/**
 * @brief sql_acclist:  Selects information about the
 *                  channelaccess of a single user from table `orion_cs_access`
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param serv The service which is holding the channel in `orion_cs_core`
 * @param channel The channel to search for the authnick's access
 * @param authnick The authnick for whom we like to get the access information (in that channel)
 * @return Acclist Returns a struct that includes the information from `orion_cs_access` for a single user
 **/
Acclist sql_acclist(Threads *thread, const char* serv, char* channel, char* authnick){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
        
	char nickdata[MAX_NICK_LEN*2], chandata[MAX_CHANNELNAME_LEN*2], query[SQL_BUF];
	/* STRUCT */ Acclist acclist; /* STRUCT */
                                                  
	acclist.fields=0;
	acclist.serv[0]='\0';
	acclist.authnick[0]='\0';
	acclist.channel[0]='\0';
	acclist.level[0]='\0';
                                                  
	if(authnick==NULL || channel==NULL || serv==NULL) return acclist;
	
	/* Acclist */
	bzero(nickdata,sizeof(nickdata));
	bzero(chandata,sizeof(chandata));
	bzero(query,sizeof(query));
	
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, nickdata, authnick, strlen(authnick)); //escape authnick -> nickdata
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, chandata, channel, strlen(channel)); //escape channel -> chandata
	snprintf(query, sizeof(query),"SELECT `serv`,`authnick`,`channel`,`level` FROM `orion_cs_access` WHERE `serv`='%s' AND `authnick`='%s' AND `channel`='%s' LIMIT 0,1",serv,nickdata,chandata); //prepare query

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to MYSQL result pointer

	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res); //
		acclist.fields=mysql_num_fields(res);
		dynncpy(acclist.serv,row[0],sizeof(acclist.serv));
		dynncpy(acclist.authnick,row[1],sizeof(acclist.authnick));
		dynncpy(acclist.channel,row[2],sizeof(acclist.channel));
		dynncpy(acclist.level,row[3],sizeof(acclist.level));
	}
	mysql_free_result_myr(res);
	return acclist;
}

/*  ************************************************************************************************  */

/**
 * @brief sql_chan:  Selects information about
 *                  a channel from table `orion_cs_core`
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The channel for which we like to have some additional information
 * @return Channel Returns a struct that includes the information from `orion_cs_core` for a single channel
 **/
Channel sql_chan(Threads *thread, char* channel){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
        
	char query[SQL_BUF], chandata[MAX_CHANNELNAME_LEN*2];//, servdata[MAX_NICK_LEN*2];
	/* STRUCT */ Channel c; /* STRUCT */
                                            
	c.fields=0;
	c.serv[0]='\0';
	c.name[0]='\0';
	c.founder[0]='\0';
	c.successor[0]='\0';
	c.datecreate=(time_t)0;
	c.regdate=(time_t)0;
	c.description[0]='\0';
	c.mode[0]='\0';
	c.mlock_positive[0]='\0';
	c.mlock_negative[0]='\0';
	c.mlock_limit=0;
	c.mlock_key[0]='\0';
	c.mlock_flood[0]='\0';
	c.mlock_redirect[0]='\0';
	c.link[0]='\0';
	c.limit=0;
	c.key[0]='\0';
	c.flood[0]='\0';
	c.last_topic[0]='\0';
	c.last_topic_setter[0]='\0';
	c.last_topic_time=(time_t)0;
	c.url[0]='\0';
	c.alimit=0;
	c.flags[0]='\0';
	c.welcome[0]='\0';
	c.last_used=(time_t)0;
	c.settings=0;
	c.email[0]='\0';
	c.expire=1;
	c.maxmemos=0;
	c.reason[0]='\0';
                                            
	if(channel==NULL) return c;
	
	/* Channel */
	bzero(chandata,sizeof(chandata));
	bzero(query,sizeof(query));
	
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, chandata, channel, strlen(channel));
	snprintf(query, sizeof(query),"SELECT `serv`,`channel`,`founder`,`successor`,`datecreate`,`description`,`mode`,`mlock_positive`,`mlock_negative`,`mlock_limit`,`mlock_key`,`mlock_flood`,`link`,`limit`,`key`,`flood`,`last_topic`,`last_topic_setter`,`last_topic_time`,`url`,`alimit`,`flags`,`welcome`,`mlock_redirect`,`last_used`,`reason`,`settings`,`email`,`expire`,`maxmemos`,`regdate` FROM `orion_cs_core` WHERE `channel`='%s' LIMIT 0,1",chandata);

	ExecuteQuery(thread,query,dynlen(query)); //execute query	
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE);	
	
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res);
		c.fields=mysql_num_fields(res);
		dynncpy(c.serv,row[0],sizeof(c.serv));
		dynncpy(c.name,row[1],sizeof(c.name));
		dynncpy(c.founder,row[2],sizeof(c.founder));
		dynncpy(c.successor,row[3],sizeof(c.successor));
		c.datecreate=(time_t)stol(row[4]);
		dynncpy(c.description,row[5],sizeof(c.description));
		dynncpy(c.mode,row[6],sizeof(c.mode));
		dynncpy(c.mlock_positive,row[7],sizeof(c.mlock_positive));
		dynncpy(c.mlock_negative,row[7],sizeof(c.mlock_negative));
		c.mlock_limit=stoi(row[9]);
		dynncpy(c.mlock_key,row[10],sizeof(c.mlock_key));
		dynncpy(c.mlock_flood,row[11],sizeof(c.mlock_flood));
		dynncpy(c.link,row[12],sizeof(c.link));
		c.limit=stol(row[13]);
		dynncpy(c.key,row[14],sizeof(c.key));
		dynncpy(c.flood,row[15],sizeof(c.flood));
		dynncpy(c.last_topic,row[16],sizeof(c.last_topic));
		dynncpy(c.last_topic_setter,row[17],sizeof(c.last_topic_setter));
		c.last_topic_time=(time_t)stol(row[18]);
		dynncpy(c.url,row[19],sizeof(c.url));
		c.alimit=stol(row[20]);
		dynncpy(c.flags,row[21],sizeof(c.flags));
		dynncpy(c.welcome,row[22],sizeof(c.welcome));
		dynncpy(c.mlock_redirect,row[23],sizeof(c.mlock_redirect));
		c.last_used=(time_t)stol(row[24]);
		dynncpy(c.reason,row[25],sizeof(c.reason));
		c.settings=stoi(row[26]);
		dynncpy(c.email,row[27],sizeof(c.email));
		c.expire=stoi(row[28]);
		c.maxmemos=stoi(row[29]);
		c.regdate=(time_t)stol(row[30]);
	}
	mysql_free_result_myr(res);
	return c;
}

/*  ************************************************************************************************  */

/**
 * @brief sql_chanlevels:  Selects information about
 *                     the channel levels for target channel from table `chanlevels`
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The channel for which we like to have some additional information
 * @return ChanLevels Returns a struct that includes the information from `chanlevels` for a single channel
 **/
ChanLevels sql_chanlevels(Threads *thread, char* channel){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
           
	char query[SQL_BUF], chan[MAX_CHANNELNAME_LEN*2];//, servdata[MAX_NICK_LEN*2];
	/* STRUCT */ ChanLevels cLev; /* STRUCT */
                  
	cLev.channel[0]='\0';
	cLev.fields=0;
	cLev.accchange=0;
	cLev.acclist=0;
	cLev.akick=0;
	cLev.autodeop=0;
	cLev.autohalfop=0;
	cLev.autoop=0;
	cLev.autoprotect=0;
	cLev.autovoice=0;
	cLev.badwords=0;
	cLev.ban=0;
	cLev.banme=0;
	cLev.assign=0;
	cLev.clear=0;
	cLev.fantasia=0;
	cLev.getkey=0;
	cLev.greet=0;
	cLev.halfop=0;
	cLev.halfopme=0;
	cLev.info=0;
	cLev.invite=0;
	cLev.kick=0;
	cLev.kickme=0;
	cLev.memo=0;
	cLev.nojoin=0;
	cLev.nokick=0;
	cLev.opdeop=0;
	cLev.opdeopme=0;
	cLev.protect=0;
	cLev.protectme=0;
	cLev.say=0;
	cLev.set=0;
	cLev.signkick=0;
	cLev.topic=0;
	cLev.unban=0;
	cLev.voice=0;
	cLev.voiceme=0;
                                            
	if(channel==NULL) return cLev;
	
	/* Channel */
	bzero(chan,sizeof(chan));
	bzero(query,sizeof(query));
	
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, chan, channel, strlen(channel));
	
	snprintf(query, sizeof(query),
        "SELECT `channel`,`cs_accchange`,`cs_acclist`,"
        "`cs_akick`,`cs_autodeop`,`cs_autohalfop`,`cs_autoop`,"
        "`cs_autoprotect`,`cs_autovoice`,`cs_badwords`,`cs_ban`,`cs_banme`,"
        "`cs_botassign`,`cs_cclear`,`cs_fantasia`,`cs_getkey`,`cs_greet`,`cs_halfop`,`cs_halfopme`,"
        "`cs_info`,`cs_invite`,`cs_kick`,`cs_kickme`,`cs_memo`,`cs_nojoin`,"
        "`cs_nokick`,`cs_opdeop`,`cs_opdeopme`,`cs_protect`,"
        "`cs_protectme`,`cs_say`,`cs_set`,`cs_signkick`,"
        "`cs_topic`,`cs_unban`,`cs_voice`,`cs_voiceme` FROM `orion_cs_core` WHERE `channel`='%s' LIMIT 0,1"
	,chan);
	
	ExecuteQuery(thread,query,dynlen(query)); //execute query	
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE);
	
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res);
		cLev.fields=mysql_num_fields(res);
		dynncpy(cLev.channel,row[0],sizeof(cLev.channel));
		
		cLev.accchange=stoi(row[1]);
		cLev.acclist=stoi(row[2]);
		cLev.akick=stoi(row[3]);
		cLev.autodeop=stoi(row[4]);
		cLev.autohalfop=stoi(row[5]);
		cLev.autoop=stoi(row[6]);
		cLev.autoprotect=stoi(row[7]);
		cLev.autovoice=stoi(row[8]);
		cLev.badwords=stoi(row[9]);
		cLev.ban=stoi(row[10]);
		cLev.banme=stoi(row[11]);
		cLev.assign=stoi(row[12]);
		cLev.clear=stoi(row[13]);
		cLev.fantasia=stoi(row[14]);
		cLev.getkey=stoi(row[15]);
		cLev.greet=stoi(row[16]);
		cLev.halfop=stoi(row[17]);
		cLev.halfopme=stoi(row[18]);
		cLev.info=stoi(row[19]);
		cLev.invite=stoi(row[20]);
		cLev.kick=stoi(row[21]);
		cLev.kickme=stoi(row[22]);
		cLev.memo=stoi(row[23]);
		cLev.nojoin=stoi(row[24]);
		cLev.nokick=stoi(row[25]);
		cLev.opdeop=stoi(row[26]);
		cLev.opdeopme=stoi(row[27]);
		cLev.protect=stoi(row[28]);
		cLev.protectme=stoi(row[29]);
		cLev.say=stoi(row[30]);
		cLev.set=stoi(row[31]);
		cLev.signkick=stoi(row[32]);
		cLev.topic=stoi(row[33]);
		cLev.unban=stoi(row[34]);
		cLev.voice=stoi(row[35]);
		cLev.voiceme=stoi(row[36]);
	}
	mysql_free_result_myr(res);
	return cLev;
}

/*  ************************************************************************************************  */

/**
 * @brief sql_aliases:  Selects information about
 *                  a single user from table `orion_ns_aliases`
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param aliasnick The nick for which we like to receive information
 * @return Aliases Returns a struct that includes the information from `orion_ns_aliases` for a single user
 **/
Aliases sql_aliases(Threads *thread, char* aliasnick){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
        
	char nickdata[MAX_NICK_LEN*2], query[SQL_BUF];
	/* STRUCT */ Aliases aliases; /* STRUCT */
                                                
	aliases.fields=0;
	aliases.nick[0]='\0';
	aliases.authnick[0]='\0';
	aliases.last_vhost[0]='\0';
	aliases.last_realname[0]='\0';
	aliases.last_quit[0]='\0';
	aliases.time_linked=(time_t)0;
	aliases.last_login=(time_t)0;
	aliases.last_used=(time_t)0;
	aliases.active=0;
	aliases.expire=0;
	aliases.forbid=0;
	aliases.linked=1;
                                                
	if(aliasnick==NULL) return aliases;
	
	/* Aliases */
	bzero(nickdata,sizeof(nickdata));
	bzero(query,sizeof(query));
	
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, nickdata, aliasnick, strlen(aliasnick)); //escape authnick -> nickdata
	snprintf(query, sizeof(query),"SELECT `nick`,`display`,`last_usermask`,`last_realname`,`last_quit`,`time_linked`,`last_login`,`last_used`,`expire`,`linked`,`active`,`forbid` FROM `orion_ns_aliases` WHERE `nick`='%s'",nickdata); //prepare query

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to MYSQL result pointer
	
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res); //fetch result to the MYSQL **row array
		aliases.fields=mysql_num_fields(res); //get number of Array entries
		dynncpy(aliases.nick,row[0],sizeof(aliases.nick));
		dynncpy(aliases.authnick,row[1],sizeof(aliases.authnick));
		dynncpy(aliases.last_vhost,row[2],sizeof(aliases.last_vhost));
		dynncpy(aliases.last_realname,row[3],sizeof(aliases.last_realname));
		dynncpy(aliases.last_quit,row[4],sizeof(aliases.last_quit));
		aliases.time_linked=(time_t)stol(row[5]);
		aliases.last_login=(time_t)stol(row[6]);
		aliases.last_used=(time_t)stol(row[7]);
		aliases.expire=stoi(row[8]);
		aliases.linked=stoi(row[9]); //0-permanent, 1-temporarily
		aliases.active=stoi(row[10]);
		aliases.forbid=stoi(row[11]);
	}
	mysql_free_result_myr(res); //free result
	return aliases; //return access struct
}

/*  ************************************************************************************************  */

/**
 * @brief sql_assign:  Selects information about
 *                 a botserv channel, if a bot is holding it from table `orion_cs_core`
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The channel we have to proof for an assigned bot (usable also for the query)
 * @return Bassign Returns a struct that includes the information from `botassign` for a single channel
 **/
Bassign sql_assign(Threads *thread, char* channel){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
	
	char chandata[MAX_CHANNELNAME_LEN*2], query[SQL_BUF];
	/* STRUCT */ Bassign assign; /* STRUCT */
                                                
	assign.fields=0;
	assign.bot[0]='\0';
	assign.channel[0]='\0';
	assign.set=-1;
                                                
	if(channel==NULL) return assign;
	
	/* BotAssign */
	bzero(chandata,sizeof(chandata));
	bzero(query,sizeof(query));
	
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, chandata, channel, strlen(channel)); //escape channel -> chandata
	snprintf(query, sizeof(query),"SELECT `botnick`,`channel`,`bs_set` FROM `orion_cs_core` WHERE `channel`='%s' AND `botnick`!=NULL",chandata); //prepare query

	ExecuteQuery(thread,query,dynlen(query)); //execute query	
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to MYSQL result pointer
	
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res); //fetch result to the MYSQL **row array
		assign.fields=mysql_num_fields(res); //get number of Array entries
		dynncpy(assign.bot,row[0],sizeof(assign.bot));
		dynncpy(assign.channel,row[1],sizeof(assign.channel));
		assign.set=stoi(row[2]);
	}
	mysql_free_result_myr(res); //free result
	return assign; //return assign struct
}

/*  ************************************************************************************************  */

/**
 * @brief sql_bkick:  Selects information about
 *                a botserv channel, and it's kick settings `orion_cs_core`
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The channel we have to proof for it's kicksettings
 * @return Bkick Returns a struct that includes the information about the kicksettings from `botkick` for a single channel
 **/
Bkick sql_bkick(Threads *thread, char* channel){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/

	char query[SQL_BUF];
	/* STRUCT */ Bkick bkick; /* STRUCT */
                                                
	bkick.fields=0;
	bkick.channel[0]='\0';
	bkick.caps_min=0;
	bkick.caps_per=0;
	bkick.flood_ln=0;
	bkick.flood_secs=0;
	bkick.repeat=0;
	bkick.colors=0;
	bkick.underlines=0;
	bkick.reverses=0;
	bkick.badwords=0;
	bkick.bolds=0;
                                                
	if(channel==NULL) return bkick;
	
	/* BotKick */
	bzero(&query,sizeof(query));
	
	snprintf(query, sizeof(query),"SELECT `channel`,`caps_min`,`caps_per`,`flood_ln`,`flood_secs`,`repeat`,`colors`,`underlines`,`reverses`,`badwords`,`bolds` FROM `orion_cs_core` WHERE `channel`='%s' AND `botnick`!=NULL",mysql_quote(thread,channel)); //prepare query

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to MYSQL result pointer
	
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res); //fetch result to the MYSQL **row array
		bkick.fields=mysql_num_fields(res); //get number of Array entries
		dynncpy(bkick.channel,row[0],sizeof(bkick.channel));
		bkick.caps_min=stoi(row[1]);
		bkick.caps_per=stoi(row[2]);
		bkick.flood_ln=stoi(row[3]);
		bkick.flood_secs=stoi(row[4]);
		bkick.repeat=stoi(row[5]);
		bkick.colors=stoi(row[6]);
		bkick.underlines=stoi(row[7]);
		bkick.reverses=stoi(row[8]);
		bkick.badwords=stoi(row[9]);
		bkick.bolds=stoi(row[10]);
	}
	mysql_free_result_myr(res); //free result
	return bkick; //return bkick struct
}

/*  ************************************************************************************************  */

/**
 * @brief count:  Counts the channels which a single user is able to access
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param serv The service which is holding target channel
 * @param authnick The authnick of target user, to count his access entries / bans
 * @return int return_val Returns the counted entries (integer)
 **/
int count(Threads *thread, const char* serv, char* authnick){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
    
	int return_val=0;
	char query[SQL_BUF], nickdata[MAX_NICK_LEN*2];
	if(authnick==NULL || serv==NULL) return 0;

	bzero(&nickdata,sizeof(nickdata));
	bzero(&query,sizeof(query));

	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, nickdata, authnick, strlen(authnick));
	snprintf(query, sizeof(query),"SELECT count(1) FROM `orion_cs_access` WHERE `authnick`='%s'",nickdata);

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE);
	
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res);
		return_val = stoi(row[0]);
	}
	else return_val=0;

	mysql_free_result_myr(res);
	return return_val;
}

/*  ************************************************************************************************  */

/**
 * @brief currmodes:  Selects the usermodes for target user
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The nick for whom we like to select the usermodes
 * @return char* modes Returns the usermodes for target user
 **/
char* currmodes(Threads *thread, char* nick){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/

#ifndef USE_THREADS
	static char modes[MAX_CHAN_MODES];
#endif
	char query[SQL_BUF], nickdata[MAX_NICK_LEN*2];
	if(nick==NULL) return NULL;

#ifndef USE_THREADS
	bzero(&modes,sizeof(modes));
#else
	bzero(&thread->modes,sizeof(thread->modes));
#endif

	bzero(&nickdata,sizeof(nickdata));
	bzero(&query,sizeof(query));

	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, nickdata, nick, dynlen(nick));
	snprintf(query, sizeof(query),"SELECT `modes` FROM `orion_core_condata` WHERE `nick`='%s'",nickdata);

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE);
	
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res);
#ifndef USE_THREADS
		dynncpy(modes,row[0],sizeof(modes));
	}
	else modes[0] = '\0';
#else
		dynncpy(thread->modes,row[0],sizeof(thread->modes));
	}
	else bzero(&thread->modes,sizeof(thread->modes));
#endif

	mysql_free_result_myr(res);

#ifndef USE_THREADS
	return modes;
#else
	return thread->modes;
#endif
}

/*  ************************************************************************************************  */

/**
 * @brief sql_users:  Used to get knowledge if a user is in a channel or not
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param chan The channel where we want to know if target nick is in
 * @param nick The nick in target channel
 * @return char* user Returns the counted entries (integer) 0 or 1
 **/
char *sql_users(Threads *thread, char* chan, char* nick){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
     
#ifndef USE_THREADS
	static char user[MAX_NICK_LEN];
#endif
	char nickdata[MAX_NICK_LEN*2], chandata[MAX_CHANNELNAME_LEN*2], query[SQL_BUF];
	if(chan==NULL || nick==NULL) return NULL;

#ifdef USE_THREADS
	bzero(&thread->user,sizeof(thread->user));
#else
	bzero(&user,sizeof(user));
#endif
	bzero(&nickdata,sizeof(nickdata));
	bzero(&chandata,sizeof(chandata));
	bzero(&query,sizeof(query));

	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, nickdata, nick, strlen(nick));
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, chandata, chan, strlen(chan));

	snprintf(query, sizeof(query),"SELECT `nick` FROM `orion_chan_users` WHERE `channel`='%s' AND `nick`='%s'",chandata,nickdata);
	
    ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE);
	
	if(mysql_num_rows(res)>0){
		row = mysql_fetch_row(res);
#ifndef USE_THREADS		
	  	dynncpy(user,row[0],sizeof(user));
	}
	else bzero(&user,sizeof(user));
#else
	  	dynncpy(thread->user,row[0],sizeof(thread->user));
	}
	else bzero(&thread->user,sizeof(thread->user));
#endif

	mysql_free_result_myr(res);
	
#ifndef USE_THREADS
	return user;
#else
	return thread->user;
#endif
}

/*  ************************************************************************************************  */

/**
 * @brief acclevel:  Selects the access level for a given channel
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param serv The service which is holding target channel
 * @param channel The channel from which we like to receive the access-level for target user
 * @param authnick The authnick of which we like to know the access-level for target channel
 * @return char* return_val Returns the access-level for target user in target channel
 **/
char* acclevel(Threads *thread, char* serv, char* channel, char* authnick){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
      
#ifndef USE_THREADS
	static char level[MAX_LEVEL_LEN];
#endif
	char query[SQL_BUF], nickdata[MAX_NICK_LEN*2], chandata[MAX_CHANNELNAME_LEN*2];
	if(authnick==NULL || channel==NULL || serv==NULL) return NULL;

#ifndef USE_THREADS
	bzero(&level,sizeof(level));
#else
	bzero(&thread->level,sizeof(thread->level));
#endif
	
	bzero(&nickdata,sizeof(nickdata));
	bzero(&chandata,sizeof(chandata));
	bzero(&query,sizeof(query));

	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, nickdata, authnick, strlen(authnick));
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, chandata, channel, strlen(channel));

	snprintf(query, sizeof(query),"SELECT `level` FROM `orion_cs_access` WHERE `serv`='%s' AND `authnick`='%s' AND `channel`='%s' LIMIT 0,1",mysql_quote(thread,serv),nickdata,chandata);

	ExecuteQuery(thread,query,dynlen(query)); //execute query
    res=mysql_store_result(MYSQL_CONNECTION_HANDLE);
    
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res);
#ifndef USE_THREADS
		dynncpy(level,row[0],sizeof(level));
	}
	else bzero(&level,sizeof(level));
#else
	  	dynncpy(thread->level,row[0],sizeof(thread->level));
	}
	else bzero(&thread->level,sizeof(thread->level));
#endif

	mysql_free_result_myr(res);
	
#ifndef USE_THREADS
	return level;
#else
	return thread->level;
#endif
}

/*  ************************************************************************************************  */

/**
 * @brief accessselect:  Selects information from any single field out of `orion_ns_core`
 *                   depending on *field
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param field The field we want to get information from
 * @param authnick The authnick from which we like to know the information which is hold by *field
 * @return char* return_val Returns the information that is hold by the given *field
 **/
char* accessselect(Threads *thread, const char* field, char* authnick){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
      
#ifndef USE_THREADS
	static char select[200];
#endif
	char query[SQL_BUF], anickdata[MAX_NICK_LEN*2], fielddata[200];
	if(authnick==NULL || field==NULL) return NULL;

#ifndef USE_THREADS
	bzero(&select,sizeof(select));
#else
	bzero(&thread->select,sizeof(thread->select));
#endif
	bzero(&anickdata,sizeof(anickdata));
	bzero(&fielddata,sizeof(fielddata));
	bzero(&query,sizeof(query));

	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, fielddata, field, strlen(field));
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, anickdata, authnick, strlen(authnick));
	snprintf(query, sizeof(query),"SELECT `%s` FROM `orion_ns_core` WHERE `authnick`='%s'",fielddata,anickdata);

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE);
	
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res);
#ifndef USE_THREADS
		dynncpy(select,row[0],sizeof(select));
	}
	else bzero(&select,sizeof(select));
#else
		dynncpy(thread->select,row[0],sizeof(thread->select));
	}
	else bzero(&thread->select,sizeof(thread->select));
#endif

	mysql_free_result_myr(res);

#ifndef USE_THREADS
	return select;
#else
	return thread->select;
#endif
}

/*  ************************************************************************************************  */

/**
 * @brief selectaliases:  Selects any information about a single user from `orion_ns_aliases`
 *                    depending on *field, which is going to be selected
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param field The fields which we like to select from `orion_ns_aliases`
 * @param nnick The nick for which we like to receive this *field
 * @return char* return_val Returns the pointer to the static (returns the data for *field)
 **/
char* selectaliases(Threads *thread, const char* field, char* nnick){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
      
#ifndef USE_THREADS
	static char select[200];
#endif
	char query[SQL_BUF], nick[MAX_NICK_LEN*2], fielddata[200];
	if(nnick==NULL || field==NULL) return NULL;

#ifndef USE_THREADS
	bzero(&select,sizeof(select));
#else
	bzero(&thread->aselect,sizeof(thread->aselect));
#endif
	bzero(&nick,sizeof(nick));
	bzero(&fielddata,sizeof(fielddata));
	bzero(&query,sizeof(query));

	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, fielddata, field, strlen(field));
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, nick, nnick, strlen(nnick));

	snprintf(query, sizeof(query),"SELECT `%s` FROM `orion_ns_aliases` WHERE `nick`='%s'",fielddata,nick);
	
	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE);
	
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res);
#ifndef USE_THREADS
		dynncpy(select,row[0],sizeof(select));
	}
	else bzero(&select,sizeof(select));
#else
		dynncpy(thread->aselect,row[0],sizeof(thread->aselect));
	}
	else bzero(&thread->aselect,sizeof(thread->aselect));
#endif

	mysql_free_result_myr(res);
#ifndef USE_THREADS
	return select;
#else
	return thread->aselect;
#endif
}

/*  ************************************************************************************************  */

/**
 * @brief SingleLevel:  Selects the level for executing target command in target channel
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param field The identifier for selecting the level
 * @param channel The channel from which we like the level for *field
 * @return int Returns the level for target channel (depending on *field)
 **/
int SingleLevel(Threads *thread, const char* field, char* channel){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
    
	int select;
	char query[SQL_BUF], chan[MAX_CHANNELNAME_LEN*2], fielddata[200];
	if(channel==NULL || field==NULL) return 0;

	select=INFINIT;
	bzero(&chan,sizeof(chan));
	bzero(&fielddata,sizeof(fielddata));
	bzero(&query,sizeof(query));

	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, fielddata, field, strlen(field));
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, chan, channel, strlen(channel));
	snprintf(query, sizeof(query),"SELECT `%s` FROM `orion_cs_core` WHERE `channel`='%s'",fielddata,chan);

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE);
	
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res);
		select=stoi(row[0]);
	}
	mysql_free_result_myr(res); //free result
	return select;
}

/*  ************************************************************************************************  */

/**
 * @brief sql_condata:  Selects connection-information for a single user
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The nick for which we like to get additional information, concerning his connect
 * @return Condata Returns a struct which holds the information selected from `orion_core_condata` for a single user
 **/
Condata sql_condata(Threads *thread, char* nick){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
        
	char query[SQL_BUF], nickdata[MAX_NICK_LEN*2];
	/* STRUCT */ Condata Cdata; /* STRUCT */
                                                
	Cdata.fields=0;
	Cdata.nick[0]='\0';
	Cdata.time=(time_t)0;
	Cdata.localtime=(time_t)0;
	Cdata.rident[0]='\0';
	Cdata.ident[0]='\0';
	Cdata.rhost[0]='\0';
	Cdata.server[0]='\0';
	Cdata.modes[0]='\0';
	Cdata.vhost[0]='\0';
	Cdata.realname[0]='\0';
	Cdata.swhois[0]='\0';
	Cdata.subnet[0]='\0';
	Cdata.ip[0]='\0';
	Cdata.ns_warntime=(time_t)0;
	Cdata.faketype=0;
                                                
	if(nick==NULL) return Cdata;
	
	/* Condata */
	bzero(&nickdata,sizeof(nickdata));
	bzero(&query,sizeof(query));

	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, nickdata, nick, strlen(nick));
	snprintf(query, sizeof(query),"SELECT `nick`,`time`,`localtime`,`rident`,`ident`,`rhost`,`srv`,`modes`,`vhost`,`realname`,`swhois`,`subnet`,`ip`,`ns_warntime`,`faketype` FROM `orion_core_condata` WHERE `nick`='%s'",nickdata);

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE);
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res);
		Cdata.fields=mysql_num_fields(res);
		
		dynncpy(Cdata.nick,row[0],sizeof(Cdata.nick));
		Cdata.time=(time_t)stol(row[1]);
		Cdata.localtime=(time_t)stol(row[2]);
		dynncpy(Cdata.rident,row[3],sizeof(Cdata.rident));
		dynncpy(Cdata.ident,row[4],sizeof(Cdata.ident));
		dynncpy(Cdata.rhost,row[5],sizeof(Cdata.rhost));
		dynncpy(Cdata.server,row[6],sizeof(Cdata.server));
		dynncpy(Cdata.modes,row[7],sizeof(Cdata.modes));
		dynncpy(Cdata.vhost,row[8],sizeof(Cdata.vhost));
		dynncpy(Cdata.realname,row[9],sizeof(Cdata.realname));
		dynncpy(Cdata.swhois,row[10],sizeof(Cdata.swhois));
		dynncpy(Cdata.subnet,row[11],sizeof(Cdata.subnet));
		dynncpy(Cdata.ip,row[12],sizeof(Cdata.ip));
		Cdata.ns_warntime=(time_t)stol(row[13]);
		Cdata.faketype=stoi(row[14]);
	}
	mysql_free_result_myr(res);
	return Cdata;
}

/*  ************************************************************************************************  */

/**
 * @brief O_sql_trust:  Selects a connection-limit for target host
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param host The host for which a limit should be selected
 * @return Trust Returns the found trust for target host
 **/
Trust O_sql_trust(Threads *thread, char *host){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
      
	Trust O_trust;

	O_trust.fields=0;
	O_trust.description[0]='\0';
	O_trust.rhost[0]='\0';
	O_trust.limit[0]='\0';

	char query[SQL_BUF], nhost[MAX_HOST_LEN*2];
	char *temprhost = NULL;
	int found=0, fields=0, i=0;

	if(host==NULL || host[0] == '\0') return O_trust;

	bzero(&nhost,sizeof(nhost));
	bzero(&query,sizeof(query));
	
	dyncpy(query,"SELECT `rhost` FROM `orion_os_trusts` WHERE `art`='1'");

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE);
	fields=mysql_num_rows(res);
	if(fields){
		
		for(i=0;i<fields;i++){
			row = mysql_fetch_row(res);
			if(streqnc(row[0],host)){
				temprhost = host;
				found=1;
				break;
			}
			if(strins(row[0],"*.")){
				char tmprepl[200];
				dynncpy(tmprepl,row[0],sizeof(tmprepl));
				strdels(tmprepl,'n',"*.");
				if(strins(host,tmprepl)){
					temprhost=row[0];
					found=1;
					break;
				}
			}
		}
	}
	mysql_free_result_myr(res);
	
	if(found && temprhost){
		bzero(&query,sizeof(query));
		//printf("found = %d, temprhost = %s\n",found,temprhost);
		mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, nhost, temprhost, strlen(temprhost));
		//query = MySQL query
		snprintf(query, sizeof(query),"SELECT `rhost`,`limit`,`description` FROM `orion_os_trusts` WHERE `rhost`='%s' AND `art`='1'",nhost);
		ExecuteQuery(thread,query,dynlen(query)); //execute query
	
		res=mysql_store_result(MYSQL_CONNECTION_HANDLE);
		fields=mysql_num_rows(res);
		
		if(fields){
			row=mysql_fetch_row(res);
			O_trust.fields=mysql_num_fields(res);
			
			dynncpy(O_trust.rhost,row[0],sizeof(O_trust.rhost));
			dynncpy(O_trust.limit,row[1],sizeof(O_trust.limit));
			dynncpy(O_trust.description,row[2],sizeof(O_trust.description));
		}
		mysql_free_result_myr(res);
		return O_trust;
	}else{
		return O_trust;
	}
}

/*  ************************************************************************************************  */

/**
 * @brief Q_userlist:  Selects information from `orion_ns_core` by using pattern_in for `nick`
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param pattern_in The pattern for a nick to use in LIKE (so pattern_in is a nick-mask)
 * @return QUserlist Returns a UserList pointer to the static, which is holding the information
 **/
QUserlist *Q_userlist(Threads *thread, char *pattern_in){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
          
	static char Q_users[100+1][MAX_NICK_LEN];
	static char *QUsers[100+1];
	static QUserlist UserList;

	char query[SQL_BUF];
	char escaped[SQL_BUF];
	char pattern[500];
	int i=0, fields=0;
	
	bzero(&pattern,sizeof(pattern));
	if(pattern_in!='\0'){ dynncpy(pattern,pattern_in,sizeof(pattern)); }

	/* Terminating Strings */
	bzero(Q_users[0],sizeof(Q_users[0]));
	UserList.users=QUsers;
	UserList.count_users=0;
	/**********************/

	if(pattern_in!='\0' && pattern[0]!='\0' && strcor(pattern,"*")){
		strrepc(pattern,'*','%');
		mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, escaped, pattern, strlen(pattern));
		snprintf(query, sizeof(query),"SELECT `authnick` FROM `orion_ns_core` WHERE `nick` LIKE '%s' LIMIT 100",escaped);

	}else if(pattern_in==NULL){
		snprintf(query, sizeof(query),"SELECT `authnick` FROM `orion_ns_core`");
	}

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	fields=mysql_num_rows(res);
	
	if(fields){
		for(i=0;i<fields;i++){
			row = mysql_fetch_row(res);
			dynncpy(Q_users[UserList.count_users],row[0],sizeof(Q_users[UserList.count_users]));
			QUsers[UserList.count_users]=Q_users[UserList.count_users];
			UserList.count_users++;
		}
		return &UserList;
	}else return '\0';
}

/*  ************************************************************************************************  */

/**
 * @brief domysql:  Executes a single query with no reponse from the SQL-Server
 *              (UPDATE, INSERT, REPLACE, DELETE) in example
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param text The query for sending it to the MySQL-Server
 * @return void
 **/
void domysql(Threads *thread, const char* text, ...){
	char query[SQL_BUF+1];
	//char what[10];
	if(text==NULL) return;

	bzero(&query,sizeof(query));

	/* preparing Query */
	va_list va;
	va_start(va, text);
	vsnprintf(query, sizeof(query)-1, text, va);
	va_end(va);
	/*******************/
	
	ExecuteQuery(thread,query,dynlen(query));
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief Cmysql_sel_LColumn:  Selects one column out of any table (but only one column! - no lines!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param SQLdata The SQL-Struct where we want to save information to
 * @param text The query for sending it to the MySQL-Server
 * @return LColumn Returns the selected column with it's information
 **/
LColumn Cmysql_sel_LColumn(Threads *thread, deepSQL *SQLdata, const char* text, ...){ //List Column - SELECT 1 FROM `table` - 100 values back in a list (LIST Column)
	int i=0;
	char query[SQL_BUF+1];
	LColumn C;
	
	C.row[0]=NULL;
	C.fields=0;
	if(text==NULL) return C;

	bzero(&query,sizeof(query));

	/* preparing Query */
	va_list va;
	va_start(va, text);
	vsnprintf(query, sizeof(query)-1, text, va);
	va_end(va);
	/*******************/

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	SQLdata->res    = mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to the result pointer
	SQLdata->fields = mysql_num_rows(SQLdata->res);
	
	if(SQLdata->fields){
		for(i=0;i<SQLdata->fields;i++){
			SQLdata->row = mysql_fetch_row(SQLdata->res);
            if(i<MAX_SQL_COLUMN_SELECT_LIST){
    			C.row[i]=SQLdata->row[0];
    			i++;
    		}
        }
		C.fields=i;
		C.row[i]='\0';
		return C;
	}else return C;
}

/*  ************************************************************************************************  */

/**
 * @brief Cmysql_sel_RColumn:  Selects one line out of any table (but only one line!! - no columns)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param SQLdata The SQL-Struct where we want to save information to
 * @param text The query for sending it to the MySQL-Server
 * @return RColumn Returns the selected line with it's information
 **/
RColumn Cmysql_sel_RColumn(Threads *thread, deepSQL *SQLdata, const char* text, ...){ //Values Column SELECT 1,2,3,4,5 FROM ... (ROW column)
	int i=0;
	char query[SQL_BUF+1];
	RColumn C;
	
	C.row[0]='\0';
	C.fields=0;
	if(text==NULL) return C;

	bzero(&query,sizeof(query));

	/* preparing Query */
	va_list va;
	va_start(va, text);
	vsnprintf(query, sizeof(query)-1, text, va);
	va_end(va);
	/*******************/
	
	ExecuteQuery(thread,query,dynlen(query)); //execute query
	
	SQLdata->res=mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to the result pointer

	if(mysql_num_rows(SQLdata->res)>0){
		C.fields = SQLdata->fields = mysql_num_fields(SQLdata->res);
		C.row[SQLdata->fields]='\0';
		SQLdata->row=mysql_fetch_row(SQLdata->res);
		for(i=0;i<C.fields;i++){
			C.row[i]=SQLdata->row[i];
		}
		return C;
	}else return C;
}

/*  ************************************************************************************************  */

/**
 * @brief mysql_sel_var:  Selects one field out of any table (but only one field!)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param text The query for sending it to the MySQL-Server
 * @return char* Returns the selected field
 **/
char* mysql_sel_var(Threads *thread, const char* text, ...){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
      
	char query[SQL_BUF+1];
#ifndef USE_THREADS
	static char selected[300];
#endif
	
	if(text==NULL) return NULL;

	bzero(&query,sizeof(query));
#ifndef USE_THREADS
	bzero(&selected,sizeof(selected));
#else
	bzero(&thread->mysql_sel_var,sizeof(thread->mysql_sel_var));
#endif

	/* preparing Query */
	va_list va;
	va_start(va, text);
	vsnprintf(query, sizeof(query)-1, text, va);
	va_end(va);
	/*******************/

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to the result pointer	
	if(mysql_num_rows(res)>0){
		row=mysql_fetch_row(res);
#ifndef USE_THREADS
		dynncpy(selected,row[0],sizeof(selected));
	}
	else bzero(&selected,sizeof(selected));
#else
		dynncpy(thread->mysql_sel_var,row[0],sizeof(thread->mysql_sel_var));
	}
	else bzero(&thread->mysql_sel_var,sizeof(thread->mysql_sel_var));
#endif
	
	mysql_free_result_myr(res); //free result
#ifndef USE_THREADS
	return selected;
#else
	return thread->mysql_sel_var;
#endif
}

/*  ************************************************************************************************  */

/**
 * @brief mysql_sel:  Counts the channels which a single user is able to access
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The nick who executes the query (Information receiver)
 * @param text The SQL-Query
 * @return void
 **/
void mysql_sel(Threads *thread, char *nick, char* text){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
     
	int fields=0, i=0;
	char query[SQL_BUF], nrow[500];
	
	if(text==NULL || nick==NULL) return;

	bzero(&query,sizeof(query));
	dynncpy(query,text,sizeof(query));

	if(mysql_real_query(MYSQL_CONNECTION_HANDLE, query, dynlen(query))){ //executing Query
		do_privmsg(thread,m->nick,nick,"ERROR %s",mysql_error(MYSQL_CONNECTION_HANDLE));
		return;
	}

	res=mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to the result pointer
	fields=mysql_num_fields(res);
	
	if(mysql_num_rows(res)>0){
		for(i=0;i<fields;i++){
			row = mysql_fetch_row(res);
			
			bzero(nrow,sizeof(nrow));
			dynncpy(nrow,row[0],sizeof(nrow));
			for(i=1;row[i][0]!='\0';i++){
				straddns(nrow," | ",sizeof(nrow));
				straddns(nrow,row[i],sizeof(nrow));
			}
			do_privmsg(thread,m->nick,nick,"SQL %s",nrow);
		}
	}
	mysql_free_result_myr(res); //free result
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief sel_columns:  Selects multiply columns and rows
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param text The query for sending it to the MySQL-Server
 * @return SQLres* Returns the SQL-Result variable
 **/
SQLres *doSelectMySQL(Threads *thread, const char* text, ...){
	char query[SQL_BUF+1];
	SQLres *result;
	if(text==NULL) return NULL;

	bzero(&query,sizeof(query));

	/* preparing Query */
	va_list va;
	va_start(va, text);
	vsnprintf(query, sizeof(query)-1, text, va);
	va_end(va);
	/*******************/

#ifdef USE_THREADS
	my_pthread_mutex_lock(&mysql_lock);
#endif

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	result = mysql_store_result(MYSQL_CONNECTION_HANDLE);
	
#ifdef USE_THREADS
	my_pthread_mutex_unlock(&mysql_lock);
#endif
	return result;
}

/*  ************************************************************************************************  */

/**
 * @brief sql_BsChan:  Selects information about
 *                     the channel levels for target channel from table `orion_cs_core`
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The nick for which we like to have some additional information
 * @param channel The channel for which we like to have some additional information
 * @param bsChan The BotServ Channel structure where the data should be loaded
 * @return void
 **/
void sql_BsChan(Threads *thread, char *nick,char* channel,BsChan *bsChan){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
           
	char query[SQL_BUF];
	//int32 i = 0;
	//int rows	= 0;
      
	bsChan->fields =0;
	bsChan->data   =0;
	
	bsChan->authnick[0]='\0';
	bsChan->qlevel=0;
	bsChan->olevel=0;
	bsChan->llevel=0;
	bsChan->slevel=0;
	bsChan->rights=0;
	bsChan->active=0;
	
	bsChan->vhost[0]='\0';
	bsChan->ident[0]='\0';
	bsChan->server[0]='\0';
	
	bsChan->name[0]='\0';
	bsChan->bot[0]='\0';
	bsChan->serv[0]='\0';
	bsChan->founder[0]='\0';
	bsChan->successor[0]='\0';
	bsChan->datecreate=(time_t)0;
	bsChan->regdate=(time_t)0;
	bsChan->description[0]='\0';
	bsChan->mode[0]='\0';
	bsChan->mlock_positive[0]='\0';
	bsChan->mlock_negative[0]='\0';
	bsChan->mlock_limit=0;
	bsChan->mlock_key[0]='\0';
	bsChan->mlock_flood[0]='\0';
	bsChan->mlock_redirect[0]='\0';
	bsChan->link[0]='\0';
	bsChan->limit=0;
	bsChan->key[0]='\0';
	bsChan->flood[0]='\0';
	bsChan->last_topic[0]='\0';
	bsChan->last_topic_setter[0]='\0';
	bsChan->last_topic_time=(time_t)0;
	bsChan->url[0]='\0';
	bsChan->alimit=0;
	bsChan->flags[0]='\0';
	bsChan->welcome[0]='\0';
	bsChan->last_used=(time_t)0;
	bsChan->settings=0;
	bsChan->email[0]='\0';
	bsChan->expire=1;
	bsChan->maxmemos=0;
	bsChan->reason[0]='\0';
	
	bsChan->level[0]='\0';
	bsChan->cu_modes[0]='\0';
	bsChan->cu_nick[0]='\0';

	bsChan->cs_accchange=0;
	bsChan->cs_acclist=0;
	bsChan->cs_akick=0;
	bsChan->cs_autodeop=0;
	bsChan->cs_autohalfop=0;
	bsChan->cs_autoop=0;
	bsChan->cs_autoprotect=0;
	bsChan->cs_autovoice=0;
	bsChan->cs_badwords=0;
	bsChan->cs_ban=0;
	bsChan->cs_banme=0;
	bsChan->cs_assign=0;
	bsChan->cs_clear=0;
	bsChan->cs_fantasia=0;
	bsChan->cs_getkey=0;
	bsChan->cs_greet=0;
	bsChan->cs_halfop=0;
	bsChan->cs_halfopme=0;
	bsChan->cs_info=0;
	bsChan->cs_invite=0;
	bsChan->cs_kick=0;
	bsChan->cs_kickme=0;
	bsChan->cs_memo=0;
	bsChan->cs_nojoin=0;
	bsChan->cs_nokick=0;
	bsChan->cs_opdeop=0;
	bsChan->cs_opdeopme=0;
	bsChan->cs_protect=0;
	bsChan->cs_protectme=0;
	bsChan->cs_say=0;
	bsChan->cs_set=0;
	bsChan->cs_signkick=0;
	bsChan->cs_topic=0;
	bsChan->cs_unban=0;
	bsChan->cs_voice=0;
	bsChan->cs_voiceme=0;
	
	bsChan->bs_set=-1;
	bsChan->bs_caps_ttb=-1;
	bsChan->bs_caps_min=-1;
	bsChan->bs_caps_per=-1;
	bsChan->bs_flood_ttb=-1;
	bsChan->bs_flood_ln=-1;
	bsChan->bs_flood_secs=-1;
	bsChan->bs_repeat_ttb=-1;
	bsChan->bs_repeat=-1;
	bsChan->bs_colors=-1;
	bsChan->bs_underlines=-1;
	bsChan->bs_reverses=-1;
	bsChan->bs_badwords=-1;
	bsChan->bs_bolds=-1;
                                            
	if(channel==NULL || nick==NULL) return;
	
	/* Channel */
	bzero(&query,sizeof(query));
	
	snprintf(query, sizeof(query),

	"SELECT "
	
		"`orion_ns_aliases`.`display`,"
		"`orion_ns_aliases`.`active`,"
		"`orion_ns_core`.`qlevel`,"
		"`orion_ns_core`.`olevel`,"
		"`orion_ns_core`.`llevel`,"
		"`orion_ns_core`.`slevel`,"
		"`orion_ns_core`.`rights`,"
			
		"`orion_chan_users`.`nick`,"
		"`orion_chan_users`.`modes`,"
		"`orion_cs_access`.`level`,"
	
		"`orion_cs_core`.`serv`,"
		"`orion_cs_core`.`botnick`,"
		"`orion_cs_core`.`channel`,"
		"`orion_cs_core`.`founder`,"
		"`orion_cs_core`.`successor`,"
		"`orion_cs_core`.`datecreate`,"
		"`orion_cs_core`.`description`,"
		"`orion_cs_core`.`mode`,"
		"`orion_cs_core`.`mlock_positive`,"
		"`orion_cs_core`.`mlock_negative`,"
		"`orion_cs_core`.`mlock_limit`,"
		"`orion_cs_core`.`mlock_key`,"
		"`orion_cs_core`.`mlock_flood`,"
		"`orion_cs_core`.`link`,"
		"`orion_cs_core`.`limit`,"
		"`orion_cs_core`.`key`,"
		"`orion_cs_core`.`flood`,"
		"`orion_cs_core`.`last_topic`,"
		"`orion_cs_core`.`last_topic_setter`,"
		"`orion_cs_core`.`last_topic_time`,"
		"`orion_cs_core`.`url`,"
		"`orion_cs_core`.`alimit`,"
		"`orion_cs_core`.`flags`,"
		"`orion_cs_core`.`welcome`,"
		"`orion_cs_core`.`mlock_redirect`,"
		"`orion_cs_core`.`last_used`,"
		"`orion_cs_core`.`reason`,"
		"`orion_cs_core`.`settings`,"
		"`orion_cs_core`.`email`,"
		"`orion_cs_core`.`expire`,"
		"`orion_cs_core`.`maxmemos`,"
	
		"`orion_cs_core`.`cs_accchange`,"
		"`orion_cs_core`.`cs_acclist`,"
	    "`orion_cs_core`.`cs_akick`,"
		"`orion_cs_core`.`cs_autodeop`,"
		"`orion_cs_core`.`cs_autohalfop`,"
		"`orion_cs_core`.`cs_autoop`,"
	    "`orion_cs_core`.`cs_autoprotect`,"
		"`orion_cs_core`.`cs_autovoice`,"
		"`orion_cs_core`.`cs_badwords`,"
		"`orion_cs_core`.`cs_ban`,"
		"`orion_cs_core`.`cs_banme`,"
	    "`orion_cs_core`.`cs_botassign`,"
		"`orion_cs_core`.`cs_cclear`,"
		"`orion_cs_core`.`cs_fantasia`,"
		"`orion_cs_core`.`cs_getkey`,"
		"`orion_cs_core`.`cs_greet`,"
		"`orion_cs_core`.`cs_halfop`,"
		"`orion_cs_core`.`cs_halfopme`,"
	    "`orion_cs_core`.`cs_info`,"
		"`orion_cs_core`.`cs_invite`,"
		"`orion_cs_core`.`cs_kick`,"
		"`orion_cs_core`.`cs_kickme`,"
		"`orion_cs_core`.`cs_memo`,"
		"`orion_cs_core`.`cs_nojoin`,"
	    "`orion_cs_core`.`cs_nokick`,"
		"`orion_cs_core`.`cs_opdeop`,"
		"`orion_cs_core`.`cs_opdeopme`,"
		"`orion_cs_core`.`cs_protect`,"
	    "`orion_cs_core`.`cs_protectme`,"
		"`orion_cs_core`.`cs_say`,"
		"`orion_cs_core`.`cs_set`,"
		"`orion_cs_core`.`cs_signkick`,"
		"`orion_cs_core`.`cs_topic`,"
		"`orion_cs_core`.`cs_unban`,"
		"`orion_cs_core`.`cs_voice`,"
		"`orion_cs_core`.`cs_voiceme`,"
		
		"`orion_cs_core`.`bs_set`,"
		"`orion_cs_core`.`bs_caps_ttb`,"
		"`orion_cs_core`.`bs_caps_min`,"
		"`orion_cs_core`.`bs_caps_per`,"
		"`orion_cs_core`.`bs_flood_ttb`,"
		"`orion_cs_core`.`bs_flood_ln`,"
		"`orion_cs_core`.`bs_flood_secs`,"
		"`orion_cs_core`.`bs_repeat_ttb`,"
		"`orion_cs_core`.`bs_repeat`,"
		"`orion_cs_core`.`bs_colors`,"
		"`orion_cs_core`.`bs_underlines`,"
		"`orion_cs_core`.`bs_reverses`,"
		"`orion_cs_core`.`bs_badwords`,"
		"`orion_cs_core`.`bs_bolds`,"
		
		"`orion_core_condata`.`vhost`,"
		"`orion_core_condata`.`ident`,"
		"`orion_core_condata`.`srv`,"
		
		"`orion_cs_core`.`regdate` "

	"FROM `orion_chan_users` "
	"LEFT OUTER JOIN `orion_core_condata` ON (`orion_chan_users`.`nick` = `orion_core_condata`.`nick`) "
	"LEFT OUTER JOIN `orion_cs_core` ON (`orion_chan_users`.`channel`=`orion_cs_core`.`channel`) "
		"LEFT OUTER JOIN `orion_ns_aliases` "
			"LEFT OUTER JOIN `orion_ns_core` "
				"LEFT OUTER JOIN `orion_cs_access` "
				"ON (`orion_ns_core`.`authnick`=`orion_cs_access`.`authnick` AND `orion_cs_access`.`channel`='%s' AND `orion_ns_core`.`authnick` IS NOT NULL) "
			"ON (`orion_ns_aliases`.`display`=`orion_ns_core`.`authnick` AND `orion_ns_aliases`.`display` IS NOT NULL) "
		"ON (`orion_chan_users`.`nick` = `orion_ns_aliases`.`nick` AND `orion_ns_aliases`.`active`=1) "
	"WHERE `orion_chan_users`.`nick` IN ('%s') AND `orion_chan_users`.`channel` = '%s' "
	"ORDER BY `orion_chan_users`.`nick` ASC"
	,mysql_quote(thread,channel),mysql_quote(thread,nick),mysql_quote(thread,channel)
	
	);

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to MYSQL result pointer
	
	bsChan->fields	= mysql_num_fields(res);
	bsChan->data	= mysql_num_rows(res);

	if(bsChan->fields && bsChan->data>0){
		row=mysql_fetch_row(res); //
		
		//for(i=0;i<bsChan->fields;i++){
		//	if(row[i]==NULL) row[i]='\0';
		//}
		
		dynncpy(bsChan->authnick,row[0],sizeof(bsChan->authnick));
		bsChan->active=stoi(row[1]);
		bsChan->qlevel=stoi(row[2]);
		bsChan->olevel=stoi(row[3]);
		bsChan->llevel=stoi(row[4]);
		bsChan->slevel=stoi(row[5]);
		bsChan->rights=stoi(row[6]);
		
		dynncpy(bsChan->cu_nick,row[7],sizeof(bsChan->cu_nick));
		dynncpy(bsChan->cu_modes,row[8],sizeof(bsChan->cu_modes));
		dynncpy(bsChan->level,row[9],sizeof(bsChan->level));
		
		dynncpy(bsChan->serv,row[10],sizeof(bsChan->serv));
		dynncpy(bsChan->bot,row[11],sizeof(bsChan->bot));
		dynncpy(bsChan->name,row[12],sizeof(bsChan->name));
		dynncpy(bsChan->founder,row[13],sizeof(bsChan->founder));
		dynncpy(bsChan->successor,row[14],sizeof(bsChan->successor));
		bsChan->datecreate=(time_t)stol(row[15]);
		dynncpy(bsChan->description,row[16],sizeof(bsChan->description));
		dynncpy(bsChan->mode,row[17],sizeof(bsChan->mode));
		dynncpy(bsChan->mlock_positive,row[18],sizeof(bsChan->mlock_positive));
		dynncpy(bsChan->mlock_negative,row[19],sizeof(bsChan->mlock_negative));
		bsChan->mlock_limit=stol(row[20]);
		dynncpy(bsChan->mlock_key,row[21],sizeof(bsChan->mlock_key));
		dynncpy(bsChan->mlock_flood,row[22],sizeof(bsChan->mlock_flood));
		dynncpy(bsChan->link,row[23],sizeof(bsChan->link));
		bsChan->limit=stol(row[24]);
		dynncpy(bsChan->key,row[25],sizeof(bsChan->key));
		dynncpy(bsChan->flood,row[26],sizeof(bsChan->flood));
		dynncpy(bsChan->last_topic,row[27],sizeof(bsChan->last_topic));
		dynncpy(bsChan->last_topic_setter,row[28],sizeof(bsChan->last_topic_setter));
		bsChan->last_topic_time=(time_t)stol(row[29]);
		dynncpy(bsChan->url,row[30],sizeof(bsChan->url));
		bsChan->alimit=stol(row[31]);
		dynncpy(bsChan->flags,row[32],sizeof(bsChan->flags));
		dynncpy(bsChan->welcome,row[33],sizeof(bsChan->welcome));
		dynncpy(bsChan->mlock_redirect,row[34],sizeof(bsChan->mlock_redirect));
		bsChan->last_used=(time_t)stol(row[35]);
		dynncpy(bsChan->reason,row[36],sizeof(bsChan->reason));
		bsChan->settings=stoi(row[37]);
		dynncpy(bsChan->email,row[38],sizeof(bsChan->email));
		bsChan->expire=stoi(row[39]);
		bsChan->maxmemos=stoi(row[40]);
		
		bsChan->cs_accchange=stoi(row[41]);
		bsChan->cs_acclist=stoi(row[42]);
		bsChan->cs_akick=stoi(row[43]);
		bsChan->cs_autodeop=stoi(row[44]);
		bsChan->cs_autohalfop=stoi(row[45]);
		bsChan->cs_autoop=stoi(row[46]);
		bsChan->cs_autoprotect=stoi(row[47]);
		bsChan->cs_autovoice=stoi(row[48]);
		bsChan->cs_badwords=stoi(row[49]);
		bsChan->cs_ban=stoi(row[50]);
		bsChan->cs_banme=stoi(row[51]);
		bsChan->cs_assign=stoi(row[52]);
		bsChan->cs_clear=stoi(row[53]);
		bsChan->cs_fantasia=stoi(row[54]);
		bsChan->cs_getkey=stoi(row[55]);
		bsChan->cs_greet=stoi(row[56]);
		bsChan->cs_halfop=stoi(row[57]);
		bsChan->cs_halfopme=stoi(row[58]);
		bsChan->cs_info=stoi(row[59]);
		bsChan->cs_invite=stoi(row[60]);
		bsChan->cs_kick=stoi(row[61]);
		bsChan->cs_kickme=stoi(row[62]);
		bsChan->cs_memo=stoi(row[63]);
		bsChan->cs_nojoin=stoi(row[64]);
		bsChan->cs_nokick=stoi(row[65]);
		bsChan->cs_opdeop=stoi(row[66]);
		bsChan->cs_opdeopme=stoi(row[67]);
		bsChan->cs_protect=stoi(row[68]);
		bsChan->cs_protectme=stoi(row[69]);
		bsChan->cs_say=stoi(row[70]);
		bsChan->cs_set=stoi(row[71]);
		bsChan->cs_signkick=stoi(row[72]);
		bsChan->cs_topic=stoi(row[73]);
		bsChan->cs_unban=stoi(row[74]);
		bsChan->cs_voice=stoi(row[75]);
		bsChan->cs_voiceme=stoi(row[76]);
		
		bsChan->bs_set=stoi(row[77]);
		bsChan->bs_caps_ttb=stoi(row[78]);
		bsChan->bs_caps_min=stoi(row[79]);
		bsChan->bs_caps_per=stoi(row[80]);
		bsChan->bs_flood_ttb=stoi(row[81]);
		bsChan->bs_flood_ln=stoi(row[82]);
		bsChan->bs_flood_secs=stoi(row[83]);
		bsChan->bs_repeat_ttb=stoi(row[84]);
		bsChan->bs_repeat=stoi(row[85]);
		bsChan->bs_colors=stoi(row[86]);
		bsChan->bs_underlines=stoi(row[87]);
		bsChan->bs_reverses=stoi(row[88]);
		bsChan->bs_badwords=stoi(row[89]);
		bsChan->bs_bolds=stoi(row[90]);
		
		dynncpy(bsChan->vhost,row[91],sizeof(bsChan->vhost));
		dynncpy(bsChan->ident,row[92],sizeof(bsChan->ident));
		dynncpy(bsChan->server,row[93],sizeof(bsChan->server));
		
		bsChan->regdate=(time_t)stol(row[94]);
	}
	mysql_free_result_myr(res);
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief sql_BsChan2:  Selects information about
 *                     the channel levels for target channel from table `orion_cs_core`
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param nick The nick for which we like to have some additional information
 * @param target The target (user) for which we like to receive the channel-access rights
 * @param channel The channel for which we like to have some additional information
 * @param bsChan The BotServ Channel structure where the data should be loaded
 * @return char* Returns the channel-access level of target user
 **/
char* sql_BsChan2(Threads *thread, char *nick, char *target, char* channel, BsChan *bsChan){
	/* Initialize SQL */
	SQLres *res = NULL;
	SQLrow  row;
	/******************/
           
	char query[SQL_BUF];
	static char returner[MAX_MODES_LEN];
	int i = 0;
	
	bzero(&returner,sizeof(returner));
      
	bsChan->fields = 0;
	bsChan->data   = 0;
	
	bsChan->authnick[0]='\0';
	bsChan->qlevel=0;
	bsChan->olevel=0;
	bsChan->llevel=0;
	bsChan->slevel=0;
	bsChan->rights=0;
	bsChan->active=0;
	
	bsChan->vhost[0]='\0';
	bsChan->ident[0]='\0';
	
	bsChan->name[0]='\0';
	bsChan->bot[0]='\0';
	bsChan->serv[0]='\0';
	bsChan->founder[0]='\0';
	bsChan->successor[0]='\0';
	bsChan->datecreate=(time_t)0;
	bsChan->regdate=(time_t)0;
	bsChan->description[0]='\0';
	bsChan->mode[0]='\0';
	bsChan->mlock_positive[0]='\0';
	bsChan->mlock_negative[0]='\0';
	bsChan->mlock_limit=0;
	bsChan->mlock_key[0]='\0';
	bsChan->mlock_flood[0]='\0';
	bsChan->mlock_redirect[0]='\0';
	bsChan->link[0]='\0';
	bsChan->limit=0;
	bsChan->key[0]='\0';
	bsChan->flood[0]='\0';
	bsChan->last_topic[0]='\0';
	bsChan->last_topic_setter[0]='\0';
	bsChan->last_topic_time=(time_t)0;
	bsChan->url[0]='\0';
	bsChan->alimit=0;
	bsChan->flags[0]='\0';
	bsChan->welcome[0]='\0';
	bsChan->last_used=(time_t)0;
	bsChan->settings=0;
	bsChan->email[0]='\0';
	bsChan->expire=1;
	bsChan->maxmemos=0;
	bsChan->reason[0]='\0';
	
	bsChan->level[0]='\0';
	bsChan->cu_modes[0]='\0';
	bsChan->cu_nick[0]='\0';

	bsChan->cs_accchange=0;
	bsChan->cs_acclist=0;
	bsChan->cs_akick=0;
	bsChan->cs_autodeop=0;
	bsChan->cs_autohalfop=0;
	bsChan->cs_autoop=0;
	bsChan->cs_autoprotect=0;
	bsChan->cs_autovoice=0;
	bsChan->cs_badwords=0;
	bsChan->cs_ban=0;
	bsChan->cs_banme=0;
	bsChan->cs_assign=0;
	bsChan->cs_clear=0;
	bsChan->cs_fantasia=0;
	bsChan->cs_getkey=0;
	bsChan->cs_greet=0;
	bsChan->cs_halfop=0;
	bsChan->cs_halfopme=0;
	bsChan->cs_info=0;
	bsChan->cs_invite=0;
	bsChan->cs_kick=0;
	bsChan->cs_kickme=0;
	bsChan->cs_memo=0;
	bsChan->cs_nojoin=0;
	bsChan->cs_nokick=0;
	bsChan->cs_opdeop=0;
	bsChan->cs_opdeopme=0;
	bsChan->cs_protect=0;
	bsChan->cs_protectme=0;
	bsChan->cs_say=0;
	bsChan->cs_set=0;
	bsChan->cs_signkick=0;
	bsChan->cs_topic=0;
	bsChan->cs_unban=0;
	bsChan->cs_voice=0;
	bsChan->cs_voiceme=0;
	
	bsChan->bs_set=-1;
	bsChan->bs_caps_ttb=-1;
	bsChan->bs_caps_min=-1;
	bsChan->bs_caps_per=-1;
	bsChan->bs_flood_ttb=-1;
	bsChan->bs_flood_ln=-1;
	bsChan->bs_flood_secs=-1;
	bsChan->bs_repeat_ttb=-1;
	bsChan->bs_repeat=-1;
	bsChan->bs_colors=-1;
	bsChan->bs_underlines=-1;
	bsChan->bs_reverses=-1;
	bsChan->bs_badwords=-1;
	bsChan->bs_bolds=-1;
                                            
	if(channel==NULL || nick==NULL || target==NULL) return returner;
	
	/* Channel */
	bzero(&query,sizeof(query));
	
	snprintf(query, sizeof(query),

	"SELECT "
	
		"`orion_ns_aliases`.`display`,"
		"`orion_ns_aliases`.`active`,"
		"`orion_ns_core`.`qlevel`,"
		"`orion_ns_core`.`olevel`,"
		"`orion_ns_core`.`llevel`,"
		"`orion_ns_core`.`slevel`,"
		"`orion_ns_core`.`rights`,"
			
		"`orion_chan_users`.`nick`,"
		"`orion_chan_users`.`modes`,"
		"`orion_cs_access`.`level`,"
	
		"`orion_cs_core`.`serv`,"
		"`orion_cs_core`.`botnick`,"
		"`orion_cs_core`.`channel`,"
		"`orion_cs_core`.`founder`,"
		"`orion_cs_core`.`successor`,"
		"`orion_cs_core`.`datecreate`,"
		"`orion_cs_core`.`description`,"
		"`orion_cs_core`.`mode`,"
		"`orion_cs_core`.`mlock_positive`,"
		"`orion_cs_core`.`mlock_negative`,"
		"`orion_cs_core`.`mlock_limit`,"
		"`orion_cs_core`.`mlock_key`,"
		"`orion_cs_core`.`mlock_flood`,"
		"`orion_cs_core`.`link`,"
		"`orion_cs_core`.`limit`,"
		"`orion_cs_core`.`key`,"
		"`orion_cs_core`.`flood`,"
		"`orion_cs_core`.`last_topic`,"
		"`orion_cs_core`.`last_topic_setter`,"
		"`orion_cs_core`.`last_topic_time`,"
		"`orion_cs_core`.`url`,"
		"`orion_cs_core`.`alimit`,"
		"`orion_cs_core`.`flags`,"
		"`orion_cs_core`.`welcome`,"
		"`orion_cs_core`.`mlock_redirect`,"
		"`orion_cs_core`.`last_used`,"
		"`orion_cs_core`.`reason`,"
		"`orion_cs_core`.`settings`,"
		"`orion_cs_core`.`email`,"
		"`orion_cs_core`.`expire`,"
		"`orion_cs_core`.`maxmemos`,"
	
		"`orion_cs_core`.`cs_accchange`,"
		"`orion_cs_core`.`cs_acclist`,"
	    "`orion_cs_core`.`cs_akick`,"
		"`orion_cs_core`.`cs_autodeop`,"
		"`orion_cs_core`.`cs_autohalfop`,"
		"`orion_cs_core`.`cs_autoop`,"
	    "`orion_cs_core`.`cs_autoprotect`,"
		"`orion_cs_core`.`cs_autovoice`,"
		"`orion_cs_core`.`cs_badwords`,"
		"`orion_cs_core`.`cs_ban`,"
		"`orion_cs_core`.`cs_banme`,"
	    "`orion_cs_core`.`cs_botassign`,"
		"`orion_cs_core`.`cs_cclear`,"
		"`orion_cs_core`.`cs_fantasia`,"
		"`orion_cs_core`.`cs_getkey`,"
		"`orion_cs_core`.`cs_greet`,"
		"`orion_cs_core`.`cs_halfop`,"
		"`orion_cs_core`.`cs_halfopme`,"
	    "`orion_cs_core`.`cs_info`,"
		"`orion_cs_core`.`cs_invite`,"
		"`orion_cs_core`.`cs_kick`,"
		"`orion_cs_core`.`cs_kickme`,"
		"`orion_cs_core`.`cs_memo`,"
		"`orion_cs_core`.`cs_nojoin`,"
	    "`orion_cs_core`.`cs_nokick`,"
		"`orion_cs_core`.`cs_opdeop`,"
		"`orion_cs_core`.`cs_opdeopme`,"
		"`orion_cs_core`.`cs_protect`,"
	    "`orion_cs_core`.`cs_protectme`,"
		"`orion_cs_core`.`cs_say`,"
		"`orion_cs_core`.`cs_set`,"
		"`orion_cs_core`.`cs_signkick`,"
		"`orion_cs_core`.`cs_topic`,"
		"`orion_cs_core`.`cs_unban`,"
		"`orion_cs_core`.`cs_voice`,"
		"`orion_cs_core`.`cs_voiceme`,"
		
		"`orion_cs_core`.`bs_set`,"
		"`orion_cs_core`.`bs_caps_ttb`,"
		"`orion_cs_core`.`bs_caps_min`,"
		"`orion_cs_core`.`bs_caps_per`,"
		"`orion_cs_core`.`bs_flood_ttb`,"
		"`orion_cs_core`.`bs_flood_ln`,"
		"`orion_cs_core`.`bs_flood_secs`,"
		"`orion_cs_core`.`bs_repeat_ttb`,"
		"`orion_cs_core`.`bs_repeat`,"
		"`orion_cs_core`.`bs_colors`,"
		"`orion_cs_core`.`bs_underlines`,"
		"`orion_cs_core`.`bs_reverses`,"
		"`orion_cs_core`.`bs_badwords`,"
		"`orion_cs_core`.`bs_bolds`,"
		
		"`orion_core_condata`.`vhost`,"
		"`orion_core_condata`.`ident`,"
		"`orion_core_condata`.`srv`,"
		
		"`orion_cs_core`.`regdate` "

	"FROM `orion_chan_users` "
	"LEFT OUTER JOIN `orion_core_condata` ON (`orion_chan_users`.`nick` = `orion_core_condata`.`nick`) "
	"LEFT OUTER JOIN `orion_cs_core` ON (`orion_chan_users`.`channel`=`orion_cs_core`.`channel`) "
		"LEFT OUTER JOIN `orion_ns_aliases` "
			"LEFT OUTER JOIN `orion_ns_core` "
				"LEFT OUTER JOIN `orion_cs_access` "
				"ON (`orion_ns_core`.`authnick`=`orion_cs_access`.`authnick` AND `orion_cs_access`.`channel`='%s' AND `orion_ns_core`.`authnick` IS NOT NULL) "
			"ON (`orion_ns_aliases`.`display`=`orion_ns_core`.`authnick` AND `orion_ns_aliases`.`display` IS NOT NULL) "
		"ON (`orion_chan_users`.`nick` = `orion_ns_aliases`.`nick` AND `orion_ns_aliases`.`active`=1) "
	"WHERE `orion_chan_users`.`nick` IN ('%s','%s') AND `orion_chan_users`.`channel` = '%s' "
	"ORDER BY FIELD(`orion_chan_users`.`nick`,'%s','%s')"
	,mysql_quote(thread,channel),mysql_quote(thread,nick),mysql_quote(thread,target),mysql_quote(thread,channel)
	,mysql_quote(thread,nick),mysql_quote(thread,target)
	
	);

	ExecuteQuery(thread,query,dynlen(query)); //execute query
	res=mysql_store_result(MYSQL_CONNECTION_HANDLE); //store result to MYSQL result pointer
	
	bsChan->fields = mysql_num_fields(res);
	bsChan->data   = mysql_num_rows(res);
	
	//printf("we've %d rows\n",bsChan->data);

	if(bsChan->fields && bsChan->data>0){
		row=mysql_fetch_row(res); //
		
		for(i=0;i<bsChan->fields;i++){
			if(row[i]==NULL) row[i]='\0';
		}
		
		//printf("MAIN NICK = %s\n",row[7]);
		//printf("MAIN NICK LEVEL = %s\n",row[9]);
		
		dynncpy(bsChan->authnick,row[0],sizeof(bsChan->authnick));
		bsChan->active=stoi(row[1]);
		bsChan->qlevel=stoi(row[2]);
		bsChan->olevel=stoi(row[3]);
		bsChan->llevel=stoi(row[4]);
		bsChan->slevel=stoi(row[5]);
		bsChan->rights=stoi(row[6]);
		
		dynncpy(bsChan->cu_nick,row[7],sizeof(bsChan->cu_nick));
		dynncpy(bsChan->cu_modes,row[8],sizeof(bsChan->cu_modes));
		dynncpy(bsChan->level,row[9],sizeof(bsChan->level));
		
		dynncpy(bsChan->serv,row[10],sizeof(bsChan->serv));
		dynncpy(bsChan->bot,row[11],sizeof(bsChan->bot));
		dynncpy(bsChan->name,row[12],sizeof(bsChan->name));
		dynncpy(bsChan->founder,row[13],sizeof(bsChan->founder));
		dynncpy(bsChan->successor,row[14],sizeof(bsChan->successor));
		bsChan->datecreate=(time_t)stol(row[15]);
		dynncpy(bsChan->description,row[16],sizeof(bsChan->description));
		dynncpy(bsChan->mode,row[17],sizeof(bsChan->mode));
		dynncpy(bsChan->mlock_positive,row[18],sizeof(bsChan->mlock_positive));
		dynncpy(bsChan->mlock_negative,row[19],sizeof(bsChan->mlock_negative));
		bsChan->mlock_limit=stol(row[20]);
		dynncpy(bsChan->mlock_key,row[21],sizeof(bsChan->mlock_key));
		dynncpy(bsChan->mlock_flood,row[22],sizeof(bsChan->mlock_flood));
		dynncpy(bsChan->link,row[23],sizeof(bsChan->link));
		bsChan->limit=stol(row[24]);
		dynncpy(bsChan->key,row[25],sizeof(bsChan->key));
		dynncpy(bsChan->flood,row[26],sizeof(bsChan->flood));
		dynncpy(bsChan->last_topic,row[27],sizeof(bsChan->last_topic));
		dynncpy(bsChan->last_topic_setter,row[28],sizeof(bsChan->last_topic_setter));
		bsChan->last_topic_time=(time_t)stol(row[29]);
		dynncpy(bsChan->url,row[30],sizeof(bsChan->url));
		bsChan->alimit=stol(row[31]);
		dynncpy(bsChan->flags,row[32],sizeof(bsChan->flags));
		dynncpy(bsChan->welcome,row[33],sizeof(bsChan->welcome));
		dynncpy(bsChan->mlock_redirect,row[34],sizeof(bsChan->mlock_redirect));
		bsChan->last_used=(time_t)stol(row[35]);
		dynncpy(bsChan->reason,row[36],sizeof(bsChan->reason));
		bsChan->settings=stoi(row[37]);
		dynncpy(bsChan->email,row[38],sizeof(bsChan->email));
		bsChan->expire=stoi(row[39]);
		bsChan->maxmemos=stoi(row[40]);
		
		bsChan->cs_accchange=stoi(row[41]);
		bsChan->cs_acclist=stoi(row[42]);
		bsChan->cs_akick=stoi(row[43]);
		bsChan->cs_autodeop=stoi(row[44]);
		bsChan->cs_autohalfop=stoi(row[45]);
		bsChan->cs_autoop=stoi(row[46]);
		bsChan->cs_autoprotect=stoi(row[47]);
		bsChan->cs_autovoice=stoi(row[48]);
		bsChan->cs_badwords=stoi(row[49]);
		bsChan->cs_ban=stoi(row[50]);
		bsChan->cs_banme=stoi(row[51]);
		bsChan->cs_assign=stoi(row[52]);
		bsChan->cs_clear=stoi(row[53]);
		bsChan->cs_fantasia=stoi(row[54]);
		bsChan->cs_getkey=stoi(row[55]);
		bsChan->cs_greet=stoi(row[56]);
		bsChan->cs_halfop=stoi(row[57]);
		bsChan->cs_halfopme=stoi(row[58]);
		bsChan->cs_info=stoi(row[59]);
		bsChan->cs_invite=stoi(row[60]);
		bsChan->cs_kick=stoi(row[61]);
		bsChan->cs_kickme=stoi(row[62]);
		bsChan->cs_memo=stoi(row[63]);
		bsChan->cs_nojoin=stoi(row[64]);
		bsChan->cs_nokick=stoi(row[65]);
		bsChan->cs_opdeop=stoi(row[66]);
		bsChan->cs_opdeopme=stoi(row[67]);
		bsChan->cs_protect=stoi(row[68]);
		bsChan->cs_protectme=stoi(row[69]);
		bsChan->cs_say=stoi(row[70]);
		bsChan->cs_set=stoi(row[71]);
		bsChan->cs_signkick=stoi(row[72]);
		bsChan->cs_topic=stoi(row[73]);
		bsChan->cs_unban=stoi(row[74]);
		bsChan->cs_voice=stoi(row[75]);
		bsChan->cs_voiceme=stoi(row[76]);	
		
		bsChan->bs_set=stoi(row[77]);
		bsChan->bs_caps_ttb=stoi(row[78]);
		bsChan->bs_caps_min=stoi(row[79]);
		bsChan->bs_caps_per=stoi(row[80]);
		bsChan->bs_flood_ttb=stoi(row[81]);
		bsChan->bs_flood_ln=stoi(row[82]);
		bsChan->bs_flood_secs=stoi(row[83]);
		bsChan->bs_repeat_ttb=stoi(row[84]);
		bsChan->bs_repeat=stoi(row[85]);
		bsChan->bs_colors=stoi(row[86]);
		bsChan->bs_underlines=stoi(row[87]);
		bsChan->bs_reverses=stoi(row[88]);
		bsChan->bs_badwords=stoi(row[89]);
		bsChan->bs_bolds=stoi(row[90]);
		
		dynncpy(bsChan->vhost,row[91],sizeof(bsChan->vhost));
		dynncpy(bsChan->ident,row[92],sizeof(bsChan->ident));
		dynncpy(bsChan->server,row[93],sizeof(bsChan->server));
		
		bsChan->regdate=(time_t)stol(row[94]);
		
		if(bsChan->data==2){
			row = mysql_fetch_row(res);
			//printf("TARGET NICK = %s\n",row[7]);
			if(row[9]!=NULL){
				dynncpy(returner,row[9],sizeof(returner));
				//printf("target NICKLEVEL = %s\n",returner);
			}
		}else bzero(&returner,sizeof(returner));
				
	}
	mysql_free_result_myr(res);
	return returner;
}

/*  ************************************************************************************************  */

/**
 * @brief AkickEnforce:  For the Akick Enforce command
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The channel for which we like to enforce the Akick-List
 * @return SQLres* Returns the SQL-Result variable
 **/
SQLres *AkickEnforce(Threads *thread, char *channel){
	/* Global variables */
	char quoted[MAX_CHANNELNAME_LEN*2];
	//SQLres *res = NULL;
	bzero(&quoted,sizeof(quoted));

	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, quoted, channel, strlen(channel));

	return doSelectMySQL(thread,
		"SELECT `orion_chan_users`.`nick`, CONCAT(`orion_core_condata`.`nick`,'!',`orion_core_condata`.`ident`,'@',`orion_core_condata`.`vhost`), `mask`, `orion_cs_akick`.`reason` "
		"FROM "
		"`orion_chan_users` "
		"LEFT JOIN `orion_cs_core` ON `orion_cs_core`.`channel`='%s'" //CHANNEL
		"LEFT JOIN `orion_core_condata` ON `orion_chan_users`.`nick`=`orion_core_condata`.`nick` "
		"LEFT JOIN `orion_cs_akick` ON (`orion_chan_users`.`channel`=`orion_cs_akick`.`channel` "
												"AND CONCAT(`orion_core_condata`.`nick`,'!',`orion_core_condata`.`ident`,'@',`orion_core_condata`.`vhost`) LIKE REPLACE(`mask`,'*','%')) "
		"LEFT JOIN `orion_ns_aliases` "
				"LEFT OUTER JOIN `orion_cs_access` "
				"ON (`authnick`=`orion_ns_aliases`.`display` AND `channel`='%s') " //CHANNEL
				"ON `orion_core_condata`.`nick`=`orion_ns_aliases`.`nick` "
		"WHERE "
				"IF((CASE `orion_cs_core`.`serv` "
				"WHEN 'ChanServ' THEN `orion_cs_access`.`level`>=IF(`orion_cs_core`.`cs_nokick`=-1000,`orion_cs_core`.`cs_autoop`,`orion_cs_core`.`cs_nokick`) "
				"WHEN 'Q'	THEN `orion_cs_access`.`level` RLIKE '(n|m|o|h|p)' "
				"END "
			"AND `orion_ns_aliases`.`active`)=1,1,0)=0 AND "
		"`orion_chan_users`.`channel`='%s' AND `mask` IS NOT NULL GROUP BY `orion_chan_users`.`nick`" //CHANNEL
		,quoted,quoted,quoted);
}

/*  ************************************************************************************************  */

/**
 * @brief AkickOnJoin:  To perform Akicks on JOIN
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The channel for which we like to enforce the Akick-List
 * @param nick The user sent by JOIN to check
 * @return SQLres* Returns the SQL-Result variable
 **/
SQLres *AkickOnJoin(Threads *thread, char *channel, char *nick){
	/* Global variables */
	char chan_quote[MAX_CHANNELNAME_LEN*2];
	char nick_quote[MAX_NICK_LEN*2];
	
	//SQLres *res = NULL;
	bzero(&chan_quote,sizeof(chan_quote));
	bzero(&nick_quote,sizeof(nick_quote));

	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, chan_quote, channel, strlen(channel));
	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, nick_quote, nick, strlen(nick));

	return doSelectMySQL(thread,
		"SELECT CONCAT(`orion_core_condata`.`nick`,'!',`orion_core_condata`.`ident`,'@',`orion_core_condata`.`vhost`) `hostmask`, `orion_cs_akick`.`mask`, `orion_cs_akick`.`reason` "
		"FROM "
		"`orion_core_condata` "
		"LEFT JOIN `orion_cs_core` ON `orion_cs_core`.`channel`='%s'" //CHANNEL
		"LEFT JOIN `orion_cs_akick` ON (CONCAT(`orion_core_condata`.`nick`,'!',`orion_core_condata`.`ident`,'@',`orion_core_condata`.`vhost`) "
								"LIKE REPLACE(`mask`,'*','%') AND `orion_cs_akick`.`channel`=`orion_cs_core`.`channel`) "
		"LEFT JOIN `orion_ns_aliases` "
				"LEFT OUTER JOIN `orion_cs_access` "
				"ON (`authnick`=`orion_ns_aliases`.`display` AND `channel`='%s') " //CHANNEL
				"ON `orion_core_condata`.`nick`=`orion_ns_aliases`.`nick` "
		"WHERE "
				"IF((CASE `orion_cs_core`.`serv` "
				"WHEN 'ChanServ' THEN `orion_cs_access`.`level`>=IF(`orion_cs_core`.`cs_nokick`=-1000,`orion_cs_core`.`cs_autoop`,`orion_cs_core`.`cs_nokick`) "
				"WHEN 'Q'	THEN `orion_cs_access`.`level` RLIKE '(n|m|o|h|p)' "
				"END "
			"AND `orion_ns_aliases`.`active`)=1,1,0)=0 AND "
		"`orion_core_condata`.`nick`='%s' GROUP BY `orion_core_condata`.`nick`" //CHANNEL
		,chan_quote,chan_quote,nick_quote);
}

/*  ************************************************************************************************  */

/**
 * @brief AkickOnSJoin:  To perform Akicks on SJOIN
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param channel The channel for which we like to enforce the Akick-List
 * @param nicks The user-list sent by SJOIN to check all users within one query
 * @return SQLres* Returns the SQL-Result variable
 **/
SQLres *AkickOnSJoin(Threads *thread, char *channel, char *nicks){
	/* Global variables */
	char quoted[MAX_CHANNELNAME_LEN*2];
	//SQLres *res = NULL;
	bzero(&quoted,sizeof(quoted));

	mysql_real_escape_string(MYSQL_CONNECTION_HANDLE, quoted, channel, strlen(channel));

	return doSelectMySQL(thread,
		"SELECT `orion_core_condata`.`nick`,CONCAT(`orion_core_condata`.`nick`,'!',`orion_core_condata`.`ident`,'@',`orion_core_condata`.`vhost`) `hostmask`, " /*GROUP_CONCAT(`mask`) as `masks`*/ "`mask`, `orion_cs_akick`.`reason` "
		"FROM "
		"`orion_core_condata` "
		"LEFT JOIN `orion_cs_core` ON `orion_cs_core`.`channel`='%s'" //CHANNEL
		"LEFT JOIN `orion_cs_akick` ON (CONCAT(`orion_core_condata`.`nick`,'!',`orion_core_condata`.`ident`,'@',`orion_core_condata`.`vhost`) "
								"LIKE REPLACE(`mask`,'*','%') AND `orion_cs_akick`.`channel`=`orion_cs_core`.`channel`) "
		"LEFT JOIN `orion_ns_aliases` "
				"LEFT OUTER JOIN `orion_cs_access` "
				"ON (`authnick`=`orion_ns_aliases`.`display` AND `channel`='%s') " //CHANNEL
				"ON `orion_core_condata`.`nick`=`orion_ns_aliases`.`nick` "
		"WHERE "
				"IF((CASE `orion_cs_core`.`serv` "
				"WHEN 'ChanServ' THEN `orion_cs_access`.`level`>=IF(`orion_cs_core`.`cs_nokick`=-1000,`orion_cs_core`.`cs_autoop`,`orion_cs_core`.`cs_nokick`) "
				"WHEN 'Q'	THEN `orion_cs_access`.`level` RLIKE '(n|m|o|h|p)' "
				"END "
			"AND `orion_ns_aliases`.`active`)=1,1,0)=0 AND "
		"`orion_core_condata`.`nick` IN(%s) GROUP BY `orion_core_condata`.`nick` ORDER BY FIELD(`orion_core_condata`.`nick`,%s)" //CHANNEL
		,quoted,quoted,nicks,nicks);
}

/*  ************************************************************************************************  */

/**
 * @brief doBufferedInsert:  This is used to buffer INSERT'S in buffer tables
 *                     This increases insert speed about 30% (used in cnick();)
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param directtable The Table where we want to buffer the Information (IF we use a directtable)
 * @param input The query we want to add to the Buffer.
 * @return size_t
 **/
size_t doBufferedInsert(Threads *thread, bool duplicate_update, size_t directtable, const char *input, ...){
	va_list va;
	char buffer [MYSQL_BUF+1];
	char *array1[ARRAY_SIZE];
	char *values	= '\0';
	char *duplication= NULL;
	size_t parts		= 0;
	int length			= 0;
	bool found			= 0;
	int lastseen		= 0;
	int i				= 0;
	
	if(input==NULL) return -1;
	bzero(buffer,sizeof(buffer));

	/* preparing Query */
	va_start(va, input);
	vsnprintf(buffer, sizeof(buffer)-1, input, va);
	va_end(va);
	/*******************/
	
	parts = strsplit(buffer,array1,' ');
	
	/* Analyze parts */
	/*for(i=6;i<parts;i++){
		for(j=0;array1[i][j]!='\0';j++){
			if(begin!=0 && end!=0) break;
			if(array1[i][j]=='(' && array1[i][j+1]=='`') begin = i;
			if(array1[i][j]=='`' && array1[i][j+1]==')') end = i;
		}
		if(begin!=0 && end!=0){
			merge(array1,begin,begin+1,end);
			begin = end = 0;
		}
		printf("Begin = %d - End = %d\n",begin,end);
		printf("array1[5] = %s\n",array1[5]);
		straddns(array1[5],array1[i],sizeof(array1[5]));
		printf("array1[5] = %s\n",array1[5]);
	}*/
	
// ON DUPLICATE KEY UPDATE `modes`=VALUES(`modes`)
//part == 1
	if(duplicate_update>0 && streqnc(array1[parts-4-duplicate_update],"on")){
		values = _dstrjoin_det(' ',array1,5,parts-4-duplicate_update);
		duplication = _dstrjoin(' ',array1,parts-4-duplicate_update);
	}
	else
		values = _dstrjoin(' ',array1,5);
	/* Analyzing done */
	
	/* Check and prepare Parts for adding the data to the Buffers */
	if(streqnc(array1[0],"insert")){
		if(directtable == NO_DIRECTTABLE){
			for(i=0;i<MAX_BUFFER_TABLES;i++){
				if(MySQLSave[i].length != -1) lastseen++;
				
				if(streqnc(MySQLSave[i].table,array1[2])){
					if(streqnc(MySQLSave[i].insertcache,array1[3])){
						found = 1;
						break;
					}
				}
				if(MySQLSave[i].lastused != -1 && MySQLSave[i].lastused+90<time(NULL)){
					/* Free Cache to be able to accelerate other processes
					   That is in fact the maximum delay for queries */
					MySQLSave[i].length   = -1;
					MySQLSave[i].lastused = -1;
					bzero(&MySQLSave[i].buffer		,sizeof(MySQLSave[i].buffer));
					bzero(&MySQLSave[i].table		,sizeof(MySQLSave[i].table));
					bzero(&MySQLSave[i].insertcache	,sizeof(MySQLSave[i].insertcache));
				}
			}
		}
		else i = directtable;
		
		if(MySQLSave[i].duplication[0]=='\0' && duplication!=NULL)
			dynncpy(MySQLSave[i].duplication,duplication,sizeof(MySQLSave[i].duplication));
		
		if(found || directtable != NO_DIRECTTABLE){
			length = dynlen(values)+1;
			
			if(MySQLSave[i].length+length >= MYSQL_BUF){
				if(MySQLSave[i].duplication[0]!='\0'){
					straddnc(MySQLSave[i].buffer,' ',sizeof(MySQLSave[i].buffer));
					straddns(MySQLSave[i].buffer,MySQLSave[i].duplication,sizeof(MySQLSave[i].buffer));
					MySQLSave[i].length   +=dynlen(MySQLSave[i].duplication)+1;
				}
				ExecuteQuery(thread,MySQLSave[i].buffer,MySQLSave[i].length);
				
				bzero (&MySQLSave[i].buffer,sizeof(MySQLSave[i].buffer));
				MySQLSave[i].length   = snprintf(MySQLSave[i].buffer, sizeof(MySQLSave[i].buffer),"INSERT INTO %s %s VALUES %s",MySQLSave[i].table,MySQLSave[i].insertcache,values);
				MySQLSave[i].lastused = time(NULL);
				return i;
			}
			else if(MySQLSave[i].length + length < MYSQL_BUF){
				
				if(MySQLSave[i].length   == -1){
					dynncpy(MySQLSave[i].insertcache ,array1[3],sizeof(MySQLSave[i].insertcache ));
					dynncpy(MySQLSave[i].table 		,array1[2],sizeof(MySQLSave[i].table 		));
					printf("MySQLSave[i].table = %s\n",MySQLSave[i].table);
					MySQLSave[i].length   = snprintf(MySQLSave[i].buffer, sizeof(MySQLSave[i].buffer),"INSERT INTO %s %s VALUES %s",MySQLSave[i].table,MySQLSave[i].insertcache,values);
				}
				else{
					straddnc(MySQLSave[i].buffer,',',sizeof(MySQLSave[i].buffer));
					straddns(MySQLSave[i].buffer,values,sizeof(MySQLSave[i].buffer));
					MySQLSave[i].length   +=length;
				}
				MySQLSave[i].lastused  = time(NULL);
				return i;
			}
		
		}
		else if(!found && lastseen < MAX_BUFFER_TABLES){
			lastseen++;
			dynncpy(MySQLSave[lastseen].table       ,array1[2],sizeof(MySQLSave[lastseen].table       ));
			dynncpy(MySQLSave[lastseen].insertcache ,array1[3],sizeof(MySQLSave[lastseen].insertcache ));
			
			MySQLSave[lastseen].length   = snprintf(MySQLSave[lastseen].buffer,sizeof(MySQLSave[lastseen].buffer),"INSERT INTO %s %s VALUES %s",MySQLSave[lastseen].table,MySQLSave[lastseen].insertcache,values);
			MySQLSave[lastseen].lastused = time(NULL);
			return lastseen;
		}
		else if(!found && lastseen == MAX_BUFFER_TABLES){
			MySQL_FlushInserts(thread,0,MAX_BUFFER_TABLES);
			dynncpy(MySQLSave[0].table       ,array1[2],sizeof(MySQLSave[0].table       ));
			dynncpy(MySQLSave[0].insertcache ,array1[3],sizeof(MySQLSave[0].insertcache ));
			
			MySQLSave[0].length   = snprintf(MySQLSave[lastseen].buffer,sizeof(MySQLSave[lastseen].buffer),"INSERT INTO %s %s VALUES %s",MySQLSave[lastseen].table,MySQLSave[lastseen].insertcache,values);
			MySQLSave[0].lastused = time(NULL);
			return 0;
		}
	}
	return -2;
}

/*  ************************************************************************************************  */

/**
 * @brief MySQL_FlushInserts:  This is used to FLush (send) the buffered INSERT'S
 *                     The Buffer is freed after send to be used again.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param begin The beginning (first table we want to flush)
 * @param end The end (last table we want to flush)
 * @return void
 **/
void MySQL_FlushInserts(Threads *thread, size_t begin, size_t end){
	size_t i = 0;
	
	for(i=begin;i<=end;i++){
		dlog(thread,2,"flushing MySQL Table [%d]\n",i);
		
		if(MySQLSave[i].buffer[0]!='\0' && MySQLSave[i].length != -1){
			if(MySQLSave[i].duplication[0]!='\0'){
				straddnc(MySQLSave[i].buffer,' ',sizeof(MySQLSave[i].buffer));
				straddns(MySQLSave[i].buffer,MySQLSave[i].duplication,sizeof(MySQLSave[i].buffer));
				MySQLSave[i].length   +=dynlen(MySQLSave[i].duplication)+1;
			}
			dlog(thread,5,"EXECUTING QUERY = %s\n",MySQLSave[i].buffer);
			ExecuteQuery(thread,MySQLSave[i].buffer,MySQLSave[i].length);
		}
		
		MySQLSave[i].length   = -1;
		MySQLSave[i].lastused = -1;
		bzero(&MySQLSave[i].buffer		,sizeof(MySQLSave[i].buffer));
		bzero(&MySQLSave[i].table		,sizeof(MySQLSave[i].table));
		bzero(&MySQLSave[i].insertcache	,sizeof(MySQLSave[i].insertcache));
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief MySQL_CleanInserts:  This is used to unitialize all buffertables
 *                     Frees them before using the first time (common C action)
 *
 * @param begin The beginning (first table we want to clear up)
 * @param end The end (last table we want to clear up)
 * @return void
 **/
void MySQL_CleanInserts(size_t begin, size_t end){
	size_t i = 0;
	
	for(i=begin;i<end;i++){
		MySQLSave[i].length   = -1;
		MySQLSave[i].lastused = -1;
		bzero(&MySQLSave[i].buffer		,sizeof(MySQLSave[i].buffer));
		bzero(&MySQLSave[i].table		,sizeof(MySQLSave[i].table));
		bzero(&MySQLSave[i].insertcache	,sizeof(MySQLSave[i].insertcache));
	}
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief DropAuthnick:  This is used to drop nicks
 *                  (automatically drops channels where he is founder and successor is NULL) etc.
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information.
 * @param authnick The Nick we want to delete.
 * @return void
 **/
void DropAuthnick(Threads *thread, char *authnick){
	/* Global Variables */
	deepSQL channels;
	char chan_delete[MAX_CHANNELNAME_LEN*15]; //hardlimit
	char chan_update[MAX_CHANNELNAME_LEN*15]; //hardlimit
	char change_succ[MAX_CHANNELNAME_LEN*15]; //hardlimit
	
	/* Initialise Variables to (NULL) */
	bzero(&channels,sizeof(channels));
	bzero(&chan_delete,sizeof(chan_delete));
	bzero(&chan_update,sizeof(chan_update));
	bzero(&change_succ,sizeof(change_succ));
	
	channels.res    = doSelectMySQL(thread,
		"SELECT "
		  "SUM(IF(`orion_chan_users`.`nick`=`orion_cs_core`.`botnick`,1,0)) as `botinchan`, "
		  "(COUNT(`orion_chan_users`.`nick`)-SUM(IF(`orion_chan_users`.`nick`=`orion_cs_core`.`botnick`,1,0))) as `count_real_users`, "
		  "IF( "
			"`orion_cs_core`.`founder`='%s', " //NICK
			"IF( "
				"`orion_cs_core`.`successor`<>'%s', " //NICK
				"IF( "
					"`orion_cs_core`.`successor` IS NOT NULL, "
					"1,IF((COUNT(`orion_chan_users`.`nick`)-SUM(IF(`orion_chan_users`.`nick`=`orion_cs_core`.`botnick`,1,0)))>0,2,3) "
				"), "
				"IF((COUNT(`orion_chan_users`.`nick`)-SUM(IF(`orion_chan_users`.`nick`=`orion_cs_core`.`botnick`,1,0)))>0,2,3) "
			"), "
		    "1 "
		  ") as `action`, "
		  "`orion_cs_core`.`serv`,`orion_cs_core`.`channel`,`orion_cs_core`.`botnick`,`orion_cs_core`.`founder`, "
		  "`orion_cs_core`.`successor`, "
		  "`orion_cs_core`.`datecreate`,`orion_cs_core`.`mode` "
		"FROM "
		  "`orion_cs_core` "
		    "LEFT JOIN `orion_chan_users` ON (`orion_cs_core`.`channel` = `orion_chan_users`.`channel`) "
		"WHERE "
		  "`orion_cs_core`.`founder`='%s' OR `orion_cs_core`.`successor`='%s' " // 2x NICK
		"GROUP BY "
			"`orion_cs_core`.`channel`"
		,mysql_quote(thread,authnick),mysql_quote(thread,authnick),mysql_quote(thread,authnick),mysql_quote(thread,authnick)
	);
	channels.fields = mysql_num_rows(channels.res);
					
	if(channels.fields){
		while((channels.row = mysql_fetch_row(channels.res))){
			if(stoi(channels.row[0])==1 && (stoi(channels.row[2])==2 || stoi(channels.row[2])==3)){ // Fraglich.. warum nicht wenn ==1 - channel delete!?
				do_rawmode(thread,channels.row[5],channels.row[4],stol(channels.row[8]) == 0 ? time(NULL) : (time_t)stol(channels.row[8]),IRC.ModeOnChannelDrop);
				orion_WRCtranslate_cmd_part(thread,channels.row[5],channels.row[4],NULL);
				domysql(thread,"DELETE FROM `orion_chan_users` WHERE `nick`='%s' AND `channel`='%s'",mysql_quote(thread,channels.row[5]),mysql_quote(thread,channels.row[4]));
			}
			else if(streqnc(channels.row[3],"chanserv") && (stoi(channels.row[2])==2 || stoi(channels.row[2])==3)){
				do_rawmode(thread,cs->nick,channels.row[4],stol(channels.row[8]) == 0 ? time(NULL) : (time_t)stol(channels.row[8]),IRC.ModeOnChannelDrop);
				do_logchan(thread,NULL,NULL,NULL,NULL,"Auto-dropped ChanServ channel %s (founder: %s)",channels.row[4],channels.row[6]);
			}
			else if(streqnc(channels.row[3],"q") && (stoi(channels.row[2])==2 || stoi(channels.row[2])==3)){
				do_rawmode(thread,Q->nick,channels.row[4],stol(channels.row[8]) == 0 ? time(NULL) : (time_t)stol(channels.row[8]),IRC.ModeOnChannelDrop);
				do_logchan(thread,NULL,NULL,NULL,NULL,"Auto-dropped Q channel %s (founder: %s)",channels.row[4],channels.row[6]);
			}
			if(stoi(channels.row[2])==3){
				straddnc(chan_delete,toascii(39),sizeof(chan_delete));
				straddns(chan_delete,mysql_quote(thread,channels.row[4]),sizeof(chan_delete));
				straddnc(chan_delete,toascii(39),sizeof(chan_delete));
				straddnc(chan_delete,',',sizeof(chan_delete));
			}else if(stoi(channels.row[2])==2){
				straddnc(chan_update,toascii(39),sizeof(chan_update));
				straddns(chan_update,mysql_quote(thread,channels.row[4]),sizeof(chan_update));
				straddnc(chan_update,toascii(39),sizeof(chan_update));
				straddnc(chan_update,',',sizeof(chan_update));
			}else if(stoi(channels.row[2])==1){
				straddnc(change_succ,toascii(39),sizeof(change_succ));
				straddns(change_succ,mysql_quote(thread,channels.row[4]),sizeof(change_succ));
				straddnc(change_succ,toascii(39),sizeof(change_succ));
				straddnc(change_succ,',',sizeof(change_succ));
			}
		}
	}
					
	chan_delete[strlen(chan_delete)-1]='\0';
	chan_update[strlen(chan_update)-1]='\0';
	change_succ[strlen(change_succ)-1]='\0';
		
	/**** Changing Founder to the Successor (We were Founder, and our Nick was dropped..) ****/
	if(change_succ[0]!='\0')
		domysql(thread,"UPDATE `orion_cs_core` SET `founder`=IF(`founder`='%s',IF(`successor`='%s',NULL,`successor`),`founder`),`successor`=NULL WHERE `channel` IN (%s)",authnick,authnick,change_succ);
	
	/* Delete Akicks for the channels! */
	if(chan_update[0]!='\0')
		domysql(thread,"DELETE FROM `orion_cs_akick` WHERE `channel` IN (%s)",chan_update);
	
	/* Delete EMPTY Channels */
	if(chan_delete[0]!='\0')
		domysql(thread,"DELETE FROM `orion_cs_core` WHERE `channel` IN (%s)",chan_delete);
	
	/* Delete our Group Information */
	domysql(thread,"DELETE `orion_ns_core`,`orion_ms_core` FROM `orion_ns_core` LEFT OUTER JOIN `orion_ms_core` ON (`orion_ns_core`.`authnick`=`orion_ms_core`.`receiver`) WHERE `orion_ns_core`.`authnick`='%s'",mysql_quote(thread,authnick));
	
	/* Update NOT EMPTY channels ! (make them not registered) */
	if(chan_update[0]!='\0')
		domysql(thread,
			"UPDATE `orion_cs_core` SET "
						
				"`botnick`=NULL,`serv`='',`founder`=NULL,`successor`=NULL,`mode`=REPLACE(`mode`,'r',''),`email`='',`settings`='',`regdate`='',`description`='',`mlock_positive`='',"
				"`mlock_negative`='',`mlock_limit`='',`mlock_key`='',`mlock_flood`='',`mlock_redirect`='',`url`='',`alimit`='',`flags`='',"
				"`welcome`='',`last_used`='',`reason`='',`expire`='',`maxmemos`='',"
				"`cs_set`='',`cs_autodeop`='',`cs_autohalfop`='',`cs_autoop`='',`cs_autoprotect`='',`cs_autovoice`='',"
				"`cs_nojoin`='',`cs_signkick`='',`cs_acclist`='',`cs_accchange`='',`cs_akick`='',`cs_ban`='',`cs_banme`='',"
				"`cs_cclear`='',`cs_getkey`='',`cs_halfop`='',`cs_halfopme`='',`cs_info`='',`cs_kick`='',`cs_kickme`='',`cs_invite`='',"
				"`cs_opdeop`='',`cs_opdeopme`='',`cs_protect`='',`cs_protectme`='',`cs_topic`='',`cs_unban`='',`cs_voice`='',"
				"`cs_voiceme`='',`cs_memo`='',`cs_botassign`='',`cs_badwords`='',`cs_fantasia`='',`cs_greet`='',`cs_nokick`='',"
				"`cs_say`='',`bs_set`='-1',`bs_caps_ttb`='-1',`bs_caps_min`='-1',`bs_caps_per`='-1',`bs_flood_ttb`='-1',`bs_flood_ln`='-1',`bs_flood_secs`='-1',`bs_repeat_ttb`='-1',`bs_repeat`='-1',"
				"`bs_colors`='-1',`bs_underlines`='-1',`bs_reverses`='-1',`bs_badwords`='-1',`bs_bolds`='-1'"			
	
			" WHERE `orion_cs_core`.`channel` IN (%s)",chan_update
		);

	/* MySQL Free Result */
	mysql_free_result_myr(channels.res);
	return;
}

/*  ************************************************************************************************  */

/**
 * @brief sql_thread_data:  Selects information for a channel
 *
 * @param thread The thread-struct (passed from active thread)- holds the threads information
 * @param starttype The start parameter for the query
 * @param parameters The number of different information which should be returned
 * @param ... following types are allowed: CHANNEL,WITH_LEVELS,WITH_BOTSERV,COUNTUSERS,OWNACCLEVEL,TARGETACCLEVEL
 * @return DB4 The Channel-Struct from the Channel-Database
 **/
DB4 sql_thread_data(Threads *thread, char* starttype, int parameters, ...){
	int i=0,j=0,adddata=0,fields=0,param_now=0,join_aliases=0;
	int start_from[MAX_CHANDATA_TYPES],needed_param[MAX_CHANDATA_TYPES],num_fields[MAX_CHANDATA_TYPES];
	
	char temp_fields[SQL_BUF];
	char select_fields[SQL_BUF];
	char select_from[SQL_BUF];
	char select_join[SQL_BUF];
	char select_where[SQL_BUF];
	char nick[MAX_NICK_LEN];
	char tempauthnick[MAX_NICK_LEN];
	char channel[MAX_CHANNELNAME_LEN];
	
	char* arg;
	deepSQL chandata;
	va_list options;
	/* STRUCT */ DB4 c; /* STRUCT */
	
	bzero(&select_fields,sizeof(select_fields));
	bzero(&select_from,sizeof(select_from));
	bzero(&select_join,sizeof(select_join));
	bzero(&select_where,sizeof(select_where));
	bzero(&nick,sizeof(nick));
	bzero(&channel,sizeof(channel));
	bzero(&c,sizeof(c));
	bzero(&start_from,sizeof(start_from));
	bzero(&needed_param,sizeof(needed_param));
	bzero(&num_fields,sizeof(num_fields));
	
	nick[0]='\0';
	channel[0]='\0';
	
	// reset the returning ChannelDB structure
	c.access.fields = 0;
	c.aliases.fields = 0;
	c.chan.fields = 0;
	c.Cdata.fields = 0;
	
	// build the SQL query...
	va_start(options, parameters);

	for(i=0;i<parameters;i++){
		adddata+=va_arg(options,int);
	}

	if(!(adddata&CHANNEL)) adddata+=CHANNEL;
	dynncpy(channel,starttype,sizeof(channel));

	if(adddata&WITH_LEVELS){
		if(!(adddata&CHANNEL)) adddata+=CHANNEL;

		c.chan.levels.accchange=0;
		c.chan.levels.acclist=0;
		c.chan.levels.akick=0;
		c.chan.levels.autodeop=0;
		c.chan.levels.autohalfop=0;
		c.chan.levels.autoop=0;
		c.chan.levels.autoprotect=0;
		c.chan.levels.autovoice=0;
		c.chan.levels.badwords=0;
		c.chan.levels.ban=0;
		c.chan.levels.banme=0;
		c.chan.levels.assign=0;
		c.chan.levels.clear=0;
		c.chan.levels.fantasia=0;
		c.chan.levels.getkey=0;
		c.chan.levels.greet=0;
		c.chan.levels.halfop=0;
		c.chan.levels.halfopme=0;
		c.chan.levels.info=0;
		c.chan.levels.invite=0;
		c.chan.levels.kick=0;
		c.chan.levels.kickme=0;
		c.chan.levels.memo=0;
		c.chan.levels.nojoin=0;
		c.chan.levels.nokick=0;
		c.chan.levels.opdeop=0;
		c.chan.levels.opdeopme=0;
		c.chan.levels.protect=0;
		c.chan.levels.protectme=0;
		c.chan.levels.say=0;
		c.chan.levels.set=0;
		c.chan.levels.signkick=0;
		c.chan.levels.topic=0;
		c.chan.levels.unban=0;
		c.chan.levels.voice=0;
		c.chan.levels.voiceme=0;

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,
				"`orion_cs_core`.`cs_accchange`,`orion_cs_core`.`cs_acclist`,"
				"`orion_cs_core`.`cs_akick`,`orion_cs_core`.`cs_autodeop`,`orion_cs_core`.`cs_autohalfop`,`orion_cs_core`.`cs_autoop`,"
				"`orion_cs_core`.`cs_autoprotect`,`orion_cs_core`.`cs_autovoice`,`orion_cs_core`.`cs_badwords`,`orion_cs_core`.`cs_ban`,`orion_cs_core`.`cs_banme`,"
				"`orion_cs_core`.`cs_botassign`,`orion_cs_core`.`cs_cclear`,`orion_cs_core`.`cs_fantasia`,`orion_cs_core`.`cs_getkey`,`orion_cs_core`.`cs_greet`,`orion_cs_core`.`cs_halfop`,`orion_cs_core`.`cs_halfopme`,"
				"`orion_cs_core`.`cs_info`,`orion_cs_core`.`cs_invite`,`orion_cs_core`.`cs_kick`,`orion_cs_core`.`cs_kickme`,`orion_cs_core`.`cs_memo`,`orion_cs_core`.`cs_nojoin`,"
				"`orion_cs_core`.`cs_nokick`,`orion_cs_core`.`cs_opdeop`,`orion_cs_core`.`cs_opdeopme`,`orion_cs_core`.`cs_protect`,"
				"`orion_cs_core`.`cs_protectme`,`orion_cs_core`.`cs_say`,`orion_cs_core`.`cs_set`,`orion_cs_core`.`cs_signkick`,"
				"`orion_cs_core`.`cs_topic`,`orion_cs_core`.`cs_unban`,`orion_cs_core`.`cs_voice`,`orion_cs_core`.`cs_voiceme`",
		sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));

		//printf("CLevels fields = %d - 36\n",findchar(temp_fields,',')+1); // 36
		num_fields[WITH_LEVELS_ARR]=findchar(temp_fields,',')+1;
		
		start_from[WITH_LEVELS_ARR]=fields;
		fields+=num_fields[WITH_LEVELS_ARR];
	}
	if(adddata&WITH_BOTSERV){
		if(!(adddata&CHANNEL)) adddata+=CHANNEL;

		c.chan.botserv.set=-1;
		c.chan.botserv.caps_ttb=-1;
		c.chan.botserv.caps_min=-1;
		c.chan.botserv.caps_per=-1;
		c.chan.botserv.flood_ttb=-1;
		c.chan.botserv.flood_ln=-1;
		c.chan.botserv.flood_secs=-1;
		c.chan.botserv.repeat_ttb=-1;
		c.chan.botserv.repeat=-1;
		c.chan.botserv.colors=-1;
		c.chan.botserv.underlines=-1;
		c.chan.botserv.reverses=-1;
		c.chan.botserv.badwords=-1;
		c.chan.botserv.bolds=-1;

		//num_fields[WITH_BOTSERV_ARR]=14;
		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,
			"`orion_cs_core`.`bs_set`,`orion_cs_core`.`bs_caps_ttb`,`orion_cs_core`.`bs_caps_min`,`orion_cs_core`.`bs_caps_per`,`orion_cs_core`.`bs_flood_ttb`,`orion_cs_core`.`bs_flood_ln`,`orion_cs_core`.`bs_flood_secs`,`orion_cs_core`.`bs_repeat_ttb`,`orion_cs_core`.`bs_repeat`,`orion_cs_core`.`bs_colors`,"
			"`orion_cs_core`.`bs_underlines`,`orion_cs_core`.`bs_reverses`,`orion_cs_core`.`bs_badwords`,`orion_cs_core`.`bs_bolds`",
		sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("BotServ fields = %d - 14\n",findchar(temp_fields,',')+1); //14
		num_fields[WITH_BOTSERV_ARR]=findchar(temp_fields,',')+1;
		
		start_from[WITH_BOTSERV_ARR]=fields;
		fields+=num_fields[WITH_BOTSERV_ARR];
	}
	if(adddata&COUNTUSERS){
		if(!(adddata&CHANNEL)) adddata+=CHANNEL;

		c.chan.countusers = -1;
		c.chan.botinchan = 0;

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,"COUNT(`orion_chan_users`.`nick`) as `count_users`, SUM(IF(`orion_chan_users`.`nick`=`orion_cs_core`.`botnick`,1,0)) as `botinchan`",sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
				
		printf("Countusers fields = %d - 2\n",findchar(temp_fields,',')+1-2); //2
		num_fields[COUNTUSERS_ARR]=findchar(temp_fields,',')+1-2;
		
		start_from[COUNTUSERS_ARR]=fields;
		fields+=num_fields[COUNTUSERS_ARR];
		needed_param[param_now++]=COUNTUSERS;
	}
	if(adddata&TARGETACCLEVEL_FROM_NICK){
		c.chan.access.targetlevel[0]='\0';
		c.chan.access.targetauthnick[0]='\0';

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,"`target`.`authnick` as `targetauthnick`,`target`.`level` as `targetacclevel`",sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("targetacclevel from nick fields = %d - 2\n",findchar(temp_fields,',')+1); //2
		num_fields[TARGETACCLEVEL_ARR]=findchar(temp_fields,',')+1;
		
		start_from[TARGETACCLEVEL_ARR]=fields;
		fields+=num_fields[TARGETACCLEVEL_ARR];
		needed_param[param_now++]=TARGETACCLEVEL;
	}
	if(adddata&OWNACCLEVEL){
		c.chan.access.ownlevel[0]='\0';
		c.chan.access.ownauthnick[0]='\0';

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,"`own`.`authnick` as `ownauthnick`, `own`.`level` as `ownacclevel`",sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("Ownacclevel fields = %d - 2\n",findchar(temp_fields,',')+1); //2
		num_fields[OWNACCLEVEL_ARR]=findchar(temp_fields,',')+1;
		
		start_from[OWNACCLEVEL_ARR]=fields;
		fields+=num_fields[OWNACCLEVEL_ARR];
		needed_param[param_now++]=OWNACCLEVEL;

	}
	if(adddata&ALIASES){
		c.aliases.nick[0]='\0';
		c.aliases.authnick[0]='\0';
		c.aliases.last_vhost[0]='\0';
		c.aliases.last_realname[0]='\0';
		c.aliases.last_quit[0]='\0';
		c.aliases.time_linked=(time_t)0;
		c.aliases.last_login=(time_t)0;
		c.aliases.last_used=(time_t)0;
		c.aliases.active=0;
		c.aliases.expire=0;
		c.aliases.forbid=0;
		c.aliases.linked=1;

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,"`orion_ns_aliases`.`nick`,`orion_ns_aliases`.`display`,`orion_ns_aliases`.`last_usermask`,`orion_ns_aliases`.`last_realname`,`orion_ns_aliases`.`last_quit`,`orion_ns_aliases`.`time_linked`,`orion_ns_aliases`.`last_login`,`orion_ns_aliases`.`last_used`,`orion_ns_aliases`.`expire`,`orion_ns_aliases`.`linked`,`orion_ns_aliases`.`active`,`orion_ns_aliases`.`forbid`",sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("Aliases fields = %d - 12\n",findchar(temp_fields,',')+1); //12
		num_fields[ALIASES_ARR]=findchar(temp_fields,',')+1;
		
		start_from[ALIASES_ARR]=fields;
		fields+=num_fields[ALIASES_ARR];
		needed_param[param_now++]=ALIASES;
	}
	if(adddata&ACCESS || adddata&ACCESS_FROM_NICK){
		int temp_arr = (adddata&ACCESS ? ACCESS_ARR : ACCESS_FROM_NICK_ARR);
		int temp = (adddata&ACCESS ? ACCESS : ACCESS_FROM_NICK);

		c.access.authnick[0]='\0';
		c.access.password[0]='\0';
		c.access.reason[0]='\0';
		c.access.qlevel=0;
		c.access.olevel=0;
		c.access.slevel=0;
		c.access.llevel=0;
		c.access.email[0]='\0';
		c.access.greet[0]='\0';
		c.access.url[0]='\0';
		c.access.icq=0;
		c.access.yahoo[0]='\0';
		c.access.location[0]='\0';
		c.access.birthday[0]='\0';
		c.access.realname[0]='\0';
		c.access.gender=0;
		c.access.interests[0]='\0';
		c.access.profession[0]='\0';
		c.access.slogan[0]='\0';
		c.access.language=m->language;
		c.access.rights=is_user;
		c.access.settings=0;
		c.access.maxmemos=0;
		c.access.failed_logins=0;
		c.access.last_failed_login_time=(time_t)0;
		c.access.last_failed_login_password[0]='\0';

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,"`orion_ns_core`.`authnick`,`orion_ns_core`.`password`,`orion_ns_core`.`reason`,`orion_ns_core`.`qlevel`,`orion_ns_core`.`olevel`,`orion_ns_core`.`slevel`,`orion_ns_core`.`llevel`,`orion_ns_core`.`email`,`orion_ns_core`.`greet`,`orion_ns_core`.`url`,`orion_ns_core`.`icq`,`orion_ns_core`.`yahoo`,`orion_ns_core`.`location`,`orion_ns_core`.`birthday`,`orion_ns_core`.`realname`,`orion_ns_core`.`gender`,`orion_ns_core`.`interests`,`orion_ns_core`.`profession`,`orion_ns_core`.`slogan`,`orion_ns_core`.`language`,`orion_ns_core`.`rights`,`orion_ns_core`.`settings`,`orion_ns_core`.`maxmemos`,`orion_ns_core`.`failed_logins`,`orion_ns_core`.`last_failed_login`,`orion_ns_core`.`last_login_password`",sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("Access fields = %d - 26\n",findchar(temp_fields,',')+1); //26
		num_fields[temp_arr]=findchar(temp_fields,',')+1;
		
		start_from[temp_arr]=fields;
		fields+=num_fields[temp_arr];
		needed_param[param_now++]=temp;
	}
	if(adddata&CHANNEL){
		c.chan.fields=0;
		c.chan.name[0]='\0';
		c.chan.bot[0]='\0';
		c.chan.serv[0]='\0';
		c.chan.founder[0]='\0';
		c.chan.successor[0]='\0';
		c.chan.datecreate=(time_t)0;
		c.chan.regdate=(time_t)0;
		c.chan.description[0]='\0';
		c.chan.mode[0]='\0';
		c.chan.mlock_positive[0]='\0';
		c.chan.mlock_negative[0]='\0';
		c.chan.mlock_limit=0;
		c.chan.mlock_key[0]='\0';
		c.chan.mlock_flood[0]='\0';
		c.chan.mlock_redirect[0]='\0';
		c.chan.link[0]='\0';
		c.chan.limit=0;
		c.chan.key[0]='\0';
		c.chan.flood[0]='\0';
		c.chan.last_topic[0]='\0';
		c.chan.last_topic_setter[0]='\0';
		c.chan.last_topic_time=(time_t)0;
		c.chan.url[0]='\0';
		c.chan.alimit=0;
		c.chan.flags[0]='\0';
		c.chan.welcome[0]='\0';
		c.chan.last_used=(time_t)0;
		c.chan.settings=0;
		c.chan.email[0]='\0';
		c.chan.expire=1;
		c.chan.maxmemos=0;
		c.chan.reason[0]='\0';
		c.chan.countusers=-1;

		c.chan.levels.fields=0;
		c.chan.access.fields=0;
		c.chan.botserv.fields=0;

		if(channel[0]!='\0'){
			straddns(select_from,"`orion_cs_core`",sizeof(select_from));
			snprintf(select_where,sizeof(select_where),"`orion_cs_core`.`channel` = '%s'",mysql_quote(thread,channel));
		}else
			needed_param[param_now++]=CHANNEL;

		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,
			"`orion_cs_core`.`serv`,`orion_cs_core`.`botnick`,`orion_cs_core`.`channel`,`orion_cs_core`.`founder`,`orion_cs_core`.`successor`,`orion_cs_core`.`datecreate`,`orion_cs_core`.`regdate`,"
			"`orion_cs_core`.`description`,`orion_cs_core`.`mode`,`orion_cs_core`.`mlock_positive`,`orion_cs_core`.`mlock_negative`,"
			"`orion_cs_core`.`mlock_limit`,`orion_cs_core`.`mlock_key`,`orion_cs_core`.`mlock_flood`,`orion_cs_core`.`mlock_redirect`,`orion_cs_core`.`link`,"
			"`orion_cs_core`.`limit`,`orion_cs_core`.`key`,`orion_cs_core`.`flood`,`orion_cs_core`.`last_topic`,`orion_cs_core`.`last_topic_setter`,"
			"`orion_cs_core`.`last_topic_time`,`orion_cs_core`.`url`,`orion_cs_core`.`alimit`,`orion_cs_core`.`flags`,`orion_cs_core`.`welcome`,"
			"`orion_cs_core`.`last_used`,`orion_cs_core`.`settings`,"
			"`orion_cs_core`.`email`,`orion_cs_core`.`expire`,`orion_cs_core`.`maxmemos`,`orion_cs_core`.`reason`",
		sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("Channel fields = %d - 32\n",findchar(temp_fields,',')+1); //32
		num_fields[CHANNEL_ARR]=findchar(temp_fields,',')+1;
		
		start_from[CHANNEL_ARR]=fields;
		fields+=num_fields[CHANNEL_ARR];
	}
	if(adddata&CONDATA){
		c.Cdata.nick[0]='\0';
		c.Cdata.time=(time_t)0;
		c.Cdata.localtime=(time_t)0;
		c.Cdata.ident[0]='\0';
		c.Cdata.rhost[0]='\0';
		c.Cdata.server[0]='\0';
		c.Cdata.modes[0]='\0';
		c.Cdata.vhost[0]='\0';
		c.Cdata.realname[0]='\0';
		c.Cdata.swhois[0]='\0';
		c.Cdata.subnet[0]='\0';
		c.Cdata.ns_warntime=(time_t)0;
		c.Cdata.faketype=0;

		if(nick[0]!='\0'){
			straddns(select_from,"`orion_core_condata`",sizeof(select_from));
			snprintf(select_where,sizeof(select_where),"`orion_core_condata`.`nick` = '%s'",mysql_quote(thread,nick));
		}else
			needed_param[param_now++]=CONDATA;
			
		if(select_fields[0]!='\0') straddnc(select_fields,',',sizeof(select_fields));
		
		bzero(&temp_fields,sizeof(temp_fields));
		dynncpy(temp_fields,"`orion_core_condata`.`nick`,`orion_core_condata`.`time`,`orion_core_condata`.`localtime`,`orion_core_condata`.`rident`,`orion_core_condata`.`ident`,`orion_core_condata`.`rhost`,`orion_core_condata`.`srv`,`orion_core_condata`.`modes`,`orion_core_condata`.`vhost`,`orion_core_condata`.`realname`,`orion_core_condata`.`swhois`,`orion_core_condata`.`subnet`,`orion_core_condata`.`ip`,`orion_core_condata`.`ns_warntime`,`orion_core_condata`.`faketype`",sizeof(temp_fields));
		straddns(select_fields,temp_fields,sizeof(select_fields));
		
		//printf("Condata fields = %d - 13\n",findchar(temp_fields,',')+1); //13
		num_fields[CONDATA_ARR]=findchar(temp_fields,',')+1;
		
		start_from[CONDATA_ARR]=fields;
		fields+=num_fields[CONDATA_ARR];
	}

	for(i=0;i<param_now;i++){
		//printf("PARAM: %d -- %d \n",i,needed_param[i]);
		switch(needed_param[i]){
			case CHANNEL:
				if(channel[0]=='\0') dynncpy(channel,arg = va_arg(options,char *),sizeof(channel));
				stradds(select_from,_dynadd(" LEFT OUTER JOIN `orion_cs_core` ON `orion_cs_core`.`channel` ='%s'",mysql_quote(thread,channel)));
				break;
			case CONDATA:
				if(nick[0]=='\0') dynncpy(nick,arg = va_arg(options,char *),sizeof(nick));
				stradds(select_from,_dynadd(" LEFT OUTER JOIN `orion_core_condata` ON `orion_core_condata`.`nick` ='%s'",mysql_quote(thread,nick)));
				break;
			case COUNTUSERS:
				stradds(select_join," LEFT OUTER JOIN `orion_chan_users` ON `orion_chan_users`.`channel` = `orion_cs_core`.`channel`");
				stradds(select_where," GROUP BY `orion_chan_users`.`channel`");
				break;
			case TARGETACCLEVEL:
				if(nick[0]=='\0') dynncpy(nick,arg = va_arg(options,char *),sizeof(nick));
				join_aliases=1;
				stradds(select_join,_dynadd(" LEFT OUTER JOIN `orion_cs_access` AS `target` ON (`orion_ns_aliases`.`display`=`target`.`authnick` AND `target`.`channel`='%s')",mysql_quote(thread,channel)));
				break;
			case OWNACCLEVEL:
				arg = va_arg(options,char *);
				bzero(&tempauthnick,sizeof(tempauthnick));
				dynncpy(tempauthnick,arg,sizeof(tempauthnick));
				stradds(select_join,_dynadd(" LEFT OUTER JOIN `orion_cs_access` AS `own` ON ( `own`.`channel` = `orion_cs_core`.`channel` AND `own`.`authnick` = '%s')",mysql_quote(thread,arg)));
				break;
			case ALIASES:
				if(nick[0]=='\0') dynncpy(nick,arg = va_arg(options,char *),sizeof(nick));
				join_aliases=1;
				break;
			case ACCESS:
				arg = va_arg(options,char *);
				stradds(select_join,_dynadd(" LEFT OUTER JOIN `orion_ns_core` ON `orion_ns_core`.`authnick`='%s'",mysql_quote(thread,arg)));
				break;
			case ACCESS_FROM_NICK:
				if(nick[0]=='\0') dynncpy(nick,arg = va_arg(options,char *),sizeof(nick));
				join_aliases=1;
				stradds(select_join," LEFT OUTER JOIN `orion_ns_core` ON `orion_ns_core`.`authnick`=`orion_ns_aliases`.`display`");
				break;
		}
	}

	va_end(options);

	if(join_aliases==1) stradds(select_from,_dynadd(" LEFT OUTER JOIN `orion_ns_aliases` ON (`orion_ns_aliases`.`nick` = '%s')"/* AND `orion_ns_aliases`.`active`=1)"*/,mysql_quote(thread,nick)));

	// get the requested chandata..
	//dlog(thread,4,"QRY: SELECT %s FROM %s%s WHERE %s LIMIT 0,1\n",select_fields,select_from,select_join,select_where);
	chandata.res = doSelectMySQL(thread,"SELECT %s FROM %s%s WHERE %s LIMIT 0,1",select_fields,select_from,select_join,select_where);
	
	if(mysql_num_rows(chandata.res)>0){
		chandata.row = mysql_fetch_row(chandata.res);
		c.fields = mysql_num_fields(chandata.res);
		
		//dlog(thread,4,"mysql_num_fields: %d\n",mysql_num_fields(chandata.res));
		
		for(i=0;i<MAX_CHANDATA_TYPES;i++){
			if(num_fields[i]==0) continue;
			j=start_from[i];
			switch((1<<i)){
				case CHANNEL:
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.serv,chandata.row[(j-1)],sizeof(c.chan.serv));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.bot,chandata.row[(j-1)],sizeof(c.chan.bot));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.name,chandata.row[(j-1)],sizeof(c.chan.name));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.founder,chandata.row[(j-1)],sizeof(c.chan.founder));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.successor,chandata.row[(j-1)],sizeof(c.chan.successor));
					if(chandata.row[j++]!=NULL) c.chan.datecreate=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.regdate=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.description,chandata.row[(j-1)],sizeof(c.chan.description));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.mode,chandata.row[(j-1)],sizeof(c.chan.mode));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.mlock_positive,chandata.row[(j-1)],sizeof(c.chan.mlock_positive));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.mlock_negative,chandata.row[(j-1)],sizeof(c.chan.mlock_negative));
					if(chandata.row[j++]!=NULL) c.chan.mlock_limit=stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.mlock_key,chandata.row[(j-1)],sizeof(c.chan.mlock_key));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.mlock_flood,chandata.row[(j-1)],sizeof(c.chan.mlock_flood));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.mlock_redirect,chandata.row[(j-1)],sizeof(c.chan.mlock_redirect));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.link,chandata.row[(j-1)],sizeof(c.chan.link));
					if(chandata.row[j++]!=NULL) c.chan.limit=stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.key,chandata.row[(j-1)],sizeof(c.chan.key));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.flood,chandata.row[(j-1)],sizeof(c.chan.flood));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.last_topic,chandata.row[(j-1)],sizeof(c.chan.last_topic));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.last_topic_setter,chandata.row[(j-1)],sizeof(c.chan.last_topic_setter));
					if(chandata.row[j++]!=NULL) c.chan.last_topic_time=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.url,chandata.row[(j-1)],sizeof(c.chan.url));
					if(chandata.row[j++]!=NULL) c.chan.alimit=stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.flags,chandata.row[(j-1)],sizeof(c.chan.flags));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.welcome,chandata.row[(j-1)],sizeof(c.chan.welcome));
					if(chandata.row[j++]!=NULL) c.chan.last_used=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.settings=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.email,chandata.row[(j-1)],sizeof(c.chan.email));
					if(chandata.row[j++]!=NULL) c.chan.expire=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.maxmemos=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.reason,chandata.row[(j-1)],sizeof(c.chan.reason));
					c.chan.fields=num_fields[i];
					break;
				case WITH_LEVELS:
					if(chandata.row[j++]!=NULL) c.chan.levels.accchange=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.acclist=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.akick=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.autodeop=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.autohalfop=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.autoop=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.autoprotect=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.autovoice=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.badwords=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.ban=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.banme=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.assign=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.clear=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.fantasia=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.getkey=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.greet=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.halfop=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.halfopme=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.info=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.invite=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.kick=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.kickme=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.memo=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.nojoin=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.nokick=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.opdeop=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.opdeopme=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.protect=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.protectme=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.say=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.set=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.signkick=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.topic=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.unban=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.voice=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.levels.voiceme=stoi(chandata.row[(j-1)]);
					c.chan.levels.fields=num_fields[i];
					break;
				case WITH_BOTSERV:
					if(chandata.row[j++]!=NULL) c.chan.botserv.set=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.caps_ttb=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.caps_min=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.caps_per=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.flood_ttb=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.flood_ln=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.flood_secs=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.repeat_ttb=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.repeat=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.colors=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.underlines=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.reverses=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.badwords=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botserv.bolds=stoi(chandata.row[(j-1)]);
					c.chan.botserv.fields=num_fields[i];
					break;
				case COUNTUSERS:
					if(chandata.row[j++]!=NULL) c.chan.countusers=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.chan.botinchan=stoi(chandata.row[(j-1)]);
					c.chan.fields=num_fields[i];
					break;
				case TARGETACCLEVEL:
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.access.targetauthnick,chandata.row[(j-1)],sizeof(c.chan.access.targetauthnick));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.access.targetlevel,chandata.row[(j-1)],sizeof(c.chan.access.targetlevel));
					c.chan.access.fields=num_fields[i];
					//c.chan.access.fields++;
					break;
				case OWNACCLEVEL:
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.access.ownauthnick,chandata.row[(j-1)],sizeof(c.chan.access.ownauthnick));
					if(chandata.row[j++]!=NULL) dynncpy(c.chan.access.ownlevel,chandata.row[(j-1)],sizeof(c.chan.access.ownlevel));
					c.chan.access.fields=num_fields[i];
					break;
				case CONDATA:
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.nick,chandata.row[(j-1)],sizeof(c.Cdata.nick));
					if(chandata.row[j++]!=NULL) c.Cdata.time=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.Cdata.localtime=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.rident,chandata.row[(j-1)],sizeof(c.Cdata.rident));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.ident,chandata.row[(j-1)],sizeof(c.Cdata.ident));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.rhost,chandata.row[(j-1)],sizeof(c.Cdata.rhost));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.server,chandata.row[(j-1)],sizeof(c.Cdata.server));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.modes,chandata.row[(j-1)],sizeof(c.Cdata.modes));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.vhost,chandata.row[(j-1)],sizeof(c.Cdata.vhost));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.realname,chandata.row[(j-1)],sizeof(c.Cdata.realname));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.swhois,chandata.row[(j-1)],sizeof(c.Cdata.swhois));
					if(chandata.row[j++]!=NULL) dynncpy(c.Cdata.subnet,chandata.row[(j-1)],sizeof(c.Cdata.subnet));
					if(chandata.row[j++]!=NULL) c.Cdata.ns_warntime=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.Cdata.faketype=stoi(chandata.row[(j-1)]);
					c.Cdata.fields=num_fields[i];
					break;
				case ALIASES:
					if(chandata.row[j++]!=NULL) dynncpy(c.aliases.nick,chandata.row[(j-1)],sizeof(c.aliases.nick));
					if(chandata.row[j++]!=NULL) dynncpy(c.aliases.authnick,chandata.row[(j-1)],sizeof(c.aliases.authnick));
					if(chandata.row[j++]!=NULL) dynncpy(c.aliases.last_vhost,chandata.row[(j-1)],sizeof(c.aliases.last_vhost));
					if(chandata.row[j++]!=NULL) dynncpy(c.aliases.last_realname,chandata.row[(j-1)],sizeof(c.aliases.last_realname));
					if(chandata.row[j++]!=NULL) dynncpy(c.aliases.last_quit,chandata.row[(j-1)],sizeof(c.aliases.last_quit));
					if(chandata.row[j++]!=NULL) c.aliases.time_linked=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.aliases.last_login=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.aliases.last_used=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.aliases.expire=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.aliases.linked=stoi(chandata.row[(j-1)]); //0-permanent, 1-temporarily
					if(chandata.row[j++]!=NULL) c.aliases.active=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.aliases.forbid=stoi(chandata.row[(j-1)]);
					c.aliases.fields=num_fields[i];
					break;
				case ACCESS:
				case ACCESS_FROM_NICK:
					if(chandata.row[j++]!=NULL) dynncpy(c.access.authnick,chandata.row[(j-1)],sizeof(c.access.authnick));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.password,chandata.row[(j-1)],sizeof(c.access.password));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.reason,chandata.row[(j-1)],sizeof(c.access.reason));
					if(chandata.row[j++]!=NULL) c.access.qlevel=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.olevel=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.slevel=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.llevel=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.access.email,chandata.row[(j-1)],sizeof(c.access.email));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.greet,chandata.row[(j-1)],sizeof(c.access.greet));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.url,chandata.row[(j-1)],sizeof(c.access.url));
					if(chandata.row[j++]!=NULL) c.access.icq=stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.access.yahoo,chandata.row[(j-1)],sizeof(c.access.yahoo));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.location,chandata.row[(j-1)],sizeof(c.access.location));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.birthday,chandata.row[(j-1)],sizeof(c.access.birthday));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.realname,chandata.row[(j-1)],sizeof(c.access.realname));
					if(chandata.row[j++]!=NULL) c.access.gender=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.access.interests,chandata.row[(j-1)],sizeof(c.access.interests));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.profession,chandata.row[(j-1)],sizeof(c.access.profession));
					if(chandata.row[j++]!=NULL) dynncpy(c.access.slogan,chandata.row[(j-1)],sizeof(c.access.slogan));
					if(chandata.row[j++]!=NULL) c.access.language=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.rights=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.settings=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.maxmemos=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.failed_logins=stoi(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) c.access.last_failed_login_time=(time_t)stol(chandata.row[(j-1)]);
					if(chandata.row[j++]!=NULL) dynncpy(c.access.last_failed_login_password,chandata.row[(j-1)],sizeof(c.access.last_failed_login_password));
					c.access.fields=num_fields[i];
					break;
			}
		}
	}
	
	//printf("c.chan.serv = %s\n",c.chan.serv);
	
	if(adddata&TARGETACCLEVEL_FROM_NICK){
		if((c.access.authnick[0]=='\0' && c.aliases.active==0) && streqnc(c.chan.serv,"chanserv"))
			unknown_access(c.chan.access.targetlevel,sizeof(c.chan.access.targetlevel));
	}
	if(adddata&OWNACCLEVEL){
		if(tempauthnick[0]=='\0' && streqnc(c.chan.serv,"chanserv"))
			unknown_access(c.chan.access.ownlevel,sizeof(c.chan.access.ownlevel));
	}
	mysql_free_result_myr(chandata.res);
	return c;
}

/*  ************************************************************************************************  */
